array set versioninfo { version 4.0 date Oct-1-2019 }
array set platforminfo { platform rhlinux }

set mswindows 0
set unixplatform 0
set macplatform 0
set msAPPDATA ""
set pbcorLastData ""
set os $tcl_platform(os)
if { [string match -nocase win* $os] } {
    set mswindows 1
} elseif [string match -nocase linux* $os] {
	set unixplatform 1
} elseif [string match -nocase Darwin* $os] {
	set macplatform 1
}

set txtwarn ""

set plotprev ""

set warnStack {}
set msgfont ""
array set dialog {}

# not using gnuplot
set GNU 0

# Use the same distibution check as for fits
set pbcorDir ""
if {$mswindows} {
	set homedir C:/Users
	set althomedir ""
} else {
	set homedir /home
	set althomedir /usr/local/lib
}
# first see if user is starting pbcor64.exe from pwd for example by double clicking pbcor64.exe in Windows
# or typein it on cmnd line in unix. Distribution must have the pbcor64 and the Tclpbs folder
set fchk [pwd]
if {$mswindows} {
	set fchk2 [file join $fchk pbcor64.exe]
} else {
	set fchk2 [file join $fchk pbcor64]
}
set fchk3 [file join $fchk Tclpbs]
if {[file exists $fchk2] && [file isdirectory $fchk3]} {
	set pbcorDir $fchk
}
# if pwd doesnt work try env then defaults
if { [string length $pbcorDir] < 2 } {
	set fchk ""
	if {[info exists env(USERNAME)] && [string length $env(USERNAME)] > 0} {
		set fchk [file join $homedir $env(USERNAME) Pbcor]
	}
	set fchk2 ""
	if {[file isdirectory $althomedir]} {
		set fchk2 [file join $althomedir Pbcor]
	}
	# first check for distribution directory defined by env
	if {[info exists env(pbcorDir)] && [file isdirectory $env(pbcorDir)]} {
		set pbcorDir $env(pbcorDir)
	# if env(pbcorDir)  fails try homedir env(USERNAME)
	} elseif {[file isdirectory $fchk]} {
		set pbcorDir $fchk
	} elseif {[file isdirectory $fchk2]} {
		set pbcorDir $fchk2
	}
}

# As of version 4.0 a distribution directory is required
# so pbcorDir may NOT be blank

if {[string length $pbcorDir] > 0} {
	lappend auto_path $pbcorDir
} else {
	if { $mswindows } {
	    set distopt "Double-clicking the pbcor executable when it is in the distribution folder should work.\n"
		append distopt "Otherwise the default distribution location is C:/Users/_username/Pbcor\n"
		append distopt "If you don't put it there run the pbcorDir.bat in your distribution folder by double clicking it.\n"
		append distopt "This creates the pbcorDir environment variable that points to your distribution folder.\n"
	} else {
		set distopt "The default location is /home/_username or /usr/local/lib\n"
		append distopt "If you don't put it there set the environment variable pbcorDir to your distribution folder.\n"
		append distopt "In bash: env pbcorDir=/yourPbcorDistributionDirectory\n"
	}
	Dialog_Warrn "ERROR: Could not locate Pbcor distribution folder with pbcor64.exe and Tclpbs folder.\n$distopt"
	exit
}

# load platforminfo and versioninfo scripts if they exist in the distribution
set pinfo [file join $pbcorDir platforminfo.tcl]
if [file exists $pinfo] { source $pinfo }
set vinfo [file join $pbcorDir versioninfo.tcl]
if [file exists $vinfo] { source $vinfo }

# check for background job resources in pbcorDir
set bgExec pbsCL
set bgWatch pbcorWatchServer
if {$mswindows} {
	set bgExec $bgExec.exe
	set bgWatch $bgWatch.exe
	# if the .bat version exists we need to use it as Windows is probably disallowing unloading libraries
	if [file exists [file join $pbcorDir $bgWatch.bat]] {
		set bgWatch $bgWatch.bat
	} else {
		set bgWatch $bgWatch.exe
	}
}
set bgExecOK 0
set bgWatchOK 0
set pbcorWatch 1
set bgExec [file join $pbcorDir $bgExec]
set bgWatch [file join $pbcorDir $bgWatch]
if [file executable $bgExec] { set bgExecOK 1 }
if [file executable $bgWatch] {set bgWatchOK 1 }

# Still building starkit on windows
set isStarKit 0
if [info exists starkit::topdir] {
	set isStarKit 1
	set savetopdir $starkit::topdir
}


########################################################################
# program startup with command line argument processing

set DEBUG 0
set debugFPT ""

# record the startup directory, which could be users DATA directory
set currentDir [pwd]
set dataDir ""
set dataDirSet 0
set saverecoverErr 0
# new flag recoverPlotBySolve can be turned off and then just plot is recovered
# request this also using cmndline arg plot
set recoverPlotBySolve 0

foreach arg $argv {
    if { [string match sav* $arg] || [string match SAV* $arg] } {
	    set saverecoverErr 1
    } elseif { [string match deb* $arg] || [string match DEB* $arg] || \
	     [string match dbg* $arg] || [string match DBG* $arg] } {
	    set DEBUG 1
		puts "set DEBUG= $DEBUG"
		# dont open debugCMDS until the directory is set i df
        #set debugFPT [open debugCMDS w]
        #puts "debugFPT= $debugFPT"
	    # this will open a window to monitor io to pbscript
	    # also for windows, open the console to get puts output
	    if { $mswindows && ! $unixplatform } {
	        console show
	    }
    } elseif { [string match dat* $arg] } {
	    if { $DEBUG } { puts "checking arg=$arg for dataDir" }
	    if { [regexp {dat[^=]*=(.*)$} $arg match dataDir] && \
	        [string length $dataDir] > 0 } {
	        if { $DEBUG } { puts "regexp got dataDir=$dataDir" }
	        if { ! [file exists "$dataDir"] || ! [file isdirectory $dataDir] } {
		        Dialog_Warn "File \"$dataDir\" doesnt exist or not directory."
	        }
	        set dataDirSet 1
	        cd $dataDir
	        set currentDir [pwd]
	    }
    } elseif { [string match noplot* $arg] } {
		set recoverPlotBySolve 0
	} elseif { [string match plot* $arg] } {
		set recoverPlotBySolve 1
    } elseif { [string match gnu* $arg] } {
    	set wantGNU 1
	    if { $DEBUG } { puts "checking arg=$arg for gnu" }
	    if { [regexp {gnu[^=]*=(.*)$} $arg match gnuprog] && \
	        [string length $gnuprog] > 0 } {
	        if { $DEBUG } { puts "regexp got gnuprog=$gnuprog" }
	        if { ! [file exists "$gnuprog"] } {
		        Dialog_Warn "File \"$gnuprog\" doesnt exist"
		        exit
	        }
	        if { ! [file executable "$gnuprog"] } {
		        Dialog_Warn \
		        "File named on command line=\"$gnuprog\" not executable!!"
	    	    exit
	        }
	        set gnuprogfound 1
	    }
    } elseif { ! $pbprogfound } {
	    if { ! [regexp {pbs[^=]*=(.*)$} $arg match pbprog] } {
	        set pbprog $arg
	    }
	    if { ! [file exists "$pbprog"] } {
	        Dialog_Warn "File \"$pbprog\" doesnt exist"
	        exit
	    }
	    if { ! [file executable "$pbprog"] } {
	        Dialog_Warn "File named on command line=\"$pbprog\" not executable!!"
	        exit
	    }
	    set pbprogfound 1
    }
}

# set to 1 if you want a scrollbar on the group-selector
# -1 to move the scrollbar to the screen center side of the frame
set gselSB -1

# global pipe io variables
set readfinished 0
# readline only used by Tclpbs
set readline ""
set readlines {}
set errlines ""
set warnlines ""
set allines ""
set nread 0
set nerr 0
set nwarn 0

set allLines 0
set prepCheck 0

set sellectlist ""

# startup plot window
set iwin 1

############### arrow bitmaps use X11 format strings

set downarrowdata "#define downarrow9x7_width 7
#define downarrow9x7_height 9
static unsigned char downarrow9x7_bits[] = \{
   0x00, 0x08, 0x08, 0x08, 0x49, 0x3e, 0x1c, 0x08, 0x00
};"


set uparrowdata "#define uparrow9x7_width 7
#define uparrow9x7_height 9
static unsigned char uparrow9x7_bits[] = \{
   0x00, 0x08, 0x1c, 0x3e, 0x49, 0x08, 0x08, 0x08, 0x00
};"

set rightarrowdata "#define rightarrow9x6_width 6
#define rightarrow9x6_height 9
static unsigned char rightarrow9x6_bits[] = \{
    0x02, 0x04, 0x0c, 0x18, 0x3f, 0x18, 0x0c, 0x04, 0x02,
};"

set leftarrowdata "#define leftarrow9x6_width 6
#define leftarrow9x6_height 9
static unsigned char leftarrow9x6_bits[] = \{
    0x10, 0x08, 0x0c, 0x06, 0x3f, 0x06, 0x0c, 0x08, 0x10,
};"


image create bitmap rightarrow -data $rightarrowdata
image create bitmap leftarrow  -data $leftarrowdata

image create bitmap uparrow -data $uparrowdata
image create bitmap downarrow  -data $downarrowdata

#filestack globals
set fptstack {}
set curFPT ""

# RWE Feb 2019 start adding background job processing stuff
set ncmnds 0

set scriptext pbcor
set recordFPT ""
set recordFile ""
set recordScriptName ""
set recordScript 0
set recordStarted 0

# Feb 2019 add background stuff
# list of running jobs by fname=unique group outfile-root
# this should be saved in pbsession so running jobs can be checked at restart of pbcor.
set bgRunning {}

set bgRan {}

array set bgRecovered {}
array set pbcorlbxs {}

array set bgjobs {}
# bgjobs is array with filename=elementname
# and value= list = group status filename
# RWE decided not to use bgjobs as it would need to be session saved and updated
# just add groups to the gui as needed and check status from the pbcorBG gui

# default settings is polling on and watchServer enabled
set pollBG 1
set watchBG 1
set pollBGsec 15
# record the polling after id so we can be sure to only ever run once
set bgPolling -1


# pbscript callback for reading
# normally skip all blank lines in output to parse correctly
# if pipeRead is used with formatted output just turn off the skipblanklines
set skipblanklines 1

# storage for group info
set ngroup 0
array set group {}
# the libPB flipper-state representation is 1=offoff 2=onon  3=onoff 4=offon
# in pbgui we use the ICP-ICE    1=offoff 2=onoff 3=offon 4=onon
# fss are the labels in the pbgui constraint matrix
# fs is used as a vector variable for BLT so it will create it as array
# so use fss instead of fs to store the standard flipper-state order
array set fss {1 offoff 2 ONoff 3 offON 4 ONON}
array set cm {}

# monochormator+filter options
set monoOptions [list "Be filter" "PGbt7 no filter" "PGbt7 PGfilter" "PGbt7 velsel" "PGbt7 velsel + PGfilter" "MACS no filter"]
# beam monitor options
set monitorOptions [list "none" "before He3 pol" "after He3 pol"]
# init option values
set monofilter [lindex $monoOptions 1]
set monitor [lindex $monitorOptions 2]

set acszero 0
set acswght 0
set cellFiles {}
set nsl1Files {}
# standard constraint labels
set nsflbls {none nsf=0 ONON=offoff offoff=ONON}
set sflbls {none sf=0 offON=ONoff ONoff=offON}
set nsfcons [lindex $nsflbls 0]
set sfcons [lindex $sflbls 0]
set npts 0
set nmsr 0
set calc 0
set isfop 0
set textf(0) ""
set textf(1) ""
set textf(2) ""
set cellmen ""
set nsl1men ""

# for tolerances
array set stndTolArr {}
array set saveStndTol {}
# notallowed agrees with nomatch in DFint Feb 2019
set notallowed {y e c ce scl fs ts Mon Time S* pt ttm tta I* chi* }
set stndTolNames {}
array set autoStndTol {}

# lbxi listbox indices when selecting or creating file lists
set lbxi {}
set nlbxs 0
array set lbxs {}
set lbxfiles {}

# flipper-state translations gui -> libPB and libPB -> gui
array set fst {1 1 2 3 3 4 4 2}
array set fsg {1 1 2 4 3 2 4 3}

array set fopgroup {}
# overall group scalefactor will get set (if non-zero)
# on first group solution
# user can change this from the Edit menu but must
# re-solve any previously solved groups to update the scale.
# list the soln-scale in the group-ranges toplevel
set grpScale 0
array set grpScales {}

set Xscan 0
set useXscan 1
# user selection of X Y columns use different globals
set XscanU 0
set useXscanU 1
set Xlabel ""
set relaxToler 1

# fast background subtraction is a separate toplevel window
set bg ""
set scl ""
set FBGcps 0
set FBGcpsErr 0

# text file fiewer globals
set tv ""
set txtw ""
set mp 1
set viewfile ""
set natn 1
array set natext {1 .A 2 .B 3 .C 4 .D}

set tvtitle "text file viewer"
set sfvm ""
array set natnlabels {1 "A offoff" 2 "B ONoff" 3 "C offON" 4 "D ONON"}
set mbbg #9999ff
set mbab #ffaa00


# group selector globals
array set srcfiles {}
set ny 0
set ngrpsets 0
set dsns {}
array set isrcplot {}
set solnfiles {}
set solnfile ""
set solnresultfiles {}
array set pset {}
array set mset {}
array set solnsets {}
array set solnresultsets {}

# column select stuff
array set viewcollabels {}
set yviewcollabels {}
set Yselect {}
set Xselect ""
set viewncol 0
set saveset 0
set yselmode 0
set lastyselmode 0
set lastxselmode 0
array set xcolumns {}
array set xvary {}
set ixlst {En x1 x2 x3 tts om Temp Bfld}
set xlst {En x1 x2 x3 ttm tta tts om Temp Bfld}

set inclhead 1
set lbout ""
set lbsrc ""
set lbY ""
set svcws {}
set yselect ""

#plotting
set maxgrpsets 0
set pVecNames {suu sdd sdu sud euu edd edu eud qx qy qz en tts om temp bfld di}
set pDatNames {Suu Sdd Sdu Sud Euu Edd Edu Eud Qx Qy Qz En TTS OM TEMP BFLD DI}
set pStdNames {y1  y2  y3  y4  e1  e2  e3  e4  x1 x2 x3 En tts om Temp Bfld di}

set mVecNames {cc ccerr qxm qym qzm enm ttsm omm fs tempm bfldm dim}
set mDatNames {Cc Ccerr Qxm Qym Qzm Enm TTSm OMm Fs TEMPm BFLDm DIm}
set mStdNames {y1  e1   x1  x2  x3  En  tts  om  fs Temp  Bfld di}
foreach pdat $pDatNames { set $pdat {} }
foreach mdat $mDatNames { set $mdat {} }
set Xpts {}
set Xmsr {}
set iX 1
set iXU 1
set Yi {}
set Src {}
set iSrc {}
set Rf {}
set iWts {}
set suuWts {}
set sddWts {}
set sduWts {}
set sudWts {}
set stylst {}
set sstylst {}
set rstylst {}
array set pen {}
array set spen {}
array set pencolor {}
array set spencolor {}
# use hexColorLst for msr e.g. green->yellow->red
set mcolorFracs {0 0.5 1}
set mcolors {00ff00 ffff00 ff0000}
set scolorFracs {0 0.2 0.4 0.6 0.8 1}
# for single y use blue i.e. frac = 0.2
set scolors {ff00ff 0000ff 00ffff 00ff00 ffff00 ff0000}
set icolor 00ff00
set mcolor ffff00
set fcolor ff0000
# use hexColor3 for dpts blueGreen->Blue->Violet
set iscolor 00ffff
set mscolor 4000ff
set fscolor ff00ff

# symbols for uu dd du ud
array set fsSymb {1 circle 2 square 3 arrow 4 triangle}
# gnu equivalent filled pointstyle numbers from wxt & windows term (pt index)
array set gfsSymb {1 7     2 5      3 11    4 9}
array set bltSymbols {
    1 plus 2 cross
    3 square 4 circle
    5 triangle 6 arrow
    7 diamond
    8 square-fill 9 circle-fill
    10 triangle-fill 11 arrow-fill
    12 diamond-fill
}
set bltSymbolsLst {
    plus cross square circle triangle arrow diamond
    square-fill circle-fill triangle-fill arrow-fill diamond-fill
}
foreach {i val} [array get bltSymbols] { set bltSymbolIndex($val) $i }


# plotting
array set setplot {}
set spsn 0
set pcxye 0
set pcxyelast 0
array set splabelsarr {}
set splabelslst {}

array set nsetfs {}
array set pcolumns {}
array set isrcplot {}
# xaxlbl indexed by iX into stndxcolnames
array set xaxlbl {1 E 2 Qx 3 Qy 4 Qz 5 tts 6 omega 7 Temp 8 Bfld}

set Pyi {}
set Myi {}

array set stndX {1 En 2 x1 3 x2 4 x3 5 tts 6 om 7 Temp 8 Bfld}

set multisoln 1
array set solnindx {}

set msetprev 0
set psetprev 0

set plotbg white
set toscreen 1
array set groupiwin {}
array set viewS {1 suu 2 sdd 3 sdu 4 sud}

array set prevplotwin {}

set g ""
array set eview {suu 1 sdd 1 sdu 1 sud 1 msr 1 R 0 sfm 1 nsfm 1}
set gvtitle "PB plotting"
set gtitlefont [list Helvetica 24 bold]
set titlefont [list Helvetica 20 bold]
set tickfont [list Helvetica 16]
set symbolsizes {0 2 4 6 8 10 12 14 16 18 20}
set imagefmts {gif bmp eps eps2 jpg miff pdf png ppm ps ps2 rgb sgi tiff xpm}
set plotimagetype "gif"

set yctslbl "pb corrected counts"
set y2ctslbl "mon cor msr counts"
set y2Rlbl "flip-ratio"

array set bltPSopt {
    -landscape 0
    -center 1
    -maxpect 0
    -decorations 0
    -colormode color
}

# gws array of BLT graph widgets
# gvs array of toplevels for BLT graphs
array set gws {}
array set gvs {}
set psEdit ""
set psPBmenu ""
set psDSmenu ""
set pbWidgets {}
set dsWidgets {}
set dseditY ""
set dseditm ""
set dsmode 0
set miwin ""
set onePlotPerGrp 1
set rgbColors {
    red
    green
    blue
    orange
    yellow
    violet
    purple
    grey
    black
    brown
}

# global menu widgets
set sfm ""
set detm ""

array set detectorsOn {}
set detectorsMin -1
set detectorsMax -1
set detectorMenu ""
set detectorMenuMin ""
set detectorMenuMax ""

set ax ""

array set plotconfigsave {}
array set plotsymbolsizesave {}

# define legend labels for the legend elements
array set leglbl {
    suu offoff
    sdu ONoff
    sud offON
    sdd ONON
    msr msr-red-green
    cor cor-blue
}

array set spcfg {
    zoom 1
    crosshairs 1
    logscale 0
    ymin 0
    ymax ""
    yautomin 1
    yautomax 1
    y2min 0
    y2max ""
    y2automin 1
    y2automax 1
    xmin ""
    xmax ""
    xautomin 1
    xautomax 1
    errs 1
    pixels 6
    xlbl X
    ylbl Y
}

array set plotconfig {
    zoom 1
    crosshairs 1
    logscale 0
    ymin 0
    ymax ""
    yautomin 1
    yautomax 1
    y2min 0
    y2max ""
    y2automin 1
    y2automax 1
    xmin ""
    xmax ""
    xautomin 1
    xautomax 1
    errmsr 1
    errpb 1
}

# default symbol sizes
array set plotsymbolsize {pbcor 6 msr 6 R 6 same 1 suu 6 sdd 6 sdu 6 sud 6}
array set plotsymbolcolor {same 1 suu blue sdd blue sdu blue sud blue}
array set plotsymbolfill {same 1 suu 1 sdd 1 sdu 1 sud 1}


set pbcorelems {suu sdd sdu sud}
set plotconfigall 1
# plotconfigall 1 means apply any plotconfig to all graphs (atleast BLT)

set globalsetlist {}

# auto group
set toltype ""
set tolerance 0
set tolmenu ""
set currentgroup 1
set samepts 0
set exclmatch 1
set instrType -2

# He3 cells
proc nsl1FileDummy {} {}
set cellFiles {}
set nsl1FilesEnable 1

# column assignments
array set colds {}
array set colLbls {}
array set colRanges {}
array set colNdif {}

# fileselector listboxes
# columns for which we get statistics to fill the lbxs
set colsrch {
x1 x2 x3 En fs H V Temp scl
}
set colsrchS {
x1 x2 x3 En H V Temp scl
}
set lbxnames {fnam gnum ts npts ny y x1 x2 x3 En Fix fs H V Temp scl ref fbg}
array set lbxfmts {
    0 ""
    1 ""
    2 {%.10g}
    3 ""
    4 ""
    5 {%.5g}
    6 {%.6g}
    7 {%.6g}
    8 {%.6g}
    9 {%8.4f}
    10 ""
    11 ""
    12 ""
    13 ""
    14 {%g}
    15 ""
    16 ""
    17 ""
}

set qxlbx 6
set qylbx 7
set qzlbx 8
set elbx 9

set QEangles 0
set QElbxs "$qxlbx $qylbx $qzlbx $elbx"
set QElbls {Qx Qy Qz E}
set QEstnd {x1 x2 x3 En}
set TTlbls {tts om ttm tta}

set rotH 0
array set Hlbls {0 H 1 rotH}

array set elbls {0 En 1 Ei 2 Ef}

set tsdate 0
array set tslbls {0 ts 1 date}

# remember the indices for ts, temperature and fast-background
set tslbx 2

set hlbx 12
set templbx 14
set fbglbx 17
set nlbxs [llength $lbxnames]
#set lbxnames [join "fnam gnum npts $colsrch"]
set gnum 1
set lbxpad 1

array set dsFile {}

# flipper-state correspondence to print in readfile table
array set fspb {1 offoff 2 ONON 3 ONoff 4 offON}
array set fbgFile {}
set a4enable 1
set ynames {}
set newfiles 0

set nDetectors 0
# readFile
set errs ""
set configQA -1
set configSC -1

array set lbxfixedwidth {}
array set calcHilite {0 white 1 green}
array set fopHilite {0 white 1 red}

set windowCaptureFile ""


set graphicsFmt jpeg

array set bld {}
array set bldtext {1 "file-select" 2 "soln-options" 3 "auto group"}

set lbxlabels {}
set curnpts ""
set curnmsr ""

set grpr ""
set nlbxsr 19
array set lbxsr {}
set lbxrlabels {}

set grplbxs ""
array set lastCwidth {}
array set lastCheight {}
set mainwidth 3

array set lbxLabels {}
array set lbxWidgets {}
set lbxwidgetlst {}

array set helpdocs {0 pbcor40QuickStartGuide.pdf 1 pbcorHelp.pdf 2 PBcorrect3.pdf 3 He3SpinTransport.pdf}

set imgfmts {jpeg gif png ppm}

set adjlbxw ""
set adjlabel ""
set adjwidth ""
set adjindex ""
set adjframe ""

set widgetsel ""

############### group coordinate-peaks 
set pks ""
set pksuid ""
set pksmin ""
set pksmax ""
set pkspos ""
set pksfwhm ""
set lbxplabels {}
# listbox array by column index
array set lbxps {}
# pks group number
set gnumpk 1
# ipks is the selection index in lbxps(0)
set ipks -1
array set pksdata {}
array set pkeToi {}

############### group file ops - subtract or average group solutions ######

set fop ""
set lbxolabels {}
set grpsop ""
set grpsub ""
array set lbxos {}
set fopcontrol 4
set useFF 0
set fopTolerSrc 2

set ifop -1
set ivc .ivc
array set ivce {}

set stndColNames {En x1 x2 x3 tts om ttm tta Ei Ef Temp Bfld}
set stndScanNames {En x1 x2 x3 tts om Temp Bfld}
set stndScanNamesC {En x1 x2 x3 tts om}
array set stndColTypes {
    En 2
    x1 -1
    x2 -2
    x3 -3
    Ei 3
    Ef 4
    ttm 10
    tta 11
    Temp 14
    tts 25
    om 26
    Bfld 32
}

set ivcEnt ""
set ivcChi 0.
set ivcOrder 1

array set groupfop {}
array set fopgroup {}
# RWE Jul19 2010 added tts om as iX=5,6
array set stndForiX {1 2 2 -1 3 -2 4 -3 5 25 6 26}
# natfs for p A D B C
array set natfs {0 ALL 2 offoff 3 ONON 4 ONoff 5 offON}
array set saveStndTol {}

set ifop -1
array set curOp {}
set editfop 0

# saving
set lastPlot ""
set sessionrecover 0
array set fopinfo {}
array set fopdata {}
array set groupsolved {}
array set pltDet {}

array set groupiwin {}
array set iwingroup {}

array set lbxs {}

# list comparison
set cmptyp -integer
set cmpind 0
# listbox sorting
set sortdirection {-increasing -decreasing}
set sortdirectionindex 0

####### include the He3-cell gui ##############
# tk app to prep edit cellfiles

# use . for toplevel standalone app
# else use .cedt to include in another app
set standalone 0
if { $standalone } {
    set celledtTop .
    set celledtPre ""
} else {
    set celledtTop .cedt
    set celledtPre .cedt
}
set overrideDB 1

# has db of cell fixed data
# June 2010  Zinfandel is reborn as ZinfandellII. New BT7 cells are under construction
# added June 2014 the third line ic Cells list

set Cells {
    ZinfandelII Sangiovese Beaujolis Chablis Syrah Mercurey
    Monastrell Barolo Valpolicella Barbaresco
    Chambourcin Grigio Gris Primitivo Serendipity Teroldego Viognier
}

array set ZinfandelII {
    nsL0 1.2789 nsL0err 0.02 tEmpty 0.86 tEmptySlope 0
    R 25 L 8.79 D 11.6 volcc  950 Pbar 1.92
}
array set Sangiovese {
    nsL0 1.33584 nsL0err 0.01 tEmpty 0.86 tEmptySlope 0
    R 25 L 10.12 D 11.0 volcc  950 Pbar 1.73
}
array set Beaujolis {
    nsL0 1.17 nsL0err 0 tEmpty 0.86 tEmptySlope 0
    R 25 L 11.7 D 10.5 volcc 1010 Pbar 1.34
}
array set Chablis {
    nsL0 1.1768625 nsL0err 0.01 tEmpty 0.86 tEmptySlope 0
    R 25 L 9.51 D 10.5 volcc 1010 Pbar 1.34
}
array set Syrah {
    nsL0 1.0220925 nsL0err 0.01 tEmpty 0.86 tEmptySlope 0
    R 25 L 9.53 D 10.5 volcc 1010 Pbar 1.34
}
array set Mercurey {
    nsL0 1.05975 nsL0err 0.01 tEmpty 0.86 tEmptySlope 0
    R 25 L 7.85 D 10 volcc 1010 Pbar 1.34
}
array set Monastrell {
    nsL0 1.04115 nsL0err 0.008 tEmpty 0.87 tEmptySlope 0
    R 25 L 6.31 D 8 volcc 300 Pbar 2.2
}
array set Barolo {
    nsL0 0.66885 nsL0err 0.008 tEmpty 0.87 tEmptySlope 0
    R 25 L 6.86 D 8 volcc 580 Pbar 1.3
}
array set Valpolicella {
    nsL0 1.1055 nsL0err 0.013 tEmpty 0.87 tEmptySlope 0
    R 25 L 6.7 D 8 volcc 300 Pbar 2.2
}
array set Barbaresco {
    nsL0 0.691275 nsL0err 0.008 tEmpty 0.87 tEmptySlope 0
    R 25 L 7.09 D 8 volcc 580 Pbar 1.3
}


# create array of entries to edit cell info
# cellName  installTimeDate  iPol iPolErr fPol fPolErr Tc  TcErr expt trans

# expt has hcol vcol mosa dsp beamAreacm smplGFdir
# trans has transeff transErr flipeff flipErr

# for H V trans may differ, requ diff cell entry

# RWE Aug 2010 add tEmpty to GUI so that can be used for Heusler and SM etc
# set overrideDB 1

# RWE May 2011 add Dia transTab S3 SD SH A4 dA4 tilt  to GUI

# RWE Oct 2011 add lpf to He3 Cells data before Tc

# entryTable will do init build of entry table into frame f
# want to add bindings to directly edit current cell params
# for imediated-mode = lockCells-mode used with simulation
set cellsColBindList [list \
			  [list 1 <Tab> {parseTimeDateRow 0}] \
			  [list 1 <Return> {parseTimeDateRow 0}] \
			  [list 12 <Tab> {checkid 0 2}] \
			  [list 12 <Return> {checkid 0 2}] \
			  [list 13 <Tab> {checkid 0 3}] \
			  [list 13 <Return> {checkid 0 3}] \
			  [list * <Tab> {directEditCells 1}] \
			  [list * <Return> {directEditCells 1}] \
			 ]

set exptColBindList [list \
			  [list * <Tab> {directEditCells 3}] \
			  [list * <Return> {directEditCells 3}] \
			 ]

set transColBindList [list \
			  [list * <Tab> {directEditCells 2}] \
			  [list * <Return> {directEditCells 2}] \
			 ]

set timedatefmt "%R %a %b %d %Y"
set datefmt "%a %b %d %Y"
set exportdatefmt "%D"
set timefmt "%R"
set utcfmt "%s"

set lockCells 0
array set curcellrow {}
array set curcellcol {}

# cellfile variable names
set cfvnames {
    name PorA date tim
    startSecs
    Energy lambda Pi Pierr
    T Terr
    beamArea
    nsL0E nsL0Eerr
    teff terr feff ferr
    tEmpty tEmptySlope
    Pbar L D R
    volcc nsL0 nsL0err nsLE nsLEerr
    rnam Hmos Vmos dsp Hcols Hcol2 VcolsDeg Vcol2Deg
    omRad
    hsigsq vsigsq xsigsq
    curvCor angcor Tf
    S3 SD SH A4 dA4 tilt rfloss table
}
set cfdeflt {
    "" P "" ""
    0
    14.7 2.351 0.7 0.1
    100 10
    10
    1 0.1
    1 0 1 0
    0.83 0.
    2 10 12 30
    900 1 0.01 1 0.01
    coarse 40 40 3.35416 40 40 2 3
    0.1
    0.1 0.1 0.1
    1 1 0
    0 0 0 0 0 0 0 ""
}
foreach cfn $cfvnames cfv $cfdeflt { set cellinfo($cfn) $cfv }

set dbchk {nsL0 nsL0err R L tEmpty tEmptySlope}
set req [concat $dbchk name PorA date tim Pi Pierr T Terr teff terr feff ferr]
# the cell array cell(1) .. cell(ncells)
array set cell {}
set ncell 0
set ntrans 0
set nexpt 0

array set transTables {}

set cfvexpt   { \
Hcols Hcol2 VcolsDeg Vcol2Deg Hmos dsp     xsigsq beamArea S3 SD SH A4 dA4 tilt}
set exptmed   { \
50    50    2        2        40   3.35416 0      6        0  0  0  0  0   0}
set exptopen  { \
90   100    2        2        40   3.35416 0      6        0  0  0  0  0   0}
set exptfine  { \
10    20    2        2        40   3.35416 0      6        0  0  0  0  0   0}

array set frames "
    1 $celledtPre.cf.d.e
    2 $celledtPre.tf.d.e
    3 $celledtPre.ef.d.e
    4 .ecfp.f.d.e
"
array set frametype {
    1 He3-cells
    2 transport
    3 experiment
    4 He-3fixedData
}
array set framearrays {
    1 celldata
    2 transdata
    3 exptdata
    4 fixedcelldata
}

set dbchk {nsL0 nsL0err R L tEmpty tEmptySlope}

set reqcell {name datetim PorA  tEmpty Pi Pierr T  Terr trans expt}
set reqcellC {0    1       2    3      4  5     10 11   12    13}
# teff terr can be blank if table is valid and vice versa
set reqtrans  {name feff ferr}
set reqtransC {0    3    4}
# for fixed polarization devices the cellfile requirements are relaxed
# The time constants Tc(T) TcErr(Terr) are not needed
# and datetim=installTimeDate not needed
# experiment resolution stuff is not needed
set reqcellF {name  PorA  tEmpty Pi Pierr  trans}
set reqcellFC {0    2     3      4  5      12}
set fixedDevices {"supermirror" "sm" "heusler" "hs" "fixed" "fix"}

set exportname ""

array set cellall {}
set ecfp .ecfp
array set fixedcelldata {}
# when user edits fixedcelldata, return results to cell
# lockCells
set ecfpColBindList [list \
			  [list 1 <Tab> {setCellFD 1}] \
			  [list 1 <Return> {setCellFD 1}] \
			  [list 2 <Tab> {setCellFD 2}] \
			  [list 2 <Return> {setCellFD 2}] \
			  [list 3 <Tab> {setCellFD 3}] \
			  [list 3 <Return> {setCellFD 3}] \
			  [list 4 <Tab> {setCellFD 4}] \
			  [list 4 <Return> {setCellFD 4}] \
			  [list 5 <Tab> {setCellFD 5}] \
			  [list 5 <Return> {setCellFD 5}] \
			  [list * <Tab> {directEditCells 4}] \
			  [list * <Return> {directEditCells 4}] \
			 ]

set nttents 0

array set celldata {}
array set transdata {}
array set exptdata {}


set curcellf ""

set ct ""
set tf ""
set ef ""

set he3titlefont [list Helvetica 20 bold]
set lblfont [list Helvetica 12]
set titlecolor red
set lblcolor blue

# the cell editor needs updating to handle
# new cell geometry parameters and transport table
# May 2011 the cellfile line now has after Tf
# SZ(He3-center-to sample-cm) = S3
# DD(sample to detector cm)   = SD
# SH(sample or beam height cm)= SH
# A4angle(fixed cell position in a4 degrees. use > 360 to disable) = A4
# offset(cell tilt angle cylinder geom same sense as a4 degrees)   = tilt
# rfLossPerFlip(needs time log of flips to implement so this is excluded here)
# teffTable(name vars data)

array set savecellerrs {}
set useParamErrs 1

array set helptext {
    .cf.m.im "when lock is selected cell editor data is used for solve or sim."
    .cf.m.perrs "when paramErrs is de-selected, the param errs are zeroed."
    .cf.d.l0 "cellName typically from database. Use Heusler, supermirror or fixed for fixed polarizing device."
    .cf.d.l1 "installTimeDate is just time-date of init polarization"
    .cf.d.l2 "P for polarizer, A for 3-ax analyzer\nAr for radial-coll analyzer\nAp for PSD analyzer\nAs SANS analyzer\nAc cylinder-wall geometry analyzer"
    .cf.d.l3 "empty-cell transmission or xtal reflectivity/transmission"
    .cf.d.l4 "initial He3 polarization\n -1 <= iPol <= 1 or neutron polarization for fixed devices"
    .cf.d.l5 "initial He3/neutron polarization error"
    .cf.d.l6 "final He3 polarization.\nUse for two time polarization msrment\nwith Tf(hr) as the final time"
    .cf.d.l7 "final He3 polarization error"
    .cf.d.l8 "final He3 pol msrment time\n> 0 for 2 pt msrment"
    .cf.d.l9 "polarization loss per He3 flip"
    .cf.d.l10 "He3 decay time constant in hours"
    .cf.d.l11 "He3 decay time constant error in hours"
    .cf.d.l12 "identifier for transport parameters"
    .cf.d.l13 "identifier for experiment resolution parameters. Expt corrections are on order of 1%"
    .tf.d.l0 "identifier for transport parameters"
    .tf.d.l1 "transport efficiency for P or A beam side\n0 <= trans <= 1"
    .tf.d.l2 "transport efficiency uncertainty >= 0"
    .tf.d.l3 "flipper efficiency for P or A beam side\n0 <= flip <= 1"
    .tf.d.l4 "flipper efficiency uncertainty >= 0"
    .tf.d.l5 "transport table name"
    .ef.d.l0 "identifier for experiment resolution parameters"
    .ef.d.l1 "horiz collimation(FWHMmin)\nnearest sample on P or A beam side"
    .ef.d.l2 "horiz collimation(FWHMmin)\nfarthest from sample on P or A beam side"
    .ef.d.l3 "vert collimation(FWHMdeg)\nnearest sample on P or A beam side"
    .ef.d.l4 "vert collimation(FWHMdeg)\nfarthest from sample on P or A beam side"
    .ef.d.l5 "crystal mosaic(FWHMmin) for monochoromator (P side) or analyzer (A side)"
    .ef.d.l6 "crystal d-spacing(Angst) for monochoromator (P side) or analyzer (A side)"
    .ef.d.l7 "sigLambda/lambda squared. calculated except for SANS"
    .ef.d.l8 "beam area(cm^2) in he3-cell for 3-axis\nelse sample area in the beam. P or A"
    .ef.d.l9 "sample to He3 cell center distance(cm)"
    .ef.d.l10 "sample to detector distance(cm)"
    .ef.d.l11 "sample height(cm)"
    .ef.d.l12 "He3 cell fixed A4 angle(deg). blank if cell moves with A4"
    .ef.d.l13 "He3 cell A4 angle range(deg)"
    .ef.d.l14 "He3 cell tilt angle(deg) cyl geom. Same sense as A4"
    .ttg.mf.ln "transport table unique name"
    .ttg.mf.lv "specify 1 or 2 variables E and/or H"
    .ttg.mf.ld "data= var1 [var2] transportEfficiency Err.  If both E and H put E first"
    .a.left.mid.exec.nsflbl "make sure boxes in the constraint table for un msrd cross-sections are checked"
    .a.left.mid.exec.sflbl "make sure boxes in the constraint table for un msrd cross-sections are checked"
    .a.left.mid.cm.en1 "make sure boxes in the constraint table for un msrd cross-sections are checked"
    .a.left.mid.cm.en2 "make sure boxes in the constraint table for un msrd cross-sections are checked"
    .a.left.mid.cm.en3 "make sure boxes in the constraint table for un msrd cross-sections are checked"
    .a.left.mid.cm.en4 "make sure boxes in the constraint table for un msrd cross-sections are checked"
    .a.left.bot.exe.defltbut "sets best guess tolerances for either BT7 or MACS"
    .a.left.bot.cb.rlx "relaxes all tolerances except for common scan variable from Qxyz,E,tts,om."
        .a.right.top.prepchk "check this to pause after solution prep allowing inspection before solving. Adds detail to warnings"
}

# find windows
set wf .wf
set wtbox ""
set wgbox ""
set wpbox ""

set wfall {}
set wfgrphs {}

# sim
set sim .sim
set simtxt ""
array set simct {}
array set simtm {}





