package provide app-pbcorWatchServer 1.0
# pbcorWatchServer is socket server to accept, display and plot
# pbcor correction data from client solvers in C progs

# The client should first send it's filename to the server upon
# connecting. In this design the client only sends data to the server
# and never reads from server.
# Then the server waits to receive pbcor data
# that gets posted in the server GUI and plotted.
# Each client pbcor correct file gets its own plot

# The GUI can switch between multiple connected clients.
set mswindows 0
if { [string match windows $tcl_platform(platform)] } { set mswindows 1 }

# pbcorWatchServer is starpacked with TclTk as a standalone executable
# Only thing it requires is the Bltxs plot package
set pbcorDir ""
set plotFolder "Bltxs"
if {[info exists env(pbcorDir)] && [file isdirectory $env(pbcorDir)]} {
	set pbcorDir $env(pbcorDir)
} elseif {$mswindows && [info exists env(USERNAME)] && [string length $env(USERNAME)] > 0} {
	set fchk [file join C:/Users $env(USERNAME)]
	set fchk2 [file join $fchk Pbcor]
	set fchk3 [file join $fchk2 $plotFolder]
	if [file isdirectory $fchk3] {
		set pbcorDir $fchk2
	} elseif [file isdirectory $fchk] {
		set fchk3 [file join $fchk $plotFolder]
		if [file isdirectory $fchk3] {
			set pbcorDir $fchk
		}
	}
} else {
	set fchk [file join /usr local lib]
	set fchk2 [file join $fchk Pbcor]
	set fchk3 [file join $fchk2 $plotFolder]
	if [file isdirectory $fchk3] {
		set pbcorDir $fchk2
	} elseif [file isdirectory $fchk] {
		set fchk3 [file join $fchk $plotFolder]
		if [file isdirectory $fchk3] {
			set pbcorDir $fchk
		}
	}
}

if {[string length $pbcorDir] < 1} {
	tk_messageBox -type ok -title "pbcorWatchServer ERROR" -message "NO Pbcor install directory found in env or $fchk. Run pbcorDir.bat in the install (unzip) directory."
	exit
}
lappend auto_path $pbcorDir

if {$mswindows} {console show}

if {[catch {package require Bltxs} bltver]} {
	tk_messageBox -type ok -title "pbcorWatchServer ERROR" -message "Failed to load Bltxs for pbcorWatchServer."
	exit
}

namespace import blt::*
#puts "Bltxs import OK"


set pbcorfExists 0
set pbcorf ""
set pbcorfmtime 0
set lastDir ""
set lastDirChanged 0


proc Scrolled_Text { f args } {
    frame $f
    eval {text $f.text \
	      -xscrollcommand [list $f.xscroll set] \
	      -yscrollcommand [list $f.yscroll set]} $args
    scrollbar $f.xscroll -orient horizontal \
	-command [list $f.text xview]
    scrollbar $f.yscroll -orient vertical \
	-command [list $f.text yview]
    grid $f.text $f.yscroll -sticky news
    grid $f.xscroll -sticky news
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1
    return $f.text
}

#puts "Scrolled_Text proc loaded OK"

proc checkDir {} {
	# convolution data directory can be located by reading last-Conv file in FitDir or backup fit-lastDir
	global pbcorDir watch
	global mswindows pbcorf pbcorfExists watch pbcorfmtime env progtitle lastDir lastDirChanged
	if { ! $pbcorfExists } {
    	set pbcorf [file join $pbcorDir last-Pbcor]
    	if [file isfile $pbcorf] {
    		set pbcorfExists 1			
    	} else {
    		# try pbcor-lastDir
    		set pbcorf [file join $pbcorDir pbcor-lastDir]
    		if [file isfile $pbcorf] { set pbcorfExists 1 }
    	}
    }
    if { $pbcorfExists } {
    	set newpbcorfmtime [file mtime $pbcorf]
    	#puts "checkDir about to open $pbcorf"
    	set fpt [open $pbcorf "r"]
    	#puts "checkDir about to gets ln from $pbcorf"
    	if {[gets $fpt ln] >= 0} {
    		if { $ln != $lastDir } {
    			#puts "checkDir about to cd $ln"
    			cd $ln
    			set lastDir [pwd]
    			set lastDirChanged 1
    			# new dir cant have convSrc yet
    			wm title $watch "$progtitle $lastDir"
    		}
    	}
    	set pbcorfmtime $newpbcorfmtime
    	close $fpt
    	return 1
    }
    return 0
}

#puts "checkDir proc loaded OK"

set pbcorChan ""
array set pbcorsocks {}
array set pbcorfiles {}
array set sockForFile {}
array set iterForFile {}
set currentIter 0
set maxitshow 1

set connectedFiles {}
set currentFile ""
set currentSock ""
set pbcorServerPort 6100


proc stopPbcor {} {
	global currentFile
	if {[file exists $currentFile.STOP]} {
		#puts "FILE: $currentFile already flagged for STOP."
		return
	}
	set fpt [open $currentFile.STOP "w"]
	catch {close $fpt}
	#puts "FILE: $currentFile NOW flagged for STOP."
}



proc startPbcorServer {} {
    global pbcorChan pbcorServerPort server
    # server is the start server button in the gui
    # start up the conv-server to listen on the given port
    $server configure -text "Stop pbcorWatch Server" -command stopPbcorServer
    set pbcorChan [socket -server pbcorAccept $pbcorServerPort]
}

proc stopPbcorServer {args} {
    global pbcorChan pbcorsocks server
    # close any open connections. This is the tricky part.
    # make sure server is not waiting for any replies from clients??
    if {$pbcorChan == ""} {
    	if {[llength $args] > 0} {exit}
    	return
    }
    foreach sock [array names pbcorsocks] {
		fileevent $sock readable {}
		# cant have the client close the socket as client only sends
		#puts $sock "convClose"
		# the client send should fail
		catch {close $sock}
		unset pbcorsocks($sock)
    }
    # close the server so it cant accept anymore connections.
    # stopConvServer also called when main window is closed by user and server may already be stopped
    close $pbcorChan
    set pbcorChan ""
    updateConnectedFilesMenu
    $server configure -text "Start pbcorWatch Server" -command startPbcorServer
    if {[llength $args] > 0} {exit}
}
set DBG 0
proc pbcorAccept {sock addr port} {
    # called when watchServer accepts a connection

    global pbcorsocks connectedFiles currentFile
    global wtxt DBG

    # client is supposed to send filename
    # use the outtxt text widget for error messages
    # so we dont need Windows console
    if { [eof $sock] || [gets $sock filename] < 0 } {
		$wtxt insert end \
	    	"SERVER: client $sock may have died. Connection failed.\n"
		catch {close $sock}
    } else {
		if { $DBG } { puts "debugAccept: sock=$sock addr=$addr port=$port" }
		fconfigure $sock -buffering line
		# send back appname OK
		# NO writing to the client
		#puts $sock "$filename OK"
		#flush $sock
		# after this the client must wait to read
		# a cmd from this server
		# record the connection and appname

		set pbcorsocks($sock) [list $addr $port]
		# updateConnectedAppsMenu
		# also preps the connectedApps list

		# if this is first connection make it currentSock through
		# trace on variable currentFile
		# client
	
		# setup the client-data handler
		# client-data on this sock will always be for filename
		fileevent $sock readable [list pbcorRead $sock]
    }
}

set wtxt ""
set gnum 0
set cg ""
set cgn -1
set words {}
set nwords 0
array set graphOFfile {}
array set gnumOFfile {}
array set nptsOFfile {}
array set ptnoOFfile {}
array set xstndOFfile {}
array set xlablOFfile {}
array set progOFfile {}

array set ixlabels {0 Qx 1 Qy 2 Qz 3 E}


array set iColor {0 red 1 #cc3300 2 #888800 3 #44bb00 4 #22ee00 5 green 6 #00cc33 7 #008888 8 #0044bb 9 blue}
proc iterColor {it} {
	global iColor
	return $iColor([expr $it % 10])
}

set titlefont [list Helvetica 20 bold]
set tickfont [list Helvetica 16]


proc pbcorRead {sock} {
    # called on fileeevent $sock readable
    global watch progtitle lastDir pbcorsrc connectedFiles currentFile
    global ptnoOFfile nptsOFfile progOFfile
    global readfinished readlines nread DBG outtxt gnumOFfile
    global connectedFiles graphOFfile gnum cg cgn pbcorfiles sockForFile
    global words nwords ixlabels ixOFfile
    global wtxt
    global titlefont tickfont
    if { [eof $sock] || [set nr [gets $sock readline]] < 0 } {
    	set cvf ""
    	if [info exists pbcorfiles($sock)] {
    		set cvf $pbcorfiles($sock)
    		unset pbcorfiles($sock)
    		unset sockForFile($cvf)
    		if {[set ifl [lsearch $connectedFiles $cvf]] >= 0} {
    			set connectedFiles [lreplace $connectedFiles $ifl $ifl]
    			updateConnectedFilesMenu
    		}
    	}
		$wtxt insert end \
		    "SERVER: client $cvf on socket $sock closed. Closing connection.\n"
		closeSock $sock
		set readfinished -2
		return
    }
    # readline success parse the readline
    # This server cen handle multiple clients (usually this would be multiple background jobs)
    # so check if convfiles($sock) already exists
    # if so set the filename and current file iter and graph stuff
    if [info exists pbcorfiles($sock)] {
    	set currentFile $pbcorfiles($sock)
    	set filename $currentFile
    	#if [info exists iterForFile(file)] {
    	#	set currentIter $iterForFile($filename)
    	#}
    	set cg $graphOFfile($filename)
    	set cgn $gnumOFfile($filename)
    }
    #puts "server received readline= $readline  sock= $sock"
    $wtxt insert end "$readline\n"
    # first data from client should be pbcor starting and sending ...src=fileName
    if [string match *src=* $readline] {
    	# pbcor src line has ..xcol=?? xlbl=... src=???  will split into 5 words-> "...npts" "nptsvalue xcol" "xcolval xlbl" "xlblvalue src" "srcval"
    	#%9.3f pbcor starting npts=%d xcol=%s xlbl=%s src=", nowWatch(), g->xStndName, g->xLabel
    	set swords [split $readline "="]
    	set filename [string trim [lindex $swords 4]]
    	set ixwrds [string trim [lindex $swords 1]]
    	set ixwrdl [split $ixwrds]
    	set nptsOFfile($filename) [string trim [lindex $ixwrdl 0]]
    	set ixwrds [string trim [lindex $swords 2]]
    	set ixwrdl [split $ixwrds]
    	set xstndOFfile($filename) [string trim [lindex $ixwrdl 0]]
    	set ixwrds [string trim [lindex $swords 3]]
    	# it is possible that the xLabel is blank
    	set xlablOFfile($filename) ""
    	if {[llength $ixwrds] > 1} {
	    	set ixwrdl [split $ixwrds]
    		set xlablOFfile($filename) [string trim [lindex $ixwrdl 0]]
    	}
    	if {[string length $xlablOFfile($filename)] < 1} {
    		set xlablOFfile($filename) $xstndOFfile($filename)
    	}
    	# changing file on current or different sock means use a different graph and start new
    	if {$filename != $currentFile} {
			wm title $watch "$progtitle $lastDir $filename"
			set currentFile $filename
			set pbcorfiles($sock) $filename
			set sockForFile($filename) $sock
		}
		#set iterForFile($filename) 0
		#set currentIter 0
		#set ixOFfile($filename) $ix
		if {[lsearch -exact $connectedFiles $filename] < 0} {
			lappend connectedFiles $filename
			updateConnectedFilesMenu
		}
		
		# make sure graph exists .g#.gf  else create new graph and vectors
		if {![info exists graphOFfile($filename)] || ![winfo exists $graphOFfile($filename)]} {
			incr gnum
			set gtop [toplevel .g$gnum]
			wm title $gtop "$progtitle $lastDir $filename"
			set g [graph $gtop.gf]
			set cg $g
			set cgn $gnum
			set graphOFfile($filename) $g
			set gnumOFfile($filename) $gnum
			pack $g -side top -fill both -expand true
			vector create x$gnum
			vector create yuu$gnum
			vector create ydd$gnum
			vector create ydu$gnum
			vector create yud$gnum
			vector create euu$gnum
			vector create edd$gnum
			vector create edu$gnum
			vector create eud$gnum
			
			$g element create uudata -xdata ::x$gnum -ydata ::yuu$gnum -yerror ::euu$gnum -linewidth 0 -pixels 5 -symbol circle -color blue -fill blue
			$g element create dddata -xdata ::x$gnum -ydata ::ydd$gnum -yerror ::edd$gnum -linewidth 0 -pixels 5 -symbol square -color blue -fill blue
			$g element create dudata -xdata ::x$gnum -ydata ::ydu$gnum -yerror ::edu$gnum -linewidth 0 -pixels 5 -symbol arrow -color red -fill red
			$g element create uddata -xdata ::x$gnum -ydata ::yud$gnum -yerror ::eud$gnum -linewidth 0 -pixels 5 -symbol triangle -color red -fill red
    		#$g element create conv.0 -xdata ::x$gnum -ydata ::yc$gnum.0 -linewidth 1 -pixels 0 -color [iterColor $currentIter] -smooth natural

    		$g axis configure x -title $xlablOFfile($filename) -titlefont $titlefont -tickfont $tickfont
    		$g axis configure y -title "S counts" -titlefont $titlefont -tickfont $tickfont
    		
    		# also create a progresbar for this filename
    		set progOFfile($filename) [ttk::progressbar .progress.p$gnum -orient horizontal -length 300 -mode determinate -value 0]
    		set progLbl [label .progress.l$gnum -text "Progress $filename"]
    		grid $progLbl $progOFfile($filename)
		} else {
			# graph exists but must reset vectors to zero length on new src cmd
			set cg $graphOFfile($filename)
			set cgn $gnumOFfile($filename)
			# reset vectors to zero length when src comes in
			::x$cgn length 0
			::yuu$cgn length 0
			::ydd$cgn length 0
			::ydu$cgn length 0
			::yud$cgn length 0
			::euu$cgn length 0
			::edd$cgn length 0
			::edu$cgn length 0
			::eud$cgn length 0

			#foreach el [$cg element names conv*] {
			#	$cg element configure $el -label ""
			#}
			#$cg element configure conv.0 -label conv.0
			$wtxt insert end "reusing gnum=$cgn and setting vector lengths to zero length x$cgn = [::x$cgn length]\n"
			$progOFfile($filename) configure -value 0
		}
		$wtxt insert end "returning from src cmd\n"
		return
	}
	
	# id readline is not src it must be data from a solved PBdatapt
	#%9.3f pbcor ptno= %d x= %.6g Suu-dd-du-ud= %.6g %.6g %.6g %.6g Euu-dd-du-ud= %.6g %.6g %.6g %.6g
	# split on = produces 5 words->  "...ptno" "ptnoval x" "xval Suu-dd-du-ud" "Suuval Sddval Sduval Sudval Euu-dd-du-ud" "Euuval Eddval Eduval Eudval"

	set readline [string trim $readline]
	set dwords [split $readline "="]
	#puts "$dwords\n"
	set nwords [llength $dwords]
	
	set ixwrds [string trim [lindex $dwords 1]]
    set ixwrdl [split $ixwrds]
   	set ptnoOFfile($filename) [string trim [lindex $ixwrdl 0]]
   	
   	set progval 0
   	if { $nptsOFfile($filename) > 0 } {
   		set progval [expr (100 * $ptnoOFfile($filename))/$nptsOFfile($filename)]
   	}
   	$progOFfile($filename) configure -value $progval

   	set ixwrds [string trim [lindex $dwords 2]]
    set ixwrdl [split $ixwrds]
    set xval [string trim [lindex $ixwrdl 0]]
    
    # will we have to test any numbers for NAN
    set ixwrds [string trim [lindex $dwords 3]]
	set suu [lindex $ixwrds 0]
	set sdd [lindex $ixwrds 1]
	set sdu [lindex $ixwrds 2]
	set sud [lindex $ixwrds 3]
	
	set ixwrds [string trim [lindex $dwords 4]]
	set tuu [lindex $ixwrds 0]
	set tdd [lindex $ixwrds 1]
	set tdu [lindex $ixwrds 2]
	set tud [lindex $ixwrds 3]
	

	# check if we can read the last word as a number
	#if catch set tst [expr $dat]
	#	$wtxt insert end "NAN $dat at npt=$npt\n"
	#	set dat 0
	
	::x$cgn append $xval
	::yuu$cgn append $suu
	::ydd$cgn append $sdd
	::ydu$cgn append $sdu
	::yud$cgn append $sud
	::euu$cgn append $tuu
	::edd$cgn append $tdd
	::edu$cgn append $tdu
	::eud$cgn append $tud


	# append the dat to the appropriate plot vector
	#puts "cgn=$cgn vector names= [vector names] dat= $dat"
	
	# update Progress

	# log incoming lines   
	# fitLsq start with convfun "%9.3f lsq starting src=%-s\n", nowWatch(), lsqd->src
	
	# lsqdata "%9.3f fitLsq iter= %d  ptno= %d  fitting= %d  chi= %.4g\n", nowWatch(), itot, i, fitting, chi
	# "%9.3f xdata ", nowWatch()
	#    for (j = 0; j <= i; j++)
	#     if (ix >= 0) fprintf(fpt, "%.6g ", lsqd->x[j][ix])
	#     else fprintf(fpt, "%d ", j)
	#	 fprintf(fpt, "\n"
	# "%9.3f ydata ", nowWatch()
	#    for (j = 0; j <= i; j++) fprintf(fpt, "%.6g ", y[j])
	#	 fprintf(fpt, "\n")
	# "%9.3f yerrs ", nowWatch()
	#    for (j = 0; j <= i; j++) fprintf(fpt, "%.6g ", e[j])
	#    fprintf(fpt, "\n")
	# "%9.3f ycalc ", nowWatch()
	#    for (j = 0; j <= i; j++) fprintf(fpt, "%.6g ", yc[j])
	#    fprintf(fpt, "\n");
	
	# each res3conv call datapt "%9.3f res3conv datapt= %d  iflg= %d\n", nowWatch(), ip, ifl
	# low level from eig1       "%9.3f res3conv eig1fun eig1val= %.6g  ndiv= %d\n", nowWatch(), eig1val, ndiv 
}


proc updateConnectedFilesMenu {} {
	# for convWatch the server only sends a reply to the client on initial connection
	# so there is no need to track the currentFile which would be used to send data to a client
    global conmb currentFile connectedFiles sockForFile
    global pbcorsocks currentSock
    set connectedFiles {}
    set found 0
    $conmb delete 0 end
    foreach fil $connectedFiles {
    	$conmb insert end command $fil
    }
    return
    #
    foreach sock [array names convfiles] {
		set fil $convfiles($sock)
		lappend connectedFiles $fil
		if { [string match $fil $currentFile] } { set found 1 }
		set sockForFile($fil) $sock
		$conmb insert end command $fil
    }
}

proc setCurrentSock {args} {
    # trace on setting currentFile
    # NOT used here. currentFile is always last one to send socket data
    global currentFile currentSock
    global sockForFile
    set currentSock $sockForFile($currentFile)
}


set readfinished 0
set readlines {}
set nread 0
set maxwait 60000
proc sendCmd {cmd} {
    # send a command to a client socket NOT
    # NOT used for convWatch
    global currentSock debugsocks
    global readfinished readlines nread
    global maxwait
    global outtxt
    global DBG

    if { $DBG } { puts "sendCmd: currentSock=$currentSock cmd=$cmd" }
    if { [string length $currentSock] < 1 } { return }
    set readfinished 0
    set readlines {}
    set nread 0
    set id [after $maxwait {set readfinished -1}]
    #puts $currentSock $cmd
    flush $currentSock
    # readfinished gets set when the result is read back, or on timeout
    vwait readfinished
    if { $readfinished < 0 } {
		if { $readfinished == -1 } {
	    	$outtxt insert end \
	    	"SERVER: reply to $cmd on $currentSock timed out. Closing socket.\n"
		} else {
	    	after cancel id
		}
		closeSock
		return 0
    } else {
		after cancel $id
		# display the result from readlines
		$outtxt insert end "RESULT:\n"
		for {set i 0} {$i < $nread} {incr i} {
	    	$outtxt insert end "[lindex $readlines $i]\n"
		}
    }
}

proc closeSock {s} {
    global pbcorsocks
    fileevent $s readable {}
    unset pbcorsocks($s)
    close $s
    updateConnectedFilesMenu
}

proc toggleLegend {} {
	# will toggle legend view for current graph
	global legonoff
	global cg cgn
	set nowstate [$cg legend cget -hide]
	set newstate [expr 1 - $nowstate]
	$cg legend configure -hide $newstate
}


# build the gui and run the loop
set watch [toplevel .watch]
#wm protocol $watch WM_DELETE_WINDOW iconifyWatch
set progtitle "watch pbcor progress"
wm title $watch $progtitle
wm iconname $watch "pbcorprogress"
wm protocol .watch WM_DELETE_WINDOW {stopPbcorServer 1}
set wt [frame .watch.t]
grid $wt -sticky ew
set wb .watch.b
set wtxt [Scrolled_Text $wb]
grid $wb -sticky news
set server [button $wt.startw -text "Start PbcorWatch Server" -command startPbcorServer -bg green]
set stopcc [button $wt.stopcc -text "Stop Current Pbcor Correction" -command stopPbcor -bg green]
set chkdir [button $wt.chkdir -text "Check Current Data Directory" -command checkDir -bg green]
set clear [button $wt.clear -text "Clear" -command "$wtxt delete 1.0 end" -bg green]
set filcon [menubutton $wt.filcon -text "Connected Files"]
set conmb [menu $filcon.menu]
$filcon configure -menu $conmb
set legonoff [button $wt.leg -text "Legend toggle" -command toggleLegend -bg green]

# build the progress toplevel
set progress [toplevel .progress]

#set maxitlb [label $wt.maxitlb -text maxit -justify left]
#set maxit [entry $wt.maxit -justify right -textvariable maxitshow]
#set pollb [label $wt.pollb -text pollSec]
#set pollsec 3
#set polle [entry $wt.polle -textvariable pollsec]
grid $server $stopcc $chkdir $clear $filcon $legonoff
#grid $probL $progB
raise .watch

startPbcorServer

update idletasks
