package provide app-pbconsole 1.0

# following set the versioninfo and platforminfo strings
# displayed in the Version menu
array set versioninfo { version 1.0 date Oct-31-2016 }
array set platforminfo { platform Windows compiler VC-2013-Community tclkit 8.5.9-win32-x86_64.exe }

##!/cygdrive/c/Tcl64/bin/wish.exe
# tcl/tk notebook application for creating He3 cellfiles from ICE experiment console
# Tab-1 console-text  drag and drop console text to the first tab or load a console file.
# Tab-2 he3-log run-parser to extract transmissin flip-ratios monitor-cts etc
# Tab-3 he3-efficiencies  extracted from the he3-log
# tab-4 cellfile
#
#    exptID            # find "experiment id" use this in cellfile constructed name. NB MACS uses date as yyyymmdd
#                         which matches folders on charlotte
#    instrument        # use to set deflt experiment geometry params. For now look for bt7 ng0
#                      # look for: Creating File .bt7 SDC TDC SmplGFRot  Det= when counting
#                      # MACS will have Creating File ... .ng0 on file writes  has SPEC= Spec= Diff=
#                          PTAI kidney etc
#
#    cell-selection    # pb|he3 P polarizer-cellName  comments   or  pb|he3 A analyzer-cellName+Type  
#                         or pb|he3 M or MT or Empty   or  pb|he3 B|2 both in beam
#                         on MACS yq does a good job of adding comments to the scans that indicate which cells
#                         but otherwise this is the biggest problem.
#                         Will have to parse and hopefully still find the transmission measurments
#                         but mark as missing cell in the he3Log
#    counts-record     # record total counts for cell generate totals statistics and sanity checking
#    transmissions     # must find recent MT to calc transmission and record with stats
#    cell-installs     # once a cell has init transmission completed it gets install time
#    flip=ratio        # can we get this from counting outside a scan??
#                        NB users can do misc counting as well
#                        so best is to use start and stop cmnds. Can change guideMode before end
#                        optional explicit flipper-states  
#                        should find move FLIP A, B ... although on MACS doenst look like BT7 script.
#                      # he3 fr
#                      # he3 flip STATE  looks like BT7 and MACS will have move FLIP for the flipstate
#                      # he3 g=guidefld Q|P   he3 bs small/large    he3 fr end
#                      # he3 fr end should calc the transport efficiency
#    flip-eff          is usually 1 but to set it use   he3 feffP/A eff
#    rfLossPerFlip     # he3 rflpf pol ana  for global values  He3 rflpfP/A for a cell
#                      # if using rflpf  must define before Tc is calculated at a following transmission
# normally only 2 point transmissions (start and end) are made.
# When multiple transmissions are made on same cell rf can be defined or not but should
# compare decay rates between multiple measurments to check if reinstall of same cell makes sense (diff Tc)
# Same for multiple FRs check for decay sanity or transport change.
#

set mswindows 0
set unixplatform 0
if { [string match -nocase windows* $tcl_platform(platform)] } {
    set mswindows 1
    console show
    puts "Windows platform"
} else { set unixplatform 1 }

set platform [tk windowingsystem]
# returns x11, win32/64 or aqua
option add *tearOff 0

set DEBUG 0
# add CL flag April 2017 to use the pbsCL command line program instead of the Tclpbs library
set CL 0
set fptCL ""
set fptCLout ""
set programCL "C:/cygwin64/usr/local/bin/pbsCL.exe"
set fileCL "pbsCLcmd"
set fileCLout "pbsCLcmdout"
set execrunning 0
set sleepms 100
# set the eventNow flag to immediately log events
# this makes debugging easier
# otherwise all events are written to txt(1) after the consoleParse
set eventNow 1

array set helpdocs {0 pbconsole.pdf}

foreach arg $argv {
    if { [string match deb* $arg] || [string match DEB* $arg] || \
	     [string match dbg* $arg] || [string match DBG* $arg] } {
	    set DEBUG 1
	    # this will open a window to monitor io to pbscript
	    # also for windows, open the console to get puts output
    }
    if { [string match *c* $arg] } { set CL 1 }
}
    
# Scrolled_Text is needed for the gui text windows

proc Scrolled_Text { f args } {
	frame $f
	eval {text $f.text \
		  -xscrollcommand [list $f.xscroll set] \
		  -yscrollcommand [list $f.yscroll set]} $args
	scrollbar $f.xscroll -orient horizontal \
	    -command [list $f.text xview]
	scrollbar $f.yscroll -orient vertical \
	    -command [list $f.text yview]
	grid $f.text $f.yscroll -sticky news
	grid $f.xscroll -sticky news
	grid rowconfigure $f 0 -weight 1
	grid columnconfigure $f 0 -weight 1
	return $f.text
}

# Dialog stuff ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
proc Dialog_Warn {msg} {
    set nl [regsub -all {\n} $msg " " result]
    if { $nl < 31 } {
	tk_messageBox -type ok -icon warning -message $msg
    } else {
	Dialog_WarnC $msg
    }
}

set warnStack {}
set msgfont ""
array set dialog {}
proc Dialog_WarnC {string} {
    # custom Dialog using message aspect = 120
    # which means width = 1.2*height
    global warn warnStack msgfont

	# RWE Jan 2020 this looks like infinite loop. Probably should have been calling Dialog_WarnC again after idle.
	# Note that this is the same code that is in pbcor.tcl ???
	# For now skip this event check
    # if  [llength [after info]] > 0 
	#    after idle [list Dialog_Warn $string]
	#    return
    #

    set f .warn
    set c $f.f
    set b $c.buttons
    set m $c.msg
    if [Dialog_Create $f "Warning"] {
	    frame $c
	    pack $c -side top -expand true -fill both
	    frame $b
	    frame $m
	    pack $m -side top -expand true -fill both
	    pack $b -side top

	    button $b.ok -text OK -command {set warn(ok) 1}
	    pack $b.ok -side top
	    message $m.msg -text $string -fg red -padx 10 -aspect 120
	    pack $m.msg -side top -expand true -fill both

	#set msgfont [$m.msg cget -font]
    } else {
	    if { [string match .warn [grab current]] } {
	        lappend warnStack $string
	        return
	    }
    }

    #set lines [split $string \n]

    #set pixwid 0
    #foreach line $lines
	#set ll [font measure $msgfont $line]
	#if { $ll > $pixwid } { set pixwid $ll }
    #
    #set pixwid [expr 10 + $pixwid]

    # sometimes dialog comes up with text and or buttons off widget??
    # try not setting the width
    # still getting truncated window size in Windows??
    $m.msg configure -text $string
    #-width $pixwid
    set warn(ok) 0
    Dialog_Wait $f warn ""
    Dialog_Dismiss $f
}


proc Dialog_Create {top title args} {
    global dialog
    if [winfo exists $top] {
	switch -- [wm state $top] {
	    normal { raise $top }
	    withdrawn -
	    iconified {
		wm deiconify $top
		catch {wm geometry $top $dialog(geo,$top)}
	    }
	}
	return 0
    } else {
	eval {toplevel $top} $args
	wm title $top $title
	return 1
    }
}
proc Dialog_Wait {top varName {focus {}} } {
    global pausing $varName
    if { ![winfo exists $top] } { return }
    set varname $varName\(ok\)
    eval set var $$varname
    bind $top <Destroy> [list set $varname $var]
    raise $top
    update idletasks
    if {[string length $focus] == 0} { set focus $top }
    set old [focus -displayof $top]
    focus $focus
    # getting hung up on tkwait visibility
    tkwait visibility $top
    grab $top

    # continually raise the dialog window (some wm's may allow hiding it)

    set ntry 0
    set pausing 0
    set var 0
    set i 0
    while { 1 } {
	set id [after 20 "setPausing $varname 1"]
	tkwait variable $varname
	# after 20 if varname not set by user then after event happens
	# which sets pausing 1 and varname 1
	eval set var $$varname
	if { ! $pausing } {
	    after cancel $id
	    break
	}
	incr i
	if { $i > 100 } { raise $top; set i 0 }
	set pausing 0
	#incr ntry
    }
    catch {grab release $top}
    focus $old
}
proc Dialog_Dismiss {top} {
    global dialog warnStack
    if { [string match .warn $top] && [llength $warnStack] > 0 } {
	set string [lindex $warnStack 0]
	set warnStack [lreplace $warnStack 0 0]
	Dialog_Warn $string
	return
    }
    catch {
	set dialog(geo,$top) [winfo geometry $top]
	wm withdraw $top
    }
}
proc setPausing {varname value} {
    upvar $varname vname
    global pausing
    set pausing 1
    set vname $value
}

############# debug interface includes gui access to pbscript and debugserver  +++++++++++++++++++++++++++++++++++++++++++++++
puts "starting debug setup"
if { $DEBUG } {
    # setup gui for debugging io to pbscript pipe.
    # use the intxt text widget to record sent commands
    # and the outtxt widget to record replies.
    # uses globals readfinished readlines from pbcor
    puts "DEBUG flag is ON. Will write degubCMNDS"
    set debugFPT [open debugCMDS w]
    set db [toplevel .debug]
    wm title $db "pbscript command debugger"
    wm iconname $db "debug"
    wm deiconify $db

    
    proc sendCur {} {
		global intxt outtxt
		global readfinished readlines
	
		if { [catch {set line \
			 [$intxt get "insert linestart" "insert lineend"]}] } {
	    	return
		}
		set line [string trim $line]
		if { [string length $line] < 1 } { return }

    	# dont have libarary yet so skip pipeWrite for now
		pipeWrite $line
		#$outtxt insert end "pipeWrite: $line\n"
		#$outtxt insert end "pipeRead:\n"
		#vwait readfinished
		# for debugging pipeRead dumps to outtxt
    }

    proc sendClr {} {
		global outtxt
		$outtxt delete 1.0 end
    }
    
    
    set sendmb [frame $db.mb]
    set sendin [frame $db.in]
    set sendout [frame $db.out]
    
    grid $sendmb -sticky news -pady 5
    grid $sendin -sticky news
    grid $sendout -sticky news
    grid rowconfigure . 1 -weight 1
    grid rowconfigure . 2 -weight 1
    grid columnconfigure . 0 -weight 1


    set sendCur [button $sendmb.ex -text "EXEC current line" \
		     -command sendCur]
    
    set sendClr [button $sendmb.clr -text "Clear Output" \
		     -command sendClr]


    grid $sendCur $sendClr -sticky news


    set inlbl [label $sendin.lbl -text "EXEC line text editor" -justify center]
    set intxt [Scrolled_Text $sendin.ftxt]
    $intxt configure -height 6 -width 60

    set outlbl [label $sendout.lbl -text "RESULT" -justify center]
    set outtxt [Scrolled_Text $sendout.ftxt]
    $outtxt configure -height 20 -width 60 -wrap none
    
    grid $inlbl -sticky news
    grid $sendin.ftxt -sticky news
    
    grid $outlbl -sticky news
    grid $sendout.ftxt -sticky news
    
    update idletasks

    # will save all input to pbscript pipe in a file
    # for later debugging by running pbscript through ddd

    # also try connection to the debugServer (send replacement via sockets)

    # this is debugClient code using socket
    # to be used by a tcl app that wishes to use the
    # debugServer to communicate with it while running.
    # source this code into a tcl script for debugging
    # start with: connectDebugServer localhost 
    # (or whatever host the server is on)
    
    # standard socket port for debugServer is 1500
    # connect to the debug Server
    # using the connectDebugServer proc
    # Once the connection is established debugStart is called to
    # send the apps name (toplevel window name) e.g. winfo name .
    # which should be echoed back by the server.
    # Then starts listening for commands. 
    # Exec any received cmnds in global context
    # and return the result followed by an END-RESULT line
    # to signal end of result
    # use catch for errors and send back the tcl stack trace as the result.
    
    # This is more flexible than sendTk as it works for a plane tcl script.
    # It is more secure, since the tcl script must agree to receive commands
    # from the server by connecting to it.
    # In fact tk send is broken since X uses xhost list

    #if { $mswindows && ! $unixplatform } { console show }

    set debugsock ""
    set debugServerPort 1500
    set debugconnect 0

    proc connectDebugServer {host} {
		global debugsock debugServerPort debugconnect
		# connect to the debugServer, and return the socket channel
		if [catch {socket $host $debugServerPort} debugsock] {
	    	puts "connectDebugServer $host $debugServerPort failed: $debugsock"
	    	return
		} else {
	    	puts "debugServer connected."
	    	set debugconnect 1
		}
		fconfigure $debugsock -buffering line
		debugStart
    }
    
    proc debugStart {} {
		global debugsock
		# send the appname and get one line back from the server
		# to confirm the connection
		puts $debugsock "[winfo name .]"
		flush $debugsock
		if { [eof $debugsock] || [gets $debugsock cmd] < 0 } {
	    	catch {close $debugsock}
	    	puts "no initial reply from debugServer"
		} else {
	    	if { [string match *OK* $cmd] } {
				puts "debugServer replied OK OK"
	    	} else {
				puts "debugServer replied without OK"
	    	}
	    # init the client cmd reader
	    	flush $debugsock
	    	fileevent $debugsock readable debugExec
		}
    }
    
    proc debugExec {} {
		global debugsock errorInfo
		if { [eof $debugsock] || [gets $debugsock cmd] < 0 } {
	    	catch {close $debugsock}
	    	puts "debugServer may have died. Socket closed."
		} else {
	    	# eval the cmd in global context
	    	# and return the result
	    	puts "debugClient socket=$debugsock received cmd: $cmd"
	    	# let the tcl app catch any errors
	    	if [catch {$cmd} rslt] {
				#puts "cmd result: $rslt"
				#if server sends debugClose, debugsock will have been closed
				if { [string match debugClose $cmd] } { return }
				puts $debugsock $rslt
				puts $debugsock "*** Tcl stack TRACE ***"
				puts $debugsock $errorInfo
	    	} else {
				if { [string match debugClose $cmd] } { return }
				puts $debugsock $rslt
	    	}
	    	puts $debugsock "END-RESULT"
	    	# do we need flush for line buffering?
	    	flush $debugsock
		}
    }

    proc debugClose {} {
	# user app can close the socket or the server will
	# request debugClose before it shutsdown
		global debugsock debugconnect
		if { ! $debugconnect } { return }
		catch {fileevent $debugsock readable {}}
		catch {close $debugsock}
    }

    # try the debugServer socket connection
    connectDebugServer localhost
}
#puts "finished debug setup"

#############++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# package loading can now use Dialogs to report errors
# so they are visible without console on Windows

# Use the same distibution check as for fits
set pbcorDir ""
if {$mswindows} {
	set homedir C:/Users
	set althomedir ""
} else {
	set homedir /home
	set althomedir /usr/local/lib
}
# first see if user is starting pbcor64.exe from pwd for example by double clicking pbcor64.exe in Windows
# or typein it on cmnd line in unix. Distribution must have the pbcor64 and the Tclpbs folder
set fchk [pwd]
if {$mswindows} {
	set fchk2 [file join $fchk pbcor64.exe]
} else {
	set fchk2 [file join $fchk pbcor64]
}
set fchk3 [file join $fchk Tclpbs]
if {[file exists $fchk2] && [file isdirectory $fchk3]} {
	set pbcorDir $fchk
}
# if pwd doesnt work try env then defaults
if { [string length $pbcorDir] < 2 } {
	set fchk ""
	if {[info exists env(USERNAME)] && [string length $env(USERNAME)] > 0} {
		set fchk [file join $homedir $env(USERNAME) Pbcor]
	}
	set fchk2 ""
	if {[file isdirectory $althomedir]} {
		set fchk2 [file join $althomedir Pbcor]
	}
	# first check for distribution directory defined by env
	if {[info exists env(pbcorDir)] && [file isdirectory $env(pbcorDir)]} {
		set pbcorDir $env(pbcorDir)
	# if env(pbcorDir)  fails try homedir env(USERNAME)
	} elseif {[file isdirectory $fchk]} {
		set pbcorDir $fchk
	} elseif {[file isdirectory $fchk2]} {
		set pbcorDir $fchk2
	}
}

# As of version 4.0 a distribution directory is required
# so pbcorDir may NOT be blank

if {[string length $pbcorDir] > 0} {
	lappend auto_path $pbcorDir
} else {
	if { $mswindows } {
	    set distopt "Double-clicking the pbcor executable when it is in the distribution folder should work.\n"
		append distopt "Otherwise the default distribution location is C:/Users/_username/Pbcor\n"
		append distopt "If you don't put it there run the pbcorDir.bat in your distribution folder by double clicking it.\n"
		append distopt "This creates the pbcorDir environment variable that points to your distribution folder.\n"
	} else {
		set distopt "The default location is /home/_username or /usr/local/lib\n"
		append distopt "If you don't put it there set the environment variable pbcorDir to your distribution folder.\n"
		append distopt "In bash: env pbcorDir=/yourPbcorDistributionDirectory\n"
	}
	Dialog_Warrn "ERROR: Could not locate Pbcor distribution folder with pbcor64.exe and Tclpbs folder.\n$distopt"
	exit
}



# if this is a starkit we need to manually package require Tk
# startkit is loaded into ActiveState wish so just
# update suto-Path in case we need to load libraries while debugging
# if not starkit, need directory for libraries Tclpbs Blts(or Bltxs)
if [file isdirectory /usr/local/lib] { lappend auto_path /usr/local/lib }
if [file isdirectory C:/Tcl/lib] { lappend auto_path C:/tcl/lib }
# for cygwin environment
if [file isdirectory C:/cygwin64/usr/local/lib] {
    lappend auto_path C:/cygwin64/usr/local/lib
} elseif [file isdirectory C:/cygwin/usr/local/lib] {
    lappend auto_path C:/cygwin/usr/local/lib
}

## load packages
#  should work in either case (starpack or not)

# tcltk libraries build with VC-Express 10 which uses
# msvcr100.dll which is not a standard system library
# unless VC-Express is installed.
# users must get vcredist_x86.exe from www.microsoft.com
# I am going to try putting the vcr.dll into the startpack

if [catch {package require Tclpbs} pbsver] {
    Dialog_Warn "Failed to load Tclpbs library.\n$pbsver"
    exit
} else {
    set useTclpbs 1
    set pbprogfound 1
    #puts "package Tclpbs loaded OK so init"
    pbsinit
}

if [catch {package require Bltxs} bltver] {
	set PLT 0
	# plotting is not yet implemented for pbconsole
    #Dialog_Warn "failed to load Bltxs which is required for plotting.\n$bltver"
} else {
    set BLT 1
    set PLT 1
    namespace import blt::*
    if [catch {package require Graphtcl} graphver] {
	    Dialog_Warn "$graphver failed to source graph.tcl for Bltxs"
    }
}

# help Doc
proc openHelp {idoc} {
    # open the pbcorHelp.pdf stored in the starkit doc directory
    # On Windows this can be done with start command which will find the
    # pdf reader
    # except that Windows cant find the vfs mounted directories
    # so use the Windows User/username/Documents
    # if the pdf is not there copy it from the starkit doc to Documents
    global mswindows
    global pbcorDir
    global helpdocs
    global isStarKit

   	set docdir [file join $pbcorDir Doc]
   	if {![file isdirectory $docdir]} {
   		Dialog_Warn "NO directory $docdir for Documents"
   		return
   	}
   	set helpdoc [file join $docdir $helpdocs($idoc)]
   	if {![file exists $helpdoc]} {
   		Dialog_Warn "Cant find $helpdocs($idoc)"
   		return
   	}
   	if {$mswindows} {
        exec {*}[auto_execok start] "" $helpdoc
    } else {
        # try evince for linux
        exec evince $helpdoc
    }
}


# getting He3Cell information +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# using builtin http package to get cellinfo from my redhat server
package require http

array set cells {}
set ncells 0
set cellcols {}
set cellnames {}
# create the nocell data
array set nocell {
	Lcm 10
	Hcm 10
	opacity1 0
	opacity1SD 0
	rCURVcm 25
	glassTrans1 1
}

set cellDir ""
proc getCellsLocal {args} {
	global cellDir cellTXT cellDBok cdbB
	if {[llength $args] > 0} {
		set cellDir [lindex $args 0]
	} else {
		set cellDir [pwd]
	}
	set ok 0
	set f [file join $cellDir he3Cells.txt]
	if {[file exists $f] && [getCellsTXT $f]} {set ok 1}
	if { $ok == 0 } {
		set f [file join $cellDir he3Cells.json]
		if {[file exists $f] && [getCellsJSON $f]} {
			set ok 1
		}
	}
	if {$ok == 1} {
		setCellTXT $cellTXT
		set cellDBok 1
       	$cdbB configure -bg green -text "he3Cell DB OK"
       	return 1
	}
	return 0
}

set cellDBok 0
set cellTXT ""

proc setCellTXT {t} {
	global txt
	$txt(8) delete 1.0 end
	$txt(8) insert 1.0 $t
}

proc getCellsServer {} {
	# call this after the GUI is setup
	global cellDBok cellDir cdbB cellTXT txt
	set cellDBok 0
	# try to load cell DB from server
    set url http://erwin.ncnr.nist.gov/cgi-bin/getHe3Cells.py
    if [ catch {::http::geturl $url -query} http] {
    	puts "http query returned\n$http"
    	Dialog_Warn "WARNING: he3Cell DB server unavailable. You will need to find the DB in a file."
    	$cdbB configure -bg red -text "NO he3Cell DB"
        return 0		
	}
    # get the state array and check that status is ok
    upvar #0 $http state
	set html [::http::data $http]
	if {[string match -nocase *failure* $html] || !([info exists state(status)] && $state(status) == "ok")}  {
    	Dialog_Warn "WARNING: he3Cell DB server unavailable. You will need to find the DB in a file."
    	$cdbB configure -bg red -text "NO he3Cell DB"
        return 0
	}

    # capture the returned database lines
    if [linesToCellDB $html] {
       	$cdbB configure -bg green -text "he3Cell DB OK"
       	set cellTXT $html
       	setCellTXT $cellTXT
    	set cellDBok 1
    	return 1
    }
    $cdbB configure -bg red -text "NO he3Cell DB"
	Dialog_Warn "FAILED to load He3Cells database from server $url. You will need to find the DB in a file."
    return 0
}

proc getCells {} {
	global cellDBok cellDir
	set cellDBok 0
	# try to load cell DB from server
    set url http://erwin.ncnr.nist.gov/cgi-bin/getHe3Cells.py
    set http [::http::geturl $url -query]
    # get the state array and check that status is ok
    upvar #0 $http state
	set html [::http::data $http]
	if {[string match -nocase *failure* $html] || !([info exists state(status)] && $state(status) == "ok")}  {
		# try local file database first .txt then .json
		if [cellsLocal] {
			set cellDBok 1
			return 1
		}
    	set selDir [tk_chooseDirectory \
			-title "select directory that contains he3Cells.txt or he3Cells.json"]
    	if { [string length [string trim $selDir]] < 1 } { return 0 }
		if [cellsLocal $selDir] {
			set cellDBok 1
			return 1
		}
    	Dialog_Warn "ERROR: server unavailable and FAILED to find He3Cells database file in $cellDir."
        return 0
	}

    # capture the returned database lines
    if [linesToCellDB $html] {
    	set cellDBok 1
    	return 1
    }
	Dialog_Warn "FAILED to load He3Cells database from server $url"
    return 0
}

proc linesToCellDB {html} {
	# html is just text lines of the database from file or DB query
    global cells
    global ncells
    global cellcols cellnames
    global nocell
    # break html database into lines
    set lines [split $html "\n"]
    # the last newline may give an empty list element
    # the first line has the column labels
    # get rid of extra white space and then split lines into words
    # create an array for each cell of its relevant parameters
    # cellname is column 3 (counting from zero) opcaity,err,glass=6,7,8
    # L,D = 11,12  R(curv)=16
    set nl [llength $lines]
    set heads [lindex $lines 0]
    # we can access the words of each database line by list index which treats all white-space as single sep
    #regsub -all {\s+} $heads " " head
    #set lbls [split $head]
    #set cellcols $lbls
    #if {[llength $lbls] < 19} {return 0}
    if {[llength $heads] < 19} {return 0}
    array set cells {}
    set ncells 0
    set relcol {6 7 8 11 12 16}
    for {set i 1} {$i < $nl} {incr i} {
        set celldatas [lindex $lines $i]
        if {[string length $celldatas] < 1} {continue}
        #regsub -all {\s+} $celldatas " " celldata
        #set cellst [split $celldata]
        #if {[llength $cellst] < 19} {continue}
        if {[llength $celldatas] < 19} {continue}
        set cellname [lindex $celldatas 3]
        # make cellname lc
        set cellname [string tolower $cellname]
		if {[lsearch $cellnames $cellname] < 0} {
	        lappend cellnames $cellname
		}
        # make the celldata array global
        global $cellname\A
        array set $cellname\A {}
        foreach ir $relcol {
            set lbl [lindex $heads $ir]
            set $cellname\A($lbl) [lindex $celldatas $ir]
        }
        set cells($cellname) $cellname\A
        incr ncells
    }
    # add the nocell
    global nocellA
    array set nocellA [array get nocell]
    set cells(nocell) nocellA
    lappend cellnames nocell
	return 1
}

proc getCellsTXT {f} {
	# the published txt version of database may have multiple entries for some cells
	# linesToCellDB will overwrite to use the last i.e. newest entry for a cell
	global cellTXT
    if { ! [file exists $f] } { return 0 }
    set fp [open $f r]
    set cellTXT [read $fp]
    close $fp
	return [linesToCellDB $cellTXT]
}


set tokenRE ""
set jsontext ""

proc getCellsJSON {f} {
	# the published Jason version of database only has most recent entry for each cell
	# look in current directoy if cant find then open selectDirectory. 
	# User must know which directory as this is not a file browser. 
    global cells
    global ncells
    global cellcols cellnames
    global nocell
    global cellTXT
    
    array set cells {}
	set ncells 0
	set cellcols {}
	set cellnames {}

    global tokenRE jsontext tokens ntok
    set celldatadir [pwd]
    set celldatafile [file join $celldatadir he3Cells.json]
    if { ! [file exists $f] } { return 0 }
    set fp [open $f r]
    set jsontext [read $fp]
    close $fp
    puts $jsontext
    set tokenRE {[{}:\[\],]|"(?:\\(?:[\"\\/bfnrt]|u[[:xdigit:]]{4}|.)|[^\\\"])*"|true|false|null|-?(?:[1-9][[:digit:]]*|0)(?:[.][[:digit:]]+)?(?:[eE][+-]?[[:digit:]]+)?}
    #puts "getCells checking file exists datafile= $datafile"
    set tokens [regexp -all -inline -- $tokenRE $jsontext]
    set ntok [llength $tokens]
    #puts $tokens
    # tokens will be list of tokens
    # index 0 is the starting item bracket first cellname is at 1
    set itok 1
    while {1 > 0} {
    	set cellname [string tolower [string trim [lindex $tokens $itok] "\""]]
    	lappend cellnames $cellname
        # make the celldata array global
        global $cellname\A
        array set $cellname\A {}
        # find the first data pair for this cell skip the colon and list starting bracket
        set itok [expr $itok + 3]
        puts "starting data for $cellname"
        while {1 > 0} {
        	set key [string trim [lindex $tokens $itok] "\""]
        	puts "key= $key"
        	# skip the : separator to get to value
        	set itoksep [expr $itok + 1]
        	puts "separator= [lindex $tokens $itoksep]"
        	set itok [expr $itok + 2]
        	set val [string trim [lindex $tokens $itok] "\""]
        	puts "value= $val"
        	set $cellname\A($key) $val
        	set itok [expr $itok + 1]
        	set sep [lindex $tokens $itok]
        	puts "token following value= $sep"
        	if [expr {$sep ne ","}] { break }
        	puts "continuing to next key-value after comma"
        	set itok [expr $itok + 1]
        }
        puts "cell loop BREAK on sep= $sep"
        set cells($cellname) $cellname\A
        incr ncells
        # on break itok should be end of list bracket, not comma
        # next token must be comma if there is another cell to parse
        # skip the end of list bracket
        set itok [expr $itok + 1]
        if {$itok >= $ntok} {break}
        # the next token after end of list bracket should be comma before next cell
        set sep [lindex $tokens $itok]
        puts "END of list separator= $sep"
        if [expr {$sep ne ","}] { break }
        set itok [expr $itok + 1]
    }
    # add the nocell
    global nocellA
    array set nocellA [array get nocell]
    set cells(nocell) nocellA
    lappend cellnames nocell
    # create the cellTXT for now just cellname < 14 chars opacity1 opacity1SD
    set cls [array names cells]
    set cls [lsort -dictionary $cls]
    set cellTXT ""
    set fmt "%-14s %-10s %-10s\n"
    set kys {cellname opacity1 opacity1SD}
	set cmd [list format $fmt]
	foreach ky $kys [lappend cmd $ky]
	set head [eval $cmd]
    append cellTXT $head
    foreach cl $cls {
    	set vals {}
    	foreach ky $kys {
    		lappend vals $cl\A($ky)
    	}
		set cmd [list format $fmt]
		foreach v $vals [lappend cmd $v]
    	set line [eval $cmd]
    	append cellTXT $line
    }
}

proc lookupCell {cel} {
	# find unique string match to lc cel*
	# in lc cellnames
	global cellnames
	foreach celn $cellnames {
		if [string match -nocase $cel* $celn] {return [list 1 $celn]}
	}
	return [list 0 $cel]	
}

#puts "about to call getCells"
#set serverOK [getCells]
#if {$serverOK == 0} {exit}


# guid aux

set labelList {}
set entryList {}
set cmndList {}
set checkList {}
set menuList {}
set radioList {}

array set wcolors {
    labelfg black
    labelbg white
    entryfg black
    entrybg "\#eeeeee"
    entrydisabledfg red
    entrydisabledbackground "\#d9d9d9"
    checkfg white
    checkbg white
    checkactivebg green
    cmndbg green
    cmndfg black
    cmndactivebg yellow
    menufg black
    menubg "\#9999ff"
    menuactivebg "\#ffaa00"
    radiofg black
    radiobg white
    radioactivebg green
    rmenufg black
    rmenubg "\#9999ff"
    rmenuactivebg "\#ffaa00"
}

proc makelabel {w text just args} {
    global wcolors labelList
    lappend labelList $w
	# dont think this needs eval so -font should survive
    label $w -text $text -justify $just \
	    -bg $wcolors(labelbg) -fg $wcolors(labelfg) \
	    -takefocus 0
	foreach {opt val} $args {
		$w configure $opt $val
	}
    return $w
}

proc makeentry {widget tv args} {
    global wcolors entryList
    global $tv
	# here we use eval to pass the textvariable name in tv
    entry $widget -justify right -textvariable $tv \
	-bg $wcolors(entrybg) -fg $wcolors(entryfg) \
	-disabledforeground $wcolors(entrydisabledfg) \
	-disabledbackground $wcolors(entrydisabledbackground)
	foreach {opt val} $args {
		$widget configure $opt $val
	}
    lappend entryList $widget
    return $widget
}

proc makecheckbutton {widget v args} {
    global wcolors checkList
    checkbutton $widget -variable $v \
	-bg $wcolors(checkbg) -selectcolor $wcolors(checkfg) \
	-activebackground $wcolors(checkactivebg)
	foreach {opt val} $args {
		$widget configure $opt $val
	}
    lappend checkList $widget
    return $widget
}

proc makeradio {widget var text value args} {
    global wcolors radioList
    radiobutton $widget -bg $wcolors(radiobg) -fg $wcolors(radiofg) \
	-activebackground $wcolors(radioactivebg) \
	-variable $var -text $text -value $value
	foreach {opt val} $args {
		$widget configure $opt $val
	}
    lappend radioList $widget
    return $widget
}

proc makebutton {widget text cmnd args} {
    # command buttons
    global cmndList wcolors
    button $widget -bg $wcolors(cmndbg) -fg $wcolors(cmndfg) \
	-activebackground $wcolors(cmndactivebg)
	foreach {opt val} $args {
		$widget configure $opt $val
	}
    if {[string length $cmnd] > 0} {
	$widget configure -command "$cmnd"
    }
    if {[string length $text] > 0} {
	$widget configure -text "$text"
    }
    lappend cmndList $widget
    return $widget
}

proc makemenubutton {widget text args} {
    global menuList wcolors
    menubutton $widget \
	-bg $wcolors(menubg) -fg $wcolors(menufg) \
	-activebackground $wcolors(menuactivebg)
	menu $widget.menu
	$widget configure -menu $widget.menu
	foreach {opt val} $args {
		$widget configure $opt $val
	}
    if { [string length $text] > 0 } { $widget configure -text $text }
    lappend menuList $widget
    return $widget
}

set currentDir ""
proc selectDataDirectory {} {
	# NB tcltk needs to have the correct directory which can be set using cd
	# and Tclpbs separately needs to have the correct directory for its file operations
	# set by  i d dirPath
	# NB dirPath must be quoted if it has spaces
    global currentDir mswindows
    global cellDBok cellTXT
    global readlines nread nerr
	global year
    set selDir [tk_chooseDirectory \
			-title "choose your DATA directory for pbconsole"]
    if { [string length [string trim $selDir]] < 1 } { return }
    set currentDir $selDir
    puts "tk_chooseDirectory returned $currentDir"
    cd "$currentDir"
    set checkcd [pwd]
    puts "current working directory is $checkcd"
    if {$checkcd != $currentDir} {
    	Dialog_Warn "ERROR: FAILED to cd to $currentDir  ended up in $checkcd"
    	return
    }

    if [winfo exists .c] {
		wm title .c "PBconsole for cellfile production   $currentDir"
    }
    if [catch {glob *console.log} cfs] {
    	Dialog_Warn "NO *console.log files in $currentDir"
    	return
    }
    puts "glob for onsole.log in $currentDir produced $cfs"

    if {[llength $cfs] == 1} {
    	set cf [lindex $cfs 0]
    	if {! [loadFile $cf]} {
    		Dialog_Warn "Failed to load $cf in $currentDir"
    		return
    	}
    } else {
    	if { ![loadConsoleFile] } { return }
    }
    if {$cellDBok == 1} {return}
    getCellsLocal
	# also check for .bt7 data files
	if [catch {glob *.bt7} dfs] {
    	Dialog_Warn "NO *.bt7 data files in $currentDir"
    	return
	}
	set ndfs [llength $dfs]
	set f [lindex $dfs 0]
	if [catch {set fp [open $f r]}] {
    	Dialog_Warn "Failed to open file $f to look for year"
    	return
	}
	for {set i 0} {$i < 4} {incr i} {
		if [catch {gets $fp ln}] { return }
	}
	# The first word should be numbersignDate
	if [string match "#Date" [lindex $ln 0]] {
		set ds [lindex $ln 1]
		set dsw [split $ds -]
		set yr [lindex $dsw 0]
		if {$yr > 2000 && $yr < 3000} {
			set year $yr
			puts "found year= $year"
		}
	}	
}

proc selectCellDirectory {} {
	# NB tcltk needs to have the correct directory which can be set using cd
	# and Tclpbs separately needs to have the correct directory for its file operations
	# set by  i d dirPath
	# NB dirPath must be quoted if it has spaces
    global currentDir mswindows
    global cellDBok cellTXT
    global readlines nread nerr
    set selDir [tk_chooseDirectory \
			-title "choose your directory for he3Cell DB"]
    if { [string length [string trim $selDir]] < 1 } { return }

    if {![file exists [file join $selDir he3Cells.txt]] && ![file exists [file join $selDir he3Cells.json]]} {
    	Dialog_Warn "NO he3Cell DB in $selDir"
    	return
    }
    getCellsLocal $selDir
}


# build the pbConsole parser gui +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
toplevel .c
wm title .c he3-Console
set m  [frame .c.top]
pack $m -fill x -expand 1
set mfb [makemenubutton $m.mfb File]
set meb [makemenubutton $m.meb Edit]
set filem $mfb.menu
set editm $meb.menu

set mhb [makemenubutton $m.mhb HELP]
set hlpm $mhb.menu
$hlpm add command -label "pbconsole guide" -command "openHelp 0"

#.c configure -menu .c.menubar

#$m add cascade -menu $filem -label File
#$m add cascade -menu $editm -label Edit
$filem add command -label "select data directory with ..console.log" -command "selectDataDirectory"
$filem add command -label "select he3Cell DB directory with he3Cell.txt or he3Cell.json" -command "selectCellDirectory"
$filem add command -label "load Console.log file" -command "loadConsoleFile"
$filem add command -label "paste clipboard to console-text" -command "pasteClipboard"
$filem add command -label "save console-text to file exptid_Console.log" -command "saveConsoleFile"
$filem add command -label "write cellfiles" -command "writeCellFiles"
#$filem add command -label "start over" -command "clearAll"
$filem add command -label quit -command quit

$editm add command -label "preferences" -command "raisePreferences"
$editm add command -label "monitorMTdb" -command "raiseMonitorDB"
$editm add command -label "parse console-text to he3-log" -command "parseConsole"
$editm add command -label "parse he3-log to he3-efficiencies" -command "parseHe3Log"
$editm add command -label "parse he3-efficiencies to cellfile" -command "parseTOcellfiles"

set conB [makebutton $m.console parse-console parseConsole]
set he3B [makebutton $m.he3 parse-he3log parseHe3Log]
set effB [makebutton $m.eff parse-efficiencies parseTOcellfiles]
set wcfB [makebutton $m.wcf write-cellfiles writeCellFiles]

set cdbB [makebutton $m.cdb cellDB gotoCellDBtab]
$cdbB configure -bg red -text "NO cellDB"

grid $mfb $meb $conB $he3B $effB $wcfB -sticky e
grid $cdbB -row 0 -column 8 -sticky w
grid $mhb -row 0 -column 13 -sticky w

proc butdisable {} {
	global conB he3B effB wcfB
	$conB configure -bg grey -state disabled
	$he3B configure -bg grey -state disabled
	$effB configure -bg grey -state disabled
	$wcfB configure -bg grey -state disabled
}
proc butenable {n} {
	global conB he3B effB wcfB
	butdisable
	if {$n == 1} {
		$conB configure -bg green -state normal
	} elseif {$n == 2} {
		$he3B configure -bg green -state normal
	} elseif {$n == 3} {
		$effB configure -bg green -state normal
	} elseif {$n == 4} {
		$wcfB configure -bg green -state normal
	}
}
butdisable

#grid $mfb -row 0 -column 0 -sticky e
#grid $meb -row 0 -column 1 -sticky e
#grid $conB -row 0 -column 3 -sticky w
#grid $he3B -row 0 -column 4 -sticky w
#grid $effB -row 0 -column 5 -sticky w

# RWE Feb 19 2020 just use overlapping sibling frames with raise to get the tab we want
#package require Ttk
#set nb [ttk::notebook .c.nb]
#pack $nb



# there are 3 scrollable text-widgets in each of 3 tabs
array set tabnames {0 console-text 1 he3-log 2 he3-efficiencies 3 cellfiles 4 error-log 5 blocks 6 monitor-trans 7 files 8 cellDB}
set ntabs [llength [array names tabnames]]

set nb [frame .c.bot]
pack $nb -fill both -expand 1
set bf [frame $nb.bf]
set tf [frame $nb.tf]
grid $bf -sticky ew
grid $tf -sticky nsew
grid rowconfigure $tf 0 -weight 1
grid columnconfigure $tf 0 -weight 1

array set txt {}
array set tbf {}
array set but {}

proc selectTab {i} {
	global tbf ntabs but
	raise $tbf($i)
	for {set j 0} {$j < $ntabs} {incr j} {
		$but($j) configure -background white
	}
	$but($i) configure -background green
}

for {set i 0} {$i < $ntabs} {incr i} {
    # NB Scrolled_Text creates the subframes for tabs. It creates and grids the text widgets
    set tbf($i) $tf.f$i
    set txt($i) [Scrolled_Text $tf.f$i -wrap none]
    $txt($i) configure -width 120 -height 40
    grid $tf.f$i -row 0 -column 0 -sticky news
    set but($i) [button $bf.b$i -activebackground green -text $tabnames($i) -command "selectTab $i"]
    grid $but($i) -row 0 -column $i
    #$txt($i) insert 1.0 $tabnames($i)
}

# Try to load the he3Cell DB from server
getCellsServer


#for {set i 0} {$i < $ntabs} {incr i} 
    # NB Scrolled_Text creates the subframes for tabs. It creates and grids the text widgets
#    set txt($i) [Scrolled_Text $nb.f$i -wrap none]
#    $txt($i) configure -width 120 -height 40
#    $nb add $nb.f$i -text $tabnames($i)

set itxt 0

# globals


# bt7 geometry
set bt7AnaTrans 340
set bt7AnaHe3ToAnaColl 25.
set bt7he3Slit 1.
# globals that can be set by pb ct  pb beam  pb fbg pb fru
# pb ct counterDefString ...  define the counters used to get counts
set counters {}

# pb fru
# frupdate 0=e 1=p 2=a  choose what gets updated after flip-ratio
set frupdate e

# set deflt fbgcpm to SDC value about 1 cpm
# pb fbg fbgcpm
set fbgcpm 1


# above are annotation settable globals the controller might want to alter
# following program flags and parameters are accesible in the gui
# frscan Gaussian 3-pt fit flag
set scanfit 0
# global flag to subtract BG if possible when using frscan
set fitScanSubBG 1
# nsig for fitting flat background
set nsigAveFit 2
# flag to treat frscan as data that can then be corrected
set scanfrdata 1
# when measuring transmission or FR for a given beam condition (cell or empty) watch for aberrant count rates defined by rejection conf interval CI
set ctrateRejectionCI 5
# flag to remove bad count rates as defined above
set removeBadCts 1
# when processing flip-ratio for efficiencies check for consistency of transport measurements with the following confidence interval
set transportEffCI 3
# minimum monitor cts to log else try batching them at least in scans
set minMonitorCt 10000
# when fitting mrat to exp throughout mrat values outside mratCI
set mratCI 3
# if mrat deacy is found compare to detector-trans decay and warn if differ by mratDecayCI
# this is fairly lenient as must expect reactor power dips that arent accounted for in the mrat decay
set mratDecayCI 5
# if fraction of removed-mrats/nmrats due to CI are > mratRejFracWarn warn
set mratRejFrac 0.2
# if chired > mratChiRed after throwing out outliers then warn


# allow share nsf counts between Q and V when doing FR
set shareNSF 1
# have one hour to complete a transmission block
set maxTransModeSecs 3600
# default max time between transmissions to be combined as 30 min
set maxtranswait 1800
# and same for flip-ratios
set maxfrwait 1800
# handling more than two transmissions for one cell
# it is possible to just add all transmissions for a cell to that cells install data
# but what if the decay rate changes after a transmission measurement?
# We dont lose much by just using 2 pt segments for transmission data and then reinstalling the
# same cell. The following flag selects the segmented treatment.
# NB the transmission data gets added to both the previous install and the new one.
set transSeg 1

# if a beam geometry parameter changes, at what accuracy do we consider it irrelevant
set geomAcc 0.05

set piRad 3.141592
set degToRad [expr $piRad/180.]
set dspPG002 3.35416
set Dneut 2.072142

# add the monitorMT toplevel
set mratDB [toplevel .mratDB]
wm protocol $mratDB WM_DELETE_WINDOW iconifyPreferences
wm title $mratDB "monitor beam MT table"
wm iconname $mratDB "mrat"
wm iconify $mratDB
set mratt [frame .mratDB.t]
set mratb [frame .mratDB.b]
grid $mratt -sticky ew
grid $mratb -sticky news
set mtxt [text $mratb.txt]
grid $mtxt
set mratLd [button $mratt.ld -text loadDB -command loadMratDB]
set mratSv [button $mratt.sv -text saveDB -command saveMratDB]
set mratEx [label $mratt.ex -text "add or edit empty beam monitor rates"]
grid $mratLd $mratSv $mratEx

proc getMratDB {} {
	global monitorMTbasePGin monitorMTbasePGout
	global monitorMTbaseVSPGin monitorMTbaseVSPGout
	if {[file exists monitorMTbt7.db]} {
		source monitorMTbt7.db
	} else {
		array set monitorMTbasePGin  {13.70 {{1000. open-50-V1H0 20.} {1340. open-80-V1H0 20.}} 14.70 {{1025. open-50-V1H0 20.} {1300. open-80-V1H0 25.}}}
		array set monitorMTbasePGout {13.70 {{2000. open-50-V1H0 35.} {2700. open-80-V1H0 35.}} 14.70 {{2100. open-50-V1H0 35.} {2800. open-80-V1H0 35.}}}
		array set monitorMTbaseVSPGin  {14.70 {{909. open-50-V1H0 15}}}
		array set monitorMTbaseVSPGout {14.70 {{1164. open-50-V1H0 20.}}}
	}
}

proc loadMratDB {} {
	global monitorMTbasePGin monitorMTbasePGout
	global monitorMTbaseVSPGin monitorMTbaseVSPGout
	global mtxt
	$mtxt delete 1.0 end
	foreach mratA {monitorMTbasePGin monitorMTbasePGout monitorMTbaseVSPGin monitorMTbaseVSPGout} {
		array set m [array get $mratA]
		$mtxt insert end "array set $mratA \{\n"
		foreach e [array names m] {
			$mtxt insert end "$e [list $m($e)]\n"
		}
		$mtxt insert end "\}\n"
	}
}

proc raiseMonitorDB {} {
	wm deiconify .mratDB
	raise .mratDB
	loadMratDB
}
proc saveMratDB {} {
	global mtxt
	set fp [open "monitorMTbt7.db" "w"]
	puts $fp [$mtxt get 1.0 end]
	close $fp
	getMratDB
}

# add the configure toplevel
array set globalParams {
	bt7AnaTrans {f 340 "bt7 analyzer translation"} \
	bt7AnaHe3ToAnaColl {f 25 "distance in cm from bt7 analyzer drum to He3Cell center"} \
	bt7he3Slit {f 1 "bt7 transmission slit size in cm"} \
	counters {l {"SDC Total:"} "add detectors speced in counters string list for trans and flipratio"} \
	fbgcpm {f 1 "fast background cpm for bt7. default is for SDC Total: detector counts"} \
	ctrateRejectionCI {f 5 "rejection confidence interval when counting trans or flipratio"} \
	removeBadCts {i 1 "flag to remove counts that fail ctrateRejectionCI"} \
	shareNSF {i 1 "flag to share non-spin-flip counts between Q and V modes for flipratio"} \
	scanfit {i 0 "flag to fit frscan with 3-pt Gaussian"} \
	fitScanSubBG {i 1 "flag to subtract BG for frscan"} \
	nsigAveFit {f 2 "confidence interval for fitting flat background in frscan"} \
	frupdate {s e "choose what to update from a fliratio measurment. default e=transport-efficiency"} \
	useMonitorForTrans {i 0 "flag to request monitor decay transmission check"} \
	envTempTol {f 0.5 "temperature tolerance used in determining if transportEff has changed"} \
	envTempZeroTol {f 3 "temperature tolerance for zero temperature used in determining if transportEff has changed"} \
	envBfldTol {f 0.05 "magnetic field tolerance in Tesla used in determining if transportEff has changed"} \
	transportEffCI {f 3 "confidence interval for transport efficiency msrments and env dependence. warn if outside CI"} \
	minMonitorCt {f 10000 "minimum monitor count to log in tracking inc beam monitor rates. If less try batching"} \
	mratCI {f 3 "when fitting mrat decay throw away mrat values outside this CI"} \
	mratDecayCI {f 5 "warn if transmission decay differs from mrat decay by this CI"} \
	mratRefFrac {f 0.2 "warn if outliers-rejected-by-mratCI/total-mrats excedes this fraction"} \
	maxTransModeSecs {f 3600 "transmission measurements must complete in this time period secs"} \
	maxtranswait {f 1800 "maximum time in sec between transmission that are to be combined"} \
	maxfrwait {f 1800 "maximum time in sec between flipratio measurments that are to be combined"} \
	transSeg {i 1 "flag to request cell decay always uses exact two-point fit between transmission blocks"} \
	geomAcc {f 0.05 "if any beam geometry parameter changes by more than geomAcc force a new cell install"} \
}

set cfg [toplevel .cfg]
wm protocol $cfg WM_DELETE_WINDOW iconifyPreferences
wm title $cfg "pbconsole configure"
wm iconname $cfg "CFG"
wm deiconify $cfg
set cfgt [frame .cfg.t]
set cfgb [frame .cfg.b]
grid $cfgt -sticky ew
grid $cfgb -sticky news

set gpnames [array names globalParams]
set gpnames [lsort $gpnames]
set ngp [llength $gpnames]
puts "ngp= $ngp"
for {set i 0} {$i < $ngp} {incr i} {
	set gp [lindex $gpnames $i]
	set gpdata $globalParams($gp)
	set val [lindex $gpdata 1]
	set dsc [lindex $gpdata 2]
	set gpl [makelabel $cfgb.l$i $gp right]
	set gpe [makeentry $cfgb.e$i $gp]
	set $gp $val
	set gpd [makelabel $cfgb.d$i $dsc left]
	grid $gpl -row $i -column 0 -sticky e
	grid $gpe -row $i -column 1
	grid $gpd -row $i -column 2 -sticky w
}

proc raisePreferences {} {
	wm deiconify .cfg
	raise .cfg
}
proc iconifyPreferences {} {
	wm iconify .cfg
}

proc pasteClipboard {} {
    global txt utc datestr
    if {[catch {clipboard get} contents]} {
        return
    }
    butdisable
    # if txt already has lines look for last non-blank line
    set last [$txt(0) index end]
    set lastxt [$txt(0) get $last "$last lineend"]
    set utc 0
    set prevtxt 0
    while {[string length $lastxt] < 15 && $last > 1.0} {
    	if {[string length $lastxt] >= 15 && [getUTC $last] > 0} {
    		set prevtxt 1
    		break
    	}
    	set last [$txt(0) index "$last - 1 lines"]
    	set lastxt [$txt(0) get $last "$last lineend"]
    }
    if {$prevtxt == 0} {
    	$txt(0) delete 1.0 end
    	$txt(0) insert 1.0 $contents
    	butenable 1
    	return
    }
    # got utc from prev text end
    set utcPrev $utc
    set datestrPrev $datestr
	# get the start utc from clipboard contents that is later than utcPrev
	set utc 0
	set starti -1
	set clines [split $contents '\n']
	set nclines [llength $clines]
	for {set i 0} {$i < $nclines} {incr i} {
		set ln [lindex $clines $i]
		set utc [datestrTOutc [string range $ln 0 14]]
		if {$utc > $utcPrev} {
			set starti $i
			break
		}
	}
	if {$starti < 0} {
		Dialog_Warn "FAILED to find later console lines in clipboard."
		return
	}
	for {set i $starti} {$i < $nclines} {incr i} {
		set ln [lindex $clines $i]\n
		$txt(0) insert end $line
	}
	butenable 1
}

set filetypes {
    {{Log Files}   {.log}}
}

proc saveConsoleFile {} {
	global txt exptid
	set filename "$exptid\_Console.log"
	if {[catch {open $filename w} fpt]} {
		tk_dialog .warn Warning "Failed to open $filename" "" 0 "OK"
		return
	}
	set contents [$txt(0) get 1.0 end]
	puts -nonewline $fpt $contents
	close $fpt
}

proc loadFile {f} {
	global txt
	if {[string length $f] < 1} { return 0 }
	if {![file exists $f]} { return 0 }
	if {[catch {open $f r} fpt]} { return 0 }
    set contents [read $fpt]
    close $fpt
    $txt(0) delete 1.0 end
    $txt(0) insert 1.0 $contents
    # add to wm title
    set wmt [wm title .c]
    wm title .c "$wmt $f"
    butenable 1
    selectTab 0
    return 1
}

proc loadConsoleFile {} {
    global filetypes
    set filename [tk_getOpenFile -filetypes $filetypes]
    if {! [loadFile $filename]} {
    	tk_dialog .warn Warning "Failed to open $filename" "" 0 "OK"
		return 0
    }
	return 1
}

proc clearAll {} {
	# to start over
    global ntabs txt
    for {set i 0} {$i < $ntabs} {incr i} {
        $txt($i) delete 1.0 end
    }
    butdisable
    pipeWrite "p cgx"
}
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


########## global data
# pb beam args   global flag for beamsize. ended up using beamsize in FR where 0 is smallBeam (pinhole) condition
set beamsize 1
set mratChired [expr $mratCI - 1.]
array set frupdates {e 0 p 1 a 2 w 3}
# pb hkl h k l  the Bragg peak for Flip Ratio is tracked so that we dont have to repeat background msrments
set hkl {0 0 0}
# flipper stuff efficiency and rfLossPerFlip
# pb feff a|p efficiency   pb fefferr a|p err   pb rfloss a|p  POLARIZATIONlossperflip
array set flipEff {0 1 1 1}
array set flipEffErr {0 0 1 0}
array set rfFlipLoss {0 0 1 0}

# globals that should be determined from the input console-text
set instr 1
set exptid 0
set startDate ""
array set instrs {0 ng0 1 bt7}
# polbeam==1 is set on any pb both|fr command. polbeam==0 is set for pb end OR pb stop
# This is used to record which datafiles are using polarized beam.
set polbeam 0
set datestr ""
set utc 0
set annotated 0

array set annotatedParsers {0 annotng0 1 annotbt7}
array set rawParsers {0 rawng0 1 rawbt7}

# beamCell is the m p a both condition string. Always init to m
set beamCell m
# in cellNames P A polarizer and analyzer O is for opacity. track the name of cell in beam
array set cellNames {p mt a mt m mt}
# track installs
set ncellInstalls 0
set cellInstalls {}
set ninstalls 0
set installs {}
# track polarizers
set npolarizers 0
set polarizers {}
set nanalyzers 0
set analyzers {}
# track cell-pairs
set npairs 0
set pairs {}
set pairsinstalls {}
set pairsfiles {}
set pairsflipratios {}
set pairsGeom {}
# record transissions
set transmissions {}
set alltransmissions {}
# set opacity=1 when measuring opacity
set opacity 0
# opacity defaults to msrment at analyzer position
set opacityPos a

# track monitor rates. 
# record monitorMT(Ei)  list of lists= [[utc moncts secs]...] which should be time independent unless inc beam changes
# record monitorHe3(Ei) list of lists= [[utc moncts secs cellName]...]
# for in and out PGfilt conditions and for VS=velocity selector
array set monitorMTPGin {}
array set monitorMTPGout {}
array set monitorMTVSPGin {}
array set monitorMTVSPGout {}

array set monitorHe3PGin {}
array set monitorHe3PGout {}
array set monitorHe3VSPGin {}
array set monitorHe3VSPGout {}
array set monitorHe3bothPGin {}
array set monitorHe3bothPGout {}
array set monitorHe3bothVSPGin {}
array set monitorHe3bothVSPGout {}
array set monitorVS {}
# store MT beam monitor-rates at 20 MW  by coll PGfilt focusV focusH cond = focusV+2*focusH+4*PGin
# eg open-50-PGin-V1H0. We assume the paerature is always wide open which is current policy
# if we cant get MT monitor rate from experiment can try this table lookup

array set monitorMTbasePGin  {}
array set monitorMTbasePGout {}
array set monitorMTbaseVSPGin {}
array set monitorMTbaseVSPGout {}
# BT7 monitor preamp adjustment changed all monitor rates by a scale factor of 1.345 approx about May 1 2018 ??? Jeff Ziegler
set monitorCHANGEutc 1525190400
set monitorCHANGEfac 1.34615

proc monitorMTbaseScale {scl ecl} {
	# return the ecl with monitor rate cps and its error scaled by scl
	set conds [lindex $ecl 1]
	set newconds {}
	foreach c $conds {
		set mrt [expr $scl*[lindex $c 0]]
		set con [lindex $c 1]
		set err [expr $scl*[lindex $c 2]]
		lappend newconds [list $mrt $con $err]
	}
	return [list [lindex $ecl 0] $newconds]
}

# estimate the OPEN value for preMonoColl in minutes by comparing open vs 25 minutes at 15 min exit coll
set openPMmin 85

proc monitorMTbaseLoad {utc} {
	# this handles the change in monitor rate for bt7
	global monitorMTbasePGin monitorMTbasePGout monitorMTbaseVSPGin monitorMTbaseVSPGout 
	global monitorCHANGEutc monitorCHANGEfac
	global openPMmin
	global c13in
	if {$utc > $monitorCHANGEutc} {set scl 1.34615}	
	if {$utc <= $monitorCHANGEutc} {set scl 1.}
	if {[file exists monitorMTbt7.db]} {
		source monitorMTbt7.db
	} else {
		set c13in [list 13.70 [list [list 65.3 25-15-V1H0 2.5] [list 219 $openPMmin-15-V1H0 6.] [list 1000. $openPMmin-50-V1H0 20.] [list 1340. $openPMmin-80-V1H0 20.]]]
		set c14in [list 14.70 [list [list 69.8 25-15-V1H0 2.5] [list 234 $openPMmin-15-V1H0 6.] [list 1025. $openPMmin-50-V1H0 20.] [list 1367. $openPMmin-80-V1H0 25.]]]
		set c13out [list 13.70 [list [list 2000. $openPMmin-50-V1H0 35.] [list 2700. $openPMmin-80-V1H0 35.]]]
		set c14out [list 14.70 [list [list 2100. $openPMmin-50-V1H0 35.] [list 2800. $openPMmin-80-V1H0 35.]]]
		set c14inv [list 14.70 [list [list 909. $openPMmin-50-V1H0 15]]]
		set c14outv [list 14.70 [list [list 1164. $openPMmin-50-V1H0 20.]]]
		array set monitorMTbasePGin  [monitorMTbaseScale $scl $c13in]
		array set monitorMTbasePGin  [monitorMTbaseScale $scl $c14in]
		array set monitorMTbasePGout [monitorMTbaseScale $scl $c13out]
		array set monitorMTbasePGout [monitorMTbaseScale $scl $c14out]
		array set monitorMTbaseVSPGin  [monitorMTbaseScale $scl $c14inv]
		array set monitorMTbaseVSPGout [monitorMTbaseScale $scl $c14outv]
	}
}
# MT monitor rate changes by monitorRateChange allow looking for better match to beam condition in the tables
# while allowing about 2 MW fluctuations
set monitorRateChange 0.1
# what is std rel err in MT monitor Table values
set monitorRateTableErr 0.035
# track monitor rate changes list utc Ei beamCondStr mrat mratErr
set incBeamCond {}
# track ct counts with list [utc cts secs]
set ctsList {0 0 0}
set timelbl "Time="
set monitorlbl "Monitor="
# for now we dont handle monitor2 which may be before He3 on bt7

# global instrument and experiment condition avariables set mostly by: pb globalName value(s)
set Ei 0
set Ef 0
# track current FLIP state
set FLIP ""
# track flipstate vs time with list of lists={utc flipstate}
set flipstatestr offoff
set flipstate {0 0}
set flipstates {{0 offoff}}
# track current flippers
set flipP 0
set flipA 0
# track current VQmode by index -1=undefined 0=V 1=Q
set VQmode 0
array set VQmodes {0 V 1 Q}
# and by letter
set VQ V
# track Qmode vs time with start stop utcs
set qmodes {}

set transangle 0
set transdetoff 0

# track inpile PG filter
set PGfilt ""
# no velocity selector yet
set VelSel ""
set focusH ""
set focusV ""
set premonocoll 0

# record utcs for flips incase want to apply rfFlipLoss
set flipsP {}
set flipsA {}

set eiPhrases {"pb ei" "moving ei "}
set eiRegex {moving ei |pb ei}
set efRegex {moving ef |pb ef}
set txtend 0
# WOW no direct way to get year from console
# If this software is used in a later year than the expt need to set the year in the console log on the exptid line by adding year=20xx
set startUTC 0
# these will year and month when analysis starts. This are likely incorrect for the data taking
# are supposed to be updated from the first data file.
set cs [clock seconds]
set year [clock format $cs -format %Y]
set lastmonth [clock format $cs -format %b]

set events {}
# events is time ordered list of he3 events
# each event is a list {utc cmnd datawords}
# cmnd is the word after #pb
# datawords depends on the cmnd
# Some global cmnds have no data: Q V offoff etc
# Some global cmnds have a single param: E ei, Lamb lambi, angle angle, detoff angle,
# a few global cmnds have more than one word: hkl h k l, beam 1cm sampleH 0.5cm
# transmission cmnds P A MT have  utc nettimesecs netCts stdErr   will need Ei beam to compute
# flip-ratio cmnd fr has          utc nettimesecs netCts stdErr  Q|V flipstate   will also need globals for pass2 calc
set pbfiles {}
# pbfiles is list of scanfiles during polarized-beam-mode
# each has datestr utc VQ flipstatestr FLIP Pname Aname fileNameRoot
set warnings {}

set endCountPhrases {"pb A" "pb P" "pb MT" "pb both" "pb O" "pb fr" "move" "scan"}

set endCountReg {pb\s+[^\s]+|\.seq|scan|stopall|moving|setproperty}
set endCount 1.0

set ctrates {}
set monrates {}
set totalCtsList {}
set totalSecList {}
set totalMonList {}

set lastMratUTC 0

set nextphrase ""
set phraseline ""

set frReg {pb\s+b|Counters:|moving flip|moving hsample}
set endFrReg {pb\s+[^bcoOqvQV].+|scan|stopall|moving temp|moving ei\s+|moving filtran|setproperty Ei velselcoupled}

set frscanReg {pb\s+b|START OF SCAN|Find Peak|Moving FLIP|Moving hsample}
set endFrscanReg {pb\s+[^bcoOqvQV].+|stopall|moving temp|moving ei\s+|moving filtran|setproperty Ei velselcoupled}


array set V0 {}
array set V1 {}
array set Q0 {}
array set Q1 {}
# for frscan need concomitant background processing so set up another set of data arrays
array set V0B {}
array set V1B {}
array set Q0B {}
array set Q1B {}
#
array set modes0 {V V0 Q Q0}
array set modes1 {V V1 Q Q1}
#
array set modes0B {V V0B Q Q0B}
array set modes1B {V V1B Q Q1B}

# no longer using modes or frbeams. Using direct access to V0 V1 Q0 Q1 etc thru FRarray procs
array set frbeams {0 modes0 1 modes1}
#
array set frbeamsB {0 modes0B 1 modes1B}

set frscanvars {Qx Qy Qz A4 A3}
set lcfrscanvars {qx qy qz a4 a3}
array set frdata {}
set endScan ""

set badrates {}

set feffA 0
set feffP 0
set feffAerr 0
set feffPerr 0

set frerrors {}
set sharedwith ""

set frctrates {}

set fitAve 0
set fitAveErr 0

# no longer support seq cmnds in pb globalZero
set globalZero {updatep updatea}
set globalOne {ei lamb angle detoff}
# globalThree have p|a float  syntax
set globalThree {rfloss feff fefferr}
# also frupdate a|p|e

# pb beam handles slit as 1 or 1cm or open beam size, sampleH and W as sh=ncm or shncm and same for sw
# also collimations as hc[ol]sm hc[ol]sa hc0=hcm is the inpile hcol hcol  horDivPFWHMmin horDivAFWHMmin
# vcol are calculated from instrument geometry sh bsh fsh
set beampats {open full 1 1cm *cm sh* sw* sm* pg* *in *out pol* anatrans* *mev hc* vc* hmos* temp* fsh* fsw* focus* vs* vel*}

set bt7seqPhrases {"Qmode.seq" "Vmode.seq" "offoff.seq" "ONoff.seq" "offON.seq" "ONON.seq"}
set bt7annotPhrases [concat $bt7seqPhrases "pb"]

# RWE Jan 31 2020 should not be parsing seq as this is just adding to queue and may in fact not be executed. Always look for moving instead
# [^\s:]+mode\.seq\.txt|[^\s:]+off\.seq\.txt|[^\s:]+ON\.seq\.txt

set bt7annotRegex {pb\s+[^\[\]]+$|moving ei |moving ef |moving flip|moving hsample|moving back|moving filtran|moving premonocoll|START OF SCAN|Find Peak|horizontal focus|vertical focus|setproperty Ei velselcoupled}
array set VQofIndex {-1 N 0 V 1 Q}

array set FLIPVSflipstate {offoff A ONoff B offON C ONON D}
array set flipstateVSFLIP {A offoff B ONoff C offON D ONON}
array set flippersVSFLIP {A {0 0} B {1 0} C {0 1} D {1 1}}
array set flipstateVSindex {0 off 1 ON}


# global env flag tracks when beamParse finds an env condition like temp
# this is currently not used. Instead use findEnv
set envflg 0
set envstr ""
array set envRanges {}
array set envSecs {}
array set envCurRange {}
array set envCurRangeStr {}
set envKeys {}
array set envReg {}
set envReg(temp) {Moving temp [^\s]+|temp=[^\s]+}
set envReg(bfld) {Moving magfield [^\s]+|Magfield=[^\s]+}

set lastTemp 0
set lastBfld 0
set lastTempIndex 1.0
set lastBfldIndex 1.0
set envTemp 0
set envBfld 0
set envTempTol 0.5
set envTempZeroTol 3
set envBfldTol 0.05

set nit 0
set maxit 0
set niterror 0
set nfiles 0
set nflipratios 0
set ntotalfiles 0
set polstops {}
set flipratios {}
set transmodeStartSecs 0
set forceinstall 0
set warnings {}

array set envSecs {}
array set envCurRange {}

# bt7 geometry in cm. First are fixed geometry
# starting with the standard translation value of analyzer trans for polarizedBeam
# Normally this is all the way back to 340 although sometimes I think it runs at 300
# for the geometry calc this only makes a small difference.
set Pi 3.14159
set radToDegFac [expr 360./(2.*$Pi)]


# the bt7PolHe3ToEnc is currently fixed by magicBox geometry
set bt7PolHe3ToEnc 37.
set bt7EncToFS 16.
# The following should not be adjustable assuming we have the correct values
set bt7SampleEncRadius 38.
set bt7SampleEncToMon 2.
set bt7AnaToDet 35.
set bt7AnaDrumRad 70.
set bt7DetH 10.
set bt7SampleToMono 206.
set bt7SampleToMon [expr $bt7SampleEncRadius + $bt7SampleEncToMon]
set bt7MonToMono [expr $bt7SampleToMono - $bt7SampleToMon]
set bt7MonDia 6.
set bt7monoH 20.

# default for bt7 back slit is 100 mm which makes it non obstructing
# This can be changed if a Moving BackSlitHeight is found
set bt7bsh 10.
set bt7bsw  5.
set bt7fsh  3.
set bt7fsw  3.

# now compute dependent geometry
set bt7SampleToAna [expr 165. + 64.*($bt7AnaTrans/360.)]
set bt7SampleToPolHe3 [expr $bt7SampleEncRadius + $bt7PolHe3ToEnc]
set bt7MonToPolHe3 [expr $bt7SampleToPolHe3 - $bt7SampleToMon]
set vertDivFWHMradMon [expr ($bt7monoH - $bt7MonDia)/$bt7MonToMono]
set vertDivFWHMdegMon [expr $radToDegFac*$vertDivFWHMradMon]
set hPmon [expr $bt7MonDia + $vertDivFWHMradMon*$bt7MonToPolHe3]
set wPmon 5.
# use ellipse beam cross-section at monitor
set beamAreaMon [expr ($Pi/4.)*$hPmon*$wPmon]
# need sample to detector distance DD
set DD [expr $bt7SampleToAna + $bt7AnaToDet]
set bt7SampleToAnaColl [expr $bt7SampleToAna - $bt7AnaDrumRad]
# back slit is about 10 cm upstream from anaColl
set bt7SampleToBS [expr $bt7SampleToAnaColl - 10]
# need sample to he3 analyzer cell center = S3
set S3 [expr $bt7SampleToAnaColl - $bt7AnaHe3ToAnaColl]
set bt7SlitToAnaHe3 [expr $S3 - $bt7SampleEncRadius]
set bt7SlitToDet [expr $DD - $bt7SampleEncRadius]
set bt7SlitToBS [expr $bt7SampleToBS - $bt7SampleEncRadius]
set bt7SlitToFS [expr 2.*$bt7SampleEncRadius - $bt7EncToFS]
set bt7SlitToPolHe3 [expr 2.*$bt7SampleEncRadius + $bt7PolHe3ToEnc]
set bt7SlitToMono [expr $bt7SampleToMono + $bt7SampleEncRadius]
# to determine the beam vertical divergence angle at the 1cm slit
# This divergence is limited by one of the detectorHeight, bsHeight, sampleH or the monoH
# since there are no slit systems between mono and sample
# monoTosample = 206 cm
# monoH = 20 cm for Vfocus
# For detector: Vdiv = DetH/slitToDet
# For sample:   Vdiv = sampleH/sampleEncRadius
# For mono:     Vdiv = monoH/slitToMono


set beamSizeFudge 0.8
# the beam vertical profile gets squared up after the slit so there
# is a sharp cutoff in neutron flux at the analyzer cell (and polarizer)
# That is the beam area calculated with vert div limits should be converted
# from squared to Gaussian by the (2ln2)/sqrt(Pi) factor = 0.782
set sampleH 1
set sampleW 1
# there are several sampleMos variables
# sampleMosMin is the one stored in pairsGeom for taking data and also used for transmission calculations
# since the effective sample mosiac may vary between transmission and data
# we have separate variables for those
set sampleMosMin 60
set sampleMosData 60
set sampleMosTran 60
set beamAreaP 1
set beamAreaA 1
set beamAreaPopen 5
set beamAreaAopen 5
# for 3-axis SH is the beam height at cell center not the actual sample height
set shPopen 1
set shAopen 1
set hP 1
set hA 1
set hPo 2
set hAo 2
# vertDivFWHMdeg is for the slit condition
# but for powder sample the front and back divergences are allowed to be different
set vertDivFWHMdeg 2.6
set vertDivPFWHMdeg 2.6
set vertDivAFWHMdeg 2.6
# max sample height for vertical focus
set bt7focusH 3
# the open beam vertical divergence determined by monochromatorHeight and focussed height
# This calcs to about 4.7 degrees

set vertDivFWHMradOPEN [expr ($bt7monoH - $bt7focusH)/$bt7SampleToMono]
set vertDivFWHMdegOPEN [expr $vertDivFWHMradOPEN*$radToDegFac]
# the horizontal divergence angles can be set in pb beam ?
# strictly the sample crystal mosaic contributes to horDiv but this is samll effect
# and we can fudge this in horDiv FWHMdeg values
# In case powder or inelastic allow front and back vertical divergence to differ
set vertDivPFWHMdegOPEN 2.6
set vertDivAFWHMdegOPEN 2.6
set horDivAFWHMmin 50
set horDivPFWHMmin 50
set horDivInPilemin 100

set incBeamUTCranges {}

set efficiencyLog {}
set transblock {}
set transpairs {}

set transblocks {}
set frpairs {}
set frblocks {}
set newfrblocks {}
set frpairlists {}

set geoATutc {}

# 3-axis specific
set openbeamCellVarsP {hPo beamAreaPopen horDivPFWHMmin vertDivPFWHMdegOPEN bt7SampleToPolHe3 DD sampleMosMin}
set openbeamCellVarsA {hAo beamAreaAopen horDivAFWHMmin vertDivAFWHMdegOPEN S3 DD sampleMosMin}
set cellfileParams {SH beamArea Hcols Vcols S3 DD SampleMos}
set flipCellVars {rfFlipLoss flipEff flipEffErr}
set cellfileParamsF {rfLossPerFlip feff ferr}

set incBeamUTCcond "unknown"

set frc 1
set frcerr 0
set frwarnings {}

set dbCols {Lcm Hcm rCURVcm glassTrans1 opacity1 opacity1SD}
set pbCols {L   D   R       tEmpty      nsl1     nsl1err}


array set effs {}
array set effsList {}
array set bflds {}
array set temps {}
array set distinctEnv {0 {} 1 {}}
array set finstallfiles {}
array set installeffs {0 {} 1 {}}
set pairsForIns {}

set cellfileLabels {cellName PorA iDate iTime iUNIXtime iPol iPolErr Thr Terrhr Acm2 tEff tErr fEff fErr tEmpty tESlope}
lappend cellfileLabels Lcm Diacm rCRVcm nsL0 nsL0err HmosMin dspA HcolsMin Hcol2Min VcolsDeg Vcol2Deg tElapsed
lappend cellfileLabels SZcm DDcm SHcm A4angle A4range offset rfLossPerFlip SampleMosMin
set cfspc 2


# END of proc globals
###############################################################################################
# START procs

proc findNextEi {istar} {
    global txt
    global Ei
    global eiPhrases eiRegex
    global nextphrase
    set eip [findNextReg $istar $eiRegex]
    #puts "findNextEi eip= $eip nextphrase= $nextphrase"
    if {$eip} {
        set nc [string length $nextphrase]
        set estar [$txt(0) index "$eip + $nc chars"]
        set etxt [$txt(0) get $estar "$estar lineend"]
        #puts "findNextEi etxt= $etxt"
    }
    if {! $eip || [scan $etxt "%g" Ei] < 1} {
        return 0
    }
    return $eip   
}
proc findNextEf {istar} {
    global txt
    global Ef
    global efPhrases efRegex
    global nextphrase
    set efp [findNextReg $istar $efRegex]
    #puts "findNextEf efp= $efp nextphrase= $nextphrase"
    if {$efp} {
        set nc [string length $nextphrase]
        set estar [$txt(0) index "$efp + $nc chars"]
        set etxt [$txt(0) get $estar "$estar lineend"]
        #puts "findNextEf etxt= $etxt"
    }
    if {! $efp || [scan $etxt "%g" Ef] < 1} {
        return 0
    }
    return $efp   
}

proc parseDetectors {ictxt} {
	global counters
    if {[llength $ictxt] > 2} {
    	set ctl [lrange $ictxt 2 end]
        set ctl [split $ctl +]
        foreach ct $ctl {
            set ctr [string trim $ct]
        	lappend counters $ctr
        }
    }
}
proc parseHfocus {foctxt} {
	global focusH
	if {[llength $foctxt] < 1} {return}
    set foccond [lindex $foctxt 0]
    set focusH 0
    if {$foccond != "FLAT"} {set focusH 1}
}
proc parseVfocus {foctxt} {
	global focusV bt7monoH
	if {[llength $foctxt] < 1} {return}
    set foccond [lindex $foctxt 0]
    set focusV 1
    if {$foccond != "SAGITTAL"} {set focusV 0}
    set bt7monoH 20.
    if {$focusV == 0} {
    	set bt7monoH 6.
    }
}
proc parseVelSel {vstxt args} {
	global VelSel datestr utc
	if {[llength $vstxt] < 4} {return}
	set valus [lindex $vstxt 3]
	if {[string match -nocase true $valus]} {
		set VelSel 1
	} elseif {[string match -nocase false $valus]} {
		set VelSel 0
	}
	if {[llength $args] > 0} {return}
	logEvent [list $datestr $utc VelSel $VelSel]
}

proc nextNonBlankLine {i} {
	global txt itxt
	set txtend [$txt($itxt) index end]
	if {$i >= $txtend} {return 0}
	set i [$txt($itxt) index "$i linestart"]
	while {$i < $txtend} {
		set ln [$txt($itxt) get $i "$i lineend"]
		if {[string length $ln] > 0} {return $i}
		set i [$txt($itxt) index "$i + 1 lines"]
	}
	return 0
}

proc getDataFileName {sti sdir} {
	# console parse a Creating file line for the datafile name
	# return a list with the lineindex and then the filename
	# sti is the start index and sdir is search direction reversed if < 0
	global txt itxt
	set sw -forwards
	set endi [$txt($itxt) index end]
	if { $sdir < 0 } {
		set sw -backwards
		set endi [$txt($txt) index 1.0]
	}
	set fi [$txt($itxt) search -nocase $sw "Creating file" $sti $endi]
	if { [string length $fi] < 1 } { return {} }
	set ln [$txt($itxt) get $fi "$fi lineend"]
	if { [llength $ln] < 3 } { return {} }
	set fis [$txt($itxt) index "$fi linestart"]
	set fil [file tail [lindex $ln 2]]
	return [list $fis $fil]
}

proc getDataFileParams {fd} {
	global nerr nwarn nread readlines errlines warnlines
	global datestr utc
	global year lastmonth months
	global focusH focusV Ei Ef PGfilt VelSel premonocoll bt7bsw bt7bsh
	puts "getDataFileParams starting for file= $fd"
    pipeWrite "f+ $fd"
    if {$nerr > 0} {
    	set etxt [join $errlines "\n"]
    	Dialog_Warn "ERROR: Failed to read datafile $fd.\n$etxt"
    	return
    }
    # OK to get info from this file
    pipeWrite "d h #Date"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set dat [lindex $rl 1]
    	set datl [split $dat -]
    	set year [lindex $datl 0]
    	set monnum [lindex $datl 1]
    	set lastmonth [lindex $months [expr $monnum - 1]]
    }
    # look for mono focussing
    pipeWrite "d h #MonoVert"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set dat [lindex $rl 1]
    	set newFocusV 0
    	if [string match SAGITTAL $dat] {set newFocusV 1}
    	if { $newFocusV != $focusV } {
    		set focusV $newFocusV
    		logEvent [list $datestr $utc beam focusV $focusV]
    	}
    }
    pipeWrite "d h #MonoHoriz"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set dat [lindex $rl 1]
    	set newFocusH 1
    	if [string match FLAT $dat] {set newFocusH 0}
    	if { $newFocusH != $focusH } {
    		set focusH $newFocusV
    		logEvent [list $datestr $utc beam focusH $focusH]
    	}
    }
    # get Ei Ef PreMonoColl Filtran Velsel from their column data
    pipeWrite "c dl Ei"
	if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set newEi [lindex $rl 1]
    	if { $newEi != $Ei } {
    		set Ei $newEi
    		logEvent [list $datestr $utc ei $Ei]
    	}
    }
    pipeWrite "c dl Ef"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set newEf [lindex $rl 1]
    	if { $newEf != $Ef } {
    		set Ef $newEf
    		logEvent [list $datestr $utc ef $Ef]
    	}
    }
    pipeWrite "c dl Filtran"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set fts [lindex $rl 1]
    	set newPGfilt "out"
    	if { [string match "IN" $fts] } { set newPGfilt "in" }
    	if { $newPGfilt != $PGfilt } {
    		set PGfilt $newPGfilt
    		logEvent [list $datestr $utc beam PGfilt $PGfilt]
    	}
    }
    pipeWrite "c dl Velsel"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set vsval [lindex $rl 1]
    	set newVelSel 0
    	if { $vsval > 0 } { set newVelSel 1 }
    	puts "getDataFileParams found Velsel val= $vsval"
    	if { $newVelSel != $VelSel } {
    		puts "getDataFileParams setting VelSel= $newVelSel"
    		set VelSel $newVelSel
    		logEvent [list $datestr $utc beam VelSel $VelSel]
    	}
    }
    pipeWrite "c dl PreMonoColl"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set val [lindex $rl 1]
    	if { $val == "OPEN" } { set val 85 }
    	set newpremonocoll $val
    	if { $newpremonocoll != $premonocoll } { updatePreMonoColl $newpremonocoll }
    }
    pipeWrite "c dl BkSltWdth"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set val [lindex $rl 1]
    	# convert to cm
    	set val [expr $val/10.]
    	set newbt7bsw $val
    	if { $newbt7bsw != $bt7bsw } {
    		set bt7bsw $newbt7bsw
    		logEvent [list $datestr $utc beam bsw $bt7bsw]
    	}
    }
    pipeWrite "c dl BkSltHght"
    if {$nread > 0} {
    	set rl [lindex $readlines 0]
    	set val [lindex $rl 1]
    	# convert to cm
    	set val [expr $val/10.]
    	set newbt7bsh $val
    	if { $newbt7bsh != $bt7bsh } {
    		set bt7bsh $newbt7bsh
    		logEvent [list $datestr $utc beam bsh $bt7bsh]
    	}
    }
}

set months {Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec}

proc parseConsole {} {
    # console text must be in global txt(0)
    # first check for instr name since cellfile prep is instr dependent
    # and find the exptid used for naming the cellfile
    # often as on MACS the exptid is turned into the experiment start date as MODYYR etc
    # also try to determine beam conditions before the first transmission is done 
    # log the open beam, check year premonocoll Ei Ef Velsel PGinpile(BT7) what about MACS for beam filtering?
    # if there is a data file written before the first transmission we can use that for bt7 at least.
    global txt itxt
    global txtend
    global exptid year startDate startUTC lastmonth months
    global instrs
    global instr
    global annotated
    global Ei Ef
    global nextphrase
    global events eventNow
    global annotatedParsers
    global rawParsers
    global utc datestr PGfilt polbeam sampleH sampleW sampleMosMin bt7monoH
    global focusH focusV VelSel
    global counters
    global nerr nwarn nread readlines

    set itxt 0
    $txt(1) delete 1.0 end
    set txtend [$txt(0) index end]
    set instr 0
    set firstline [nextNonBlankLine 1.0]
    # look for the first transmission annotation
    set fpb [$txt(0) search -nocase "pb m" 1.0 end]
    if { [string length $fpb] < 1 } {
    	set fpb [$txt(0) search -nocase "pb p" 1.0 end]
    }
    if { [string length $fpb] < 1 } {
    	Dialog_Warn "There are no annotated transmissions (pb m or pb p) in this conole.log"
    	exit
    }
    # look for the first data file
    # deflt instr is bt7 but check data file name
    set instr 1
    set fdl [getDataFileName 1.0 1]
    if { [llength $fdl] < 2 } {
    	Dialog_Warn "There are no datafiles created for this console.log"
    } else {
    	puts "getDataFileName returned fdl= $fdl"
    	set fd [lindex $fdl 1]
    	set fdi [lindex $fdl 0]
    	getUTC $fdi
    	# whether or not the first file is before the transmission measurent, init the beam conditions year month from that file
    	# Any moves before the first transmission should handle changes when parsing the console log.
    	getDataFileParams $fd
    	set instrname [file extension $fd]
    	if { $instrname == ".bt7" } {
    		set instr 1
    	} elseif { $instrname == ".ng0" } {
    		set instr 2
    	} else {
    		Dialog_Warn "Unknown instrument data file extension $instrname"
    	}
    }
    getUTC $firstline
    #if {[$txt(0) search -nocase bt7 1.0] ne ""} { set instr 1 }
    set idp [$txt(0) search -nocase -regexp {experiment id [^0-9]*[0-9]+} 1.0]
    set exptid 0
    if {$idp ne ""} {
		#puts "found experiment id line" has experiment id set to idnum year=YEAR
        set idtxt [$txt(0) get $idp "$idp lineend"]
		#puts "exptidtxt= $idtxt"
		set idstr [lindex $idtxt 4]
		if {[scan $idstr %d exptidtest] > 0} {
		    set exptid $exptidtest
		}
		if {[llength $idtxt] > 5} {
		    set yrstr [lindex $idtxt 5]
		    if {[scan $yrstr year=%d yeartest] > 0} {
		        set year $yeartest
		        if {$year < 2000} {set year [expr $year + 2000]}
		        # reget the UTC if year is set here
		        getUTC $firstline
		    }
			# year is used in getUTC		    
		}
        set idok [regexp {\s+([0-9]+)} $idtxt whole idstr]
		#puts "exptid regexpOK= $idok"
        if {$idok} {
			set exptid $idstr
        	# dump the whole id line to he3 log
        	getUTC $idp
        	if {$startDate == ""} {set startDate [clock format $utc -format %b%d_%Y]}
		}
        logEvent [list $datestr $utc id $exptid]
        logEvent [list $datestr $utc instr $instr $instrs($instr)]
    }
    #getUTC $firstline
    set startUTC $utc
    # once we get startUTC set up base monitor table which has changed over time
    monitorMTbaseLoad $startUTC
    puts "parseConsole: after set startUTC datestr= $datestr"
    if {$utc > 0 && $startDate == ""} {set startDate [clock format $utc -format %b%d_%Y]}
    set wmt [wm title .c]
    wm title .c "$wmt exptid= $exptid  exptDate= $startDate"
    # search for a pb c counters definition
    # multiple detector phrases can be speced using + as separator
    # use the exact word(s) that precede the countsNumber in the Counters: block
    # for example SDC TOTAL:  or SDC0=  etc
    set ict [$txt(0) search -nocase "pb ct" 1.0]
    if {$ict ne ""} {
        set ictxt [$txt(0) get $ict "$ict lineend"]
        if {[llength $ictxt] > 2} {parseDetectors $ictxt}
    }
    # log the initial polbeam state=off=stopped
    set polbeam 0
    logEvent [list $datestr $utc stop "polarized beam"]
    # log the default sampleHW
    logEvent [list $datestr $utc sampleHW $sampleH $sampleW]
    # make sure we can get initial Ei
    # from first occurence of pb Ei; moving Ei value with nocase findNext
    
    set foundEi [findNextEi 1.0]
    set foundEf [findNextEf 1.0]

	if {! $foundEi && ! $foundEf && $Ei == 0 && $Ef == 0 } {
        Dialog_Warn "WARNING: Console doesnt define Ei or Ef. Both default to 14.7"
        set Ei 14.7
        set Ef 14.7
	} elseif {! $foundEi && $Ei == 0 } {
        logWARNING "Console doesnt define Ei defaults to Ef value= $Ef\n"
        set Ei $Ef
    } elseif {! $foundEf && $Ef == 0 } {
        logWARNING "Console doesnt define Ef defaults to Ei value= $Ei\n"
        set Ef $Ei    
    }
    logEvent [list $datestr $utc ei $Ei]
    logEvent [list $datestr $utc ef $Ef]

    # set init beam condition here set default sample mosaic
    beamParse [list open sm=$sampleMosMin]
    
    # following is bt7 specific
    # Cant be sure of initial PG filter state
    # assume PGin if we cant detect a change
    # look for a filtran
    
    
    # RWE Feb 2020  now that we init from first datafile the critical beam param parsing can be moved to annotbt7
    # and parsed in order.
    if { [string length $PGfilt] < 1 } {
    	set PGfilt "in"
    	set filti [$txt(0) search -nocase "moving filtran" 1.0]
    	if {[llength $filti] > 0 && $filti != 0} {
        	set filttxt [$txt(0) get $filti "$filti lineend"]
        	set filtlst [split $filttxt]
        	#puts "parseConsole filtlst= $filtlst"
        	set pgpos [string tolower [lindex $filtlst 1]]
        	#puts "parseConsole pgpos= $pgpos"
        	# if first filtran moves in the init to out
        	if {[string match -nocase *in* $pgpos]} {
        	    set PGfilt "out"
        	}
        	#puts "parseConsole logging event PG $PGfilt"
    	} else {
        	logWARNING "Console doesnt set the PG inpile filter position. Defaults to IN\n"
        	puts "logWARNING for PG finished"
    	}
    	puts "about to logEvent for PG $PGfilt"
    	logEvent [list $datestr $utc PG $PGfilt]
    }
    #return
    # same for the bt7 velocity selector
    # if we cant find a setting via setproperty Ei velselcoupled assume it is out (this may change)
    if { [string length $VelSel] < 1 } {
    	puts "starting VS search"
    	set vsi [$txt(0) search -nocase "setproperty Ei velselcoupled" 1.0]
    	if {$vsi != ""} {
    		set vstxt [$txt(0) get $vsi "$vsi lineend"]
    		parseVelSel $vstxt 0
    		# WHY did I reverse the value. coupled True parses to 1 which means VS is used. False should be 0 as the parser gets.
    		# so turn off this reversal
    		#set VelSel [expr 1 - $VelSel]
    		logEvent [list $datestr $utc VelSel $VelSel]
    	}	
    	puts "logged VS OK"
    }
    #return
    # parseVelSel always logs the vevent  logEvent [list $datestr $utc Velsel $VelSel]
    # check for monochromator focus
	# ICE started with DFM in FLAT horizontal focus mode
	# ICE started with DFM with Phi = 0.000000
	# ICE started with DFM in SAGITTAL vertical focus mode
	# I dont know how to handle changini focus condition
	if { [string length $focusV] < 1 } {
		set focusH 0
		set focusV 1
    	set focushi [$txt(0) search "horizontal focus" 1.0]
    	if {[llength $focushi] > 0 && $focushi != 0} {
    		set foctxt [$txt(0) get "$focushi - 4 chars wordstart" "$focushi lineend"]
    		parseHfocus $foctxt
    	}
    	set focusvi [$txt(0) search "vertical focus" 1.0]
    	if {[llength $focusvi] > 0 && $focusvi != 0} {
    		set foctxt [$txt(0) get "$focusvi - 4 chars wordstart" "$focusvi lineend"]
			parseVfocus $foctxt
    	}
    	logEvent [list $datestr $utc focusH $focusH]
    	logEvent [list $datestr $utc focusV $focusV]
    	puts "logged focussing OK"
    }
    # Now check if the console is pb annotated
    # This allows to choose a parser by instr and annotated
    # NB search for hashtag may cause problems. So advise leave it off on the console
    puts "start search for pb strings"
    set ap [$txt(0) search -nocase -regexp {pb\s+} 1.0 end]
    puts "search found ap= $ap"
    #return
    if {$ap != ""} {
    	puts "starting $annotatedParsers($instr)"
        $annotatedParsers($instr)
    } else {
        $rawParsers($instr)
    }
    puts "inserting events"
    if {! $eventNow} {
        foreach event $events {
            $txt(1) insert end $event\n
        }
    }
}

    # search returns start position or fails with empty return
    # next is to find transmission measurements
    # first check if the console has been anotated using #pb lowercase comments
    # for transmissions
    # pb P polarizerName     puts only polarizer in beam and names it if name is supplied check for newcell
    # pb A analyzerName
    # pb MT                 sets empty beam condition
    # pb both               sets both cells condition
    #
    # for flipping ratio
    # pb fr
    # pb offoff|ONoff|offON|ONON
    # pb Q
    # pb V
    # pb beam descr slitsize 1cm samplesize SH SW
    #
    # globals are also set
    # VQmode can be obtained from script cmnds
    # pb Q  Qmode
    # pb V  Vmode
    
    # pb feff P/A eff efferr   for He3 these can default to 1 0
    # pb rfloss P/A lossperflip
    # pb hkl h k l   Bragg position for FR
    # pb beam 1cm small open etc to set trans or exp condition
    # pb Ei ei
    # pb Lamb lambi
    # pb angle  kidneyAngleDeg   for MACS transmission
    # pb detoff detoffDeg for MACS transmission detector selection
    #
    # NB flipper-states and VQmode are set from scripts on BT7
    # we can look for set FLIP to determine the flip-states
    # A=offoff B=ONoff C=offON D=ONON?
    # also the scripts for VQmode set Vsample Hsample on BT7
    # and MACS has only Vmode.
    # for BT7 Vmode Vsample = about -10 Amps
    #         Qmode Hsample = about 8-10 Amps
    # also Ei possible from A2= or FixedE= move Ei etc


proc logEvent {event} {
    global txt itxt
    global events
    global eventNow
    # only log console parse events not when parsing the he3-log
    if {$itxt != 0} {return}
    if {$eventNow} {
        $txt(1) insert end $event\n
    }
    lappend events $event
}

proc utcTOdatestr {utc cons} {
    # if cons flag is 1: produce console   datestr  %d-%b %T
    # else               produce cellfile  datestr  %m/%d/%y %H:%M
    if {$cons == 1} {
        return [clock format $utc -format "%d-%b %T"]
    } else {
        return [clock format $utc -format "%m/%d/%y %H:%M"]
    }
}
proc datestrTOutc {dstr} {
    global year lastmonth
    # tcl clock cant scan the ICE datetime string correctly so we need to specify the format
    # add the year to dstr in case expt year is not current year
    # get the month from datestring
    set dm [lindex $dstr 0]
    set ddm [split $dm -]
    set currentmonth [lindex $ddm 1]
    if { $lastmonth == "Dec" && $currentmonth == "Jan" } {
    	set year [expr $year + 1]
    }
    set lastmonth $currentmonth
    set dstry "$dstr $year"
    if { [catch {clock scan $dstry -format "%d-%b %T %Y"} t] } { return 0 }
    return $t
}
proc getUTC {line} {
    # if line in txt(0) has dd-bbb hr:mn:sc> return utc
    # if fails leave utc and datestr as is
	global txt
	global datestr
	global utc
	global lastmonth year
	set datestrchk [$txt(0) get "$line linestart" "$line lineend"]
	if {[string length $datestrchk] < 15} {return $utc}
	set datestrchk [$txt(0) get "$line linestart" "$line linestart + 15 chars"]
	set utcchk [datestrTOutc $datestrchk]
	if {$utcchk > 0} {
	    set utc $utcchk
	    set datestr $datestrchk
	}
	return $utc
}


proc findNextReg {istar ex} {
    # text search looks to be slow and doesnt work at all for strings with #
    # as if it stops reading arguments of the search cmnd and assumes the start is 1.0
    # RWE text search wraps if no endIndex is specified, so added txtend
    # returns index of start of matching regex. phraseline is from match to end of line
    # and nextphrase is the part that actually matches the regex=ex
    global txt itxt
    global txtend
    global nextphrase phraseline
    set nextphrase ""
    set it [$txt($itxt) search -nocase -regex $ex $istar $txtend]
    if {$it eq ""} { return 0 }
    # redo the regexp to find the matching phrase
    set phraseline [$txt($itxt) get $it "$it lineend"]
    regexp -nocase $ex $phraseline nextphrase
    return $it
}

proc findNextRangeReg {istar iend ex} {
    # text search looks to be slow and doesnt work at all for strings with #
    # as if it stops reading arguments of the search cmnd and assumes the start is 1.0
    # RWE text search wraps if no endIndex is specified, so added txtend
    global txt itxt
    global nextphrase
    set nextphrase ""
    set it [$txt($itxt) search -nocase -regex $ex $istar $iend]
    if {$it eq ""} { return 0 }
    # redo the regexp to find the matching phrase
    set phraseline [$txt($itxt) get $it "$it lineend"]
    regexp -nocase $ex $phraseline nextphrase
    return $it
}
proc findPrevRangeReg {istar iend ex} {
    # text search looks to be slow and doesnt work at all for strings with #
    # as if it stops reading arguments of the search cmnd and assumes the start is 1.0
    # RWE text search wraps if no endIndex is specified, so added txtend
    global txt itxt
    global nextphrase
    set nextphrase ""
    set it [$txt($itxt) search -backwards -nocase -regex $ex $istar $iend]
    if {$it eq ""} { return 0 }
    # redo the regexp to find the matching phrase
    set phraseline [$txt($itxt) get $it "$it lineend"]
    regexp -nocase $ex $phraseline nextphrase
    return $it
}


proc findNext {istar targets} {
	# search txt(0) from istar for closest (smallest index) occurence of a phrase from targets list
	# return the found index or zero if none
	# findNext is OK for short target lists (one or two)
	# but for long lists use findNextReg so the search can go line-by-line instead of repeating the search
	global txt
	global txtend
	global nextphrase
	set next 0
	set nextphrase ""
	foreach phrase $targets {
		set it [$txt(0) search -nocase $phrase $istar $txtend]
		if {$it eq ""} { continue }
		if { $it < $istar } {
		    puts "ERROR txt0 search for phrase= $phrase from istar=$istar return it=$it"
		}
		if {$next == 0} {
			set next $it
			set nextphrase $phrase
		} elseif {$it < $next} {
			set next $it
			set nextphrase $phrase
		}
		#puts "findNext phrase= $phrase it= $it nextphrase= $nextphrase"
	}
	return $next
}
proc findNextRange {istar iend targets} {
	# search txt(0) from istar for closest (smallest index) occurence of a phrase from targets list
	# return the found index or zero if none
	# findNext is OK for short target lists (one or two)
	# but for long lists use findNextReg so the search can go line-by-line instead of repeating the search
	global txt
	global txtend
	global nextphrase
	set next 0
	set nextphrase ""
	foreach phrase $targets {
		set it [$txt(0) search -nocase $phrase $istar $iend]
		if {$it eq ""} { continue }
		if { $it < $istar } {
		    puts "ERROR txt0 search for phrase= $phrase from istar=$istar return it=$it"
		}
		if {$next == 0} {
			set next $it
			set nextphrase $phrase
		} elseif {$it < $next} {
			set next $it
			set nextphrase $phrase
		}
		#puts "findNext phrase= $phrase it= $it nextphrase= $nextphrase"
	}
	return $next
}


proc parseCounts {istar fr} {
	# start at index istar in ICE console txt(0) and process Counters: blocks
	# from a ct command in ICE
	# adding up all the counts and count times using counters
	# until the detemined end of block endCount
	# return the number of ct blocks processed
	# will return text index < 0 on parse error
	# sets the global ctsList and does monitor logging
	# the fr flag just tells us which error list to append the errors to as
	# parsCounts is used in transmissions and console flip-ratio measurments
	global txt
	global counters
	global ctsList
	global endCount
	global endCountPhrases
	global endCountReg
	global timelbl monitorlbl
	global beamCell
	global cellNames
	global PGfilt
	global Ei
	global nit maxit niterror nextphrase
	global ctrates monrates
	global totalCtsList totalSecList totalMonList
	global frerrors warnings badrates removeBadCts ctrateRejectionCI
	# find the next prompt line starting from istar
	# that has   datetime> ... Counters:
	# All count blocks start this way and end with  
	#  Time= ...
	#---------------------
	#
	# Look for these Counters blocks until the beam state is changed by
	# another pb A/P/MT/both/O
	# annotator may forget both for example so also terminate on
	# pb fr or any  move or scan lines
	set ncts 0
	set ctsList {0 0 0}
	set ctrates {}
	set monrates {}
    # find where the counters block starts
	set startCount [findNext $istar {"Counters:"}]
	if { $startCount == 0 } { return 0 }
	set nextline [$txt(0) index "$startCount linestart + 1 lines"]
	set endCount [findNextReg $nextline $endCountReg]
	if {$endCount == 0} { set endCount [$txt(0) index end] }
	#puts "parsCounts istar= $istar  endCount= $endCount"
	# check for pb beam commands that are entered after a transmission/fr is started and log them
	set beami [findNextRange $istar $endCount {"pb beam"}]
	if {$beami != 0} {
	    set beamline [$txt(0) get $beami "$beami lineend"]
	    set arglst [split $beamline]
	    set arglst [lreplace $arglst 0 1]
	    beamParse $arglst
	}

	set utc 0
	set totalCts 0
	set totalSecs 0
	set totalMons 0
	set totalCtsList {}
	set totalSecList {}
	set totalMonList {}
    set ic $istar
	while {1} {
	    # find the next Counters line
		set startCount [findNext $ic {"Counters:"}]
		#puts "parsCounts loop ncts= $ncts  startCount= $startCount"
		if {$startCount > $endCount || $startCount == 0} {
		    break
		}
		# update utc everytime we find a Counters: block
		set utc [getUTC $startCount]
		#puts "parsCounts about to loop counters= $counters"
		set cval 0
		foreach counter $counters {
			set nextcount [findNext $startCount [list $counter]]
			if {$nextcount == 0} { break }
			set nt [string length $counter]
			set nstar [$txt(0) index "$nextcount + $nt chars"]
			set nend  [$txt(0) index "$nextcount lineend"]
			set nextnumb [$txt(0) get $nstar $nend]
			#puts "parsCounts numbertext= $nextnumb"
			if {[scan $nextnumb "%g" numb] < 1} { return -$nstar }
			#lappend totalCtsList $cval
			set cval [expr $cval + $numb]
			#set totalCts [expr $totalCts + $cval]
			#puts "parsCounts totalCts updated to= $totalCts"
		}
		#puts "parseCount PASSED counter loop"
		#if {$nit >= $niterror && $ncts > 4} {return 0}
		set nexttime [findNext $startCount [list $timelbl]]
		if {$nexttime == 0} { break }
		# set the next counters search point after the Time
		set ic [$txt(0) index "$nexttime linestart + 1 lines"]
		set ns [string length $timelbl]
		set nstar [$txt(0) index "$nexttime + $ns chars"]
		set nend  [$txt(0) index "$nexttime lineend"]
		set nextsecs [$txt(0) get $nstar $nend]
		if {[scan $nextsecs "%g" sval] < 1} { return -$nstar }
		#lappend totalSecList $sval
		#set totalSecs [expr $totalSecs + $sval]

		set nextmon [findNext $startCount [list $monitorlbl]]
		if {$nextmon == 0} { break }
		set ns [string length $monitorlbl]
		set nstar [$txt(0) index "$nextmon + $ns chars"]
		set nend  [$txt(0) index "$nextmon lineend"]
		set nextmons [$txt(0) get $nstar $nend]
		if {[scan $nextmons "%g" monct] < 1} { return -$nstar }
		
		if {$sval > 0 && $cval > 0 && $monct > 0} {
		    set thisrate [expr $cval/$sval]
		    set thisrateErr [expr sqrt($cval)/$sval]
		    lappend ctrates [list $thisrate $thisrateErr]
		
		    set thismonrate [expr $monct/$sval]
		    set thismonrateErr [expr sqrt($monct)/$sval]
		    lappend monrates [list $thismonrate $thismonrateErr]
		    
		    lappend totalCtsList $cval
		    lappend totalSecList $sval
		    lappend totalMonList $monct
		    incr ncts
		}
		#set totalMons [expr $totalMons + $monct]
		# update the ctsList
	    set ctsList [list $utc $totalCts $totalSecs]
		incr ncts
		#puts "parseCount FINISHED first while loop reset ic= $ic ncts= $ncts"
		#if {$nit >= $niterror} {return 0}
	}
	# finished count block parsing
	# check for badCounts
	set errline [$txt(0) get "$istar linestart" "$istar lineend"]
	set willdel ""
	if {$removeBadCts == 1} {set willdel "BAD count-rates will be removed "}
	set errlst warnings
	if {$fr} {set errlst frerrors}
	# badCountRates can remove outliers if removeBadCts flag is set==1
	if [badCountRates] {
		lappend $errlst "WARNING: count rates or monitor rates vary by more than $ctrateRejectionCI std dev at $istar\:\n$willdel\with badrates= $badrates\n$errline"
	}
	# compute totalCts and totalSecs to return via globqal ctsList
	set ncts [llength $totalCtsList]
	foreach ct $totalCtsList sc $totalSecList mn $totalMonList {
		set totalCts [expr $totalCts + $ct]
		set totalSecs [expr $totalSecs + $sc]
		set totalMons [expr $totalMons + $mn]
	}
	set ctsList [list $utc $totalCts $totalSecs]
	# when parsing complete mark the end of the count block
    set endCount $ic
    # record monitor rates
    recordMonitor $utc $totalMons $totalSecs
	return $ncts
}

proc logERROR {stxt} {
	global txt
	$txt(4) insert end "ERROR: $stxt"	
}
proc logWARNING {stxt} {
	global txt
	$txt(4) insert end "WARNING: $stxt"	
}

proc monitorByVSPG {} {
	global PGfilt VelSel
}
# find array
proc recordMonitor {utcc totalMons totalSecs} {
    global beamCell cellNames PGfilt VelSel Ei npolarizers
    global monitorHe3PGin monitorHe3PGout monitorHe3VSPGin monitorHe3VSPGout
    global monitorMTPGin monitorMTPGout monitorMTVSPGin monitorMTVSPGout
    global monitorHe3bothPGin monitorHe3bothPGout monitorHe3bothVSPGin monitorHe3bothVSPGout
    global monitorMTbasePGin monitorMTbasePGout monitorMTbaseVSPGin monitorMTbaseVSPGout
    global incBeamCond
    global horDivPFWHMmin
    global monitorRateChange
    global minMonitorCt
    global lastMratUTC
    global txt
    # both is of interest since it likely represents the experiment data where we want
    # to check on the monitor
    # changed logging the He3 case by polarizer-number=cellName
    if {$totalSecs <= 0 || $totalMons <= $minMonitorCt} {return}
    set mrat [expr $totalMons/$totalSecs]
    set lastMratUTC $utcc
    set mratErr [expr sqrt($totalMons)/$totalSecs]
	set Eir [format %.2f $Ei]
    set newentry [list $utcc $mrat $mratErr $Eir]
    #puts "recordMonitor mrat= $mrat  beamCell= $beamCell"
	if [regexp -nocase {both} $beamCell] {
	    if {$PGfilt == "in"} {
	    	if {$VelSel == 0} {
    	    	lappend monitorHe3bothPGin($npolarizers=$cellNames(p)) $newentry
    	    } else {
    	    	lappend monitorHe3bothVSPGin($npolarizers=$cellNames(p)) $newentry
    	    }
    	} else {
    		if {$VelSel == 0} {
    	    	lappend monitorHe3bothPGout($npolarizers=$cellNames(p)) $newentry
    	    } else {
    	    	lappend monitorHe3bothVSPGout($npolarizers=$cellNames(p)) $newentry
    	    }
    	}
	} elseif [regexp -nocase {p} $beamCell] {
	    if {$PGfilt == "in"} {
	    	if {$VelSel == 0} {
    	    	lappend monitorHe3PGin($npolarizers=$cellNames(p)) $newentry
    	    } else {
    	    	lappend monitorHe3VSPGin($npolarizers=$cellNames(p)) $newentry
    	    }
    	} else {
    		if {$VelSel == 0} {
    	    	lappend monitorHe3PGout($npolarizers=$cellNames(p)) $newentry
    	    } else {
    	    	lappend monitorHe3VSPGout($npolarizers=$cellNames(p)) $newentry
    	    }
    	}
	} else {
	    # empty can include analzyer transmissions=a
	    # use no He3 measurements to check on incident beam flux consistency
	    # and assign the incbeamCond by looking up expected mrat for Ei and PGstate at collimations-focussing
	    # for MT tables use rounded Ei to nearest 0.1
	    
	    if {$PGfilt == "in"} {
	    	if {$VelSel == 0} {
	    		array set mlog [array get monitorMTPGin]
	    		array set mtab [array get monitorMTbasePGin]
	    	} else {
	    		array set mlog [array get monitorMTVSPGin]
	    		array set mtab [array get monitorMTbaseVSPGin]
	    	}
	    } else {
	    	if {$VelSel == 0} {
	    		array set mlog [array get monitorMTPGout]
	    		array set mtab [array get monitorMTbasePGout]
	    	} else {
	    		array set mlog [array get monitorMTVSPGout]
	    		array set mtab [array get monitorMTbaseVSPGout]
	    	}
	    }
	    # try to get beamCond for current mrat from base table or last incBeamCond
	    set ok -1
	    #puts "recordMonitor checking mtab($Eir) exists mtab=\n[array get mtab]"
	    if [info exists mtab($Eir)] {
	    	set ok [getMonitorTableMatch $mtab($Eir) $mrat $mratErr]
	    }
	    set nowCond "unknown"
	    set oldCond "none"
	    set datestr [utcTOdatestr $utcc 1]
	    if {$ok >= 0} {
	    	#puts "  Exists"
	    	set nowCond [lindex [lindex $mtab($Eir) $ok] 1]
	    	if {[llength $incBeamCond] > 0} {
	    		set oldCond [lindex [lindex $incBeamCond end] 2]
	    	}
	    	if {$nowCond != $oldCond} {
	    		logWARNING "beam condition change to $nowCond $datestr\n"
	    		logEvent [list $datestr $utcc incBeam $nowCond]
	    		# NB incBeam is not parsed from the parse-console tab but incBeamCond list is used which only has utc
	    	}
	    } else {
	    	logWARNING "unknown MT beam monitor rate $mrat at Ei= $Eir $datestr\n"
	    }
	    set newCond [list $utcc $Eir $nowCond $mrat $mratErr PG=$PGfilt VS=$VelSel]
	    set newinc 0
	    if {[llength $incBeamCond] < 1} {
	    	lappend incBeamCond $newCond
	    	set newinc 1
	    } else {
	    	set lastCond [lindex [lindex $incBeamCond end] 2]
	    	# if nowcond is found ok >= 0 just compare with lastCond
	    	if {$ok >= 0} {
	    		if {$nowCond != $lastCond} {
	    			lappend incBeamCond $newCond
	    			set newinc 1
	    		}
	    	} else {
		    	# if cant find nowCond check if we have previous mrat and compare with now rate
			    # if monitorMT at Eir already has an element compare with last value
			    # to see if we need to change beamCond
	    		if {[info exists mlog($Eir)] && [llength $mlog($Eir)] > 0} {
	    			set lastMon [lindex $mlog($Eir) end]
	    			#puts "recordMonitor last log for E=$Eir = $lastMon"
	    			set lastR [lindex $lastMon 1]
	    			set lastE [lindex $lastMon 2]
	    			set diffR [expr abs($lastR - $mrat)]
	    			set diffE [expr sqrt($lastE*$lastE + $mratErr*$mratErr)]
	    			if {$diffR > [expr $monitorRateChange*$lastR + $diffE]} {
						# record the new beam condition which is unknown
	    				lappend incBeamCond $newCond
	    				set newinc 1
	    			}
	    		}
	    	}
	    }
	    if {$newinc == 1} {
	    	if [regexp {[^-]+-([^-]+)-.+} $nowCond wm hc] {
	    		set horDivPFWHMmin $hc
	    	}
	    }
	    lappend mlog($Eir) [list $utcc $mrat $mratErr]
	    if {$PGfilt == "in"} {
	    	if {$VelSel == 0} {
	    		array set monitorMTPGin [array get mlog]
	    	} else {
	    		array set monitorMTVSPGin [array get mlog]
	    	}
	    } else {
	    	if {$VelSel == 0} {
	    		array set monitorMTPGout [array get mlog]
	    	} else {
	    		array set monitorMTVSPGout [array get mlog]
	    	}
	    }
	}
}

proc getMonitorTableMatch {tb mrat mratErr} {
	global monitorRateTableErr
	puts "getMonitorTableMatch tb= $tb mrat= $mrat"
	set ntb [llength $tb]
	for {set i 0} {$i < $ntb} {incr i} {
		set mi [lindex $tb $i]
		set m [lindex $mi 0]
		set me [expr $m*$monitorRateTableErr]
		set de [expr sqrt($me*$me + $mratErr*$mratErr)]
		puts "  i=$i table mrat= $m de= $de"
		if {[expr abs($mrat - $m)] <= $de} {return $i}
	}
	return -1
}

proc monitorReportMT {pgstate} {
    global monitorMTPGin monitorMTPGout
    global monitorHe3bothPGin monitorHe3bothPGout
    global monitorHe3PGin monitorHe3PGout
    # for MT puts for each Ei secsRange aveRate relVar
    # for both    for each Ei eachPolCell: secsRange startRate endRate
    puts "MTPG$pgstate"
    set aname monitorMTPG$pgstate
    array set la [array get $aname]
    set anames [array names la]
    puts $anames
    foreach ei $anames {
        set seci 9999999999
        set secf 0
        set nmsr 0
        set ave 0
        set msq 0
        set var 0
        set maxrt 0
        set maxrterr 0
        set minrt 9999999999
        set minrterr 0
        foreach m [set ${aname}($ei)] {
            set utc [lindex $m 0]
            set mon [lindex $m 1]
            set sec [lindex $m 2]
            set mrt [expr $mon/$sec]
            set mrterr [expr sqrt($mon)/$sec]
            if {$mrt > $maxrt} {
                set maxrt $mrt
                set maxrterr $mrterr
            }
            if {$mrt < $minrt} {
                set minrt $mrt
                set minrterr $mrterr
            }
            if {$utc > $secf} {set secf $utc}
            if {$utc < $seci} {set seci $utc}
            incr nmsr
            set ave [expr $ave + $mrt]
            set msq [expr $msq + $mrt*$mrt]
        }
        set secrange [expr $secf - $seci]
        if {$nmsr > 0} {
            set ave [expr $ave/$nmsr]
        }
        if {$nmsr > 1} {
            set var [expr sqrt($msq/$nmsr - $ave*$ave)]
        }
        set ei [format %.2f $ei]
        puts "ei=$ei nmsr= $nmsr  secRange= $secrange  avemonrate= $ave  var= $var  maxrt= $maxrt +- $maxrterr  minrt= $minrt +- $minrterr"
    }
}

proc monitorReportHe3 {pgstate} {
    global monitorMTPGin monitorMTPGout
    global monitorHe3bothPGin monitorHe3bothPGout
    global monitorHe3PGin monitorHe3PGout
    # for MT puts for each Ei secsRange aveRate relVar
    # for both    for each Ei eachPolCell: secsRange startRate endRate
    puts "He3PG$pgstate"
    set aname monitorHe3bothPG$pgstate
    array set la [array get $aname]
    set anames [array names la]
    puts $anames
    set nmsr 0
    foreach ei $anames {
        set prevcel ""
        set nmsr 0
        set msrlst {}
        foreach m [set ${aname}($ei)] {
            set cel [lindex $m 3]
            if {$cel != $prevcel} {
                if {$nmsr > 0} {
                    # print previous
                    set endrt $mrt
                    set endrterr $mrterr
                    set secrange [expr $secf - $seci]
                    set ave [expr $ave/$nmsr]
                    if {$nmsr > 1} {
                        set var [expr sqrt($msq/$nmsr - $ave*$ave)]
                    }
                    set ei [expr round(1000.*$ei)/1000.]
                    puts "ei=$ei pol= $prevcel secRange= $secrange msr= $msrlst"
                }
                set secf 0
                set nmsr 1
                set ave 0
                set msq 0
                set var 0
                set maxrt 0
                set maxrterr 0
                set minrt 9999999999
                set minrterr 0
                set prevcel $cel
                set utc [lindex $m 0]
                set mon [lindex $m 1]
                set sec [lindex $m 2]
                set seci $utc
                set mrt [expr $mon/$sec]
                set mrterr [expr sqrt($mon)/$sec]
                set startrt $mrt
                set startrterr $mrterr
                set msrlst {}
                lappend msrlst [list [format "%.2f" $mrt] [format "%.2f" $mrterr] $utc]
            } else {
                set utc [lindex $m 0]
                set mon [lindex $m 1]
                set sec [lindex $m 2]
                set mrt [expr $mon/$sec]
                set mrterr [expr sqrt($mon)/$sec]
                if {$mrt > $maxrt} {
                    set maxrt $mrt
                    set maxrterr $mrterr
                }
                if {$mrt < $minrt} {
                    set minrt $mrt
                    set minrterr $mrterr
                }
                if {$utc > $secf} {set secf $utc}
                if {$utc < $seci} {set seci $utc}
                incr nmsr
                lappend msrlst [list [format "%.2f" $mrt] [format "%.2f" $mrterr] $utc]
                set ave [expr $ave + $mrt]
                set msq [expr $msq + $mrt*$mrt]
            }
        }
        # print the last one
        if {$nmsr > 0} {
            # print previous
            set endrt $mrt
            set endrterr $mrterr
            set secrange [expr $secf - $seci]
            set ave [expr $ave/$nmsr]
            if {$nmsr > 1} {
                set var [expr sqrt($msq/$nmsr - $ave*$ave)]
            }
            set ei [expr round(100.*$ei)/100.]
            puts "ei=$ei pol= $prevcel secRange= $secrange msr= $msrlst"
        }
    }
}


# make sure to ignore pb state cmnds as these are now all processed through seq
# NB fr will also end on another pb fr cmd
# pb fbg must be outside of a fr block

proc monotonicList {nums} {
    set nn [llength $nums]
    if {$nn < 2} {return 1}
    set num0 [lindex $nums 0]
    set num1 [lindex $nums 1]
    if {$num1 > $num0} {
        set nmax $num1
        for {set i 2} {$i < $nn} {incr i} {
            set numi [lindex $nums $i]
            if {$numi <= $nmax} {return 0}
            set nmax $numi
        }
        return 1
    }
    if {$num1 < $num0} {
        set nmin $num1
        for {set i 2} {$i < $nn} {incr i} {
            set numi [lindex $nums $i]
            if {$numi >= $nmin} {return 0}
            set nmin $numi
        }
        return 1
    }
    return 0
}
proc maxList {nums} {
    set nn [llength $nums]
    if {$nn < 1} {return [list 0 0]}
    set nmax [lindex $nums 0]
    set imax 0
    for {set i 1} {$i < $nn} {incr i} {
        set num [lindex $nums $i]
        if {$num > $nmax} {
            set nmax $num
            set imax $i
        }
    }
    return [list $imax $nmax]
}

proc maxCountRate {det sec} {
    set nn [llength $det]
    if {$nn < 1} {return [list 0 0]}
    set ct [lindex $det 0]
    set sc [lindex $sec 0]
    set rt [expr $ct/$sc]
    set imax 0
    set rtmax $rt
    for {set i 1} {$i < $nn} {incr i} {
        set ct [lindex $det $i]
        set sc [lindex $sec $i]
        set rt [expr $ct/$sc]
        if {$rt > $rtmax} {
            set rtmax $rt
            set imax $i
        }
    }
    return [list $imax $rtmax]
}

proc listRange {lst} {
    set nl [llength $lst]
    if {$nl < 1} {return [list 0 0]}
    set lstmin [lindex $lst 0]
    set lstmax [lindex $lst 0]
    for {set i 1} {$i < $nl} {incr i} {
        set v [lindex $lst $i]
        if {$v < $lstmin} {set lstmin $v}
        if {$v > $lstmax} {set lstmax $v}
    }
    return [list $lstmin $lstmax]
}

proc inListRange {lst val} {
    set lstminmax [listRange $lst]
    if {$val < [lindex $lstminmax 0] || $val > [lindex $lstminmax 1]} return 0
    return 1
}



proc emptyFRarrays {} {
    global V0 Q0 V1 Q1
    global V0B Q0B V1B Q1B
    foreach state {A B C D} {
        array set V0 [list $state {}]
        array set V1 [list $state {}]
        array set Q0 [list $state {}]
        array set Q1 [list $state {}]
        array set V0B [list $state {}]
        array set V1B [list $state {}]
        array set Q0B [list $state {}]
        array set Q1B [list $state {}]
    }
}
# init the fr arrays although
emptyFRarrays

proc appendFRarray {bg bs vq st vlst} {
	# append vlst to the FRlist for bg bs vq st
	global V0 V1 Q0 Q1 V0B V1B Q0B Q1B
	global sharedwith
	set ar $vq$bs
	if {$bg == 1} {set ar $ar\B}
	set islist [string is list $vlst]
	set ll [llength $vlst]

	# get the list for st
	set arlst [set ${ar}($st)]
	# append vlst to this list
	lappend arlst $vlst
	# reset array(st) to new arlst
	set ${ar}($st) $arlst
	#puts "appendArray arglistcheck= $islist length=$ll   array=$ar inlst= $vlst  outlst= $arlst"
}
proc setFRarray {bg bs vq st vlst} {
	# append vlst to the FRlist for bg bs vq st
	global V0 V1 Q0 Q1 V0B V1B Q0B Q1B
	set ar $vq$bs
	if {$bg == 1} {set ar $ar\B}
    set ${ar}($st) $vlst
}
proc getFRarray {bg bs vq st} {
	# return the Frlist for bg bs vq st
	global V0 V1 Q0 Q1 V0B V1B Q0B Q1B
	global sharedwith
	set ar $vq$bs
	if {$bg == 1} {set ar $ar\B}
	set retlst [set ${ar}($st)]
	if {$sharedwith == $vq} {
	    #puts "getFRarray $vq $st sharedwith=$sharedwith returning $retlst"
	}
	return $retlst
}


proc parseScan {istar fr fpk} {
    # istar is line with START OF SCAN
    # if flip-ratio flag is set collect the scan data in frdata array by columns
    global txt
    global endScan
    global datestr utc
    global warnings
    global Ei Ef
    global cellNames beamCell
    global monitorHe3in monitorHe3out monitorMTin monitorMTout
    global VQ flipstatestr FLIP cellNames
    global frdata
    global frcts frsecs frvar frpos frbgcts frbgsecs frscanvars
    global minMonitorCt
    global scanfrdata
    global pbfiles
    global cellNames
    global envTemp envBfld degToRad dspPG002 piRad Dneut
    # added env info. Find env before the scan starts
    #puts "parseScan start istar=$istar   envTemp= $envTemp"
    findEnv $istar
    #puts "parseScan findEnv envTemp= $envTemp"
    
	set nextline [$txt(0) index "$istar linestart + 1 lines"]
	set endScan [findNextReg $nextline {END OF SCAN|stopall|kill}]
	if {! $endScan } {
		puts "parseScan FAILED END OF SCAN lookup"
		return 0
	}
	set endScan [$txt(0) index "$endScan linestart"]
	#puts "parseScan endScan= $endScan"
    # find the Fixed devices:   and  Creating File fname  lines
    # NB in fpk Fixed gets replaced by Constant
    set fixdline [findNextReg $nextline {Fixed devices:|Constant devices:}]
    if {! $fixdline} {
    	puts "parseScan FAILED Fixed devices lookup"
    	return 0 
    }
    set fileline [findNextReg $nextline {Creating file}]
    if {! $fileline} {
    	puts "parseScan FAILED Creating File lookup"
    	return 0
    }
    set ftxt [$txt(0) get "$fileline" "$fileline lineend"]
    #puts "parseScan ftxt= $ftxt"
    set flst [split $ftxt]
    if {[llength $ftxt] < 3 || [regexp {.*\/(.+)$} [lindex $ftxt 2] wm fn] < 1} {
        lappend warnings "Failed to parse creating file at $istar"
        return 0
    }
    #puts "parseScan FILENAME= $fn"
    # RWE Jan 2019 check for first data line after Creating file
    set dataline [$txt(0) index "$fileline linestart + 1 lines"]
    set datatxt [$txt(0) get $dataline "$dataline lineend"]
    # third word should be var=val
    if {[llength $datatxt] < 3} {
        lappend warnings "Failed to parse data line after creating file at $dataline"
        return 0
    }
    set varval [lindex $datatxt 2]
    set varvall [split $varval =]
    set scanvar [lindex $varvall 0]

    
    set dtxt [$txt(0) get "$fixdline" "$fixdline lineend"]
    set dlst [split $dtxt]
    # remove the first 2 words Fixed devices:
    set dlst [lreplace $dlst 0 1]
    set fixE 0
    set fixedE $Ei
    if {[lsearch $dlst "Ef"] >= 0} {
        set fixE 1
        set fixedE $Ef
    }
    if {[lsearch $dlst "A2"] >= 0 || [lsearch $dlst "Ei"] >= 0} {
        set fixE 0
        set fixedE $Ei
    }
	
	# now loop for datalines with datetime> fld=num ...

	set ndata 0
	set nmon 0
	set totalmon 0
	set totaltim 0
	set eichanged 0
	set efchanged 0
	array unset frdata
	array set frdata {}
	while {1} {
	    puts "parseScan loop nextline= $nextline"
	    if {$nextline >= $endScan} {break}
	    if {[getUTC $nextline] == 0} {
	        set nextline [$txt(0) index "$nextline + 1 lines"]
	        continue
        }
        set datastr [$txt(0) get "$nextline + 17 chars" "$nextline lineend"]
        # strip white space blocks to single space
        regsub -all {\s+} $datastr " " datastrs
        set datawds [split $datastrs]
        # RWE Mar 2020 datawds must have device=value
        if {![string match *=* [lindex $datawds 0]]} {
	        set nextline [$txt(0) index "$nextline + 1 lines"]
	        continue
        }        
        if {$fpk && $ndata > 1 && [string match "*Fit Results:*" $datastrs]} {
        	# look for Fit Results: BG = bgValue. Then Height =   Pos =  Width =  Chi^2/dof =  lines
        	set frdata(BG) 0
        	if {[llength $datawds] > 4} {set frdata(BG) [lindex $datawds 4]}
        	set frdata(Height) 0
        	set frdata(Pos) 0
        	set frdata(Width) 0
        	set frdata(ChiSqRed) 0
        	set frdata(fitOK) 0
        	set nextline [$txt(0) index "$nextline + 1 lines"]
        	set nexttxt [$txt(0) get $nextline "$nextline lineend"]
        	if {! [string match *Height* $nexttxt]} {continue}
        	set frdata(Height) [lindex $nexttxt 2]
        	set nextline [$txt(0) index "$nextline + 1 lines"]
        	set nexttxt [$txt(0) get $nextline "$nextline lineend"]
        	if {! [string match *Pos* $nexttxt]} {continue}
        	set frdata(Pos) [lindex $nexttxt 2]
        	set nextline [$txt(0) index "$nextline + 1 lines"]
        	set nexttxt [$txt(0) get $nextline "$nextline lineend"]
        	if {! [string match *Width* $nexttxt]} {continue}
        	set frdata(Width) [lindex $nexttxt 2]
        	set nextline [$txt(0) index "$nextline + 1 lines"]
        	set nexttxt [$txt(0) get $nextline "$nextline lineend"]
        	if {! [string match *Chi* $nexttxt]} {continue}
        	set frdata(ChiSqRed) [lindex $nexttxt 2]
        	set frdata(fitOK) 1
        	set endline [$txt(0) index "$nextline + 4 lines"]
        	set endtxt [$txt(0) get $endline "$endline lineend"]
        	# RWE add handling of aborted findpeak Jan 2020
        	if {[string match "*END OF SCAN*" $endtxt]} {
        		set accline [$txt(0) index "$endline + 1 lines"]
        		set acctxt [$txt(0) get $accline "$accline lineend"]
        		if {! [string match *AcceptFindPeak* $acctxt]} {
        			set scvar $frdata(var)
        			# set to last scanned value
        			set frdata(Pos) [lindex $frdata($scvar) [expr $ndata - 1]]
        		}
        	} else {
        		#set scvar $fradata(var)
        		# set to last scanned value
        		set frdata(Pos) [lindex $frdata($scanvar) [expr $ndata - 1]]
        	}
        	# 4th line after chi2 should be END OF SCAN or was aborted
        	# next line should be AcceptFindPeak to move to the fit value
        	set nextline [$txt(0) index "$nextline + 1 lines"]
        	continue
        }
        
        # recall that split will produce empty list elements if for multiple whitespace separation
        set dataok 0
        #puts "parseScan ndatawds= [llength $datawds]"
        # could just treat datastr as the word list because index into list skips white space
        foreach wd $datawds {
            #puts "parseScan checking wd= $wd"
            if {[string length $wd] < 1} {continue}
            if [string match *=* $wd] {
                set dataok 1
            } else {
                set dataok 0
                break
            }
        }
        if {! $dataok} {
        	set nextline [$txt(0) index "$nextline + 1 lines"]
	        continue
        }

        #puts "parseScan about to incr ndata from ndata= $ndata"
        incr ndata
        # scans do not declare EI EF but may have A2 A6 so check for these
        set ia2 [lsearch $datawds A2=*]
        set ia6 [lsearch $datawds A6=*]
        if { $ia2 >= 0 } {
        	if {[scan [lindex $datawds $ia2] "A2=%g" A2] == 1} {
        		set lambi [expr 2.*$dspPG002*sin($degToRad*$A2/2.)]
        		set ei [expr $Dneut*pow(2.*$piRad/$lambi,2)]
        		set ei [expr round($ei*10)/10.]
        		if {$ei != $Ei} {
        			set Ei $ei
	        		set eichanged 1
	        	}
        	}
        }
        if { $ia6 >= 0 } {
        	if {[scan [lindex $datawds $ia6] "A6=%g" A6] == 1} {
        		set lambf [expr 2.*$dspPG002*sin($degToRad*$A6/2.)]
        		set ef [expr $Dneut*pow(2.*$piRad/$lambf,2)]
        		set ef [expr round($ef*10)/10.]
        		if {$ef != $Ef} {
        			set Ef $ef
        			set efchanged 1
        		}
        	}
        }
        # now check for Mon= and Time= E= to log a monitor rate
        set im [lsearch $datawds Mon=*]
        set it [lsearch $datawds Time=*]
        set id [lsearch $datawds Det=*]
        if { $im < 0 || $it < 0 || $id < 0 } {
        	set nextline [$txt(0) index "$nextline + 1 lines"]
         	continue 
        }
        if {[scan [lindex $datawds $im] "Mon=%g" mon] < 1} {
        	set nextline [$txt(0) index "$nextline + 1 lines"]
        	continue
        }
        if {[scan [lindex $datawds $it] "Time=%g" tim] < 1} {
        	set nextline [$txt(0) index "$nextline + 1 lines"]
        	continue
        }
        # for fr collect data
        # decide to collect data always so we can check for env changes
        #if $fr > 0
        set firstCol 0
        foreach wd $datawds {
            if {[string length $wd] < 1} {continue}
            if {![regexp {(.+)=(.+)} $wd wm col val]} {continue}
            if {! $firstCol} {
            	set frscan(var) $col
            	set frdata(var) $col
            	set firstCol 1
            }
            if {![info exists frdata($col)]} {set frdata($col) {}}
            lappend frdata($col) $val
        }

        # if fixE==1 we need E= to calculated Ei
        set ei $Ei
        if {$fixE == 1} {
            # cant log monitor rate wthout Ei and thus Ef E
            set ie [lsearch -glob $datawds E=*]
            if {$ie >= 0} {
                if {[scan [lindex $datawds $ie] "E=%g" e] > 0} {
                    set ei [expr $Ef + $e]
                    # elastic fr check
                    if {$fr > 0 && $e != 0.} {return 0}
                    set Ei $ei
                }
            }
        }
        # log the monitor in monitorHe3(ei)
        incr nmon
        if {$mon < $minMonitorCt} {
        	set totalmon [expr $totalmon + $mon]
        	set totaltim [expr $totaltim + $tim]
        } else {
        	recordMonitor $utc $mon $tim
        }
		set nextline [$txt(0) index "$nextline + 1 lines"]
	}
	if {$eichanged > 0} {
		logEvent [list $datestr $utc Ei $Ei]
	}
	if {$efchanged > 0} {
		logEvent [list $datestr $utc Ef $Ef]
	}
	#puts "parseScan records nmon=$nmon for utc=$utc file=$fn polarizer=$cellNames(p)"
	if {$totalmon >= $minMonitorCt && $totaltim > 0.} {
		recordMonitor $utc $totalmon $totaltim
	}
	set temprange 0
	set bfldrange 0
	if {[info exists frdata(Temp)] && [llength $frdata(Temp)] > 0} {
	    set tempminmax [listRange $frdata(Temp)]
	    set tempmin [lindex $tempminmax 0]
	    set tempmax [lindex $tempminmax 1]
	    set envTemp [format %.3f [expr 0.5*($tempmin + $tempmax)]]
	    set temprange [format %.3f [expr $tempmax - $tempmin]]
	}
	if {[info exists frdata(MagField)] && [llength $frdata(MagField)] > 0} {
	    set bfldminmax [listRange $frdata(MagField)]
	    set bfldmin [lindex $bfldminmax 0]
	    set bfldmax [lindex $bfldminmax 1]
	    set envBfld [format %.3f [expr 0.5*($bfldmin + $bfldmax)]]
	    set bfldrange [format %.3f [expr $bfldmax - $bfldmin]]
	}
	# fn is the filename. log as data if non-fr or the scanfrdata flag is set
	if {$fr <= 0 || $scanfrdata == 1} {
	    lappend pbfiles [list $datestr $utc $VQ $flipstatestr $FLIP $cellNames(p) $cellNames(a) $fn $envBfld $bfldrange $envTemp $temprange]
	}
	return $ndata
}

proc getCountRate {ctlst} {
    # return countrate and error from ctlst =  fr utc cts secs  with fr optional
    set ctr 0
    set ctrerr 0
    #puts "getCountrate ctlst= $ctlst"
    set sec 0
    set cts 0
    set lstl [llength $ctlst]
    if {[llength $ctlst] < 3} {return [list 0 0]}
    if {$lstl == 3} {
    	set cts [lindex $ctlst 1]
    	set sec [lindex $ctlst 2]
    } elseif {$lstl == 4} {
        set cts [lindex $ctlst 2]
    	set sec [lindex $ctlst 3]
    }
    if {$sec > 0 && $cts > 0} {
        set ctr [expr $cts/$sec]
        set ctrerr [expr sqrt($cts)/$sec]
    }
    return [list $ctr $ctrerr]
}

proc averageANDerr {xelst} {
	set sum 0.
	set sumerr 0.
	set nxe [llength $xelst]
	if {$nxe < 1} {return [list 0 0]}
    for {set i 0} {$i < $nxe} {incr i} {    
        set ctri [lindex $xelst $i]
        set ctrnext [lindex $ctri 0]
        set ctrnextErr [lindex $ctri 1]
        set ctrnextErrSq [expr $ctrnextErr*$ctrnextErr]
        set sum [expr $sum + $ctrnext/$ctrnextErrSq]
        set sumerr [expr $sumerr + 1./$ctrnextErrSq]
    }
    set ave [expr $sum/$sumerr]
    set err [expr 1./sqrt($sumerr)]
    return [list $ave $err]
}


proc badCountRates {} {
    global ctrates monrates badrates
    global totalCtsList totalMonList totalSecList
    global ctrateRejectionCI
    global removeBadCts
    global txt

	# ctrates and monrates totalCtsList... must all have same list length
    set nctr [llength $ctrates]
    if {$nctr < 2 || [llength $monrates] != $nctr || [llength $totalCtsList] != $nctr} {return 0}

    # init the range from ctrateRejectionCI * ctrateErr

    set badrates {}
    set badindices {}

    # get the average and error of all list elements
    set aveerr [averageANDerr $ctrates]
    set ctrAve [lindex $aveerr 0]
    set ctrAveErr [lindex $aveerr 1]
    set aveerr [averageANDerr $monrates]
    set monAve [lindex $aveerr 0]
    set monAveErr [lindex $aveerr 1]

	
    # Now find the outliers starting with worst of either ctrates or monrates and updating average
    set nrem $nctr
    while {$nrem > 1} {
    	set ctrCI [expr $ctrateRejectionCI * $ctrAveErr]
    	set monCI [expr $ctrateRejectionCI * $monAveErr]
    	set dbad 0
    	set ibad -1
    	for {set i 0} {$i < $nctr} {incr i} {
    		if {[lsearch $badindices $i] >= 0} {continue}
        	set ctri [lindex $ctrates $i]
        	set ctrnext [lindex $ctri 0]
        	set dct [expr abs($ctrnext - $ctrAve)/$ctrAveErr]
        	if {$dct > $ctrateRejectionCI && $dct > $dbad} {
        		set ibad $i
        		set dbad $dct
        		set ctrbad [list ctr $ctrnext]
        	}
        	set moni [lindex $monrates $i]
        	set monnext [lindex $moni 0]
        	set dmn [expr abs($monnext - $monAve)/$monAveErr]
        	if {$dmn > $ctrateRejectionCI && $dmn > $dbad} {
        		set ibad $i
        		set dbad $dmn
        		set ctrbad [list mon $monnext]
        	}
        }
        # break if there are no outliers left
        if {$ibad < 0} {break}
        lappend badindices $ibad
        # remove the outlier
        lappend badrates [list $ibad $ctrbad $dbad]
        #set rates [lreplace $rates $ibad $ibad]
        incr nrem -1
        # recompute the average without the outlier
        set okcts {}
        set okmon {}
        set okTotalCts {}
        set okTotalSec {}
        set okTotalMon {}
    	for {set i 0} {$i < $nctr} {incr i} {
    		if {[lsearch $badindices $i] >= 0} {continue}
    		lappend okcts [lindex $ctrates $i]
    		lappend okmon [lindex $monrates $i]
    		lappend okTotalCts [lindex $totalCtsList $i]
    		lappend okTotalSec [lindex $totalSecList $i]
    		lappend okTotalMon [lindex $totalMonList $i]    		
    	}        
    	set aveerr [averageANDerr $okcts]
    	set ctrAve [lindex $aveerr 0]
    	set ctrAveErr [lindex $aveerr 1]
    	set aveerr [averageANDerr $monrates]
    	set monAve [lindex $aveerr 0]
    	set monAveErr [lindex $aveerr 1]
    }
    set nbad [llength $badrates]
    if {$removeBadCts == 1 && $nbad > 0} {
    	set ctrates $okcts
    	set monrates $okmon
    	set totalCtsList $okTotalCts
    	set totalSecList $okTotalSec
    	set totalMonList $okTotalMon
    }
    return $nbad
}


proc computeFlipEffs {} {
    global frctrates
    global flipEff flipEffErr
    global feffP feffPerr
    global feffA feffAerr
    set flipEff(0) 1
    set flipEff(1) 1
    set flipEffErr(0) 0
    set flipEffErr(1) 0
    set feffP 1
    set feffA 1
    set feffPerr 0
    set feffAerr 0
    set sts {A B C D}
    array set ctrt {}
    array set ctrterr {}
    array set sigsq {}
    for {set i 0} {$i < 4} {incr i} {
        set ctr [lindex $frctrates $i]
        set st [lindex $sts $i]
        set ctrt($st) [lindex $ctr 0]
        set ctrterr($st) [lindex $ctr 1]
        set sigsq($st) [expr $ctrterr($st)*$ctrterr($st)]
    }
    set den [expr $ctrt(A) - $ctrt(C)]
    set num [expr $ctrt(D) - $ctrt(B)]
    if {$den > 0 && $num > 0} {
        set effP21 [expr $num/$den]
        set effP21err [expr ($sigsq(A) + $sigsq(C))/($den*$den) + ($sigsq(D) +$sigsq(B))/($num*$num)]
        set effP21err [expr $effP21*sqrt($effP21err)]
        set feffPerr [expr 0.5*$effP21err]
        set feffP [expr 0.5*(1. + $effP21)]
    }
    set den [expr $ctrt(A) - $ctrt(B)]
    set num [expr $ctrt(D) - $ctrt(A)]
    if {$den > 0 && $num > 0} {
        set effA21 [expr $num/$den]
        set effA21err [expr ($sigsq(A) + $sigsq(B))/($den*$den) + ($sigsq(D) +$sigsq(C))/($num*$num)]
        set effA21err [expr $effA21*sqrt($effA21err)]
        set feffAerr [expr 0.5*$effA21err]
        set feffA [expr 0.5*(1. + $effA21)]
    }
    set flipEff(0) $feffP
    set flipEff(1) $feffA
    set flipEffEff(0) $feffPerr
    set flipEffErr(1) $feffAerr
}


proc parseFR {istar cmd} {
	global txt events
	global endFrReg frReg
	global nextphrase
	global endCount endFR
	global ctsList
	global VQ VQmode VQofIndex flipstate flipstatestr flipstateVSFLIP FLIP Ei
	global utc datestr beamsize
    global V0 Q0 V1 Q1
    global modes0 modes1
    global frbeams
    global ctrates
    global frerrors ctrateRejectionCI
    global frupdate
    global fbgcpm
    global shareNSF sharedwith
    global npairs nfiles
    global feffA feffP feffAerr feffPerr
    global envTemp envBfld
    

    emptyFRarrays
    # fr block starting at istar is hard to parse since it may contain flipstate commands and count blocks
	# any VQmode change starts a new fr in frmode, add the last one to events
	# And of course flipstate seq are allowed
	# The only pb cmnd allowed  in an frmode block is pb beam
	# Anyother pb cmnd exits this parseFR
	# frmsr array by beam size (-1 0 1) of array by VQmode (V Q) of array of msrments
	# so there are four arrays V0 V1 Q0 Q1
	# accessed from frbeams(0) = modes0  frbeams(1) = modes1
	# modes0(V) = V0 modes0(Q) = Q0  modes1(V) = V1 modes1(Q) = Q1
	# each of V0 V1 Q0 Q1 is array by FLIP of ctsList = utc totalCts totalSec
	
	set istari [$txt(0) index "$istar linestart - 1 lines"]
	$txt(5) insert end "flip-ratio block starts at line:\n"
	$txt(5) insert end [$txt(0) get $istari "$istari lineend"]
	$txt(5) insert end "\n"
	set nextline [$txt(0) index "$istar linestart + 1 lines"]
	set endFR [findNextReg $nextline $endFrReg]


	if { $endFR } {
		$txt(5) insert end "and ends at phrase: $nextphrase  and line:\n"
	 	set iendi [$txt(0) index "$endFR linestart"]
	 	$txt(5) insert end [$txt(0) get $iendi "$iendi lineend"]
	 	$txt(5) insert end "\n"
	} else {
		set endFr [$txt(0) index end]
		$txt(5) insert end "and ends at end of console text\n"
	}
	$txt(5) insert end "\n"
	 
	if {! $endFR } { set endFR [$txt(0) index end] }
	#puts "parseFR endFR= $endFR"
	# now loop for Counters: or ON.seq off.seq etc

	set frflipstates {}
	set frerrors {}
	while {1} {
	    #puts "parseFR loop nextline= $nextline"
		set fri [findNextReg $nextline $frReg]
		if {! $fri || $fri >= $endFR } {break}
		#puts "parseFR at fri= $fri  nextphrase= $nextphrase"
		getUTC $fri
		set friline [$txt(0) get "$fri linestart" "$fri lineend"]
		set frtxt [$txt(0) get $fri "$fri lineend"]
        if {[string match "pb b*" $nextphrase] && [string match *\]* $frtxt] == 0} {
            set nextline [$txt(0) get $fri "$fri lineend"]
            set beamargs [split $nextline]
            set beamargs [lreplace $beamargs 0 1]
            #puts "parseFR beamargs= $beamargs"
            beamParse $beamargs
            set nextline [$txt(0) index "$fri linestart + 1 lines"]
        } elseif {[string match "pb fbg*" $nextphrase] && [string match *\]* $frtxt] == 0} {
            set nextline [$txt(0) get $fri "$fri lineend"]
            set fpgcpm [lindex $nextline 2]
            set nextline [$txt(0) index "$fri linestart + 1 lines"]
		} elseif [string match "Moving FLIP*" $nextphrase] {
			puts "parseFR found moving flip at frtxt= $frtxt"
            if {[flipParse [lindex $frtxt 2]] > 0} {
            	logEvent [list $datestr $utc FLIP $flipstateVSFLIP($FLIP) $FLIP]
            }
            set nextline [$txt(0) index "$fri linestart + 1 lines"]
		} elseif [string match "Moving Hsample*" $nextphrase] {
			puts "parseFR found MODE change at $friline"
            set nmri [$txt(0) index "$fri linestart + 2 lines"]
            set nmrln [$txt(0) get $nmri "$nmri lineend"]
            if {[string match -nocase "*Moving Efcancel*" $nmrln]} {
            	set amps [lindex $frtxt 2]
            	set mods Qmode
            	if {$amps == 0} {set mods Vmode}
            	modeParse $mods
            	logEvent [list $datestr $utc $VQ]
            } else {
            	lappend warnings "WARNING: moving Hsample without moving efcancel ignores mode change at\n$apline"
            }
			
            modeParse $nextphrase
            set nextline [$txt(0) index "$fri linestart + 1 lines"]
        } elseif [string match Counters:* $nextphrase] {
			set ncts [parseCounts $fri 1]
			# parsCounts handles badCountRates checking
			if {$ncts < 0} {
				lappend frerrors "ERROR: parsing flip-ratio counts at $fri\:\n$friline"
				return
			}
			puts "parseFR Counters ncts= $ncts VQ=$VQ FLIP=$FLIP"
			if {$ncts > 0} {
				appendFRarray 0 $beamsize $VQ $FLIP [list fr $utc $FLIP $ctsList]
                #set beamarr $frbeams($beamsize)
                #eval set modearr $$beamarr\($VQ)
                #array set lmod [array get $modearr]
                #lappend lmod($FLIP) [list fr $utc $FLIP $ctsList]
                #array set $modearr [array get lmod]
                
			    logEvent [list $datestr $utc $cmd $ctsList beam=$beamsize $VQ $flipstate $FLIP]
				if {[lsearch -exact $frflipstates $flipstatestr] < 0} {
					lappend frflipstates $flipstate
				}
			}
			set nextline $endCount
		} else {
		    set nextline [$txt(0) index "$fri linestart + 1 lines"]
		}
	}
	# compute the fr result by first combining the measurement lists in V|Q0|1(FLIP)
    # RWE added nfiles Jan 13 2017
	set fresult [list $datestr $utc fresult $cmd pair=$npairs $frupdate]
	foreach bs {0 1} {
	    #set beamarr $frbeams($bs)
	    # if sharing nsf between V and Q modes is allowed (deflt) then check here to see if V or Q had nsf missing
	    set sharedwith ""
	    if {$shareNSF} {
	        array set nsfVQ {V {} Q {}}
	        array set sfVQ {V {} Q {}}
	        foreach md {V Q} {
	            #array set modearr [array get $beamarr]
	            #array set lmod [array get modearr($md)]
	            foreach st {A D} {
	            	set lmodlst [getFRarray 0 $bs $md $st]
	                if {[llength $lmodlst] > 0} {
	                    set nsfVQ($md) $lmodlst
	                    break
	                }
	            }
	            foreach st {B C} {
	            	set lmodlst [getFRarray 0 $bs $md $st]
	                if {[llength $lmodlst] > 0} {
	                    set sfVQ($md) $lmodlst
	                    break
	                }
	            }
	        }
	        if {[llength $sfVQ(V)] > 0 && [llength $nsfVQ(V)] < 1 && [llength $nsfVQ(Q)] > 0} {
	            #set modearr $beamarr(V)
	            foreach st {A D} {
	            	set lmodlst [getFRarray 0 $bs Q $st]
	            	foreach mod $lmodlst {
	            	    appendFRarray 0 $bs V $st $mod
	            	}
	            }
	            #puts "shared bs=$bs AandD Q with V"
	            set sharedwith V
	            #appendFRarray 0 $bs V set lmod $nsfVQ(Q)
	            #array set $modearr [array get lmod]
	        }
	        if {[llength $sfVQ(Q)] > 0 && [llength $nsfVQ(Q)] < 1 && [llength $nsfVQ(V)] > 0} {
	            #set modearr $beamarr(Q)
	            foreach st {A D} {
	            	set lmodlst [getFRarray 0 $bs V $st]
	            	#puts "shared bs=$bs $st V with Q before appendFRarray lmodlst from getFRarray= $lmodlst"
	            	foreach mod $lmodlst {
	            	    appendFRarray 0 $bs Q $st $mod
	            	}
	            }
	            #puts "shared bs=$bs AandD V with Q so now getFRarray A for Q= [getFRarray 0 $bs Q A]"
	            set sharedwith Q
	            #array set lmod $nsfVQ(V)
	            #array set $modearr [array get lmod]
	        }
	    }
	    # end hsareNSF
	    # now can combine counts
	    foreach md {V Q} {
	        #eval set modearr $$beamarr\($md)
	        array unset lmod
	        array set lmod {A {} B {} C {} D {}}
	        #puts "lmod for bs=$bs md=$md is [array get lmod]"
            array unset nst
	        array set nst {}
	        set nstot 0
	        set nmsr 0
	        foreach st {A B C D} {
	        	set lmod($st) [getFRarray 0 $bs $md $st]
	        	if {$sharedwith == $md} {
	        	    #puts "sharedwith=$sharedwith lmod($st)= $lmod($st)"
	        	}
	        	set llmod [llength $lmod($st)]
	            set nmsr [expr $nmsr + $llmod]
	            if {$sharedwith == $md} {
	                #puts "sharedwith=$sharedwith nmsr=$nmsr before combineCts llength lmod($st)=$llmod"
	            }
	            set lmod($st) [combineCounts $lmod($st) 1 0]
	            if {$sharedwith == $md} {
    	            #puts "sharedwith=$sharedwith after combinCounts lmod($st) = $lmod($st)"
    	        }
	            #after combineCounts lmod(st) is list of utc st ctlist
	            set nst($st) 0
	            if {[llength $lmod($st)] > 0} {
	                set nst($st) 1
	                incr nstot
	            }
	        }

	        #puts "parseFR about to call frCtRates VQ=$md nmsr=$nmsr nst = [array get nst] lmod= [array get lmod]"
            set fresultlst [frCtRates $nmsr [array get nst] [array get lmod]]
	        if {$nstot == 4 && $sharedwith == ""} {
	            # compute flipper efficiencies and log them
	            computeFlipEffs
	            logEvent [list $datestr $utc flipeffP $feffP $feffPerr flipeffA $feffA $feffAerr]
	        }

	        lappend fresult $md$bs
	        # subtract fbgcpm from both

            lappend fresult $fresultlst
	    }
	}
	# as sanity check append ei
	lappend fresult ei=$Ei nfiles=$nfiles bfld=$envBfld temp=$envTemp
    logEvent $fresult
	# log an $cmd\end to clearly define a fr block
	logEvent [list $datestr $utc $cmd\end]
    return [llength $frflipstates]
}


proc oneValueList {lst} {
	set n [llength $lst]
	if {$n <= 1} {return 1}
	set iv [lindex $lst 0]
	foreach v $lst {
		if {$v != $iv} {return 0}
	}
}

proc parseFRscan {istar cmd arglst} {
	global txt events
	global endFrscanReg frscanReg
	global nextphrase
	global endCount endFR endScan
	global ctsList
	global VQ VQmode VQofIndex flipstate flipstatestr FLIP Ei
	global flipstateVSFLIP
	global utc datestr beamsize
    global V0 Q0 V1 Q1 V0B Q0B V1B Q1B
    global modes0 modes1 modes0B modes1B
    global frbeams frbeamsB
    global ctrates
    global frerrors ctrateRejectionCI
    global frscanvars lcfrscanvars
    global npairs nfiles
    global frupdate
    global frdata
    global fbgcpm
    global scanfit scanfrdata
    global scanBG scanCtPeak scanCtBG
    global envTemp envBfld

    emptyFRarrays

    # flip-ratio scan may not reliably reproduce the coordinate position of Breagg peak
    # use the scanfit flag to request fits to single Gaussian near max cts for all scans
    # NB this requires at least 4 points in the scan to fit 3 params H*exp(-fac((x-x0)/FWHM)^2)
    # If this flag is not set, use the max cts to find the peak position from NSF scan for given VQ smallBeam
    # This means post processing all the scans in the block since we have to find NSF 

    # The arglist can pass the scanvar column name from A4 A3 Qx Qy Qz frscanvars  v[ar]=colname
    # can pass the scanfit flag f[it]=0|1
    # can pass the peak value of scanvar as p[k]=coord 
    
    # istar is index of line after pb frscan  or pb frbscan
    # parseFRscan is similar to parseFR except using scans via START OF SCAN
    # fr is hard to parse since it may contain flipstate seq commands and count blocks
	# any VQmode change starts a new fr in frmode, add the last one to events
	# And of course flipstate seq are allowed
	# The only pb cmnd allowed  in an frmode block is pb beam
	# Anyother pb cmnd exits this parseFR
	# frmsr array by beam size (-1 0 1) of array by VQmode (V Q) of array of msrments
	# so there are four arrays V0 V1 Q0 Q1
	# accessed from frbeams(0) = modes0  frbeams(1) = modes1
	# modes0(V) = V0 modes0(Q) = Q0  modes1(V) = V1 modes1(Q) = Q1
	# each of V0 V1 Q0 Q1 is array by FLIP of ctsList = utc totalCts totalSec
	
	set istari [$txt(0) index "$istar linestart - 1 lines"]
	$txt(5) insert end "scan flip-ratio block starts at line:\n"
	$txt(5) insert end [$txt(0) get $istari "$istari lineend"]
	$txt(5) insert end "\n"
	set nextline [$txt(0) index "$istar linestart + 1 lines"]
	set endFR [findNextReg $nextline $endFrscanReg]


	if { $endFR } {
		$txt(5) insert end "and ends at phrase: $nextphrase  and line:\n"
	 	set iendi [$txt(0) index "$endFR linestart"]
	 	$txt(5) insert end [$txt(0) get $iendi "$iendi lineend"]
	 	$txt(5) insert end "\n"
	} else {
		set endFr [$txt(0) index end]
		$txt(5) insert end "and ends at end of console text\n"
	}
	$txt(5) insert end "\n"

	#puts "parseFR endFR= $endFR"
	# now loop for Counters: or ON.seq off.seq etc
	# init the flags lscanfit from global scanfit
	set lscanfit $scanfit
	set frscanvar ""
	set frscanpos ""
	if {[llength $arglst] > 0} {
	    set is [lsearch $arglst f*]
	    if {$is >= 0} {
	        set fs [lindex $arglst $is]
	        if [regexp {^.+=(\d+)$} $fs wm ival] {
	            set lscanfit $ival
	        }
	    }
	    set is [lsearch $arglst v*]
	    if {$is >= 0} {
	        set vs [lindex $arglst $is]
	        if [regexp {^.+=(.*+)$} $vs wm col] {
	            set col [string tolower $col]
	            set icol [lsearch $lcfrscanvars $col]
	            if {$icol >= 0} {
	                set frscanvar [lindex $frscanvars $icol]
	            }
	        }
	    }
	    set is [lsearch $arglst p*]
	    if {$is >= 0} {
	        set ps [lindex $arglst $is]
	        set ie [string first "=" $ps]
	        set ie1 [expr $ie + 1]
	        if {ie >= 0 && [string length $ps] > $ie1} {
	            set vs [string range $ie1 end]
	            if [scan $vs "%g" val] {
	                set frscanpos $val
	            }
	        }
	    }
	}

	set frflipstates {}
	set frerrors {}
	set ndatansf 0
	set hasfpk 0
	while {1} {
	    #puts "parseFR loop nextline= $nextline"
		set fri [findNextReg $nextline $frscanReg]
		set fpk 0
		if {! $fri || $fri >= $endFR } {break}
		#puts "parseFR at fri= $fri  nextphrase= $nextphrase"
		getUTC $fri
		set frtxt [$txt(0) get $fri "$fri lineend"]
        if {[string match "pb b*" $nextphrase] && [string match *\]* $frtxt] == 0} {
            set nextline [$txt(0) get $fri "$fri lineend"]
            set beamargs [split $nextline]
            set beamargs [lreplace $beamargs 0 1]
            #puts "parseFR beamargs= $beamargs"
            beamParse $beamargs
            set nextline [$txt(0) index "$fri linestart + 1 lines"]
		} elseif [string match *\.seq* $nextphrase] {
            seqParse $nextphrase
            set nextline [$txt(0) index "$fri linestart + 1 lines"]
        } elseif [string match -nocase "moving flip" $nextphrase] {
            #puts "annotbt7 calling seqParse"
            # check for nmr cmnd 2 lines down
            set nmri [$txt(0) index "$fri linestart + 2 lines"]
            set nmrln [$txt(0) get $nmri "$nmri lineend"]
            if {[string match "*3He polarizer state*" $nmrln] || [string match "*3He analyzer state*" $nmrln]} {
            	set FLIPs [lindex $frtxt 2]
            	seqParse $flipstateVSFLIP($FLIPs)
            } else {
            	lappend frerrors "WARNING: moving FLIP without nmr commands ignores flipstate change"
            }
            set nextline [$txt(0) index "$fri linestart + 1 lines"]
        } elseif [string match -nocase "moving hsample" $nextphrase] {
            #puts "annotbt7 calling seqParse"
            set nmri [$txt(0) index "$fri linestart + 2 lines"]
            set nmrln [$txt(0) get $nmri "$nmri lineend"]
            if {[string match "*Moving Efcancel*" $nmrln]} {
            	set amps [lindex $frtxt 2]
            	set mods Qmode
            	if {$amps == 0} {set mods Vmode}
            	seqParse $mods
            } else {
            	lappend frerrors "WARNING: moving Hsample without moving efcancel ignores mode change"
            }
            set nextline [$txt(0) index "$fri linestart + 1 lines"]            
        } elseif {[string match "START OF SCAN" $nextphrase] || [set fpk [string match "Find Peak" $nextphrase]]} {
            # parseScan will set the frdata array when called with flip-ratio flag
            # trying to handle normal scan or findPeak scan
            # fitOK should be in the frdata array if fpk is used
            puts "parseFRscan $datestr parseScan starting with FLIP= $FLIP"
            set ndata [parseScan $fri 1 $fpk]
			if {$ndata < 1} {
			    lappend frerrors "WARNING: no data for flip-ratio scan at $fri"
            } else {
                if {$ndata < 3 && $lscanfit} {
                    lappend frerrors "WARNING: less than 3 datapoints turns off scanfit"
                    set lscanfit 0
                }
                if {$fpk} {
                	set hasfpk 1
                	if [info exists frdata(var)] {set frscanvar $frdata(var)}
                	if [info exists frdata(Pos)] {set frscanpos $frdata(Pos)}
                }
                #set beamarr $frbeams($beamsize)
                #eval set modearr $$beamarr\($VQ)
                #array set lmod [array get $modearr]
                # turn the column data array=frdata into a list of colname valueslist ... as frdatalist
                set frdatalist [array get frdata]
                appendFRarray 0 $beamsize $VQ $FLIP [list fr $utc $FLIP $frdatalist]
                # find nsf data in case we need post processing
                if {($FLIP == "A" || $FLIP == "D") && $ndata > $ndatansf} {
                    set datansf $frdatalist
                    set ndatansf $ndata
                }
                #array set $modearr [array get lmod]
                # ctsList hasnt been prepared from frdatalist yet so no frscan logging
			    #logEvent [list $datestr $utc $cmd $ctsList beam=$beamsize $VQ $flipstate $FLIP]
				if {[lsearch -exact $frflipstates $flipstatestr] < 0} {
					lappend frflipstates $flipstate
				}
			}
			set nextline $endScan
		} else {
		    set nextline [$txt(0) index "$fri linestart + 1 lines"]
		}
	}
	# in order to skip preprocessing the data we need to know
	# the scanvar which can only come in through arglst or findPeak
	# This means findPeak is preferred
	# even if we know the scan variable, if not scanfit and the coord is not passed in arglst
	# then we need to preprocess to find the scan position
    # so the cases where no processing is need are
    # scanvar passed and scanfit is passed or global scanfit is set
    # scanvar passed and position is passed

    if {$ndatansf < 1} {
        lappend frerrors "ERROR: flip-ratio scans have NO nsf data at $istar"
        return 0
    }

    if {$lscanfit && $ndatansf < 3} {
        lappend frerrors "ERROR: scanfit selected but data has less than 3 points. Will use max cts"
        set lscanfit 0
    }

    array set data $datansf
    if {[string length $frscanvar] < 2} {
        # use the nsfdata
        set frvar ""
        foreach v $frscanvars {
            if {![info exists data($v)]} {continue}
            if [monotonicList $data($v)] {
                set frvar $v
                break
            }
        }
        if {$frvar == ""} {
            lappend frerrors "ERROR: failed to determine a scanned coordinate at $istar"
            return 0
        }
        set frscanvar $frvar
    }
    # is fitScanData fails (for example the peak is at end of scan) we want to be able to fall back to maxcts or passed value
    # of if not fitting for peak positon so we need a passed value or get it from data max cts
    if {[string length $frscanpos] < 1 || ![scan $frscanpos %g pos] || ![inListRange $data($frscanvar) $frscanpos]} {
        set mxdl [maxCountRate $data(Det) $data(Time)]
        set imx [lindex $mxdl 0]
        set frscanpos [lindex $data($frscanvar) $imx]
    }
    # now go back thru all the data and turn the frdatalist into ctsList = fr utc totalCts totalSecs
    array set lmod {A {} B {} C {} D {}}
	foreach bs {0 1} {
	    #set beamarr $frbeams($bs)
	    #set beamarrB $frbeamsB($bs)
	    foreach md {V Q} {
	        #eval set modearr $$beamarr\($md)
	        #eval set modearrB $$beamarrB\($md)
	        #array set lmod [array get $modearr]
	        #array set lmodB [array get $modearrB]
	        foreach st {A B C D} {
	        	set newlmod {}
	        	set newlmodB {}
	        	set lmod($st) [getFRarray 0 $bs $md $st]	        	
	            foreach scd $lmod($st) {
	            	puts "parseFRscan processing st=$st scd= $scd"
	                # the frdatalist scandatalist is index 3. change it so that it is ctList= fr utc totalCts totalSec
	                set scandatalist [lindex $scd 3]
	                set scanutc [lindex $scd 1]
	                array set data $scandatalist
	                # ready to convert data to ctList including possible bg ctList
	                if {[info exists data(fitOK)] && [info exists data(Pos)] && [info exists data(Height)] && [info exists data(BG)]} {
	                	# this is findPeak so extract Counts as Height parameter
	                	# if ct against time it is easy to get the ctRate as all times are equal
	                	# but for count against monitor needs to add the monitor uncertainty as in fitScanData
	                	# if Ei is not varied and total scan time is less than 30 min probably can average count secs
	                	#set peakTime [interpTime $data(Pos) $data($data(var)) $data(Time)]
	                	set fxEi 1
	                	if {[info exists data(E)] && ! [oneValueList $data(E)]} {set fxEi 0}
	                	if [peakScanData $data(Height) $data(Pos) $data(Width) $data(BG) $data($frscanvar) $data(Det) $data(Time) $data(Mon) $fxEi] {
	                		set ctl [list $cmd $scanutc [lindex $scanCtPeak 0] [lindex $scanCtPeak 1]]
	                		set scd [lreplace $scd 3 3 $ctl]
	                		# pretty sure this makes a copy of scd for newlmod
	                		lappend newlmod $scd	
		                	if {$scanBG} {
		                		set ctl [list $cmd\bg $scanutc [lindex $scanCtBG 0] [lindex $scanCtBG 1]]
		                		set scd [lreplace $scd 3 3 $ctl]
		                		lappend newlmodB $scd	                
		                	}	                		             	
	                	}
	                } else {	                
	                	if {$lscanfit} {
	                    	# try the Gaussian peak model now 3 pt exact
	                    	if {![fitScanData $data($frscanvar) $data(Det) $data(Time) $data(Mon) 1 0]} {
	                        	# revert to frscanpos
	                        	fitScanData $data($frscanvar) $data(Det) $data(Time) $data(Mon) 0 $frscanpos
	                    	}
	                	} else {
	                    	fitScanData $data($frscanvar) $data(Det) $data(Time) $data(Mon) 0 $frscanpos
	               		}
	                	set ctl [list $cmd $scanutc [lindex $scanCtPeak 0] [lindex $scanCtPeak 1]]
	                	puts "parseFRscan st=$st got ctl= $ctl"
	                	set scd [lreplace $scd 3 3 $ctl]
	                	# pretty sure this makes a copy of scd for newlmod
	                	lappend newlmod $scd
	                	if {$scanBG} {
	                		set ctl [list $cmd\bg $scanutc [lindex $scanCtBG 0] [lindex $scanCtBG 1]]
	                		set scd [lreplace $scd 3 3 $ctl]
	                		lappend newlmodB $scd	                
	                	}
	            	}
	            }
	            set lmod($st) $newlmod
	            set lmodB($st) $newlmodB
	        }
	        foreach st {A B C D} {
	        	setFRarray 0 $bs $md $st $lmod($st)
	        	setFRarray 1 $bs $md $st $lmodB($st)
	        }
        }
    }
    # RWE Dec 18 need to fix below
	# compute the fr result by first combining the measurement ct lists in V|Q0|1(FLIP)
	set fresult [list $datestr $utc fresult $cmd pair=$npairs $frupdate]
	set fresultB [list $datestr $utc frbresult $cmd pair=$npairs $frupdate]
	set bgSize 0
	foreach bs {0 1} {
	    foreach md {V Q} {
            array unset nst
	        array set nst {}
            array unset nstB
	        array set nstB {}
	        set nmsr 0
	        set nmsrB 0
	        foreach st {A B C D} {
    	        set lmod($st) [getFRarray 0 $bs $md $st]
    	        set lmodB($st) [getFRarray 1 $bs $md $st]
	        	set nlmod [llength $lmod($st)]
	        	set nlmodB [llength $lmodB($st)]
	            set nmsr [expr $nmsr + $nlmod]
	            set nmsrB [expr $nmsrB + $nlmodB]
	            set lmod($st) [combineCounts $lmod($st) 1 0]
	            set lmodB($st) [combineCounts $lmodB($st) 1 0]
	            puts "parseFRscan bsmd=$bs$md after combinCounts lmod($st) = $lmod($st)"
	            # after combineCounts lmod(st) is list of  cmd utc st ctlist
	            set nst($st) 0
	            if {$nlmod > 0} {set nst($st) 1}
	            set nstB($st) 0
	            if {$nlmodB > 0} {set nstB($st) 1}
	        }
	        # rlst= nmsr [nsf sf] utc fr frerr ctrlst=[nsfctr nsfctrerr sfctr sfctrerr]
	        set rlst [frCtRates $nmsr [array get nst] [array get lmod]]
	        puts "parseFRscan bsmd=$bs$md rlst= $rlst"
	        set rlstB [frCtRates $nmsrB [array get nstB] [array get lmodB]]
	        
	        lappend fresult $md$bs	        
	        lappend fresultB $md$bs
	        lappend fresult $rlst
	        lappend fresultB $rlstB
	        set bgSize [expr $bgSize + [llength $rlstB]]
	    }    
	}
	# as sanity check append ei
	lappend fresult ei=$Ei nfiles=$nfiles bfld=$envBfld temp=$envTemp
	lappend fresultB ei=$Ei nfiles=$nfiles bfld=$envBfld temp=$envTemp
    logEvent $fresult
	# log an $cmd\end to clearly define a fr block
	if {$bgSize > 0} {
		logEvent $fresultB
	}
	logEvent [list $datestr $utc $cmd\end]
    return [llength $frflipstates]
}



proc frCtRates {nmsr nstlst lmodlst} {
    # pass nst lmod as array get lists and reconstruct them as local arrays
	global fbgcpm
	global frctrates
	global warnings utc datestr
	# ths will construct the back half of the fresult list for ABCD states of one of the VQ modes
	set nsf 0
	set sf 0
	#puts "nstABCD for $md$bs= $nst(A) $nst(B) $nst(C) $nst(D)"
	set sfctr 0
	set sfctrerr 0
	set nsfctr 0
	set nsfctrerr 0
	#set utc 0

    array set nst $nstlst
	array set lmod $lmodlst
	#puts "frCtRates nmsr=$nmsr nstlst=$nstlst lmodlst=$lmodlst"
	set ctra {}
	set ctrb {}
	set ctrc {}
	set ctrd {}
	#puts "frCtRates starting lmodlst= $lmodlst"
	if {$nst(A) && $nst(D)} {
		set nsf 3
		set utc [lindex $lmod(A) 1]
		set utcd [lindex $lmod(D) 1]
	    if {$utcd > $utc} {set utc $utcd}
		set ctra [getCountRate [lindex $lmod(A) 3]]
		set ctrd [getCountRate [lindex $lmod(D) 3]]
		set cta [lindex $ctra 0]
		set ctd [lindex $ctrd 0]
		set nsfctr [expr 0.5*($cta + $ctd)]
		set erra [lindex $ctra 1]
		set errd [lindex $ctrd 1]
		set nsfctrerr [expr 0.5*sqrt($erra*$erra + $errd*$errd)]
		if {[expr abs($cta - $ctd)] > [expr 3.*$nsfctrerr]} {
			lappend warnings "WARNING: nsf ctrates for A and D differ by more that 3 sdev at $utc $datestr"
		}
	} elseif {$nst(A)} {
		set nsf 1
		set utc [lindex $lmod(A) 1]
		set ctra [getCountRate [lindex $lmod(A) 3]]
		set nsfctr [lindex $ctra 0]
		set nsfctrerr [lindex $ctra 1]
	} elseif {$nst(D)} {
		set nsf 2
		set utc [lindex $lmod(D) 1]
		set ctrd [getCountRate [lindex $lmod(D) 3]]
		set nsfctr [lindex $ctrd 0]
		set nsfctrerr [lindex $ctrd 1]
	}
	if {$nst(B) && $nst(C)} {
		set sf 3
		set utcb [lindex $lmod(B) 1]
		set utcc [lindex $lmod(C) 1]
		if {$utcb > $utc} {set utc $utcb}
		if {$utcc > $utc} {set utc $utcc}
		set ctrb [getCountRate [lindex $lmod(B) 3]]
		set ctrc [getCountRate [lindex $lmod(C) 3]]
		set ctb [lindex $ctrb 0]
		set ctc [lindex $ctrc 0]
		set sfctr [expr 0.5*($ctb + $ctc)]
		set errb [lindex $ctrb 1]
		set errc [lindex $ctrc 1]
		set sfctrerr [expr 0.5*sqrt($errb*$errb + $errc*$errc)]
		if {[expr abs($ctb - $ctc)] > [expr 3.*$sfctrerr]} {
			lappend warnings "WARNING: sf ctrates for B and C differ by more that 3 sdev at $utc $datestr"
		}
	} elseif {$nst(B)} {
		set sf 1
		set utcb [lindex $lmod(B) 1]
		if {$utcb > $utc} {set utc $utcb}
		set ctrb [getCountRate [lindex $lmod(B) 3]]
		set sfctr [lindex $ctrb 0]
		set sfctrerr [lindex $ctrb 1]
	} elseif {$nst(C)} {
		set sf 2
		set utcc [lindex $lmod(C) 1]
		if {$utcc > $utc} {set utc $utcc}
		set ctrc [getCountRate [lindex $lmod(C) 3]]
		set sfctr [lindex $ctrc 0]
		set sfctrerr [lindex $ctrc 1]
	}
	set frctrates [list $ctra $ctrb $ctrc $ctrd]
	#puts "frCtRates nsf=$nsf sf=$sf sfctr=$sfctr  frctrates= $frctrates"
	if {$nsf < 1 || $sf < 1 || $sfctr <= 0.} {
		set fresult {}
	} else {
		incr nsf -1
		incr sf -1
		set nsfctr [expr $nsfctr - ($fbgcpm/60.)]
		set sfctr [expr $sfctr - ($fbgcpm/60.)]
		set ctrlst [list $nsfctr $nsfctrerr $sfctr $sfctrerr]
		set fr [expr $nsfctr/$sfctr]
		set nsfre 0.
		if {$nsfctr > 0.} {set nsfre [expr $nsfctrerr/$nsfctr]}
		set sfre [expr $sfctrerr/$sfctr]
		set frerr [expr $fr*sqrt($nsfre*$nsfre + $sfre*$sfre)]
		set fresult [list $nmsr [list $nsf $sf] $utc $fr $frerr $ctrlst]
	}
	return $fresult
}

proc peakScanData {ht pos fw bg xin yin tin min fxEi} {
	# handle findPeak for frscan
	# does sanity checks on pos fwhm to make sure that ht is reasonable
	# returns 0 on error
	#$data(Pos) $data($frscanvar) $data(Det) $data(Time) $data(Mon) $fxEi
    global fitScanSubBG
    global scanBG
    global scanCtPeak scanCtBG
    
    set scanCtBG {}
    set scanBG 0
    # check for all equal times
    set nx [llength $xin]
    set nx1 [expr $nx - 1]
    set ny [llength $yin]
    set nt [llength $tin]
    if {$nx != $ny || $nx != $nt || $nx == 0} {return 0}
    # if pos is out of x range means trouble for using as flip ratio
    if {![inListRange $xin $pos]} {return 0}
    set xran [listRange $xin]
    set xrang [expr [lindex $xran 1] - [lindex $xran 0]]
    # cant even trust height if findpeak doesnt msr in background
    if {$fw > [expr $xrang/2.]} {return 0}
    # check to see if there are at least 2 data points yin that are within statistics of bg
    set scanBG 0
    set nbg 0
    foreach y $yin {
    	if {$y <= 0} {continue}
    	set yerr [expr sqrt($y)]
    	if {[expr abs($y - $bg)] <= $yerr} {incr nbg}
    }
    if {$nbg >= 2} {set scanBG 1}
    
    # make sure data sorted low to high on x
    # we checked for monotonicity before calling
    set ilst [lsort -indices $xin]
    set ilstn {}
    for {set i 0} {$i < $nx} {incr i} {lappend ilstn $i}
    if [expr {$ilst eq $ilstn}] {
        set x $xin
        set y $yin
        set t $tin
        set m $min
    } else {
        set x {}
        set y {}
        set t {}
        set m {}
        for {set i 0} {$i < $nx} {incr i} {
            set ii [lindex $ilst $i]
            lappend x [lindex $xin $ii]
            lappend y [lindex $yin $ii]
            lappend t [lindex $tin $ii]
            lappend m [lindex $min $ii]
        }
    }

    set t0 [lindex $t 0]
    set t2 {}
    set totalTime 0
    for {set i 0} {$i < $nx} {incr i} {
        lappend t2 $t0
        set totalTime [expr $totalTime + [lindex $t $i]]
    }
    set tdif 1
    if [expr {$t2 eq $t}] {set tdif 0}
	# even if tdif is 1 (ct against monitor)
	# we should be ok unless fxEi is 0
	# if total tin is > 1 hr also interpolate
	if {$tdif == 1 && $fxEi == 1 && $totalTime <= 3600} {
		# in this usual case we can just fix up for added uncertainty of counting vs mon
		set tdif 0
		set t0 [expr $totalTime/$nx]
		set ctrt [expr $ht/$t0]
		# we assume the t0 error is small
		set m0 [lindex $min 0]
		set ctrterr [expr $ctrt*sqrt(1./$ht + 1./$m0)]
		# scale the ctrt (ht and t0) so that ctrerr is sqrt of ctrt
		set sc [expr $ctrt/($ctrterr*$ctrterr)]
		set ht [expr $sc*$ht]
		set t0 [expr $sc*$t0]
		# also must scale bg by same scale factor
		set bg [expr $sc*$bg]
	}
	if {$tdif == 0} {
		set scanCtPeak [lreplace $scanCtPeak 0 0 $ht]
		set scanCtPeak [lreplace $scanCtPeak 1 1 $t0]
		# findpeak doesnt give us the param errors, so I assume the bg error is zero
		# and fudge the scaling to make the error small
		lappend scanCtBG [expr 10000*$bg]
		lappend scanCtBG [expr 10000*$t0]
		return $nx
	}
	# handle ct against monitor where Ei varies possibly or long scan in time.
	# This is rare so maybe just find the time nearest to x-pos
	set t0 [lindex $tin 0]
	for {set xi [lindex $xin 0]; set i 0} {incr i} {$i < [expr $nx - 1]} {
		set xe [lindex $xin [expr $i + 1]]
		if {$pos >= $xi && $pos <= $xe} {
			set t0 [lindex $tin $i]
			break
		}
	}
	set ctrt [expr $ht/$t0]
	# we assume the t0 error is small
	set m0 [lindex $min 0]
	set ctrterr [expr $ctrt*sqrt(1./$ht + 1./$m0)]
	# scale the ctrt (ht and t0) so that ctrerr is sqrt of ctrt
	set sc [expr $ctrt/($ctrterr*$ctrterr)]
	set ht [expr $sc*$ht]
	set t0 [expr $sc*$t0]
	# also must scale bg by same scale factor
	set bg [expr $sc*$bg]
	set scanCtPeak [lreplace $scanCtPeak 0 0 $ht]
	set scanCtPeak [lreplace $scanCtPeak 1 1 $t0]
	# findpeak doesnt give us the param errors, so I assume the bg error is zero
	# and fudge the scaling to make the error small
	lappend scanCtBG [expr 10000*$bg]
	lappend scanCtBG [expr 10000*$t0]
	return $nx
}


proc fitScanData {xin yin tin min g pos} {
    global fitScanSubBG
    global scanBG
    global scanCtPeak scanCtBG
    # get peak height from some kind of fit or other algorithm from scan data xin yin tin min = x cts sec mon
    # if g is set use 3 pt Gaussian peak
    # else just compute cts at pos which is supposed to be within scan range
    # to actually fit a Gaussian plus background
    # requires non-linear fitter which I can add at some point
    # but there are still issues that would have to handled if the
    # peak is not single Gaussian. This could also be handled by checking
    # goodness of fit. These issues are for MACS where due to motor reproduciblity issues
    # they always do a scan.
    # In principle  a wide enough scan can also provide a background measurement

    # For now we do some simple math to get position peakheight FW as
    # exact three point solution to y(x) = H*exp(-fac((x-X)/F)^2)
    # where fac accounts for using FWHM=F instead of sigma.
    # The 3 points are ordered in increasing x with max ctRate at x2
    # and equal steps in x assumed.
    # For the 3-point solution then the peak position, X is
    # X = 1/2 (D21(x1+x2) + D23(x2+x3))/(D21 + D23)
    # where D21 = fac/(ln(y2) - ln(y1))  D23 = fac/(ln(y2) - ln(y3))
    # NB this requires y2 > y1  y2 > y3
    # Then FWHM^2 = F^2 = 2 D21 delt (X - (x1 + x2)/2))  where delt is the x-spacing = x2 - x1 = x3 - x2
    # And ln(H) = ln(y2) + fac*(x2 - X)^2/F^2
    # From 3 pts it is stretching to believe that F will reflect the data.
    # But we can look for flat line fit of y at the data edges to try to get background
    # in this simple model. In fact we should do this first and subtract the flatbackground (if any)
    # to get a better Gaussian fit. We would need to decide if we are then going to use
    # the background and the cts above background for the FR peak signal.

    # first check for variable t. If no variation (counting vs time) just fit y(x)
    # else must convert to ctRate and fit that.
    # We need actual cts and seconds for later corrections. just use the nearest pt
    # t to convert back to Cts Secs

    # construct the scanCtList
    #set scanCtPeak [list $H $secs]
    # do the background if fitScanSubBG flag is set and enough pts
    set scanCtBG {}
    set scanBG 0
    # check for all equal times
    set nx [llength $xin]
    set nx1 [expr $nx - 1]
    set ny [llength $yin]
    set nt [llength $tin]
    if {$nx != $ny || $nx != $nt || $nx == 0} {return 0}
    if {$nx < 3 && $g} {
    	# cant get Gaussian from less than 3 pts
    	set g 0
    	if {![inListRange $xin $pos]} {
    		set posl [maxList $xin]
    		set pos [lindex $xin [lindex $posl 0]]
    	}
    }
    # make sure data sorted low to high on x
    # we checked for monotonicity before calling
    set ilst [lsort -indices $xin]
    set ilstn {}
    for {set i 0} {$i < $nx} {incr i} {lappend ilstn $i}
    if [expr {$ilst eq $ilstn}] {
        set x $xin
        set y $yin
        set t $tin
        set m $min
    } else {
        set x {}
        set y {}
        set t {}
        set m {}
        for {set i 0} {$i < $nx} {incr i} {
            set ii [lindex $ilst $i]
            lappend x [lindex $xin $ii]
            lappend y [lindex $yin $ii]
            lappend t [lindex $tin $ii]
            lappend m [lindex $min $ii]
        }
    }

    
    set t0 [lindex $t 0]
    set t2 {}
    set yerr {}
    for {set i 0} {$i < $nx} {incr i} {
        lappend t2 $t0
        set yi [lindex $y $i]
        set yierr 1
        if {$yi > 0} {set yierr [expr sqrt($yi)]}
        lappend yerr $yierr
    }
    set tdif 1
    if [expr {$t2 eq $t}] {set tdif 0}
    set yeff $y
    set yefferr $yerr
    if {$tdif} {
        # ct against monitor so convert to count rate cps
        set yeff {}
        set yefferr {}
        for {set i 0} {$i < $nx} {incr i} {
            set yi [lindex $y $i]
            set ti [lindex $t $i]
            set mi [lindex $m $i]
            set yr [expr $yi/$ti]
            lappend yeff $yr
            set yrerr [expr $yr*sqrt(1./$yi + 1./$mi)]
            lappend yefferr $yrerr
        }
    }
    if {$g < 1} {
    	# use pos
    	set ipos 0
    	set ix [lindex $x 0]
    	set dx [expr abs($pos - $ix)]
    	set i 1
    	while {$i < $nx} {
    		set ix [lindex $x $i]
    		set dxi [expr abs($pos - $ix)]
    		if {$dxi < $dx} {
    			set ipos $i
    			set dx $dxi
    		}
    	}
    	set i2 $ipos
    	set i1 [expr $i2 - 1]
    	set i3 [expr $i2 + 1]
    	if {$i1 < 0} {set i1 0}
    	if {$i3 > $nx1} {set i3 $nx1}
    	set H [lindex $yeff $ipos]
    } else {
    	set iym [maxList $yeff]
    	# if the max value is not bigger than both adjacent values check to see
    	# if using the next pt will fix the problem.
    	set i2 [lindex $iym 0]

    	if [$i2 == 0 || $i2 == $nx1] {return 0}
    	set y2 [lindex $yeff $i2]
    	set x2 [lindex $x $i2]
    	set i1 [expr $i2 - 1]
    	set i3 [expr $i2 + 1]
    	set y1 [lindex $yeff $i1]
    	set x1 [lindex $x $i1]
    	set y3 [lindex $yeff $i3]
    	set x3 [lindex $x $i3]
    	if {$y3 >= $y2} {
        	if ($i3 == $nx1) {return 0}
        	set i3 [expr $i3 + 1]
        	set y3 [lindex $yeff $i3]
        	set x3 [lindex $x $i3]
        	if {$y3 >= $y2} {return 0}
    	}
    	if {$y1 >= $y2} {
        	if ($i1 == 0) {return 0}
        	set i1 [expr $i1 - 1]
        	set y1 [lindex $yeff $i1]
        	set x1 [lindex $x $i1]
        	if {$y3 >= $y2} {return 0}
    	}
    	# ready to solve
    	set x21 [expr $x2 - $x1]
    	set x32 [expr $x3 - $x2]
    	set y21 [expr log($y2) - log($y1)]
    	set y23 [expr log($y2) - log($y3)]
    	set d21 [expr $x21/$y21]
    	set d32 [expr $x32/$y23]
    	set den [expr $d21 + $d32]
    	set num [expr ($x1 + $x2)*$d21 + ($x2 + $x3)*$d32]
    	set X [expr 0.5*$num/$den]
    	set FWsqOverFac [expr 2.*$x21*($X - 0.5*($x1 + $x2))/$y21]
    	set dx [expr $X - $x2]
    	set H [expr $y2*exp($dx*$dx/$FWsqOverFac)]
    	set fac [expr 4.*log(2.)]
    	set FW [expr sqrt($FWsqOverFac*$fac)]
  	}  
    
    # get count time for peak
    set secs [lindex $t $i2]
    if {$tdif} {
        # this restores ctrate to raw counts
        set H [expr $H*$secs]
    }
    # construct the scanCtList
    set scanCtPeak [list $H $secs]
    # do the background if fitScanSubBG flag is set
    if {$fitScanSubBG && ($i1 > 2 || $i3 < [expr $nx - 3])} {
        # bg subtract flag set needs at least 3 bg pts for zero slope line
        # look for zero slope line fit
        set bgl 0
        set bgr 0
        if {$i1 > 2} {
            set ok 0
            set ny 2
            set bgl 0
            while {$ny < $i1} {
                if {![aveFit $yeff $yefferr 0 $ny]} {break}
                set bgl $fitAve
                set bglerr $fitAveErr
                set ok 1
                incr ny
            }
        }
        if {$i3 < [expr $nx - 3]} {
            set ok 0
            set ny [expr $nx - 3]
            set bgr 0
            while {$ny > $i3} {
                if {![aveFit $yeff $yefferr $ny $nx1]} {break}
                set bgr $fitAve
                set bgrerr $fitAveErr
                set ok 1
                incr ny -1
            }
        }
        set bg 0
        if {$bgl > 0 && $bgr > 0} {
            set bg $bgl
            set bgerr $bglerr
            set bgsecs [lindex $t 0]
            if {$bgr < $bgl} {
                set bg $bgr
                set bgerr $bgrerr
                set bgsecs [lindex $t $nx1]
            }
        } elseif {$bgl > 0} {
            set bg $bgl
            set bgerr $bglerr
            set bgsecs [lindex $t 0]
        } elseif {$bgr > 0} {
            set bg $bgr
            set bgerr $bgrerr
            set bgsecs [lindex $t $nx1]
        }
        if {$bg > 0} {
            # this will correct H for background but with the current algorithm
            # we cantdo the error proagation
            # maybe better to just record the background which is near to raw count value
            # and do the error prop after any combine counts
            # first convert ctrate to raw counts by mulitplying by secs at 
            if {$tdif} {
                set bg [expr $bg*$bgsecs]
                set bgerr [expr $bgerr*$bgsecs]
            }
            # scale up the raw counts and bgsecs so that bgerr^2 = bg
            set bgerrsq [expr $bgerr*$bgerr]
            set scl [expr $bgerrsq/$bg]
            set bg $bgerrsq

            set bgsecs [expr $scl*$bgsecs]
            set scanCtBG [list $bg $bgsecs]
            set scanBG 1
        }
    }
    return $nx
}



proc aveFit {y yerr istar iend} {
    global nsigAveFit
    global fitAve fitAveErr
    set ny [llength $y]
    set nreq [expr $iend - $istar + 1]
    if {$ny < $nreq} {return 0}
    set sum 0
    set sumsq 0
    for {set i $istar} {$i <= $iend}{incr i} {
        set yi [lindex $y $i]
        set yierr [lindex $yerr $i]
        set yierrsq [expr $yierr*$yierr]
        set sum [expr $sum + $yi/$yierrsq]
        set sumsq [expr $sumsq + 1./$yierrsq]
    }
    set ave [expr $sum/$sumsq]
    for {set i $istar} {$i <= $iend}{incr i} {
        set yi [lindex $y $i]
        set yierr [lindex $yerr $i]
        if {[expr abs($yi - $ave)] > [expr $nsigAveFit*$yerr]} {return 0}
    }
    set fitAve $ave
    set fitAveErr [expr sqrt(1./$sumsq)]
    return 1
}


proc modeParse {m} {
    global utc
    global VQmode VQ qmodes
    set m [string toupper [string index $m 0]]
    set nm [llength $qmodes]
    if {$m == "Q"} {
        if {$nm < 1 || $VQmode == 0} {lappend qmodes $utc}
        set VQmode 1
        set VQ Q
    } elseif {$m == "V"} {
        if {$nm > 0 && $VQmode == 1} {lappend qmodes $utc}
        set VQmode 0
        set VQ V
    }
}


proc flipParse {s} {
    global utc
    # s should be A=offoff B=ONoff C=offON D=ONON
    global FLIP flipstates flipstate flipstatestr
    global flipsP flipsA
    global flipP flipA
    global flipstateVSFLIP flippersVSFLIP
    if {! [regexp {A|B|C|D} $s] } { return }
	set flipstatestr $flipstateVSFLIP($s)
    lappend flipstates [list $utc $flipstatestr]
    set flipstate $flippersVSFLIP($s)
    set flipP [lindex $flipstate 0]
    set flipA [lindex $flipstate 1]
    set oldflippers $flippersVSFLIP($FLIP)
    set oldflipP [lindex $oldflippers 0]
    set oldflipA [lindex $oldflippers 1]
    # log flips vs utc for Polarizer and Analyzer
    set changed 0
    if {$flipP != $oldflipP} {
         lappend flipsP $utc
         pipeWrite "p cgff 0 $utc"
         set changed 1
    }
    if {$flipA != $oldflipA} {
        lappend flipsA $utc
        pipeWrite "p cgff 1 $utc"
        set changed 1
    }
    set FLIP $s
    return $changed
}

proc initFlipper {PorA} {
    # when a cell gets replaced it polarzation is supposed to be unflipped
    # so reset all the flip data
    global utc
    global FLIP
    global flipP flipA flipstate flipstates flipstatestr
    global flipstateVSindex FLIPVSflipstate
    #puts "initFlipper starting flipstate= $flipstate"
    set pora [string tolower $PorA]
    if {$pora == "p"} {
        set flipP 0
        set flipstate [lreplace $flipstate 0 0 0]
    } elseif {$pora == "a"} {
        set flipA 0
        set flipstate [lreplace $flipstate 1 1 0]
    }
    set flipstatestr $flipstateVSindex($flipP)$flipstateVSindex($flipA)
    set FLIP $FLIPVSflipstate($flipstatestr)
    lappend flipstates [list $utc $flipstate]
}

proc seqParse {seq} {
    global events
    global datestr utc
    set lastevent [lindex $events end]
    set lastvalue [lindex $lastevent end]
    if [string match -nocase *Qmode* $seq] {
        if {$lastvalue != "Q"} {logEvent [list $datestr $utc Q]}
        modeParse Q
    } elseif [string match -nocase *Vmode* $seq] {
        if {$lastvalue != "V"} {logEvent [list $datestr $utc V]}
        modeParse V
    } elseif [string match -nocase *OffOff* $seq] {
        if {$lastvalue != "A"} {logEvent [list $datestr $utc offoff A]}
        flipParse A
    } elseif [string match -nocase *ONOff* $seq] {
        if {$lastvalue != "B"} {logEvent [list $datestr $utc ONoff B]}
        flipParse B
    } elseif [string match -nocase *OffON* $seq] {
        if {$lastvalue != "C"} {logEvent [list $datestr $utc offON C]}
        flipParse C
    } elseif [string match -nocase *ONON* $seq] {
        if {$lastvalue != "D"} {logEvent [list $datestr $utc ONON D]}
        flipParse D
    }
}


proc beamParse {arglst} {
    global events itxt
    global datestr utc
    global beampats beamargs
    global beamsize bt7he3Slit bt7AnaTrans
    global bt7fsh bt7fsw bt7EncToFS
    global bt7bsh bt7bsw bt7monoH
    global sampleH sampleW sampleMosMin sampleMosData sampleMosTran
    global focusV focusH bt7monoH
    global PGfilt VelSel
    global opacity opacityPos
    global horDivPFWHMmin horDivAFWHMmin horDivInPilemin
    global envflg envstr envRanges envSecs envKeys envCurRange envCurRangeStr envReg
    
	set beamargs {}
	set nargs 0
	set envflg 0
	set changed 0
	#puts "beamParse arglst= $arglst"
	foreach a $arglst {
	    if {[string match killed $a] || [string match opacity $a]} {
	        set opacity 1
	        continue
	    }
	    foreach pat $beampats {
		    if [string match $pat $a] {
		        lappend beamargs $a
		        incr nargs
		    }
	        #lappend warnings "invalid beam argument $a at $ap
        }
	}
	if {$nargs < 1} {return 0}
	
	#puts "beamParse beamargs= $beamargs"
	if {$itxt == 0} {
	    logEvent [list $datestr $utc beam $beamargs]
	}
	
	if {[lsearch -regexp $beamargs {1|1cm}] >= 0} {
	    if {$beamsize != 0} {
	        set changed 1
	        set beamsize 0
	        logEvent [list $datestr $utc beamsize 0]
	    }	    
	} elseif {[set icm [lsearch -regexp $beamargs {.+cm}]] >= 0} {
	    set bargstr [lindex $beamargs $icm]
	    if {[scan $bargstr "%fcm" slitsize] == 1} {
	        if {$bt7he3Slit != $slitsize} {
	            set changed 1
	            set bt7he3Slit $slitsize
	            logEvent [list $datestr $utc slitsize $bt7he3Slit]
	        }
	        if {$beamsize != 0} {
	            set changed 1
	            set beamsize 0
	            logEvent [list $datestr $utc beamsize 0]
	        }
	    }
	} elseif {[lsearch -regexp $beamargs {open|full}] >= 0} {
	    if {$beamsize != 1} {
	        set changed 1
	        set beamsize 1
	        logEvent [list $datestr $utc beamsize 1]
	    }
	}
	# opacity positions should be set with the pb po|ao commands so this is really not needed here
	# instead use ana for the analyzer translation setting
	set iana [lsearch -glob $beamargs ana*]
	if {$iana >= 0} {
		set anas [lindex $beamargs $iana]
		set iequ [string first = $anas]
	    if {[scan [string range $anas [expr $iequ + 1] end] "%f" anatrans] > 0} {
	        if {$bt7AnaTrans != $anatrans} {
	            set changed 1
	            set bt7AnaTrans $anatrans
	            logEvent [list $datestr $utc anatrans $bt7AnaTrans]
	        }	        
	    }		
	}
	#if [lsearch -glob $beamargs pol*] {set opacityPos p}
	# thought about allowing Ei spec from meV but this should be already available
	# do want to set sampleH sampleW
	set ih [lsearch -glob $beamargs s*]
	if {$ih >= 0} {
		set sams [lindex $beamargs $ih]
		if {[regexp {(s[^0123456789\.]+)([0123456789\.]+)} $sams wh pre num] == 1} {
			if [string match sh* $pre] {
				set changed 1
				set sampleH $num
				logEvent [list $datestr $utc sampleH $sampleH]
			} elseif [string match sw* $pre] {
				set changed 1
				set sampleW $num
				logEvent [list $datestr $utc sampleW $sampleW]
			} elseif [string match sm* $pre] {
				set changed 1
				if [string match smd* $pre] {
					set sampleMosData $num
					# will set sampleMosMin to this value whenever startPolBeam unless zero is set
					logEvent [list $datestr $utc sampleMosData $sampleMosData]
				} elseif [string match smt* $pre] {
					set sampleMosTran $num
					# will set sampleMosMin to this value whenever stopPolBeam unless zero is set
					logEvent [list $datestr $utc sampleMosTran $sampleH]
				} else {
					set sampleMosMin $num
					logEvent [list $datestr $utc sampleMos $sampleMosMin]
				}
			}
		}
	}

	# velocity selector out=0 in=1 use vs or vel
	set iv [lsearch -glob $beamargs v*]
	if {$iv >= 0} {
	    set vs [lindex $beamargs $iv]
	    if {[scan $vs "vs%d" vi] > 0 || [scan $vs "vs=%d" vi] > 0 || [scan $vs "vel%d" vi] > 0 || [scan $vs "vel=%d" vi] > 0} {
	        if {$VelSel != $vi} {
	            set changed 1
                set VelSel $vi
                logEvent [list $datestr $utc VelSel $VelSel]
            }
	    }
	}

	# front slit. The computer readable front slit getsemoved for polarized beam so any slit values must be
	# entered as annotation in pb beam
	set ih [lsearch -glob $beamargs fsh*]
	if {$ih >= 0} {
	    set hts [lindex $beamargs $ih]
	    if {[scan $hts "fsh%f" ht] > 0 || [scan $hts "fsh=%f" ht] > 0} {
	        if {$bt7fsh != $ht} {
	            set changed 1
                set bt7fsh $ht
                logEvent [list $datestr $utc fsh $bt7fsh]
            }
	    }
	}
	set iw [lsearch -glob $beamargs fsw*]
	if {$iw >= 0} {
	    set wts [lindex $beamargs $iw]
	    if {[scan $wts "fsw%f" wt] > 0 || [scan $wts "fsw=%f" wt] > 0} {
	        if {$bt7fsw != $wt} {
	            set changed 1
                set bt7fsw $wt
                logEvent [list $datestr $utc fsw $bt7fsw]
            }
	    }
	}
	set il [lsearch -glob $beamargs fsl*]
	if {$il >= 0} {
	    set lts [lindex $beamargs $il]
	    if {[scan $lts "fsl%f" lt] > 0 || [scan $lts "fsl=%f" lt] > 0} {
	        if {$bt7EncToFS != $lt} {
	            set changed 1
                set bt7EncToFS $lt
                logEvent [list $datestr $utc fsl $bt7EncToFS]
            }
	    }
	}
	set ih [lsearch -glob $beamargs bsh*]
	if {$ih >= 0} {
	    set hts [lindex $beamargs $ih]
	    if {[scan $hts "bsh%f" ht] > 0 || [scan $hts "bsh=%f" ht] > 0} {
	        if {$bt7bsh != $ht} {
	            set changed 1
                set bt7bsh $ht
                logEvent [list $datestr $utc bsh $bt7fsh]
            }
	    }
	}
	set iw [lsearch -glob $beamargs bsw*]
	if {$iw >= 0} {
	    set wts [lindex $beamargs $iw]
	    if {[scan $wts "bsw%f" wt] > 0 || [scan $wts "bsw=%f" wt] > 0} {
	        if {$bt7bsw != $wt} {
	            set changed 1
                set bt7bsw $wt
                logEvent [list $datestr $utc bsw $bt7fsw]
            }
	    }
	}
	set iv [lsearch -glob $beamargs focus*]
	if {$iv >= 0} {
		puts "parsed pb beam focus [lindex $beamargs $iv]"
	    set wts [lindex $beamargs $iv]
	    if {[scan $wts "focusv%d" wt] > 0 || [scan $wts "focusv=%d" wt] > 0} {
	    	puts "found focusv word"
	        if {$focusV != $wt} {
	            set changed 1
                set focusV $wt
                logEvent [list $datestr $utc focusV $focusV]
                set bt7monoH 20.
                if {$focusV == 0} {set bt7monoH 6.}
            }
	    }
	    if {[scan $wts "focush%d" wt] > 0 || [scan $wts "focush=%d" wt] > 0} {
	        if {$focusH != $wt} {
	            set changed 1
                set focusH $wt
                logEvent [list $datestr $utc focusH $focusH]
            }
	    }
	}
    # handle hcols hcol2 hmos. Vcol are computed in this program assuming 1 cm slit
    set hclst [lsearch -glob -all $beamargs hc*]
    foreach hc $hclst {
    	set hcs [lindex $beamargs $hc]
       	if {[string match hcms* $hcs] || [string match hcsm* $hcs]} {
       		if {[scan $hcs "hcms%f" wd] > 0 || [scan $hcs "hcms=%f" wd] > 0 || [scan $hcs "hcsm%f" wd] > 0 || [scan $hcs "hcsm=%f" wd] > 0} {
       		    if {$horDivPFWHMmin != $wd} {
       		        set changed 1
       			    set horDivPFWHMmin $wd
       			    logEvent [list $datestr $utc hcsm $horDivPFWHMmin]
       		    }
       		}
       	} elseif {[string match hcas* $hcs] || [string match hcsa* $hcs]} {
       		if {[scan $hcs "hcas%f" wd] > 0 || [scan $hcs "hcas=%f" wd] > 0 || [scan $hcs "hcsa%f" wd] > 0 || [scan $hcs "hcsa=%f" wd] > 0} {
       		    if {$horDivAFWHMmin != $wd} {
       		        set changed 1
       			    set horDivAFWHMmin $wd
       			    logEvent [list $datestr $utc hcsa $horDivAFWHMmin]
       		    }
       		}
       	} elseif [string match hc0* $hcs] {
       		# this is annotation to set premonocol which then shouold update beamCond calc horDivFWHMPmin from beamCond
       		if {[scan $hcs "hc0%f" wd] > 0 || [scan $hcs "hc0=%f" wd] > 0} {
       		    if {$horDivInPilemin != $wd} {
       		        # this is not used for beamGeo but may be used to check monitor rates  NO so we just set the global
       			    set horDivInPilemin $wd
       			    logEvent [list $datestr $utc hc0 $horDivInPilemin]
       		    }
       		}
       	}
	}
	# PG filter is normally tracked via filtran.  In case filtran is missing can manually set PGfilter state in pb beam args
	# this is not a geometry change to dont set the change flag
	if {[lsearch $beamargs in] >= 0} {
	    set PGfilt "in"
	    #puts "beamParse logging event PG in"
	    if {$itxt == 0} {
	        logEvent [list $datestr $utc PG in]
	    }
	} elseif {[lsearch $beamargs out] >= 0} {
	    set PGfilt "out"
	    #puts "beamParse logging event PG out"
	    if {$itxt == 0} {
	        logEvent [list $datestr $utc PG out]
	    }
	}
	# handle env specifications. NB this is now handled by findEnv
	set ienv [lsearch -glob $beamargs temp*]
	if {$ienv >= 0} {
	    # define new environment range
	    set envs [lindex $beamargs $ienv]
	    if {[scan $envs "temp%f-%f" xi xf] > 1 || [scan $envs "temp=%f-%f" xi xf] > 1} {
	        set envflg 1
	        set envstr $envs
	        set envRanges($envs) [list $xi $xf]
	        set envSecs($envs) {}
	        if {[lsearch $envKeys temp] < 0} {
	            lappend envKeys temp
	            # the range has been defined but we dont know if current temp falls in this range
	            # so init curRange
	            set envCurRange(temp) [list 0 0]
                set envCurRangeStr(temp) none
	        }
	    }
	}
	return $changed
}

proc valueInRange {val rang} {
    set xi [lindex $rang 0]
    set xf [lindex $rang 1]
    if {$val < $xi || $val > $xf} {return 0}
    return 1
}
proc valueInMinMax {val xi xf} {
    if {$val < $xi || $val > $xf} {return 0}
    return 1
}
proc rangesOverlap {xi xf yi yf} {
    if {$xf < $yi || $xi > $yf} {return 0}
    return 1
}


proc findEnv {api} {
    # look for last temp bfld setting before api in txt
	global envReg
    global envTemp lastTemp lastTempIndex
    global envBfld lastBfld lastBfldIndex
    global txt itxt nextphrase utc

	set nenv 0
	if {$api <= $lastTempIndex} {
		set envTemp $lastTemp
	} else {
    	set ap [findPrevRangeReg $api $lastTempIndex $envReg(temp)]
    	if {! $ap} {
    		set envTemp $lastTemp
    		set lastTempIndex $api
    	} else {
			# must have found a moving temp or temp= at ap
    		set nextphrase [string tolower $nextphrase]
    		if [string match moving* $nextphrase] {
        		set envTemp [lindex $nextphrase 2]
    		} else {
        		scan $nextphrase "temp=%f" envTemp

    		}
    		set lastTemp $envTemp
    		set lastTempIndex [$txt($itxt) index "$ap linestart + 1 lines"]
    		set nenv 1
    	}
    }

	if {$api <= $lastBfldIndex} {
		set envBfld $lastBfld
	} else {
    	set ap [findPrevRangeReg $api $lastBfldIndex $envReg(bfld)]
    	if {! $ap} {
    		set envBfld $lastBfld
    		set lastBfldIndex $api
    	} else {
			# must have found a moving bfld or bfld= at ap
    		set nextphrase [string tolower $nextphrase]
    		if [string match moving* $nextphrase] {
        		set envBfld [lindex $nextphrase 2]
    		} else {
        		scan $nextphrase "bfld=%f" envBfld
    		}
    		set lastBfld $envBfld
    		set lastBfldIndex [$txt($itxt) index "$ap linestart + 1 lines"]
    		incr nenv 2
    	}
    }
	return nenv
}

proc trackEnv {api apf} {
    # look for environment changes in txt(0) range api apf
    # his is replaced by findEnv
    global envKeys envRanges envSecs envCurRange envCurRangeStr
    global envTemp
    global txt itxt nextphrase utc
    foreach key $envKeys {
        # find ranges for key
        set keyranges [array names envRanges $key*]
        if {[llength $keyranges] < 1} {continue}
        set ap $api
        while {$ap <= $apf} {
            set ap [findNextRangeReg $ap $apf $envReg(key)]
            if {! $ap} break
            # currently only handling temp env
            if {$key == "temp"} {
                set nextphrase [string tolower $nextphrase]
                if [string match moving* $nextphrase] {
                    set tval [lindex $nextphrase 2]
                } else {
                    [scan $nextphrase "temp=%f" tval]
                }
                getUTC $ap
                if [valueInRange $tval $envCurRange(temp)] {
                    continue
                } else {
                    # check for exiting current valid env range
                    if [info exists envSecs($envCurRangeStr)] {
                        lappend envSecs($envCurRangeStr) $utc
                    }
                }
                foreach kr $keyranges {
                    if [valueInRange $tval $envRanges($kr)] {
                        lappend envSecs($kr) $utc
                        set envCurRange(temp) $envRanges($kr)
                        set envCurRangeStr $kr
                        break
                    }
                }
            }
            set ap [$txt($itxt) index "$ap linestart + 1 lines"]
        }
    }
}

proc addNocellTrans {pa} {
	global alltransmissions transmissions polstops
    set utclimit 0
    if {[llength $polstops] > 0} {set utclimit [lindex $polstops end]}
	set nt [llength $transmissions]
	# get the utc from the other of pa or mt which which should have been measured

    for {set i [expr $nt - 1]} {$i >= 0} {incr i -1} {
        set t [lindex $transmissions $i]
        set typ [lindex $t 2]
        if {$typ == $pa} {continue}
        set cel [lindex $t 4]
        set newtrans $t
        set newtrans [lreplace $newtrans 2 2 $pa]
        set newtrans [lreplace $newtrans 3 3 {}]
        set newtrans [lreplace $newtrans 4 4 "nocell"]
        lappend transmissions $newtrans
        return 1
    }
    puts "addNocellTrans pa= $pa failed to find a non $pa transmission in transmissions"
    return 0
}

proc startPolBeam {apline} {
    global datestr utc
    global forceinstall
    global npairs pairs pairsGeom pairsinstalls
    global ninstalls installs npolarizers polarizers nanalyzers analyzers
    global cellNames
    global sampleMosMin sampleMosData
    global transSeg transmissions polstops
    global readlines nread
    global warnings

    # transmissions are for last transmission block before startPolBeam
    #set transev [list $datestr $utc $logcmd $ctsList $cellName beam=$beamsize ei=$Ei]
    # logEvent [list $datestr $utc install=$ninstalls $cmd $cellName]


	if {$sampleMosData > 0} {
		if {$sampleMosData != $sampleMosMin} {
			set sampleMosMin $sampleMosData
			logEvent [list $datestr $utc sampleMos $sampleMosMin]
		}
	}
		    
    set new 0
    set utclimit 0
    if {[llength $polstops] > 0} {set utclimit [lindex $polstops end]}
    #puts "startPolBeam p=$cellNames(p) a=$cellNames(a) pairs= $pairs"
    # put both cells in beam
    pipeWrite "p cgb 3"
    if {$npairs == 0} {
        lappend pairs [list $datestr $utc $cellNames(p) $cellNames(a)]
        # currentGem returns a list with two lists, one list for polarizer and analyzer of the cellkey value pairs.
        # We put this currentGeom list inside another list in case the geometry changes during the polarized beam block.
        # log the nfiles counter (0 when we startPolBeam) and the utc as well
        lappend pairsGeom [list [list 0 $utc [currentGeom]]]
        set new 1
        set npairs 1
    } else {
        set lastpair [lindex $pairs end]
        set lastp [lindex $lastpair 2]
        set lasta [lindex $lastpair 3]
        set nt [llength $transmissions]
        if {$lastp != $cellNames(p) || $lasta != $cellNames(a) || $transSeg == 1} {
            lappend pairs [list $datestr $utc $cellNames(p) $cellNames(a)]
            lappend pairsGeom [list [list 0 $utc [currentGeom]]]
            set new 1
            incr npairs
        }
        # warn if lastp lasta transmissions were not taken
        set tpok 0
        set taok 0
        for {set i [expr $nt - 1]} {$i >= 0} {incr i -1} {
            set t [lindex $transmissions $i]
            set utci [lindex $t 1]
            if {$utci < $utclimit} {break}
            set typ [lindex $t 2]
            set cel [lindex $t 4]
            if {$tpok == 0 && $typ == "p" && $cel == $lastp} {
                set tpok 1
            }
            if {$taok == 0 && $typ == "a" && $cel == $lasta} {
                set taok 1
            }
            if {$tpok == 1 && $taok == 1} {break}
        }
        if {$tpok == 0} {
        	# if lastp is nocell just add a unity fixup transmission event for it
        	if {$lastp == "nocell"} {
        		if {! [addNocellTrans p] } {
        			lappend warnings "WARNING: no secondary trans for last polarizer= $lastp\n$apline"
        		} else {
        			# log the added nocell transmission
        			logEvent [lindex $transmissions end]
        		}
        	} else {
            	if {$transSeg == 1} {
                	lappend warnings "WARNING: 2-point transmissions requested but missing trans for last polarizer= $lastp\n$apline"
            	} else {
                	lappend warnings "WARNING: no follow on transmission made for last polarizer= $lastp\n$apline"
            	}
        	}
        }
        if {$taok == 0} {
        	if {$lasta == "nocell"} {
        		if {! [addNocellTrans a] } {
					lappend warnings "WARNING: no secondary trans for last analyzer= $lastp\n$apline"        		
        		}  else {
        			# log the added nocell transmission
        			logEvent [lindex $transmissions end]
        		}
        	} else {
            	if {$transSeg == 1} {
                	lappend warnings "WARNING: 2-point transmissions requested but missing trans for last analyzer= $lasta\n$apline"
            	} else {
                	lappend warnings "WARNING: no follow on transmission made for last analyzer= $lasta\n$apline"
            	}
        	}
        }
        # also if the same cell is used
        # relog the transmission as an install if transSeg is set
        # and last same cell wasnt a force install ai pi
        if {$transSeg == 1 && ($lastp == $cellNames(p) || $lasta == $cellNames(a))} {
            #puts "checking $lastp == $cellNames(p)  $lasta == $cellNames(a)"
            
            if {$lastp == $cellNames(p)} {
                #puts "reusing polarizer"
                # find the recent transmission for polarizer $cellNames(p)
                for {set i [expr $nt - 1]} {$i >= 0} {incr i -1} {
                    set t [lindex $transmissions $i]
                    set utci [lindex $t 1]
                    if {$utci < $utclimit} {break}
                    set typ [lindex $t 2]
                    set cel [lindex $t 4]
                    if {$typ == "p" && $cel == $lastp} {
                        set t [lreplace $t 2 2 pi]
                        #set t [lreplace $t 0 0 $datestr]
                        set dsi [lindex $t 0]
                        set utci [lindex $t 1]
                        #puts "startPolBeam logging duplicate polarizer= $t"
                        logEvent $t
                        #set transev [list $datestr $utc $logcmd $ctsList $cellName beam=$beamsize ei=$Ei]
                        # logEvent [list $datestr $utc install=$ninstalls $cmd $cellName]
			            incr npolarizers
			            lappend polarizers [list $utci $cellNames(p) $ninstalls]
			            incr ninstalls
			            lappend installs [list $dsi $utci $typ $cellNames(p)]
			            logEvent [list $dsi $utci install=$ninstalls $typ $cellNames(p)]
                        break
                    }
                }
            }
            if {$lasta == $cellNames(a)} {
                #puts "reusing analyzer"
                # find the recent transmission for polarizer $cellNames(p)
                for {set i [expr $nt - 1]} {$i >= 0} {incr i -1} {
                    set t [lindex $transmissions $i]
                    set utci [lindex $t 1]
                    if {$utci < $utclimit} {break}
                    set typ [lindex $t 2]
                    set cel [lindex $t 4]
                    #puts "checking $typ == a and $cel == $lasta"
                    if {$typ == "a" && $cel == $lasta} {
                        set t [lreplace $t 2 2 ai]
                        set dsi [lindex $t 0]
                        set utci [lindex $t 1]
                        #set t [lreplace $t 0 0 $datestr]
                        #set t [lreplace $t 1 1 $utc]
                        #puts "startPolBeam logging duplicate analyzer= $t"
                        logEvent $t
			            incr nanalyzers
			            lappend analyzers [list $utci $cellNames(a) $ninstalls]
			            incr ninstalls
			            lappend installs [list $dsi $utci $typ $cellNames(a)]
			            logEvent [list $dsi $utci install=$ninstalls $typ $cellNames(a)]
                        break
                    }
                }
            }
        }
    }
    # update pairsinstalls by looking back through current installs to find most recent p and a install
    set pok 0
    set aok 0
    for {set i [expr $ninstalls - 1]} {$i >= 0} {incr i -1} {
        set inst [lindex $installs $i]
        set typ [string index [lindex $inst 2] 0]
        if {$aok == 0 && $typ == "a"} {
            set aok 1
            set ain $i
        } elseif {$pok == 0 && $typ == "p"} {
            set pok 1
            set pin $i
        }
        if {$aok == 1 && $pok == 1} {break}
    }
    if {$aok == 0} {
        Dialog_Warn "ERROR: no installed analyzers"
        return
    }
    if {$pok == 0} {
        Dialog_Warn "ERROR: no installed polarizers"
        return
    }
    lappend pairsinstalls [list $pin $ain]
    
    if {$new} {
        #puts "startPolBeam: new cell so add new pair"
        logEvent [list $datestr $utc cellpair=$npairs $cellNames(p) $cellNames(a)]
        #pipeWrite "p cgp"
        #pipeWrite "p cgpp"
    }
}

proc stopPolBeam {} {
    global nfiles
    global polstops
    global datestr utc
    global sampleH sampleW
    global sampleMosMin sampleMosTran
    global transmodeStartSecs
    global nfiles nflipratios
    global pairsfiles pairsflipratios
    
	if {$sampleMosTran > 0} {
		if {$sampleMosTran != $sampleMosMin} {
			set sampleMosMin $sampleMosTran
			logEvent [list $datestr $utc sampleMos $sampleMosMin]
		}
	}
    lappend pairsfiles $nfiles
    lappend pairsflipratios $nflipratios
    set nfiles 0
    set nflipratios 0
    logEvent [list $datestr $utc "stop" "polarized beam"]
    lappend polstops $utc
    logEvent [list $datestr $utc sampleHW $sampleH $sampleW]
}


proc updatePairsGeom {} {
	# handles beam geometry changes during cell-pair block
    global npairs pairsGeom nfiles utc
    # pairsGeom has to have at least one element when this is called since polbeam=1
    set pgeom [lindex $pairsGeom end]
    set pgn [lindex $pgeom end]
    set nfil [lindex $pgn 0]
    if {$nfiles > $nfil} {
        # there were data files since last geom log this polrun so add another geom entry
        # NB flipping ratios get processed with current geometry so we only need to
        # consider what is needed to analyze data files using cellfile
        lappend pgeom [list $nfiles $utc [currentGeom]]
        set pairsGeom [lreplace $pairsGeom end end $pgeom]
    } else {
    	# with no files in previous geom replace it with the current
    	set pgn [list $nfiles $utc [currentGeom]]
    	set pgeom [lreplace $pgeom end end $pgn]
    	set pairsGeom [lreplace $pairsGeom end end $pgeom]
    }
}

proc updatePreMonoColl {newColl} {
	global datestr utc
	global incBeamCond
	global premonocoll
	set premonocoll $newColl
	set lastColl 0
    if {[llength $incBeamCond] > 0} {
    	set lastIB [lindex $incBeamCond end]
        if {[llength $lastIB] > 2} {
        	set lastIBC [lindex $lastIB 2]
            set lastIBCl [split $lastIBC -]
            set lastColl [lindex $lastIBCl 0]
        }
        puts $fp "lastColl= $lastColl"
        if { $lastColl != $newColl } {
            set lastIBCl [lreplace $lastIBCl 0 0 $newColl]
            set lastIBC [join $lastIBCl -]
            set lastIB [lreplace $lastIB 2 2 $lastIBC]
            set lastIB [lreplace $lastIB 0 0 $utc]
            set lastIB [lreplace $lastIB 1 1 $Ei]
            lappend incBeamCond $lastIB
            logWARNING "beam condition change to $lastIBC $datestr\n"
	    	logEvent [list $datestr $utc incBeam $lastIBC]
	    	# update the FWHMmin
        }
	}
	logEvent [list $datestr $utc beam premonocoll $newColl]
}
                

proc annotbt7 {} {
    global txt nb conB
    global counters endCount endFR
    global ctsList
    global events
    global beamCell
    global cellNames
    global PGfilt Ei Ef bt7bsh bt7bsw openPMmin
    global VelSel
    global globalZero
    global globalOne globalThree
    global events
    global incBeamCond
    global beampats beamargs beamsize polbeam opacity opacityPos sampleH sampleW focusV focusH
    global nextphrase
    global bt7annotPhrases
    global bt7annotRegex
    global datestr
    global utc
    global fbgcpm
    global VQ FLIP flipstatestr frupdate flips
    global flipstateVSFLIP
    global pbfiles
    global nit maxit niterror
    global nfiles ntotalfiles nflipratios
    global maxTransModeSecs
    global ctrateRejectionCI
    global ninstalls installs npolarizers polarizers nanalyzers analyzers npairs pairs pairsGeom pairsfiles pairsflipratios pairsinstalls transmissions polstops
    global alltransmissions
    global transSeg transmodeStartSecs forceinstall
    global envflg envstr envSecs envKeys envCurRange
    global envTemp envBfld
    global warnings
    
  	puts "annotbt7 starting"
    if {[llength $counters] < 1} {lappend counters "SDC Total:"}
    set ap 1.0
	set endtxt [$txt(0) index "end linestart - 1 lines"]
    #puts "annotbt7 loop starting with endtxt= $endtxt"
	if {$endtxt < 10} {return 0}
	set niterror 10000
    set maxit 10000
    set nit 0
    set warnings {}
    set transmode 0
    set transmodeStartSecs 0

    set ncellInstalls 0
    set cellInstalls {}
    # track polarizers
    set npolarizers 0
    set polarizers {}
    set nanalyzers 0
    set analyzers {}
    # track cell-pairs
    set npairs 0
    
    set pairs {}
    set pairsfiles {}
    set pairsflipratios {}
    
    set pairsinstalls {}
    set pairsGeom {}
    # record transissions
    set alltransmissions {}
    set transmissions {}
    set polstops {}
    # track flipratio utcs
    set flips {}
    set blockflips {}
    set flipratios {}
	puts "ready to start loop"
	if {[file exists parse.log]} {
		set fp [open parse.log w]
		close $fp
	}
	set fp [open parse.log a]
	
	# if you do not want the file log set fp to stdout
	#return
    while {$ap < $endtxt && $nit < $maxit} {
        #puts "annotbt7 calling findNextReg bt7annotRegex from ap= $ap"
        #if $nit >= $niterror  break 
        set apold $ap
        set ap [findNextReg $ap $bt7annotRegex]
        #if $ap < 500.
	    puts "findNextReg nit=$nit ap= $ap nextphrase= $nextphrase"
	    #
        if {$nit >= $niterror} {
        	set apline [$txt(0) get "$ap linestart" "$ap lineend"]
        	puts "nit exit at ap=$ap apline= $apline"
        	puts "exiting annot bt7 loop for niterror at nit= $nit"
         	break
        }
        
        if {! $ap} break
        # do env tracking in the block we just passed by
        # this is changed to use findEnv whenever an fr is parsed
        #trackEnv $apold $ap
        
        set aptxt [$txt(0) get $ap "$ap lineend"]
        set aps [$txt(0) index "$ap linestart"]
        set apline [$txt(0) get $aps "$aps lineend"]
        if {$nit == [expr $niterror - 1]} {
        	puts "just before niterror= $niterror  ap= $ap apline= $apline"
        	puts "exiting annot bt7 loop for niterror at nit= $nit"
        	break
        }
        set utc [getUTC $aps]
        set apnext [$txt(0) index "$ap linestart + 1 lines"]
        
        puts $fp "datestr= $datestr nextphrase= $nextphrase"
        #puts "annotbt7 findNextReg returned ap= $ap  nextphrase= $nextphrase  utc= $utc  aptxt= $aptxt"
        # first handle polbeam seq cmnds
        
        # RWE Jan 2020 should not parse seq
        # if [string match *\.seq* $nextphrase]
            #puts "annotbt7 calling seqParse"
        #   seqParse $nextphrase
        #   set ap $apnext
        #    incr nit
        #    continue
        
        # added look for moving flip  and moving hsample to set flipstate and mode outside a sequence
        # this happens when user pastes the standard pb sequence contents into their sequence.
        if {[string match -nocase "moving flip" $nextphrase]} {
            puts "annotbt7 found moving flip at $aptxt"
            #puts "moving flip at $ap line=\n$apline\nsetting apnext= $apnext"
            # the offON script had the move FLIP C after the nmr cmnds. This has been fixed but need to handle the case
            set nmri [$txt(0) index "$ap linestart + 2 lines"]
            set nmrln [$txt(0) get $nmri "$nmri lineend"]
            set flipOK 0
            if {[string match "*3He polarizer state*" $nmrln] || [string match "*3He analyzer state*" $nmrln]} {
            	set FLIPs [lindex $aptxt 2]
            	set ch [flipParse $FLIPs]
            	set flipOK 1
            } else {
            	set nmri [$txt(0) index "$ap linestart - 1 lines"]
            	set nmrln [$txt(0) get $nmri "$nmri lineend"]
            	if {[string match "*3He polarizer state*" $nmrln] || [string match "*3He analyzer state*" $nmrln]} {
            		set FLIPs [lindex $aptxt 2]
            		set ch [flipParse $FLIPs]
            		set flipOK 1
            	} else {
            		lappend warnings "WARNING: moving FLIP without nmr commands ignores flipstate change at\n$apline"
            	}
            }
            if {$flipOK == 1} {
            	# flipParse will set FLIP so we do not really have to do that
            	#set FLIP $FLIPs
            	#lappend flips [list $utc $FLIP]
            	#lappend blockflips [list $utc $FLIP]
            	if {$ch == 1} {logEvent [list $datestr $utc FLIP $flipstateVSFLIP($FLIPs) $FLIPs]}
            }
            #puts "moved FLIP $datestr $utc FLIP= $FLIP"
            set ap $apnext
            incr nit
            continue
        }
        if {[string match -nocase "*moving hsample*" $nextphrase]} {
            #puts "annotbt7 calling seqParse"
            set nmri [$txt(0) index "$ap linestart + 2 lines"]
            set nmrln [$txt(0) get $nmri "$nmri lineend"]
            if {[string match -nocase "*Moving Efcancel*" $nmrln]} {
            	set amps [lindex $aptxt 2]
            	set mods Qmode
            	if {$amps == 0} {set mods Vmode}
            	modeParse $mods
            	logEvent [list $datestr $utc $VQ]
            } else {
            	lappend warnings "WARNING: moving Hsample without moving efcancel ignores mode change at\n$apline"
            }
            set ap $apnext
            incr nit
            continue
        }
        # look for PG filtran in/out
        if {[string match -nocase "*moving filtran*" $nextphrase]} {
            set aplst [split $aptxt]
            set pgpos [string tolower [lindex $aplst 2]]
            #puts "annotbt7 filtran pgpos= $pgpos  PGfilt= $PGfilt"
            if {$pgpos != $PGfilt} {
                #puts "annotbt7 logging event PG $pgpos"
                logEvent [list $datestr $utc PG $pgpos]
            }
            set PGfilt $pgpos
            set ap $apnext
            incr nit
            continue
        }
        # look for moving premonocol ei ef backslitheight

        if {[string match -nocase "*moving premonocoll*" $nextphrase]} {
        	puts $fp "found moving premonocoll"
            if {[llength $aptxt] < 3 } {
                lappend warnings "Failed to parse moving PreMonoColl at\n$apline"
            } else {
            	set newColl [lindex $aptxt 2]
            	set newCollOK 0
            	puts $fp "premonocoll newColl is $newColl"
            	if {[expr {$newColl ne "OPEN"}]} {
            		if {![scan $newColl "%dMIN" minColl]} {
            			lappend warnings "Failed to parse moving PreMonoColl at\n$apline"
            		} else {
            			set newColl $minColl
            			set newCollOK 1
            		}
            	} else {
            		set newColl $openPMmin
            		set newCollOK 1
            	}
            	# NB incBeam is determined from monitor rates and Ei so knowing preMonoColl will for now only serve as a check
            	# just display it on the log
            	# get the last beam condition
            	puts $fp "newCollOK= $newCollOK"
            	if {$newCollOK == 1} {
            		set lastColl 0
            		if {[llength $incBeamCond] > 0} {
            			set lastIB [lindex $incBeamCond end]
            			if {[llength $lastIB] > 2} {
            				set lastIBC [lindex $lastIB 2]
            				set lastIBCl [split $lastIBC -]
            				set lastColl [lindex $lastIBCl 0]
            			}
            			puts $fp "lastColl= $lastColl"
            			if { $lastColl != $newColl } {
            				set lastIBCl [lreplace $lastIBCl 0 0 $newColl]
            				set lastIBC [join $lastIBCl -]
            				set lastIB [lreplace $lastIB 2 2 $lastIBC]
            				set lastIB [lreplace $lastIB 0 0 $utc]
            				set lastIB [lreplace $lastIB 1 1 $Ei]
            				lappend incBeamCond $lastIB
            				logWARNING "beam condition change to $lastIBC $datestr\n"
	    					logEvent [list $datestr $utc incBeam $lastIBC]
	    					# update the FWHMmin
            			}
            		}
            	}
                logEvent [list $datestr $utc "beam PreMonoColl" $newColl]
            }
            set ap $apnext
            incr nit
            continue
        }
        if {[string match -nocase "moving ei *" $nextphrase]} {
            if {[llength $aptxt] < 3 || [scan [lindex $aptxt 2] "%g" Eichk] < 1} {
                lappend warnings "Failed to parse moving Ei at\n$apline"
            } else {
                if {$Eichk != $Ei} {
                    set Ei $Eichk
                    logEvent [list $datestr $utc ei $Ei]
                }
            }
            set ap $apnext
            incr nit
            continue
        }
        if {[string match -nocase "moving ef *" $nextphrase]} {
            if {[llength $aptxt] < 3 || [scan [lindex $aptxt 2] "%g" Efchk] < 1} {
                lappend warnings "Failed to parse moving Ef at\n$apline"
            } else {
                if {$Efchk != $Ef} {
                    set Ef $Efchk
                    logEvent [list $datestr $utc ef $Ef]
                }
            }
            set ap $apnext
            incr nit
            continue
        }
        if {[string match -nocase "moving back*" $nextphrase]} {
            # this is the only beam geometry change that we can read from ICE cmnd
            # if geometry changes during polbeam mode=1 and nfiles
        	#puts "found moving back aptxt= $aptxt"
        	set changed 0
            if {[llength $aptxt] < 3 || [scan [lindex $aptxt 2] "%g" bt7bshChk] < 1} {
                lappend warnings "Failed to parse moving backslit at\n$apline"
            } else {
                set bt7bshChk [expr $bt7bshChk/10.]
                if [string match -nocase *height [lindex $aptxt 1]] {
                	if {$bt7bshChk != $bt7bsh} {
                	    set changed 1
                    	set bt7bsh $bt7bshChk
                    	logEvent [list $datestr $utc bsh $bt7bsh]
                	}
                }
                if [string match -nocase *width [lindex $aptxt 1]] {
                	if {$bt7bshChk != $bt7bsw} {
                	    set changed 1
                    	set bt7bsw $bt7bshChk
                    	logEvent [list $datestr $utc bsw $bt7bsw]
                	}
                }
            }
            if {$polbeam == 1 && $changed == 1} {
                # get currentPair geom
                updatePairsGeom
            }
            set ap $apnext
            incr nit
            continue
        }
		if {[string match -nocase "horizontal focus*" $nextphrase]} {
    		set foctxt [$txt(0) get "$ap - 4 chars wordstart" "$ap lineend"]
    		parseHfocus $foctxt
    		logEvent [list $datestr $utc focusH $focusH]
    		set ap $apnext
            incr nit
            continue
		}
		if {[string match -nocase "vertical focus*" $nextphrase]} {
    		set foctxt [$txt(0) get "$ap - 4 chars wordstart" "$ap lineend"]
    		parseVfocus $foctxt
    		logEvent [list $datestr $utc focusV $focusV]
    		set ap $apnext
            incr nit
            continue
		}
		if {[string match -nocase "setproperty Ei velselcoupled" $nextphrase]} {
			#puts "bt7annot found setproperty Ei velselcoupled at\n$apline with aptxt= $aptxt"
			parseVelSel $aptxt
    		set ap $apnext
            incr nit
            continue
		}
        # look for Creating file  to start scans and record the file rootname and monitorVStime info
        # if the polbeam flag is set
        set fpk 0
        if {$polbeam == 1 && ([string match "START OF SCAN" $nextphrase] || [set fpk [string match "Find Peak" $nextphrase]])} {
            puts "annotbt7 START OF SCAN try parseScan"
            parseScan $ap 0 $fpk
            set ap $apnext
            incr nit
            incr ntotalfiles
            incr nfiles
            puts "parseScan END set ap= $apnext  nit= $nit"
            continue
        }
        
        # all thats left are pb annot cmnds. handle the pb cmnds
        
        # get rid of the leading pb part to isolate the cmd and args
        # also skip any pb that has a following right-square-bracket
        # also skip any pbx post excluded commands
        # pbx is used to remove pb commands without deleting them from the console. For example if a transmission is done incorrectly.
        if {[string match *\]* $aptxt] || [string match -nocase pbx* $aptxt] || ([regexp -nocase {pb\s+(.+)$} $aptxt whole cmdargs] == 0)} {
            #lappend warnings "Failed to parse $aptxt at $ap"
            set ap $apnext
            incr nit
            continue
        }
        set aplst [split $cmdargs]
        set nw [llength $aplst]

        if {$nw < 1} {
            set ap $apnext
            incr nit
            continue
        }
		# there is one command which we dont want to recase that is the global parameter setter: pb gp globalName value
		set cmd [lindex $aplst 0]
		if {$cmd == "gp" || $cmd == "GP"} {
			if {$nw < 3} {
				set ap $apnext
				continue
			}
			set gpName [lindex $aplst 1]
			set gpValu [lindex $aplst 2]
			global $gpName
			# just catch the set global cmd. User can check that it gets updated in the params gui
			if {! [catch {set $gpName $gpValu}] } {
				logWARNING "set global param $gpName to $gpValu\n"
			}
			set ap $apnext
			continue
		}
        # convert all cmnds to lower case
        # convert all to lower case
        set nap 0
        set aplstL {}
        for {set i 0} {$i < $nw} {incr i} {
            set apl [lindex $aplst $i]
            if {[string length $apl] < 1} {continue}
            lappend aplstL [string tolower $apl]
        }
        set aplst $aplstL
        set nw [llength $aplst]
        set cmd [lindex $aplst 0]
        set arglst {}
        set argstr ""
        if {$nw > 1} {
            set arglst [lrange $aplst 1 end]
            #puts "arglst= $arglst"
            set argstr [join $arglst]
        }
		# for transmissions P A MT need to parse for cts
		# the way to annotate opacity is with ao or po.
		# o alone doesnt spec the opacityPos so then try to get pol or ana from args
		# also just p or a with killed or opacity in the args will do
		# There are several special cases when doing transmissions
		# If there is a long wait between transmissions in a transmission block
		# make sure to insert an empty fr block start-stop so that transmissions are not combined
		# and the delayed ones are treated as follow on.
		# Using the same cell in the same position consecutively is supposed to be follow on
		# If instead the same cell has been repolarized and used again in the same position
		# the annotator should mark this measurment with i for new install.
		# If a transmission measurment is in error because of eg PG out condition, mark
		# the next correct transmission as a restart with r. This will remove previous trans measurements
		# in the trans block for that cell/pos This must also be done in Tclpbs celldata
		set forceinstall 0
		if [regexp -nocase {^p.*|^a.*|^m.*} $cmd] {
		    puts "FOUND transmission cmd= $cmd at $datestr"
		    #if $nit >= $niterror break
		    if [string match *x $cmd] {
		    	# this is post processing logfile to eliminate incorrectly taken trans eg filter IN
		    	# supposed to be caught above
		    	set ap $apnext
            	incr nit
            	continue
		    }
		    # force install even if same cell is used will not use the current trans
		    # as follow on for previous trans. This is usually if same cell was repolarized
		    set forceinstall 0
		    if [string match *i $cmd] {
		        set forceinstall 1
		    }
		    set opacity 0
		    if [string match *o* $cmd] {
		        set opacity 1
		        if [string match *p* $cmd] {
		            set opacityPos p
		        }
		        if [string match *a* $cmd] {
		            set opacityPos a
		        }
		    }
			if {$polbeam == 1} {
				# turn off polbeam mode to measure transmissions
		        set polbeam 0
		        # start the transmission block timer
		        logEvent [list $datestr $utc files $nfiles]
		        logEvent [list $datestr $utc blockflips [llength $blockflips]]
		        set blockflips {}
		        # empty the transmissions list so it records only transmissions for this block
		        if {[llength $transmissions] > 0} {lappend alltransmissions $transmissions}
		        set transmissions {}
		        # reset the file counter for next cell pair
		        stopPolBeam
		        set transmodeStartSecs $utc

		    } else {
		        # already in transmode polbeam=0 (or at beginning of parsing)
		        # if transmodeStartSecs is already set > 0 make sure this new transmission msrment
		        # is withing time limit. If it is out of limit restart and stop polarized beam to indicate
		        # block change and then reset for transmission
		        #puts "trans cmnd in polbeam=0 transmodeStartSecs= $transmodeStartSecs utc= $utc  maxTransSecs= $maxTransModeSecs"
		        # RWE Feb14 2020 think timelimit for transmission should only apply to remeasuring same cell
		        # in any case should try to determine cell install time. Update cell install time after each cell count block
		        # Original logic that exceeding time limit requires trans restart, should be only to measure same cell.
		        set checkTimeLimit 0
		        if {$checkTimeLimit > 0 && $transmodeStartSecs > 0} {
		            if {[expr $utc - $transmodeStartSecs] > $maxTransModeSecs} {
		                # start a new transblock
		                startPolBeam $apline
		                logEvent [list $datestr $utc "start" "polarized beam"]
		                logEvent [list $datestr $utc files 0]
						logEvent [list $datestr $utc blockflips 0]
						set blockflips {}
		                stopPolBeam
		                set transmodeStartSecs $utc
		                # also restart transmissions list
		        		if {[llength $transmissions] > 0} {lappend alltransmissions $transmissions}
		                set transmissions {}
                        lappend warnings "WARNING: transmission out of time limit range. Restarting transmissions.\n$apline"
                    }
		        } else {
		            set transmodeStartSecs $utc
		        }
		    }
		    #$txt(5) insert end "transmission block start line:\n"
		    #$txt(5) insert end "$apline\n"
		    set beamCell [string index $cmd 0]
		    #puts $apline
		    #puts "cmd= $cmd beamCell= $beamCell"
		    # if cellname not entered we will use last if available
		    set cellName ""
		    if [info exists cellNames($beamCell)] {
		        #puts "before mt check cellNames($cmd)= $cellNames($cmd)"
		        set cellName $cellNames($beamCell)
		    }
		    set samecell 0
            set newinstall 0
		    if {![string match m $beamCell] && $nw > 1} {
		        set cellName [lindex $arglst 0]
		        # lookupCell returns the lowercase name
		        set chklst [lookupCell $cellName]
		        if {[lindex $chklst 0] == 0} {
		            lappend warnings "WARNING: cellname $cellName is NOT in the cell database"
		        } else {
		            set cellName [lindex $chklst 1]
		        }
		        #puts "empty check cellName= $cellName"
		        # init flipstate for any new cell
		        # when p or a cellName changes this indicates newinstall
		        # NB this implies that the old cell transmission is always measured before the new cell
		        # transmission. I think this must always be the case.
                if {$cellName == $cellNames($beamCell)} {set samecell 1}
		        if {$cellName != $cellNames($beamCell) || $forceinstall == 1} {
		            #puts "cellName= $cellName prevcellName= $cellNames($beamCell) forceinstall= $forceinstall"
		            initFlipper $beamCell
		            # record a new cell install
		            set newinstall 1
		            #puts "set newinstall= $newinstall"
		        }
		        set cellNames($beamCell) $cellName
		        # remove cellname from arg list
		        set arglst [lreplace $arglst 0 0]
		    }
		    # in case user logs the beam condition with the pb cell cmnd, parse the remaining args as beam
		    # killed is allowed in beamargs if the opacity flag is set change cmd to po or ao
		    beamParse $arglst
		    if {$PGfilt == "out"} {
		        lappend warnings "WARNING: transmission $cmd with PG out at $ap\:\n$apline"
		    }
		    #puts "PASSED beamParse arglst= $arglst opacity= $opacity"
		    #if $nit >= $niterror break
		    set logcmd $cmd
		    if {$opacity == 1} {
		        # for opacity we want to log either ao or po as the command
		        set logcmd ao
		        if {$opacityPos == "p"} {set logcmd "po"}
		    }
		    # there is nothing else to do for mt since cellNames and beamCell condition are separate
		    #puts "annotbt7 calling parsCounts"
		    # dont count the nocell
		    set err 0
		    set ctsList {}
		    if {$cellName != "nocell"} {
				set err [parseCounts $ap 0]
			}
			#puts "PASSED parsCounts ap= $ap  err= $err"
			#if $nit >= $niterror break
			if {$err < 0} {
			    set err [expr -$err]
			    lappend warnings "ERROR: parseCounts error at $err\:\n$apline"
			    #puts "parsCounts ERROR"
			} else {
                set transev [list $datestr $utc $logcmd $ctsList $cellName beam=$beamsize ei=$Ei]
			    if {$opacity != 1} {
			        #puts "opacity NOT 1  newinstall= $newinstall"
			        #if $samecell == 1 && $forceinstall != 1 && $transSeg == 1
			            # handle segmented follow on protocol when we parseHe3Log and have all the trans together
			            # log a duplicate for the followon transmission
			            # making sure to augment the install cmd with the i=install so
			            # the duplicate wont get combined
			            #logEvent list $datestr $utc $cmd $ctsList $cellName beam=$beamsize ei=$Ei
			            # then force a new install
			            #set newinstall 2
			            # add install index which appears in pairsinstalls to polarizers analyzers
			        if {$newinstall > 0} {
			            if {[string index $cmd 0] == "p"} {
			                incr npolarizers
			                lappend polarizers [list $utc $cellName $ninstalls]
			            } elseif {[string index $cmd 0] == "a"} {
			                incr nanalyzers
			                lappend analyzers [list $utc $cellName $ninstalls]
			            }
			            incr ninstalls
			            lappend installs [list $datestr $utc $cmd $cellName]
			            # here we make sure the next segment start for samecell (newinstall==2) has i in cmd
			            #if $newinstall == 2 && ![string match *i $cmd] set cmd $cmd\i
			            #puts "logging install=$ninstalls for $cellName"
			            logEvent [list $datestr $utc install=$ninstalls $cmd $cellName]
                    }
			        lappend transmissions $transev
			    }
			    logEvent $transev
			}
			# make sure we dont end up in infinite loop. Always advance the line index
			if {$endCount > $apnext} {
			    set apnext $endCount
			}
		} elseif {$cmd == "end" || $cmd == "stop" || $cmd == "pause"} {
		    # stop polarized beam. This is usually obviated by transmission cmnds
		    # but in case eg we take cells offline
		    # use pause if not changing cells
		    set beamCell mt
		    if {$polbeam == 1} {
		        set polbeam 0
		        if {$cmd != "pause"} {
		        	set transmodeStartSecs 0
		        	logEvent [list $datestr $utc files $nfiles]
		        	logEvent [list $datestr $utc blockflips [llength $blockflips]]
		        	set blockflips {}
		        	# reset the file counter for next cell pair
		        	stopPolBeam
		        }
		    }
		} elseif [string match fbg* $cmd] {
		    set fbgcpm [lindex $argstr 0]
		    logEvent [list $datestr $utc fbgcpm $fpbcpm]
		} elseif {[string match bo* $cmd] || $cmd == "begin" || $cmd == "start"} {
			# start polarized beam
		    set beamCell both
		    logEvent [list $datestr $utc both]
		    if {$polbeam == 0} {
		        set polbeam 1
		        set transmodeStartSecs 0
		        startPolBeam $apline
    		    logEvent [list $datestr $utc "start" "polarized beam"]
    		    set nfiles 0
    		}
		} elseif {[lsearch $globalZero $cmd] >= 0} {
		    # globalZero are zero arg cmds flipstate V Q
		    # flipstate and V Q are obtained from seq parsing
		    logEvent [list $datestr $utc $cmd]
		} elseif {[lsearch $globalOne $cmd] >= 0} {
		    # these have single float args pb ei  pb lamb  pb 
		    if {$nw < 2 || [scan $argstr "%g" fval] < 1} {
		        lappend warnings "ERROR: $cmd requires float arg at $ap\:\n$apline"
		    } else {
		        logEvent [list $datestr $utc $cmd $fval]
		    }
		} elseif {[lsearch $globalThree $cmd] >= 0} {
		    # rfloss feff fefferr p|a
		    # these are critical params like geom and will require updatePairsGeom if they change during polbeam=1
		    set changed 0
			if {$nw < 3 || [scan $argstr "%s %g" ap fval] < 2} {
				lappend warnings "ERROR: $cmd requires a|p float args at $ap\:\n$apline"
			} else {
				logEvent [list $datestr $utc $cmd $ap $fval]
				set api 0
				if [string match a* $ap] {set api 1}
				if {$cmd == "rfloss"} {
				    if {$rfFlipLoss($api) != $fval} {
				        set changed 1
				        set rfFlipLoss($api) $fval
				        pipeWrite "p cgfr $api $fval"
				    }
				} elseif {$cmd == "feff"} {
				    if {$flipEff($api) != $fval} {
				        set changed 1
				        set flipEff($api) $fval
				        pipeWrite "p cgfe $api $fval"
				    }
				} elseif {$cmd == "fefferr"} {
				    if {$flipEffErr($api) != $fval} {
				        set changed 1
				        set flipEffErr($api) $fval
    				    pipeWrite "p cgfee $api $fval"
    				}
				}
			}
			if {$polbeam == 1 && $changed == 1} {
			    updatePairsGeom
			}
		} elseif [string match fru* $cmd] {
			if {$nw < 2 || ![regexp -nocase {^a.*|^p.*|^e.*|^w.*} $argstr]} {
				lappend warnings "WARNING: frupdate requires a|p|e|w as arg at $ap\:\n$apline"
			} else {
			    set frupdate [string index $argstr 0]
				logEvent [list $datestr $utc frupdate $frupdate]
			}
		} elseif {$cmd == "hkl"} {
		    # must be three floats
		    if {$nw < 4 || [scan $argstr "%g %g %g"] < 3} {
		        #lappend warnings "$cmd requires 3 floags at $ap"
		    } else {
		        logEvent [list $datestr $utc $cmd $argstr]
		    }
		} elseif {$cmd == "beam"} {
		    # beam args can be open full 1cm sh# sw# fsh fsw fsl ana 
		    set changed [beamParse $arglst]
		    if {$polbeam == 1 && $changed == 1} {
		        updatePairsGeom
		    }
		} elseif {$cmd == "fr" || [string match frb* $cmd] || [string match frs* $cmd]} {
		    # This is flip ratio msr from cmnd line or scan
		    # fr args should not be Q V as Q V should be set by seq, but can be beam
		    # frbg is a background flip-ratio measurment
			# after fr frbg there are flipstate cmnds VQmode commands  beam cmnd
			# where does fr end?
			# Any VQ change ends the current fr as does any beam (change)
			# any move or scan or other pb cmnd.
			# should only be flip.seq and Counters:

			# any beam args will go in the pairsGeom appended at startPolBeam
			beamParse $arglst
			if {$polbeam == 0} {
		        set polbeam 1
		        set transmodeStartSecs 0
		        startPolBeam $apline
    		    logEvent [list $datestr $utc "start" "polarized beam"]
    		    set nfiles 0
    		}
    		# if user forgets the pb both add it here
    		if {$beamCell != "both"} {
    		    set beamCell both
    		    logEvent [list $datestr $utc both]
    		}
    		# log the fr cmnd
    		logEvent [list $datestr $utc $cmd $arglst]
            
            # check to see if beamParse found an envflg condition for this fr
            # envflg check is now replace with findEnv to update envTemp and envBfld
            #if $envflg == 1
            # whenever a condition is defined any previous logging of this same condition gets destroyed
            # init the time log for this condition
            #set envSecs $envstr
            #set envCurRange lindex $envKeys end list 0 0
            findEnv $ap
            
		    if {$PGfilt == "out"} {
		        lappend warnings "WARNING: flip-ratio $cmd with PG out at $ap\:\n$apline"
		    }
    		if [string match frs* $cmd] {
    		    set nflip [parseFRscan $apnext $cmd $arglst]
    		} else {
    			puts "calling parseFR at $datestr $utc FLIP= $FLIP"
		        set nflip [parseFR $apnext $cmd]
            }
			if {$nflip < 0} {
				set nflip -$nflip
				lappend warnings "ERROR: parseFR error at $nflip\:\n$apline"
			} elseif {$nflip < 2} {
				lappend warnings "WARNING: parseFR measured less than two flipstates at line $ap\:\n$apline"
			} else {
			    incr nflipratios
			    lappend flipratios $utc
			}
			if {$endFR > $apnext} {
			    set apnext $endFR
			}
		} elseif {[string match com* $cmd]} {
		    logEvent [list $datestr $utc comment $argstr]
		}
        set ap $apnext
        incr nit
    }
	# can we check the empty monitor logs here for unknown beam conditions
	if {[llength $incBeamCond] == 1} {
		checkMonitorMT
	}
    if {[llength $warnings] > 0} {
    	$txt(4) insert end "bt7 console ERRORS and WARNINGS:\n\n"
        foreach w $warnings {
            $txt(4) insert end $w\n\n
        }
        #Dialog_Warn [join $warnings "\n"]
    }
    butenable 2
    #$conB configure -bg grey -state disabled
    #nb select 1
    selectTab 1
    close $fp
    return $nit
}

proc aveMonitors {mlst} {
	# mrat is monitors per sec
	set utcmx 0
	set utcmn 9999999999
	set avemrat 0
	set rms 0
	set nmsr 0
	foreach m $mlst {
		set mutc [lindex $m 0]
		if {$mutc < $utcmn} {set utcmn $mutc}
		if {$mutc > $utcmx} {set utcmx $mutc}
		set mrat [lindex $m 1]
		set mrate [lindex $m 2]
		set avemrat [expr $avemrat + $mrat]
		set rms [expr $rms + $mrate*$mrate]
		incr nmsr
	}
	if {$nmsr < 1} {return {}}
	set avemrat [expr $avemrat/$nmsr]
	set rms [expr sqrt($rms)/$nmsr]
	return [list [list $utcmn $utcmx] $avemrat $rms]	
}

proc checkMonitorMT {} {
	# finished parsing console, so now check if recorded MT monitor logs are matched in base tables
	# below averaging aassumes MT is always at same condition which it usually is
	# check incBeamCond. If there are changes no point in checking average
	global txt
	global monitorMTPGin monitorMTPGout
	global monitorMTVSPGin monitorMTVSPGout
	global monitorMTbasePGin monitorMTbasePGout
	global monitorMTbaseVSPGin monitorMTbaseVSPGout
	set mlogs {monitorMTPGin monitorMTPGout monitorMTVSPGin monitorMTVSPGout}
	set mtabs {monitorMTbasePGin monitorMTbasePGout monitorMTbaseVSPGin monitorMTbaseVSPGout}
	foreach mt  $mlogs mb $mtabs {
		# skip empty logs
		if {[llength [array get $mt]] < 1} {continue}
		puts "checkMonitorMT mt= $mt mA= [array get $mt]"
		array set mA {}
		array set mA [array get $mt]
		array set mB {}
		array set mB [array get $mb]
		set bn [array names mB]
		foreach e [array names mA] {
			puts "checkMonitorMT: e= $e for mA= $mt"
			set am [aveMonitors $mA($e)]
			if {[llength $am] < 3} {continue}
			set utcrng [lindex $am 0]
			set amrat [lindex $am 1]
			set amrate [lindex $am 2]
			set Eir [format %.2f $e]
			if {[lsearch $bn $Eir] < 0} {
				$txt(4) insert end "WARNING: inc Ei= $Eir is not in base monitor MT table $mb.\n"
				continue
			}
			# check if this is a known beam condition
			set ok [getMonitorTableMatch $mB($Eir) $amrat $amrate]
			if {$ok < 1} {
				$txt(4) insert end "WARNING: cant match averaged MT monitor rate= $amrat at Ei= $Eir with table $mb.\n"
			}
		}
	}
}

proc nextline {istar} {
    global txt itxt
    set nxt [$txt($itxt) index "$istar linestart + 1 lines"]
    return $nxt
}


proc monitorArea {} {
	# what is beam area at He3Pol as counted by monitor
	# without focusV the vert div is approx 10cm / 120 cm or about 5 deg
	# with focusV
	global vertDivFWHMradMon vertDivFWHMdegMon
	global hPmon wPmon beamAreaMon focusV
	global bt7monoH bt7MonDia bt7MonToMono
	global radToDegFac Pi bt7MonToPolHe3
	set vertDivFWHMradMon 0.16
	if {$focusV > 0} { set vertDivFWHMradMon 0.20 }
	#set vertDivFWHMradMon [expr (20. - $bt7MonDia)/$bt7MonToMono]
	set vertDivFWHMdegMon [expr $radToDegFac*$vertDivFWHMradMon]
	set hPmon [expr $bt7MonDia + $vertDivFWHMradMon*$bt7MonToPolHe3]
	set wPmon 5.
	# use ellipse beam cross-section at monitor
	set beamAreaMon [expr ($Pi/4.)*$hPmon*$wPmon]
}

proc beamAreas {} {
    # calculate He3 cell beam cross-section areas for BT7 with 1 cm back of enclosure slit and without slit (open)
    # We are always considering elastic Bragg scattering for trans and FR
    # so vertical divergence is a straight line path limited by the various scattering heights
    # monoHeight frontslit sample he3slit backslit detectorHeight.
    # For inelastic attering and some elastic scattering Qz can change at the sample but we dont consider this case
    # since DD=sample to det = 253 cm and MS=monoTosample = 206 cm
    # without 1cm slit (open condition) use top of sample to top of detector for vertdivergence
    # with the slit we can use a pinhole approx from the slit to top and bottom of detector
    # to get the divergence angle
    # So the divergence we are using to compute He3beamsize is smaller without the slit
    # but the sampleHeight is added directly
    # slit to detector = D0 = 253 - 37 = 216
    # so for slit vertDiv = DH/D0 = 10/216 = .046 rad
    # and beamHeight = .046 x slitToHe3 = .046*48 cm = 2.2 cm
    # for slit the vertical divergence angle can be limited by sample height
    # so we calc sample height intercepted by .046 degrees from slit to sample (37 cm) gives 1.7 cm
    # We use the same divergence from slit back to He3Polarizer = 112 cm  .046 x 112 = 5.1 cm
    # again limited possibly by sample height.
    #
    # Now when the beam is open the divergence from top of sample to top of det is
    # (DH - SH)/2/DD and 
    # beamHeightAnalyzer = SH + S3*(DH - SH)/DD = DH*(S3/DD) + SH*(1 - S3/DD)
    # with S3 = 123 cm
    # This should be the same divergence angle from sample to he3Polarizer just use the sampleTOhe3PolDistance
    # = 75 cm = S3p
    # so he3pol beamheightOpen = SH + S3p*(DH - SH)/DD that is just replace S3 by S3p
    # beamHeightPolarizer = SH + S3p*(DH - SH)/DD = DH*(S3p/DD) + SH*(1 - S3p/DD)
    # 
    global beamAreaP beamAreaA
    global beamAreaPopen beamAreaAopen
    global shPopen shAopen hP hA hPo hAo
    global sampleH sampleW sampleMosMin
    global focusV
    global vertDivPFWHMdeg vertDivAFWHMdeg
    global vertDivPFWHMdegOPEN vertDivPFWHMradOPEN
    global vertDivAFWHMdegOPEN vertDivAFWHMradOPEN
    global beamSizeFudge
    global bt7PolHe3ToEnc bt7AnaToDet bt7AnaDrumRad bt7AnaHe3ToAnaColl
    global bt7SampleEncRadius bt7SlitToDet bt7SlitToMono bt7SlitToBS bt7SlitToFS bt7EncToFS
    global bt7bsh bt7bsw bt7fsh bt7fsw
    global bt7AnaTrans bt7SampleToAna bt7SampleToPolHe3 bt7SampleToMono bt7SampleToBS
    global bt7SlitToAnaHe3 bt7SlitToPolHe3
    global bt7DetH
    global bt7monoH
    global bt7he3Slit
    global S3 DD
    global radToDegFac
    global horDivAFWHMmin horDivPFWHMmin


    # recompute basic geo in case ana trans changes
    set bt7SampleToAna [expr 165. + 64.*($bt7AnaTrans/360.)]
    set bt7SampleToPolHe3 [expr $bt7SampleEncRadius + $bt7PolHe3ToEnc]
    # need sample to detector distance DD
    set DD [expr $bt7SampleToAna + $bt7AnaToDet]
    set bt7SampleToAnaColl [expr $bt7SampleToAna - $bt7AnaDrumRad]
    # back slit is about 10 cm upstream from anaColl
    set bt7SampleToBS [expr $bt7SampleToAnaColl - 10]
	set bt7SampleToFS [expr $bt7SampleEncRadius - $bt7EncToFS]
    # need sample to he3 analyzer cell center = S3
    set S3 [expr $bt7SampleToAnaColl - $bt7AnaHe3ToAnaColl]
    set bt7SlitToAnaHe3 [expr $S3 - $bt7SampleEncRadius]
    set bt7SlitToDet [expr $DD - $bt7SampleEncRadius]
    set bt7SlitToBS [expr $bt7SampleToBS - $bt7SampleEncRadius]
    set bt7SlitToFS [expr 2.*$bt7SampleEncRadius - $bt7EncToFS]
    set bt7SlitToPolHe3 [expr 2.*$bt7SampleEncRadius + $bt7PolHe3ToEnc]

    # for small beam pinhole wee calculate vert divergence angles starting
    # at the pinhole and ending at polarizer of analyzer he3 cell.
    # This most often using a crystal Bragg peak where front and
    # back neutron beam ray is the same. But we add a powder option
    # just in case. We had to do Al powder once.
    # start by finding the limiting pinhole vertDiv angle
    # The bt7he3Slit should be the effective pinhole where the elastic
    # beam rays will cross. Start on the backend where powder or xtal sample doesnt matter
    set vertDivAFWHMrad [expr $bt7DetH/$bt7SlitToDet]
    set vlim Det
    set bsDiv [expr $bt7bsh/$bt7SlitToBS]
    if {$bsDiv < $vertDivAFWHMrad} {
        set vertDivAFWHMrad $bsDiv
        set vlim BS
    }
    set sampleDiv [expr $sampleH/$bt7SampleEncRadius]
    if {$sampleDiv < $vertDivAFWHMrad} {
        set vertDivAFWHMrad $sampleDiv
        set vlim Sam
    }

    # on front side if powder mosiac allow vertDiv independent of back side
    set monoHeff $bt7monoH
    if {$focusV == 1} {
    	if {$sampleH < 3} {set monoHeff [expr $bt7monoH - (3 - $sampleH)]}
    } else {
    	set monoHeff $sampleH
    }
    if {$sampleMosMin > 900} {
        set fsDiv [expr ($bt7fsh + $sampleH)/$bt7sampleToFS]
        set vertDivPFWHMrad $fsDiv
        # mono focus on sample so use 1/2?
        set monoDiv [expr ($monoHeff + $sampleH)/$bt7SsampleToMono]
        if {$monoDiv < $vertDivPFWHMrad} {
            set vertDivPFWHMrad $monoDiv
        }
    } else {
        # crystal case beam doesnt change vertical direction at sample except for sampleMos
        set vertDivPFWHMrad $vertDivAFWHMrad
        set fsDiv [expr $bt7fsh/$bt7SlitToFS]
        if {$fsDiv < $vertDivPFWHMrad} {
            set vertDivPFWHMrad $fsDiv
            set vlim FS
        }
        # we can safely assume mono is vertically focussing onto a 3 cm tall sample
        # if the sample is smaller we can reduce the effective monH
        # the slit is farther from mono than sample and is at least as small so
        # we dont need to check the sample restriction
        set monoDiv [expr $monoHeff/$bt7SlitToMono]
        if {$monoDiv < $vertDivPFWHMrad} {
            set vertDivPFWHMrad $monoDiv
            set vlim Mono
        }
        set vertDivAFWHMrad $vertDivPFWHMrad
    }
    # pinhole tends to make vert dependence of flux more uniform than Gaussian
    # misalignment will decrease this divergence and the limits calculated here
    # overestimate the Gaussian FWHM, although as we mentioned the beam distibution is less Gaussian
    # so the fudge factor is a guess
    set vertDivPFWHMrad [expr $beamSizeFudge*$vertDivPFWHMrad]
    set vertDivPFWHMdeg [expr $vertDivPFWHMrad*$radToDegFac]
    set vertDivAFWHMrad [expr $beamSizeFudge*$vertDivAFWHMrad]
    set vertDivAFWHMdeg [expr $vertDivAFWHMrad*$radToDegFac]
    #puts "pinhole vertDivFWHMdeg= $vertDivFWHMdeg  limited by $vlim"
    # now compute the he3 beamHeight at analyzer and polarizer
    set hA [expr $vertDivAFWHMrad*$bt7SlitToAnaHe3]
    set hP [expr $vertDivPFWHMrad*$bt7SlitToPolHe3]
    # due to Soller collimator blade spacing being small (1mm) the
    # beam width is the smaller of the sampleW or slitWidth
    # with horizontalDiv component added
    # we assume the sample is fully illuminated
    # use the composite collimator horDiv from A and P side
    set horDivFWHMdeg [expr (1./60.)*($horDivAFWHMmin*$horDivPFWHMmin)/($horDivAFWHMmin + $horDivPFWHMmin)]
    set horDivFWHMrad [expr $horDivFWHMdeg/$radToDegFac]
    set wA [expr $bt7he3Slit + 0.5*$horDivFWHMrad*$bt7SlitToAnaHe3]
    if {$sampleW < 0.1} {set sampleW 0.1}
    if {$sampleW < $bt7he3Slit} {
        set hlimrad [expr ($bt7he3Slit - $sampleW)/$bt7SampleEncRadius)]
        if {$horDivFWHMrad < $hlimrad} {set hlimrad $horDivFWHMrad}
        set wAs [expr $sampleW + 0.5*$hlimrad*$S3]
        if {$wAs > $wA} {set wA $wAs}
    }
    # if the sample is too wide neutrons scatterred from the sample edges wont make it
    # through the slit due to restricted horDiv
    # so compute an effective sampleWe if this is the case for purpose of finding beam width at Pol
    # NB we are ignoring the fsw here since we believe the he3slit will always be limiting
    set sampleWe [expr $bt7he3Slit + $horDivFWHMrad*$bt7SampleEncRadius]
    if {$sampleW < $sampleWe} {set sampleWe $sampleW}
    # now compute the beam width at Polarizer
    set wP [expr $sampleWe + $horDivFWHMrad*$bt7SampleToPolHe3]  
    #puts "he3pinholebeamsizes hA= $hA wA= $wA  hP= $hP wP= $wP"
    set beamAreaP [expr $hP*$wP]
    set beamAreaA [expr $hA*$wA]


    # also treat the open beam case vertical divergence
    # without pinhole no sense in using angles Just translate aperatures to cell positions
    # take the sampleH assumed fully illuminated and add the contribution from full beam divergence
    set SHeff $sampleH
    if {$sampleH > 3} {set SHeff 3}
    # to treat inelastic data the beamArea y be better approximated by the powder case
    if {$sampleMosMin > 900} {
        set vertDivAFWHMradOPEN [expr ($SHeff + $bt7DetH)/$DD]
        set bsDivAFWHMradOPEN [expr ($SHeff + $bt7bsh)/$bt7SampleToBS]
        if {$bsDivAFWHMradOPEN < $vertDivAFWHMradOPEN} {
            set vertDivAFWHMradOPEN $bsDivAFWHMradOPEN
        }
        # fron divergence is same as in slit case for powder
        set fsDiv [expr ($bt7fsh + $sampleH)/$bt7sampleToFS]
        set vertDivPFWHMradOPEN $fsDiv
        # focus on sample so use 1/2
        set monoDiv [expr 0.5*($monoHeff + $sampleH)/$bt7SsampleToMono]
        if {$monoDiv < $vertDivPFWHMradOPEN} {
            set vertDivPFWHMradOPEN $monoDiv
        }
        set hPo [expr $vertDivPFWHMradOPEN*$bt7SampleToPolHe3]
        set hAo [expr $vertDivAFWHMradOPEN*$S3]
    } else {
    	# even for xtal elastic scattering the sampleMos can change the vertical divergence of a beam path
    	# ray tracing the extremal beam paths allows us to avoid including the sampleMos directly
        # find the beam edge angle from top of sample to top of detector or top of BS if smaller angle
        # On backend we are tracing from detector to mono with positive angles down
        set SDDangle [expr 0.5*($bt7DetH - $sampleH)/$DD]
        set SBSangle [expr 0.5*($bt7bsh - $sampleH)/$bt7SampleToBS]
        if {$SBSangle < $SDDangle} {set SDDangle $SBSangle}
        # this SDDangle provides the primary Analyzer beam height
        # trace this same beam edge back to the monochromator
        # and if it limited by fs or monoHeff set a secondary SDD2angle. This would be the zero sampMos case
        # check backend edge beam at the monochromator where the beam has crossed z=0
        set SDD2angle $SDDangle
        set monoHeff2 [expr $monoHeff/2.]
        set hMoeA [expr $sampleH - 2.*$SDD2angle*$bt7SampleToMono]
        if {[expr abs($hMoeA)] > $monoHeff2} {
        	# reduce SDD2angle to get monoHeff
        	if {$hMoeA < 0} {
        		set SDD2angle [expr 0.5*($monoHeff2 + $sampleH)/$bt7SampleToMono]
        	} else {
        		set SDD2angle [expr 0.5*($sampleH - $monoHeff2)/$bt7SampleToMono]
        	}
        }
        #also check limiting by front slit
        set hFoeA [expr $sampleH - 2.*$SDD2angle*$bt7SampleToFS]
        set fsh2 [expr $bt7fsh/2.]
        if {[expr abs($hFoeA)] > $fsh2} {
        	if {$hFoeA < 0} {
        		set SDD2angle [expr 0.5*($fsh2 + $sampleH)/$bt7SampleToFS]
        	} else {
        		set SDD2angle [expr 0.5*($sampleH - $fsh2)/$bt7SampleToFS]
        	}
        }
        # compute the beamheight at Analyzer for both SDD and SDD2
        set hAoe [expr $sampleH + 2.*$SDDangle*$S3]
        set hAoe2 [expr $sampleH + 2.*$SDD2angle*$S3]
        

        
		# same edge ray tracing starting from top of sample to top of monoHeff or top of FS if smaller
		# here we are again tracing towards mono but now positive angles are up
        set SMangle [expr 0.5*($monoHeff - $sampleH)/$bt7SampleToMono]
        set SFSangle [expr 0.5*($bt7fsh - $sampleH)/$bt7SampleToFS]
        if {$SFSangle < $SMangle} {set SMangle $SFSangle}
        # as before create second SM2angle that must also pass the backend slit and detector
        set SM2angle $SMangle
        # check frontend edge beam at the detector where the beam has crossed z=0
        set hDoeP [expr $sampleH - 2.*$SM2angle*$DD]
        set detH2 [expr $bt7DetH/2.]
        if {[expr abs($hDoeP)] > $detH2} {
        	if {$hDoeP < 0} {
        		set SM2angle [expr 0.5*($detH2 + $sampleH)/$DD]
        	} else {
        		set SM2angle [expr 0.5*($sampleH - $detH2)/$DD]
        	}
        }
        set hBoeP [expr $sampleH - 2.*$SM2angle*$bt7SampleToBS]
        set bsh2 [expr $bt7bsh/2.]
        if {[expr abs($hBoeP)] > $bsh2} {
        	if {$hBoeP < 0} {
        		set SM2angle [expr 0.5*($bsh2 + $sampleH)/$bt7SampleToBS]
        	} else {
        		set SM2angle [expr 0.5*($sampleH - $bsh2)/$bt7SampleToBS]
        	}
        }
        #compute the two beam heights
        set hPoe [expr $sampleH + 2.*$SMangle*$bt7SampleToPolHe3]
        set hPoe2 [expr $sampleH + 2.*$SM2angle*$bt7SampleToPolHe3]
        
		# now we have the he3 beam heights for unrestricted (inelastic or powder) and for direct ray tracing (elastic mos=0)
		# average these together

		# use SDDangle and SMangle as the edge beam (large He3 area) divergence angles 
		# get the sampleMos in radians as well
		set smosRad [expr ($sampleMosMin/60.)/$radToDegFac]
		set m2 [expr $SMangle*$SMangle]
		set s2 [expr $smosRad*$smosRad]
		set a2 [expr $SDDangle*$SDDangle]
		set xP [expr $m2/($m2 + $s2)]
		set xA [expr $a2/($a2 + $s2)]   		
		# We estimate the beam size at He3 by interpolating between the large mosiac and small mosaic cases
		# using the mosiac compared to the divergence as the interpolating param
		# actually using m2/(m2 + s2) etc
		# as the smos gets big compared to ray traced vert div the beamsize approaches the large edge traced size
		set hPo [expr (1. - $xP)*$hPoe + $xP*$hPoe2]
		set hAo [expr (1. - $xA)*$hAoe + $xA*$hAoe2]
		# now set the divergence angles from the beam heights
		set vertDivPFWHMradOPEN [expr $hPo/$bt7SampleToPolHe3]
		set vertDivAFWHMradOPEN [expr $hAo/$S3]

		# The previous version of this calc was following
        # assume that full sample height and detector height are illuminated
        #set vertDivAFWHMradOPEN [expr 0.5*($SHeff + $bt7DetH)/$DD]
        # usually BS wont be restrictive but check anyway
        #set bsDivFWHMradOPEN [expr 0.5*($SHeff + $bt7bsh)/$bt7SampleToBS]
        #if $bsDivFWHMradOPEN < $vertDivFWHMradOPEN
        #   set vertDivFWHMradOPEN $bsDivFWHMradOPEN
        
        # check the FS restriciton
        #set fsDivFWHMradOPEN [expr 0.5*($SHeff + $bt7fsh)/$bt7SampleToFS]
        #if $fsDivFWHMradOPEN < $vertDivFWHMradOPEN
        #    set vertDivFWHMradOPEN $fsDivFWHMradOPEN
        
        # beam is vertically focussed and so crosses z=0 find out where the beam limiting ray hits the mono
        #set zMono [expr $vertDivFWHMradOPEN*($DD + $bt7SampleToMono)- 0.5*$bt7DetH]
        #if $zMono > 10
    	#    set vertDivFWHMradOPEN [expr (10. + 0.5*$bt7DetH)/($DD + $bt7SampleToMono)]
        #set vertDivFWHMdegOPEN [expr $vertDivFWHMradOPEN*$radToDegFac]
    }
    set vertDivPFWHMdegOPEN [expr $vertDivPFWHMradOPEN*$radToDegFac]
    set vertDivAFWHMdegOPEN [expr $vertDivAFWHMradOPEN*$radToDegFac]
    set wAo [expr $sampleW + $horDivFWHMrad*$S3]
    set wPo [expr $sampleW + $horDivFWHMrad*$bt7SampleToPolHe3]
    #puts "he3openbeamsizes hAo= $hAo wAo= $wAo  hPo= $hPo wPo= $wPo"
    set beamAreaPopen [expr $hPo*$wPo]
    set beamAreaAopen [expr $hAo*$wAo]
    set shPopen $hPo
    set shAopen $hAo
}

proc setBeamGeoD {celli alst} {
    pipeWrite "p cd $celli Hmos 40"
    pipeWrite "p cd $celli Vcol2 9999."
    pipeWrite "p cd $celli Hcol2 9999."
    foreach {key val} $alst {
    	pipeWrite "p cd $celli $key $val"
    }
}

proc setMonitorGeo {celli} {
	global hPmon beamAreaMon vertDivFWHMdegMon horDivPFWHMmin
    pipeWrite "p cd $celli Hmos 40"
    pipeWrite "p cd $celli Vcol2 9999."
    pipeWrite "p cd $celli Hcol2 9999."
    pipeWrite "p cd $celli Hcols $horDivPFWHMmin"
    pipeWrite "p cd $celli Vcols $vertDivFWHMdegMon"
    pipeWrite "p cd $celli beamArea $beamAreaMon"
	pipeWrite "p cd $celli SH $hPmon"
}

proc setBeamGeo {celli isa bs} {
    global shPopen shAopen hP hA hPo hAo
    global vertDivFWHMdeg vertDivFWHMdegOPEN
    global vertDivPFWHMdeg vertDivAFWHMdeg
    global vertDivPFWHMdegOPEN vertDivAFWHMdegOPEN
    global beamAreaP beamAreaA beamAreaPopen beamAreaAopen
    global horDivPFWHMmin horDivAFWHMmin
    
    pipeWrite "p cd $celli Hmos 40"
    pipeWrite "p cd $celli Vcol2 9999."
    pipeWrite "p cd $celli Hcol2 9999."

    if {$isa == 0} {
        pipeWrite "p cd $celli Hcols $horDivPFWHMmin"
    } else {
        pipeWrite "p cd $celli Hcols $horDivAFWHMmin"
    }
    if {$bs == 0} {
        if {$isa == 0} {
        	pipeWrite "p cd $celli Vcols $vertDivPFWHMdeg"
			pipeWrite "p cd $celli SH $hP"
            pipeWrite "p cd $celli beamArea $beamAreaP"
        } else {
        	pipeWrite "p cd $celli Vcols $vertDivAFWHMdeg"
            pipeWrite "p cd $celli beamArea $beamAreaA"
			pipeWrite "p cd $celli SH $hA"
        }
    } else {        
        if {$isa == 0} {
			pipeWrite "p cd $celli Vcols $vertDivPFWHMdegOPEN"
            pipeWrite "p cd $celli beamArea $beamAreaPopen"
            pipeWrite "p cd $celli SH $hPo"
        } else {
			pipeWrite "p cd $celli Vcols $vertDivAFWHMdegOPEN"
            pipeWrite "p cd $celli beamArea $beamAreaAopen"
            pipeWrite "p cd $celli SH $hAo"
        }
    }
}



proc currentGeom {} {
    global openbeamCellVarsP openbeamCellVarsA cellfileParams
    global flipCellVars cellfileParamsF 
    global hPo hAo
    global vertDivPFWHMdegOPEN vertDivAFWHMdegOPEN
    global beamAreaPopen beamAreaAopen
    global horDivPFWHMmin horDivAFWHMmin
    global rfFlipLoss flipEff flipEffErr
    global S3 DD bt7SampleToPolHe3 sampleMosMin
    # call beamAreas then save the keyed list of cell params and values needed in cellfile
    # we only need OPEN beam values for the cellfile
    beamAreas
    set plist {}
    foreach vname $openbeamCellVarsP cname $cellfileParams {
        set value [set $vname]
        lappend plist $cname $value
    }
    foreach vname $flipCellVars cname $cellfileParamsF {
        set value [set $vname\(0)]
        lappend plist $cname $value
    }
    set alist {}
    foreach vname $openbeamCellVarsA cname $cellfileParams {
        set value [set $vname]
        lappend alist $cname $value
    }
    foreach vname $flipCellVars cname $cellfileParamsF {
        set value [set $vname\(1)]
        lappend alist $cname $value
    }
    return [list $plist $alist]
}

proc beamCheckTran {tran} {
	set bs [lindex $tran 5]
	if [string match beam=1 $bs] {return 1}
	return 0
}
proc beamCheckTrans {translst} {
	set nt [llength $translst]
	if {$nt < 1} {return 0}
	set n1 0
	set lastutc 0
	foreach trans $translst {
		set bc [beamCheckTran $trans]
		incr n1 $bc
		set tutc [lindex $trans 1]
		if {$tutc > $lastutc} {
			set lastutc $tutc
			set lastbc $bc
		}
	}
	if {$n1 == $nt} {return [list 1 $translst]}
	if {$n1 == 0} {return [list 0 $translst]}
	# remaining case is mixed. reset all to lastbc latest in utc condition
	set newtranslst {}
	foreach trans $translst {
		set trans [lreplace $trans 5 5 "beam=$lastbc"]
		#if [beamCheckTran $trans] == 1 continue
		lappend newtranslst $trans
	}
	return [list -1 $newtranslst]
}


proc getGEOutcr {mutc geosutcr} {
	global geoATutc
	set ng [llength $geosutcr]
	foreach g $geosutcr {
		set utci [lindex $g 0]
		set utcf [lindex $g 1]
		if {$mutc >= $utci && $mutc <= $utcf} {
			set geoATutc [lindex $g 2]
			return [list $utci $utcf]
		}
	}
	return {}
}


proc getINCutcr {mutc} {
	# find incBeamCond at mutc. Set that condition in incBeamUTCcond. return the utc range
	global incBeamUTCranges incBeamUTCcond
	set ng [llength $incBeamUTCranges]
	foreach g $incBeamUTCranges {
		set utci [lindex $g 0]
		set utcf [lindex $g 1]
		set incBeamUTCcond [lindex $g 2]
		if {$mutc >= $utci && $mutc <= $utcf} {
			return [list $utci $utcf]
		}
	}
	return {}
}
proc getINCutcrr {mutcr} {
	# get best incBeamCond that covers utc range mutcr
	global incBeamUTCranges incBeamUTCcond
	set mutci [lindex $mutcr 0]
	set mutcf [lindex $mutcr 1]
	set dm [expr double ($mutcf - $mutci)]
	set ng [llength $incBeamUTCranges]
	set bestfrac 0
	for {set i 0} {$i < $ng} {incr i} {
		set g [lindex $incBeamUTCranges $i]
		set utci [lindex $g 0]
		set utcf [lindex $g 1]
		set incBeamUTCcond [lindex $g 2]
		set frac 0
		set fraci 0
		set fracf 0
		if {$mutci >= $utci && $mutci <= $utcf} {
			set fraci 1.
			if {$mutcf > $utcf} {set fraci [expr ($utcf - $mutci)/$dm]}
			set rangei [list $mutci [expr $mutci + $fraci*$dm]]
			puts "getINCutcrr found mutci=$mutci in range $utci $utcf fraci=$fraci"
		}
		if {$mutcf >= $utci && $mutcf <= $utcf} {
			set fracf 1
			if {$mutci < $utci} {set fracf [expr ($mutcf - $utci)/$dm]}
			set rangef [list [expr $mutcf - $fracf*$dm] $mutcf]
			puts "getINCutcrr found mutcf=$mutcf in range $utci $utcf fracf=$fracf"
		}
		if { $fraci > 0 } {
			set frac $fraci
			set range $rangei
		}
		if { $fracf > $fraci } {
			set frac $fracf
			set range $rangef
		}
		if {$frac > $bestfrac} {
			puts "setting bestfrac= $bestfrac"
			set bestfrac $frac
			set bestrange $range
			set bestcond $incBeamUTCcond
		}
		if {$utci > $mutcf} {break}
	}
	if {$bestfrac > 0} {
		set incBeamUTCcond $bestcond
		return $bestrange
	}
	return {}
}


proc getMratMT {e pgin vs utcr} {
	global monitorMTPGin monitorMTPGout monitorMTbasePGin monitorMTbasePGout
	global monitorMTVSPGin monitorMTVSPGout monitorMTbaseVSPGin monitorMTbaseVSPGout
	global incBeamUTCcond
	set mn [lindex $utcr 0]
	set mx [lindex $utcr 1]
	puts "getMratMT looking for e=$e pgin=$pgin in utcrange= $mn $mx"
	set mres {}
	if {$vs == 0} {
		if {$pgin == 1 && [info exists monitorMTPGin($e)]} {
			foreach m $monitorMTPGin($e) {
				set mu [lindex $m 0]
				if {$mu < $mn || $mu > $mx} {continue}
				# append the msrd MT mrat and its error as a list
				lappend mres [list [lindex $m 1] [lindex $m 2]]			
			}
		} elseif {$pgin == 0 && [info exists monitorMTPGout($e)]} {
			foreach m $monitorMTPGout($e) {
				set mu [lindex $m 0]
				if {$mu < $mn || $mu > $mx} {continue}
				# append the msrd MT mrat and its error as a list
				lappend mres [list [lindex $m 1] [lindex $m 2]]			
			}
		}
	} else {
		if {$pgin == 1 && [info exists monitorMTVSPGin($e)]} {
			foreach m $monitorMTVSPGin($e) {
				set mu [lindex $m 0]
				if {$mu < $mn || $mu > $mx} {continue}
				# append the msrd MT mrat and its error as a list
				lappend mres [list [lindex $m 1] [lindex $m 2]]			
			}
		} elseif {$pgin == 0 && [info exists monitorMTVSPGout($e)]} {
			foreach m $monitorMTVSPGout($e) {
				set mu [lindex $m 0]
				if {$mu < $mn || $mu > $mx} {continue}
				# append the msrd MT mrat and its error as a list
				lappend mres [list [lindex $m 1] [lindex $m 2]]			
			}
		}
	}
	set nmres [llength $mres]
	puts "getMratMT mres= $mres"
	if {$nmres == 1} {return [lindex $mres 0]}
	if {$nmres > 1} {return [averageANDerr $mres]}
	# if all else fails try the base tables for MT mrat with incbeamcond=incBeamUTCcond
	if {$vs == 0} {
		puts "getMratMT using base PG tables"
		if {$pgin == 1 && [info exists monitorMTbasePGin($e)]} {
			foreach m $monitorMTbasePGin($e) {
				set bc [lindex $m 1]
				if {$bc == $incBeamUTCcond} {
					return [list [lindex $m 0] [lindex $m 2]]
				}
			}
		} elseif {$pgin == 0 && [info exists monitorMTbasePGout($e)]} {
			foreach m $monitorMTbasePGout($e) {
				set bc [lindex $m 1]
				if {$bc == $incBeamUTCcond} {
					return [list [lindex $m 0] [lindex $m 2]]
				}
			}
		}
	} else {
		puts "getMratMT using base VS tables"
		if {$pgin == 1 && [info exists monitorMTbaseVSPGin($e)]} {
			foreach m $monitorMTbaseVSPGin($e) {
				set bc [lindex $m 1]
				if {$bc == $incBeamUTCcond} {
					return [list [lindex $m 0] [lindex $m 2]]
				}
			}
		} elseif {$pgin == 0 && [info exists monitorMTbaseVSPGout($e)]} {
			foreach m $monitorMTbaseVSPGout($e) {
				set bc [lindex $m 1]
				if {$bc == $incBeamUTCcond} {
					return [list [lindex $m 0] [lindex $m 2]]
				}
			}
		}	
	}
	return {}
}

proc rangeOverlap {r1 r2} {
	set mn1 [lindex $r1 0]
	set mx1 [lindex $r1 1]
	set mn2 [lindex $r2 0]
	set mx2 [lindex $r2 1]
	if {$mn1 >= $mx2 || $mx1 <= $mn2} {return {}}
	set mn $mn1
	if {$mn2 > $mn1} {set mn $mn2}
	set mx $mx1
	if {$mx2 < $mx1} {set mx $mx2}
	return [list $mn $mx]
}

array set transa {}
set geosBYutcr {}


# parse He3 Log to He3-Efficiencies
proc parseHe3Log {} {
    # for now this is bt7 specific and requires modification for MACS
    # The He3-log consists of consecutive stop-start start-stop ... polbeam blocks
    # polbeam is stopped whenever transmissions are made or cells are removed from the beam
    # and then polbeam is start when both cells are in the beam (both or fr)
    #
    # for transmissions need to set up a cell that has transmission geometry
    # on bt7 this is now a 1 cm square slit at the sample enclosure exit
    # 1cm slit to detector (with anaTrans = 300) is 216 cm
    # with sample to detector DD = 253 cm
    # so for 10 cm tall detector this is 10/216 = 0.046 radians as vertical divirgence limit
    # using the slit as a pinhole since it is much smaller than the detector height or incident beam height. 
    # slit to He3Analyzer = 48 cm
    # slit to He3Polarizer= 112 cm
    # This gives maximal beam height at He3 as beamAH=2.2 cm  beamPH=5.1 cm
    # This maximum limits are only reached if sample is tilt aligned to go through the slit
    # slit to sample is 37 cm so sample height that matches this optimal vertical divirgence is 2.6 deg FWHM
    # 0.046 x 37 = 1.7 cm
    # if sample height is less than 1.7 cm we have to further reduce the slit divergence and
    # proportionately the beam height at He3.
    # Most sample are less than 1 cm wide so that the slit width defines the
    # beam width at the He3.
    # allowing 1 deg horizontal divergence gives 1 cm + 48cm/57 = 2 cm A  and 1cm + 112cm/57 = 3 cm P
    # If the sample width is less than 1 cm we would reduce these beam widths proportionally
    # We are going to add a fudge factor from optimum divirgences that
    # is supposed to take into account that the beam has umbra, and sample tilt misalignment
    # this fudge multiplies the maximal beam areas
    #
    # We also need the vertical div for open beam for FR openbeam
    # In this case  the back end is still limiting (by beam traversal distance sample to detector)
    # Instead of pinhole we take top of sample to top of detector to define the analyzer beamarea
    # openBeamHeight = SH + 2*S3*(DH/2 - SH/2)/DD  where S3 is sample to cellcenter = 123 cm
    # multiply this limiting angle by 2ln2/sqrt(Pi) to convert to Gaussian FWHM with same height
    # which is 0.782
    # With back end limiting again we have the vertdiv limits as +-(SH + DH)/2/DD
    # take detector height DH = 10 cm
    
    global txt itxt nb he3B
    global readlines
    global endtxt
    global sampleH sampleW sampleMosMin sampleMosData sampleMosTran
    global focusV focusH
    global beamAreaP beamAreaA
    global horDivAFWHMmin horDivPFWHMmin
    global vertDivFWHMdeg
    global pairsGeom
    global incBeamCond incBeamUTCranges incBeamUTCcond geoATutc
    global startDate startUTC lastMratUTC
    global Ei
    global frupdate frupdates
    global fbgcpm
    global maxfrwait
    global frwarnings
    global efficiencyLog
    global flipEff flipEffErr
    global rfFlipLoss
    global transblock
    global transblocks transpairs
    global transa
    global npolarizers polarizers
    global frblocks newfrblocks frpairs
    global frpairlists
	global geosBYutcr
	global useMonitorForTrans
    global monitorHe3bothPGin monitorHe3bothPGout monitorMTPGin monitorMTPGout
    global mratCI mratRejFrac mratChired mratDecayCI
    global readlines nread nwarn nerr warnlines errlines
    
    
    # set defaults for frupdate flipEff flipEfferr etc
    # flip eff and errors
    pipeWrite "p cgfe 0 $flipEff(0)"
    pipeWrite "p cgfe 1 $flipEff(1)"
    pipeWrite "p cgfee 0 $flipEffErr(0)"
    pipeWrite "p cgfee 1 $flipEffErr(1)"
    # flipratio measured states
    #pipeWrite "p cgfc 0 0"
    # rf flip loss per flip pol ana
    pipeWrite "p cgfr 0 $rfFlipLoss(0)"
    pipeWrite "p cgfr 1 $rfFlipLoss(1)"
    # frupdate deflts to efficiency
    pipeWrite "p cgu $frupdate"

	# set up utc ranges for each incBeamCond
	set incBeamUTCranges {}
	set nincB [llength $incBeamCond]
	if {$nincB > 0} {
		set ibc [lindex $incBeamCond 0]
		set incutc [lindex $ibc 0]
		set incbc [lindex $ibc 2]
		for {set i 1} {$i < $nincB} {incr i} {
			set ibc [lindex $incBeamCond $i]
			set incutci [lindex $ibc 0]
			set incbci [lindex $ibc 2]
			lappend incBeamUTCranges [list $incutc $incutci $incbc]
			set incutc $incutci
			set incbc $incbci
		}
		lappend incBeamUTCranges [list $incutc $lastMratUTC $incbc]
	} else {
		lappend incBeamUTCranges [list $startUTC $lastMratUTC unknown]
	}
    
    set efficiencyLog {}
    set itxt 1
    set ap 1.0
	set endtxt [$txt(1) index "end linestart - 1 lines"]
    #puts "annotbt7 loop starting with endtxt= $endtxt"
	if {$endtxt < 10} {return 0}
	set niterror 1000
    set maxit 1000
    set nit 1
    set warnings {}
    # start at the initial stop
    set stop [findNextReg $ap {stop \{polarized beam\}}]
    if {! $stop} {
        Dialog_Warn "ERROR: He3-Log has no stop"
        return 0
    }
    #$txt(2) insert end [$txt(1) get "$stop linestart" "$stop lineend"]\n
    # init beamAreas
    beamAreas
    set tline $stop
    set warnings {}
    # we are going to save all flip-ratio stuff and process it all at the end of parsing
    # store them as a list of lists with each sublist containing all flip-ratios for a cellpair
    set frpairlists {}
    set transblocks {}
    set frblocks {}
    set frpairs {}
    set transpairs {}
    
    while {$ap < $endtxt && $nit < $maxit} {
        set start [findNextReg $stop {start \{polarized beam\}}]
        if {! $start} {set start $endtxt}
        set start [$txt(1) index "$start linestart"]
        set startline [$txt(1) get $start "$start lineend"]
        set startutc [lindex $startline 1]
        #$txt(2) insert end [$txt(1) get $start "$start lineend"]\n
        #puts "parseHe3Log loop nit= $nit ap=$ap start=$start"

        # now process transmissions between stop and start
        # we only need to look for the transmission cmnds a p mt o
        # and beam definitions
        # make a list of all transmissions so we can check utc sanity
        # and parse any beam args and beam cmnds
        # we assume the beam condition is fixed over all of the treansmissions
        # even no matter where in the block the beam definitions are 
        # except for opacity where inline beam spec is applied
        set ap [$txt(1) index "$stop linestart + 1 lines"]
        array unset transa
        array set transa {}
        # in transmission block we should be able to set smallBeam flag=0 for BT7
        pipeWrite "p cgs 0"
        set tnames {}
        #puts "starting trans block at ap= $ap  start= $start"

        set mtelem ""
        set nitt 1
        set maxnitt 12
        #$txt(2) insert end "transmission block starting\n"
        set locinstalls {}
        set locpair ""
        while {$ap < $start} {
            set line [$txt(1) get $ap "$ap lineend"]
            if {[string length $line] < 1 || [llength $line] < 3} {
                set ap [nextline $ap]
                incr nitt
                continue
            }
            set cmd [lindex $line 2]
            if {[string match PG $cmd] || [string match Q $cmd] || [string match V $cmd]} {
                set ap [nextline $ap]
                incr nitt
                continue
            }
            #puts "trans loop starting nitt= $nitt"
            # firts line after stop is supposed to be sampleHW
            # so that we always get beam geometry into the transmission cells
            if {[string match sampleH* $cmd] || [string match sampleW* $cmd] || [string match sampleM* $cmd]} {
                if [string match sampleHW* $cmd] {
            	    set sampleH [lindex $line 3]
            	    set sampleW [lindex $line 4]
            	} elseif [string match sampleH $cmd] {
            	    set sampleH [lindex $line 3]
            	} elseif [string match sampleW $cmd] {
            	    set sampleW [lindex $line 3]
            	} elseif [string match sampleMosD* $cmd] {
            	    set sampleMosData [lindex $line 3]
            	} elseif [string match sampleMosT* $cmd] {
            	    set sampleMosTran [lindex $line 3]
            	} elseif [string match sampleM* $cmd] {
            	    set sampleMosMin [lindex $line 3]
            	}
            	beamAreas
            	#puts "called beamAreas OK"
            	# set up the transmission cells with updated small beam 0 geometry
            	# NB we are not yet handling monitor transmissions in the console.
            	# -4 is the transmission calc polarizer -3 is analyzer. second number isAnalyzer=0(for polarizer) last number is bs 0 means small beam slit in
            	setBeamGeo -4 0 0
            	setBeamGeo -3 1 0
            } elseif {[string match slit* $cmd] || [string match ana* $cmd] || [string match bs* $cmd] || [string match fs* $cmd] || [string match hc* $cmd] || [string match focus* $cmd]} {
            	# we can handle the rest of the beam goemetry stuff by sending it through beamParse
            	# proc beamParse
            	set gval [lindex $line 3]
            	beamParse [list $cmd\=$gval]
            	beamAreas
            } elseif {[string match ei $cmd]} {
                set Ei [lindex $line 3]
                pipeWrite "p cge $Ei"
                #puts "called p cge $Ei OK"
            } elseif {[string match fru* $cmd]} {
            	set frupdate [lindex $line 3]
            	pipeWrite "p cgu $frupdates($frupdate)"
            } elseif {[string match install* $cmd]} {
                lappend locinstalls $line
            } elseif {[string match cellpair* $cmd]} {
                set locpair $line
            } elseif [regexp -nocase {^m.*|^a.*|^p.*} $cmd] {
            	# still need pi ai to force install when same cellname is repolarized for reuse
            	# otherwise it would be added as a followon
            	# we check for transmission time limit in bt7annot parser
            	set cel [lindex $line 4]
            	#puts "map not PG ap=$ap cmd=$cmd"
            	#if {$nit >= $niterror} {return}
            	# cel will be blank for m mt cmnds
            	#  there is r in cmd if means restart for this cell(or mt)
            	# that is previous trans is invalid and should be ignored
            	# so empty that transa after stripping the r
            	if [string match *r* $cmd] {
            	    set ri [string first r $cmd]
            	    set cmd [string replace $cmd $ri $ri]
            	    array unset transa($cmd$cel)
            	    # maybe good idea to notify that we are donig this restart
            	    lappend warnings "WARNING: restart transmission for $cmd$cel at $ap\nline= $line"
            	}
            	#puts "transmission block cmd=$cmd cel=$cel"
            	set elem $cmd$cel
            	if [regexp -nocase {^m.*} $cmd] {
            	    #puts "found empty elem= $elem"
            	    set mtelem $elem
            	}
				lappend transa($elem) $line
				# tnames must list each elem once
				if {[lsearch $tnames $elem] < 0} {
					lappend tnames $elem
				}
				set tline $line
			}
			set ap [nextline $ap]
			incr nitt
			#if {$nitt > $maxnitt} {return}
        }
        # end ap < start loop

        # combine multiple msrments
        # tnames are in order of measurement
        #puts "transmission block about to combine msrs tnames= $tnames mtelem= $mtelem nit=$nit"

		# for normal transmissions turn on the autofit he3pol flag
		pipeWrite "p cfa 1"
		
        if {[string length $mtelem] < 1} {
            if {[llength $tnames] < 1} {
                lappend warnings "WARNING: no transmissions in transmission block at\n$tline"
            } else {
                lappend warnings "ERROR: transmission block has NO EMPTY at\n$tline"    
        	}

        	set ap [nextline $start]
            # DO not continue but proceed to fr block to check for additional fresults
        } else {
            foreach cel $tnames {
            	puts "parseHe3 about to combineCounts $transa($cel) 0 1"
        	    set chk [combineCounts $transa($cel) 0 1]
        	    if {[string length $chk] > 0} {
        		    lappend warnings $chk
        	    }
            }
            #puts "return after combineCounts chk"
            #return
            # set up Tclpbs for transmission calculations
            # via p ct

            # nw process empty first. combine returns a single translist
            set bchk [beamCheckTrans $transa($mtelem)]
            if {[lindex $bchk 0] == -1} {
            	set transa($mtelem) [lindex $bchk 1]
            	lappend warnings "WARNING: multiple MT transmissions with mixed beam conditions set to the last one at\n$tline"
            }
            set transa($mtelem) [combineCounts $transa($mtelem) 0 0]
            set bi [beamCheckTran $transa($mtelem)]
            set mtutc [lindex $transa($mtelem) 1]
            # set echk from mtmt
            set echks [lindex $transa($mtelem) 6]
            set ei 0
            if {[scan $echks ei=%g eival] > 0} {set ei $eival}

            # save each transblock in transblocks
            set transblock {}
            # do error checking before we log the efficiencies
            foreach cel $tnames {
                if {$cel == $mtelem} {continue}
                # combine multiple msrments into one
                #puts "about to combineCounts for $cel trans($cel)= $transa($cel) llength transa= [llength $transa($cel)]"
            	set bchk [beamCheckTrans $transa($cel)]
            	if {[lindex $bchk 0] == -1} {
            		set transa($cel) [lindex $bchk 1]
            		lappend warnings "WARNING: multiple transmissions for cel=$cel mixed beam conditions set tp last one at\n$tline"
            	}
            	set transa($cel) [combineCounts $transa($cel) 0 0]
            	set bc [beamCheckTran $transa($cel)]
            	set celutc [lindex $transa($cel) 1]
            	if {$bc != $bi} {
            		if {$celutc > $mtutc} {
            			set lastbc $bc
            			# fix the mt
            			set transa($mtelem) [lreplace $transa($mtelem) 5 5 "beam=$bc"]
            		} else {
            			set lastbc $bi
            			# fix the cel
            			set transa($cel) [lreplace $transa($cel) 5 5 "beam=$bi"]
            		}
            		if {$mtutc > $celutc} {set lastbc $bi}
            		set tmsg "WARNING: transmission-beam= $bc for cel=$cel is different than empty-beam= $bi"
            		set tmsg "$tmsg\nwill assume later condition $lastbc applies. If this is not correct edit the console text at\n$tline"
					lappend warnings $tmsg
            	}
            	#puts "after combineCounts for $cel llength transa($cel) = [llength $transa($cel)]"
                set echks [lindex $transa($cel) 6]
                #puts "return after set echks=$echks ei=$ei"
                #return
                if {[scan $echks ei=%g eival] > 0} {
                    if {[expr abs($ei - $eival)] > 0.02} {
                        lappend warnings "ERROR inconsistent inc energies for transmissions at\n$tline"
                    }
                }
			}
			
            # logEfficiency returns the logged event
            logEfficiency ct $transa($mtelem)

			foreach cel $tnames {
                if {$cel == $mtelem} {continue}
            	#puts "return before logEfficiency combineCounts for $cel gives transa($cel)= $transa($cel)"
            	#return
            	
                #if $nit == $niterror
                #puts "parseHe3Log returning before logEfficiency ct transa($cel) at nit= $nit"
                #return
                
            	logEfficiency ct $transa($cel)
            	
                puts "calling for cel=$cel logEfficiency tr $transa($cel)"
            	logEfficiency tr $transa($cel) $transa($mtelem)
            	#puts "parseHe3Log return after logEfficiency tr transa cel=$cel"
            	#return
            }
        }

        # end log efficiency block for detector transmissions
        # if this transblock has a preceding frblock where monitor was measured
        # handle the monitor transmissions for the polarizer cell and add them to the transblock
        # must be at least one frblocks already parsed and in frpairlist to have data monitor counts
        while {$useMonitorForTrans == 1 && [llength $frblocks] > 0} {
        	puts "parseHe3log starting monitors block"
        	# get polarizer used in previous frblock
        	set pname [lindex [lindex $frpairlist 2] 0]
        	set cel $pname
        	set typ p
        	set celli [lindex [lindex $frpairlist 1] 0]
        	set futci [lindex $frpairlist 4]
        	set frstartdate [utcTOdatestr $futci 1]
        	set futcf [lindex $frpairlist 5]
        	if {$futci <= 0 || $futcf <= 0} {
        		lappend warnings "ERROR: previous frblock ill-defined time range at $frstartdate"
        		break
        	}
        	set dutc [expr $futcf - $futci]
        	# find the polarizer number in polarizers for polarizer in beam for this frblock that starts at futc
        	# so its utci should be just before futci
        	set pnum 0
        	for {set i $npolarizers} {$i > 0} {incr i -1} {
        		set ip [expr $i - 1]
        		set poli [lindex $polarizers $ip]
        		set utci [lindex $poli 0]
        		set pnamei [lindex $poli 1]
        		set pinsindex [lindex $poli 2]
        		if {$utci < $futci} {
        			if {$pname != $pnamei} {
        				lappend warnings "ERROR: polarizer cellname in frpair= $pname is not the same as installed= $pnamei at $frstartdate"
        			} else {
        				set pnum $i
        			}
        			break
        		}
        	}
        	#puts "parseHe3Log monitors pnum= $pnum pname= $pname"
        	if {$pnum == 0} {break}
        	# NOT will need the beam geom for the cellpair in this previous frblock
        	# the pair index is just current length of frpairlists - 1
        	set pairi [expr [llength $frpairlists] - 1]
        	# DONT NEED beam geometry for MONITOR transmissions
        	# The relevant geometry doesnt change except for focus conditions
        	set geolst [lindex $pairsGeom $pairi]
        	# setup list of  utcrange geo  to do the monitor transmissions in this previous frblock
        	set geosBYutcr {}
        	set geo [lindex $geolst 0]
        	set utcg [lindex $geo 1]
        	set utcgi $utcg
        	set geop [lindex $geo 2]
        	set geoP [lindex $geop 0]
        	set geoPi $geoP
        	for {set i 1} {$i < [llength $geolst]} {incr i} {
        		set geoi [lindex $geolst $i]
        		set utcgi [lindex $geoi 1]
        		set geopi [lindex $geoi 2]
        		set geoPi [lindex $geopi 0]
        		lappend geosBYutcr [list $utcg $utcgi $geoP]
        		set geoP $geoPi
        		set utcg $utcgi
        	}
        	lappend geosBYutcr [list $utcg $futcf $geoP]
        	# NB we want to pass the monitor-trans beam geom directly to Tclpbs
        	# bypassing the current geometry globals which we are tracking sequentially via the he3log
        	# so write an equivalent to setBeamGeo that takes the tclpbs list from pairsGeom
        	set pstr $pnum\=$pname
        	#setBeamGeoD
        	
			set bestmei 0
			set bestrangei 0
			set inexists [info exists monitorHe3bothPGin($pstr)]

        	puts "parseHe3Log checking exists monitorHe3bothPGin($pstr)= $inexists  futci=$futci futcf=$futcf"
			if {$inexists == 1} {
				set monI $monitorHe3bothPGin($pstr)
				#puts "parseHe3Log nmonI= [llength $monI]"
				array unset monIe
				array set monIe {}
				#puts "parseHe3Log monitors PGin list= $monI"
				foreach m $monI {
					#puts "parseHe3Log process pgin mrat= $m"
					set mutc [lindex $m 0]
					# all should be in the frblock time range due to pstr selection but sanity check anyway
					if {$mutc < $futci || $mutc > $futcf} {
						#puts " getMratMT rejected mutc= $mutc"
						continue
					}
					set mrat [lindex $m 1]
					set mrate [lindex $m 2]
					set me [format %.2f [lindex $m 3]]
					lappend monIe($me) [list $mutc $mrat $mrate]
				}
				puts "parseHe3Log monIe= [array get monIe]"
				foreach me [array names monIe] {
					set mutcmx 0
					set mutcmn 9999999999
					foreach m $monIe($me) {
						set mutc [lindex $m 0]
						if {$mutc < $mutcmn} {set mutcmn $mutc}
						if {$mutc > $mutcmx} {set mutcmx $mutc}
					}
					if {$mutcmn >= $mutcmx}	{continue}
					set incutcr [getINCutcrr [list $mutcmn $mutcmx]]
					if {[llength $incutcr] < 2} {
						puts "getINCutcrr failed for $mutcmn $mutcmx"
						continue
					}
					set range [expr [lindex $incutcr 1] - [lindex $incutcr 0]]
					puts "getINCutcrr returned $incutcr"
					# can we get MT mrat for this energy
					set mratMT [getMratMT $me 1 0 $incutcr]
					puts "getMratMT returned $mratMT"
					if {[llength $mratMT] < 2} {continue}
					if {$range < $bestrangei} {continue}
					set bestrangei $range
					set bestmei $me
					set bestMTi $mratMT
				}
			}
			set bestmeiVS 0
			set bestrangeiVS 0
			set inexistsVS [info exists monitorHe3bothVSPGin($pstr)]
			if {$inexistsVS == 1} {
				set monI $monitorHe3bothVSPGin($pstr)
				#puts "parseHe3Log nmonI= [llength $monI]"
				array unset monIeVS
				array set monIeVS {}
				#puts "parseHe3Log monitors PGin list= $monI"
				foreach m $monI {
					#puts "parseHe3Log process pgin mrat= $m"
					set mutc [lindex $m 0]
					# all should be in the frblock time range due to pstr selection but sanity check anyway
					if {$mutc < $futci || $mutc > $futcf} {
						#puts " getMratMT rejected mutc= $mutc"
						continue
					}
					set mrat [lindex $m 1]
					set mrate [lindex $m 2]
					set me [format %.2f [lindex $m 3]]
					lappend monIeVS($me) [list $mutc $mrat $mrate]
				}

				foreach me [array names monIeVS] {
					set mutcmx 0
					set mutcmn 9999999999
					foreach m $monIeVS($me) {
						set mutc [lindex $m 0]
						if {$mutc < $mutcmn} {set mutcmn $mutc}
						if {$mutc > $mutcmx} {set mutcmx $mutc}
					}
					if {$mutcmn >= $mutcmx}	{continue}
					set incutcr [getINCutcrr [list $mutcmn $mutcmx]]
					set range [expr [lindex $incutcr 1] - [lindex $incutcr 0]]
					# can we get MT mrat for this energy
					set mratMT [getMratMT $me 1 1 $incutcr]
					if {[llength $mratMT] < 2} {continue}
					if {$range < $bestrangeiVS} {continue}
					set bestrangeiVS $range
					set bestmeiVS $me
					set bestMTiVS $mratMT
				}
			}
			# which is better noVS or VS
			set vsi 0
			if {$bestrangeiVS > $bestrangei} {
				set bestrangei $bestrangeiVS
				set bestmei $bestmeiVS
				set bestMTi $bestMTiVS
				set vsi 1
				unset monIe
				array set monIe [array get monIeVS]
			}

			set bestmeo 0
			set bestrangeo 0
			set outexists [info exists monitorHe3bothPGout($pstr)]
        	#puts "parseHe3Log checking exists monitorHe3bothPGout($pstr)= $outexists"
			if {$outexists == 1} {
				set monO $monitorHe3bothPGout($pstr)
				array unset monOe
				array set monOe {}
				foreach m $monO {
					#puts "parseHe3Log process pgin mrat= $m"
					set mutc [lindex $m 0]
					# all should be in the frblock time range due to pstr selection but sanity check anyway
					if {$mutc < $futci || $mutc > $futcf} {continue}
					set mrat [lindex $m 1]
					set mrate [lindex $m 2]
					set me [format %.2f [lindex $m 3]]
					lappend monOe($me) [list $mutc $mrat $mrate]
				}
				foreach me [array names monOe] {
					set mutcmx 0
					set mutcmn 9999999999
					foreach m $monOe($me) {
						set mutc [lindex $m 0]
						if {$mutc < $mutcmn} {set mutcmn $mutc}
						if {$mutc > $mutcmx} {set mutcmx $mutc}
					}
					if {$mutcmn >= $mutcmx}	{continue}
					set incutcr [getINCutcrr [list $mutcmn $mutcmx]]
					set range [expr [lindex $incutcr 1] - [lindex $incutcr 0]]
					# can we get MT mrat for this energy
					set mratMT [getMratMT $me 0 0 $incutcr]
					if {[llength $mratMT] < 2} {continue}
					if {$range < $bestrangeo} {continue}
					set bestrangeo $range
					set bestmeo $me
					set bestMTo $mratMT
				}
			}
			set bestmeoVS 0
			set bestrangeoVS 0
			set outexistsVS [info exists monitorHe3bothVSPGout($pstr)]
        	#puts "parseHe3Log checking exists monitorHe3bothPGout($pstr)= $outexists"
			if {$outexistsVS == 1} {
				set monO $monitorHe3bothVSPGout($pstr)
				array unset monOeVS
				array set monOeVS {}
				foreach m $monO {
					#puts "parseHe3Log process pgin mrat= $m"
					set mutc [lindex $m 0]
					# all should be in the frblock time range due to pstr selection but sanity check anyway
					if {$mutc < $futci || $mutc > $futcf} {continue}
					set mrat [lindex $m 1]
					set mrate [lindex $m 2]
					set me [format %.2f [lindex $m 3]]
					lappend monOeVS($me) [list $mutc $mrat $mrate]
				}
				foreach me [array names monOeVS] {
					set mutcmx 0
					set mutcmn 9999999999
					foreach m $monOeVS($me) {
						set mutc [lindex $m 0]
						if {$mutc < $mutcmn} {set mutcmn $mutc}
						if {$mutc > $mutcmx} {set mutcmx $mutc}
					}
					if {$mutcmn >= $mutcmx}	{continue}
					set incutcr [getINCutcrr [list $mutcmn $mutcmx]]
					set range [expr [lindex $incutcr 1] - [lindex $incutcr 0]]
					# can we get MT mrat for this energy
					set mratMT [getMratMT $me 0 1 $incutcr]
					if {[llength $mratMT] < 2} {continue}
					if {$range < $bestrangeoVS} {continue}
					set bestrangeoVS $range
					set bestmeoVS $me
					set bestMToVS $mratMT
				}
			}
			# which is better noVS or VS
			set vso 0
			if {$bestrangeoVS > $bestrangeo} {
				set bestrangeo $bestrangeoVS
				set bestmeo $bestmeoVS
				set bestMTo $bestMToVS
				set vso 1
				unset monOe
				array set monOe [array get monOeVS]
			}


			puts "parseHe3Log bestrangei= $bestrangei  bestrangeo= $bestrangeo"
			if {$bestrangei <= 0 && $bestrangeo <= 0} {break}

			set montranslst {}
        	if {$bestrangeo > $bestrangei} {
        		# use the monOe transmissions at e=bestmeo  sort on utc should not be necessary
        		# but we need to examine the time dependence so do it to make sure
        		set mratlst [lsort -integer -index 0 $monOe($bestmeo)]
        		set mratmt [lindex $bestMTo 0]
        		set mratmte [lindex $bestMTo 1]
        		set pgst "u"
        		set pgsi 1
        		if {$vso == 1} {
        			set pgst "v"
        			set pgsi 3
        		}
        		set vs $vso
        	} else {
        		set mratlst [lsort -integer -index 0 $monIe($bestmei)]
        		set mratmt [lindex $bestMTi 0]
        		set mratmte [lindex $bestMTi 1]
        		set pgst "p"
        		set pgsi 2
        		if {$vsi == 1} {
        			set pgst "P"
        			set pgsi 4
        		}
        		set vs $vsi
        	}
        	#puts "monitors for $pstr = $mratlst"
        	# examine the mratlst for decreases that might indicate reactor power loss
        	# or sudden cell polarization loss (these have occured unexplained from time to time)
        	# if the rate recovers, it must be a reactor power loss.
        	# If the rate drop does not recover this may be a problem. Report this
        	# If the rate increases this also needs examination.
        	
        	# from start of mratlst find how many points to get a reasonable negative slope
        	# Then project next mrat and compare with measured. If within CI update the slope estimate
        	# and continue.
        	# For anomalous rate find the next point that satisfies projection with exponential correction
        	set nmrat [llength $mratlst]
        	puts "try to estimate decay from mrat list= $mratlst"
        	if {$nmrat < 2} {
        		lappend warnings "Cant estimate decay from single monitor rate at\n$tline\n$frstartdate"
        		break
        	}
        	set lnrs [linefit $mratlst 3 0 0 {}]
        	if {[llength $lnrs] < 1} {
        		lappend warnings "Failed to estimate decay from monitor rates at\n%tline\n$frstartdate"
        		break
        	}
        	set slopee [lindex $lnrs 0]
        	set slope [lindex $slopee 0]
        	if {$slope >= 0.} {
        		lappend warnings "Failed to find negative slope for mrat time-dep at\n$tline"
        		break
        	}
        	set chi [lindex $lnrs 2]
        	set outliers [lindex $lnrs 3]
        	#puts "monitors for $pstr have linefit chi= $chi  outliers= $outliers"
			# and expfit with refit both at CI k=3
			set ers [expfit $mratlst 3 1 3 {}]
			if {[llength $ers] < 2} {break}
			puts "expfit returned $ers"
			set Tcl [lindex $ers 0]
			set Mlst [lindex $ers 1]
			set Tc [lindex $Tcl 0]
			set Tchrs [expr $Tc/3600.]
			# if Tc comes back negative or too small we must have thrown out datapoints with reduced mrat that made the linefit
			# give negative slope
			# so punt and try the expfit with zero CI and no refit
			set nooutliers 0
			if {$Tchrs <= 5} {
				set nooutliers 1
				set ers [expfit $mratlst 0 0 0 {}]
				puts "expfit second time returned $ers"
				if {[llength $ers] < 2} {break}
				set Tcl [lindex $ers 0]
				set Mlst [lindex $ers 1]
				set Tc [lindex $Tcl 0]
				set Tchrs [expr $Tc/3600.]
				if {$Tchrs <= 5} {
					lappend warnings "ERROR: Failed to find reasonable decay from monitor rates at\n$tline"
					break
				}
			}
			set M [lindex $Mlst 0]
			set chie [lindex $ers 2]
			# the refit outliers
			set eoutliers [lindex $ers 3]
			# the outliers from the original fit. These two lists should have different sets of excluded indices in mratlst
			set preveoutliers [lindex $ers 4]
			#puts "monitors for $pstr have expfit chie= $chie Tc= $Tc Y0= $M  outliers= $eoutliers prevoutliers= $preveoutliers"
			set excli {}
			foreach prev $preveoutliers {
				lappend excli [lindex $prev 0]
			}
			foreach out $eoutliers {
				set io [lindex $out 0]
				if {[lsearch $excli $io] < 0} {lappend excli $io}
			}
			set nexcli [llength $excli]
			set rejfrac [format %.2f [expr 1.0*$nexcli/$nmrat]]
			if {$rejfrac >= $mratRejFrac} {
				lappend warnings "WARNING: fraction of mrats rejected= $rejfrac  for ci >= $mratCI excedes $mratRejFrac of total at $frstartdate"
			}
			if {$chie > $mratChired} {
				lappend warnings "WARNING: exp fit mrats has chi= $chie even after outliers removed at $frstartdate"
			}
        	set ree [expr $mratmte/$mratmt]
        	for {set i 0} {$i < $nmrat} {incr i} {
        		if {$nooutliers == 0 && [lsearch $excli $i] >= 0} {continue}
        		set m [lindex $mratlst $i]
        		set mutc [lindex $m 0]
        		set mrat [lindex $m 1]
        		set mrate [lindex $m 2]
        		set mtrans [expr $mrat/$mratmt]
        		set re [expr $mrate/$mrat]
        		set mtranse [expr $mtrans*sqrt($re*$re + $ree*$ree)]
        		lappend montranslst [list $mutc $mtrans $mtranse]   	
        	}
        	# make sure sorted on increasing utc
        	set montranslst [lsort -integer -index 0 $montranslst]
        	#puts "parseHe3Log monitors for $pstr translst= $montranslst"
        	if {[llength $montranslst] < 2} {
        		break
        		# cant get decay from less than two transmissions
        	}

			# turn off autofit to collect add monitor data
			pipeWrite "p cfa 0"
        	# make sure monitorArea is uptodate
        	monitorArea
        	setMonitorGeo $celli
        	# clear all previous monitor data for this cell
        	pipeWrite "p cx$pgst $celli"
        	set broken 0
        	foreach mtr $montranslst {
        		set mutc [lindex $mtr 0]
				#set geoutcr [getGEOutcr $mutc $geosBYutcr]
				# this also set the global geoATutc as current geomlist
				#if {[llength $geoutcr] < 2} {continue}
				set t [lindex $mtr 1]
				set te [lindex $mtr 2]
				pipeWrite "p ct$pgst $celli $mutc $t $te"
        		if {$nerr > 0 || $nwarn > 0} {
            		if {$nerr > 0} {
            		    Dialog_Warn [join $errlines "\n"]
            		} elseif {$nwarn > 0} {
                		Dialog_Warn [join $warnlines "\n"]
            		}
            		set broken 1
            		break
        		}
        	}
        	if {$broken == 1} {break}
        	# print all the data
			pipeWrite "p ct$pgst $celli a"

            # there are two header lines followed by the data
            # secs ei trans transerr he3pol he3polErr
            if {$nread < 3} {
                Dialog_Warn "monitor transmission for cel= $cel has NO data"
                break
            }
            # print the mp mrat to transmission data in txt(6)
            $txt(6) insert end "monitor trans-he3pol for PG=$pgst VS=$vs $pstr  rejected $nexcli outliers\n"
            set nd [expr $nread - 2]
            set il [expr $nread - 1]
            for {set k 2} {$k < $nread} {incr k} {
            	set rl [lindex $readlines $k]
            	set sec [lindex $rl 0]
            	set ds [utcTOdatestr $sec 1]
            	set ei [lindex $rl 1]
            	set tr [lindex $rl 2]
            	set trerr [lindex $rl 3]
            	set pol [lindex $rl 4]
            	set polerr [lindex $rl 5]
            	if {[catch {set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-6.4f %-6.4f  %-6.4f %-6.4f  %-6g" $ds $sec mtp $typ $cel $tr $trerr $pol $polerr $ei]}]} {
            		set estr "failed rl= $rl"
            	}
            	set event [concat $ds $sec $cel $typ [lrange $rl 1 5]]
            	#lappend transblock $estr\n
            	#$txt(2) insert end $estr\n
            	$txt(6) insert end $estr\n
            	#lappend efficiencyLog $event
            	# check for decay if we get 2 data points
            }
            if {$nd < 2} {break}
            # now fit these data
            #puts "About to fit monitor he3pol"
            pipeWrite "p cf $celli $pgsi"
        	if {$nerr > 0 || $nwarn > 0} {
            	if {$nerr > 0} {
            	    Dialog_Warn [join $errlines "\n"]
            	} elseif {$nwarn > 0} {
                	Dialog_Warn [join $warnlines "\n"]
            	}
            	break
        	}
            # get back the decay for these data
            pipeWrite "p ch$pgst $celli"
            # return header line and data line:
            # isfit Pi Pierr Pf Pferr T Terr deltaHrs cellname
            if {$nread < 2} {
            	puts "NO monitor decay"
            	break
            }
        	if {$nerr > 0 || $nwarn > 0} {
            	if {$nerr > 0} {
            	    Dialog_Warn [join $errlines "\n"]
            	} elseif {$nwarn > 0} {
                	Dialog_Warn [join $warnlines "\n"]
            	}
            	break
        	}
            set rl [lindex $readlines 1]

            set pi [lindex $rl 1]
            set pie [lindex $rl 2]
            set pf [lindex $rl 3]
            set pfe [lindex $rl 4]
            set T [lindex $rl 5]
            set Te [lindex $rl 6]
            set dT [lindex $rl 7]
            set event [concat $ds $sec $typ decay $rl]
            if {[catch {set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-6.4f %-6.4f  %-6.4f %-6.4f  %-6g %-6g  %-6g" $ds $sec mpd $typ $cel $pi $pie $pf $pfe $T $Te $dT]}]} {
            	set estr "format failed for rl= $rl"
            }
            lappend transblock $estr\n
        	
        	break
        }
        
        # when transmission block is finished write out the installs cellpair and start polbeam
        # find the number of files just before stop
        set stop [findNextReg $start stop]
        if {! $stop} {set stop $endtxt}
        set stop [$txt(1) index "$stop linestart"]
        set stoptxt [$txt(1) get $stop "$stop lineend"]
        set stoputc [lindex $stoptxt 1]
        set fileln [$txt(1) index "$stop - 1 lines"]
        set filetxt [$txt(1) get $fileln "$fileln lineend"]
        set filewd [lsearch $filetxt "file"]
        #if {$filewd < 0} {set filetxt ""}
        foreach ins $locinstalls {
            lappend transblock $ins\n
            #$txt(2) insert end $ins\n
        }
        # 0 is not a valid pair number 1:Npair
        set cellpairn 0
        #$txt(2) insert end $locpair\n
        lappend transblock $locpair\n
        if {[llength $locpair] > 2} {
        	set cellpairtxt [lindex $locpair 2]
        	scan $cellpairtxt cellpair=%d cellpairn
        }
        #puts "cellpairn= $cellpairn"
        #lappend transblock "cellpair= $cellpairn\n"
        lappend transblocks $transblock
        lappend transpairs $cellpairn
        

        set frblock {}
        lappend frblock $startline\n
        lappend frblock $filetxt\n
        #$txt(2) insert end "start pol beam\n"

        # save the total block nfiles until the end of the frblock
        #$txt(2) insert end $filetxt\n

        #puts "return after trans block"
        #return
        # finished parsing transission block from start to stop
        # so we should be at start or endtxt
        # now starting at start look for next stop 
        lappend frblock [$txt(1) get $stop "$stop lineend"]\n
        lappend frblocks $frblock
        # we will inert the fresults into frblocks during post processing
        #$txt(2) insert end [$txt(1) get $stop "$stop lineend"]\n
        set ap [$txt(1) index "$start linestart + 1 lines"]
        set startfr $ap
        set startline [$txt(1) get $ap "$ap lineend"]
        #puts "starting polarized-beam block at start= $ap  stop= $stop \n  startline= $startline"
        #if {$nit == $niterror} {return}
        #return
        # process fr fr ... frend  blocks
        # put all fr upto frend else stop into array for procesing
        # note that pb fr o pb frb cmnds in console mark the end of previous fr block
        # so fr and frb blocks dont mix.
        # eventually we will have to handle frscan where the cts are in scans instead of Counter:
        # and where frscan pkpos  can be speced
        set frpairlist {}
        #puts "init frpairlist empty"
        set fr {}
        set nfiles 0
        # set the first fr flag
        # only have to process frupdate and fresult
        set firstfr 1
        while {$ap < $stop} {
            set line [$txt(1) get $ap "$ap lineend"]
            #puts "frblock ap=$ap processing line= $line"
            if {[llength $line] < 3} {
                set ap [nextline $ap]
                continue
            }
            set cmd [lindex $line 2]
            if [string match files $cmd] {
                set nfiles [lindex $line 3]
            } elseif [string match fru* $cmd] {
                # if there is no data skip

            	set frupdate [lindex $line 3]
            	pipeWrite "p cgu $frupdates($frupdate)"
            } elseif [string match fresult $cmd] {
                set hasdata 0
                foreach md {V0 Q0 V1 Q1} {
                    set imd [lsearch $line $md]
                    if {$imd < 0} {continue}
                    set dlst [lindex $line [expr $imd + 1]]
                    if {[llength $dlst] < 1} {continue}
                    set hasdata 1
                    break
                }
                if {$hasdata == 0} {
                    set ap [nextline $ap]
                    continue
                }
                lappend fr $line
            }
            set ap [nextline $ap]
        }
        set nfr [llength $fr]
        set ap [nextline $stop]
        if {$nfiles < 1} {
            lappend warnings "WARNING: NO data files in datablock $start to $stop  nfr= $nfr at\n$line"
        } elseif {$nfr < 1} {
            lappend warnings "WARNING: NO flip-ratio in datablock $start to $stop  nfiles= $nfiles at\n$line"
            #continue
        }

        # have fr to process
        # the first fr for a mode bs should always be used to get the transport efficiency
        # there can be a consecutive frb that goes with the first fr
        # subsequent fr are either for transport checks or polarization consistency checks
        # and this choice is via a global flag frupdate
        # If only one of VQ is measured we have to assume transport is same for other mode.
        #
        set i 0
        set n 0
        set nbg 0
        # make sure we tell tclpbs that both cells are in the beam
        pipeWrite "p cgb 3"
        # get cellindex and names for this frblock
        # get back the indices of cells in the beam        
        pipeWrite "p cgbn"
        # writes PAcellindices polindex anaindex
        set rl [lindex $readlines 0]
        set celli [lindex $rl 1]
        set cellf [lindex $rl 2]
        # and the cellnames
        pipeWrite "p cgb"
        set rl [lindex $readlines 1]
        set pname [lindex $rl 1]
        set aname [lindex $rl 3]
        # store cells and fr in frpairlist
        #puts "before addto frpairlists frpairlists= $frpairlists"
        lappend frpairlist $cellpairn
        lappend frpairlist [list $celli $cellf]
        lappend frpairlist [list $pname $aname]
        lappend frpairlist $fr
        lappend frpairlist $startutc $stoputc
        lappend frpairlists $frpairlist
        lappend frpairs $cellpairn
        #puts "after frpairlists= $frpairlists"
        incr nit
        # NB we already set ap above for the next line in the loop
        #puts "finished fr block ap=$ap nit=$nit stop=$stop"
    }
    # finished transmission and fr blocks
    
    # now process the frblocks in frpairlists
    #puts "frpairlists= $frpairlists"
    set newfrblocks {}
    #puts "starting fr loop: nfrpairlists= [llength $frpairlists] nfrblocks= [llength $frblocks]"
    foreach frpl $frpairlists frblock $frblocks {
        # each pairlist (frpl) can have multiple flipratio measurements in its fr element
    	set cellpn [lindex $frpl 0]
    	set cellpi [expr $cellpn - 1]
    	# dont skip invalid pairs
    	#if $cellpi < 0 continue
    	set cellilst [lindex $frpl 1]
    	set celli [lindex $cellilst 0]
    	set cellf [lindex $cellilst 1]
    	set cellnlst [lindex $frpl 2]
    	set fr [lindex $frpl 3]       
    	# make sure the cellpair exists in tclpbs and has celli cellf cell install indices
    	set i 0
    	set msrdstates {}
    	if {$cellpi >= 0} {
    	    pipeWrite "p cgp $cellpi $celli $cellf"
    	    set nfr [llength $fr]
    	} else {           
            set nfr 0
        }
        set n 0
        set nbg 0
        set frlines {}
        while {$i < $nfr} {
            # loop throuth the frs for this pair
            # find frb fr pair else fr
        	set fi [lindex $fr $i]
        	set fiutc [lindex $fi 1]
        	# fit is the fittype= fr or frb
        	set fit [lindex $fi 3]
        	# get the nfiles bfld temp as well
        	set nfilestr [lindex $fi 15]
        	set nfil 0
        	if {[scan $nfilestr nfiles=%d nfi] == 1} {set nfil $nfi}
        	# this will get nfiles into the fr data in tclpbs
        	pipeWrite "p cgfn $nfi"
        	set bstr [lindex $fi 16]
        	set bfld 0
        	if {[scan $bstr bfld=%f fld] == 1} {set bfld $fld}
        	# this will get nfiles into the fr data in tclpbs
        	pipeWrite "p cgfb $bfld"
        	set tstr [lindex $fi 17]
        	set temp 0
        	if {[scan $tstr temp=%f tem] == 1} {set temp $tem}
        	# this will get nfiles into the fr data in tclpbs
        	pipeWrite "p cgft $temp"
        	
        	set fib 0
        	if [string match frb* $fit] {
        	    # find the companion fr
        	    set fib 1
        	    set fin 0
        	    set i1 [expr $i + 1]
        	    while {$i1 < $nfr} {
        	        set fi1 [lindex $fr $i1]
        	        set fi1utc [lindex $fi1 1]
        	        set fi1t [lindex $fi1 3]
        	        if {$fi1t == "fr"} {
        	            set fin 1
        	            set fn $fi1
        	            set fnutc [lindex $fn 1]
        	            break
        	        }
        	        incr i1
        	    }
        	    # i1 is the fr normal index if fin flag is set
        	    if {$fin == 0} {
        	        lappend warnings "ERROR: No fr for frb=\n$fi"
        	        # break the i < nfr loop as we are done processing fr frb for this block
        	        break
        	    }

        	    # check if the nearest frb is within timelimit
        	    set fb1 [lindex $fr [expr $i1 - 1]]
        	    set fb1utc [lindex $fb1 1]
        	    if {[expr abs($fnutc - $fb1utc) > $maxfrwait]} {
        	        lappend warnings "ERROR: background frb=\n$fb1\ndiscarded as older by more than $maxfrwait secs than fr=\n$fn"
        	        set fib 0
        	    } elseif {[expr $i1 - 2] >= $i} {
        	        set fb2 [lindex $fr [expr $i1 -2]]
        	        set fb2utc [lindex $fb2 1]
        	        if {[expr abs($fnutc - $fb2utc) > $maxfrwait]} {
        	            lappend warnings "WARNING: secondary background frb=\n$fb2\ndiscarded as older by more than $maxfrwait secs than fr=\n$fn"
                        set fib 1
                        set fb $fb1
        	        } else {
        	            set fib 1
        	            set fb [combineFR $fb1 $fb2]
        	            lappend warnings "WARNING: combining two backgrounds\n$fb1\n$fb2"
        	        }
        	    } else {
        	        set fib 1
        	        set fb $fb1
        	    }
        	    set i [expr $i1 + 1]
            } else {
                # $i must be a normal fr result. Check for up to 2 following frb backgrounds
                set fin 1
                set fn $fi
                set fnutc $fiutc
                set i1 [expr $i + 1]
                set nbg 0
                while {$i1 < $nfr && $nbg < 3} {
                    set fi1 [lindex $fr $i1]
                    set fi1utc [lindex $fi1 1]
                    set fi1t [lindex $fi1 3]
                    if [string match frb* $fi1t] {
         	            if {[expr abs($fnutc - $fi1utc) > $maxfrwait]} {break}
         	            incr nbg                 
                    } else {
                        break
                    }
                }
                if {$nbg == 2} {
                    set fb1 [lindex $fr [expr $i + 1]]
                    set fb2 [lindex $fr [expr $i + 2]]
                    set fib 1
                    set fb [combineFR $fb1 $fb2]
                } elseif {$nbg == 0} {
                    set fib 0
                } elseif {$nbg == 1} {
                    set fib 1
                    set fb [lindex $fr [expr $i + 1]]
                }
                set i [expr $i + $nbg + 1]
            }

            # process the beam0 frs at index V0=6 Q0=8
            setBeamGeo $celli 0 0
            setBeamGeo $cellf 1 0
            #set msrdstates {}
            foreach {st vq} {V0 0 Q0 1} {
                # the result list is just after the identifier
                set sti [expr [lsearch $fi $st] + 1]
                set frlst [lindex $fi $sti]
                set fblst {}
                if {$fib} {set fblst [lindex $fb $sti]}
                if {[llength $frlst] > 0} {
                    processFRlst $cellpi $frlst $fblst 0 $vq
                    if {[llength $frwarnings] > 0} {
                        set warnings [concat $warnings $frwarnings]
                    }
                    set msrd 0$vq
                    if {[lsearch $msrdstates $msrd] < 0} {lappend msrdstates $msrd}
                }
            }
            setBeamGeo $celli 0 1
            setBeamGeo $cellf 1 1
            foreach {st vq} {V1 0 Q1 1} {
                # the result list is just after the identifier
                set sti [expr [lsearch $fi $st] + 1]
                set frlst [lindex $fi $sti]
                set fblst {}
                if {$fib} {set fblst [lindex $fb $sti]}
                if {[llength $frlst] > 0} {
                    processFRlst $cellpi $frlst $fblst 1 $vq
                    if {[llength $frwarnings] > 0} {
                        set warnings [concat $warnings $frwarnings]
                    }
                    set msrd 1$vq
                    if {[lsearch $msrdstates $msrd] < 0} {lappend msrdstates $msrd}
                }
            }

            #incr i
   		}
   		# add to frlines list puts results from this frpair
        foreach md $msrdstates {
            set sb [string index $md 0]
            set vq [string index $md 1]
            #puts "about to p crp celi=$cellpi sb=$sb vq=$vq 0"
            pipeWrite "p crp $cellpi $sb $vq 0"
            foreach rl $readlines {
                lappend frlines $rl\n
                #puts $rl
            }
        }
   		# ends processing frpl for one frpairlist frblock
        # recreate frblock
        set newfrblock {}
        lappend newfrblock [lindex $frblock 0]
        foreach frl $frlines {lappend newfrblock $frl}
        lappend newfrblock [lindex $frblock 1]
        lappend newfrblock [lindex $frblock 2]
        lappend newfrblocks $newfrblock
   	}
   	# now output to txt(2) alternating transblocks and newfrblocks
   	set ntb [llength $transblocks]
   	set nfb [llength $newfrblocks]
	puts "writing to He3-efficiencies ntransblocks= $ntb  nfrblocks= $nfb"
   	for {set i 0} {$i < $ntb} {incr i} {
   		set tbl [lindex $transblocks $i]
   		#set tpn [lindex $transpairs $i]
   		#if {$tpn < 1} {continue}
   		#set fpn 0
   		#set ifpn [lsearch $frpairs $tpn]
   		#if $ifpn >= 0
   	    #set fpn 1
   		set fbl [lindex $newfrblocks $i]
		$txt(2) insert end "transmission block\n"
   	    foreach tb $tbl {
   	        $txt(2) insert end $tb
   	    }
   	    $txt(2) insert end "fr block\n"
   	    foreach fb $fbl {
   	        $txt(2) insert end $fb
   	    }
   	}

    if {[llength $warnings] > 0} {
    	$txt(4) insert end "\n\nHe3-log ERRORS and WARNINGS\n"
        foreach w $warnings {
            $txt(4) insert end $w\n\n
        }
        #Dialog_Warn [join $warnings "\n"]
    }
    butenable 3
    #$he3B configure -bg grey -state disabled
	#nb select 2
	selectTab 2
}


proc frbgCor {ctrlst ctrlstb} {
    global frwarnings
    global frc frcerr
    set nsfctr [lindex $ctrlst 0]
    set nsfctrerr [lindex $ctrlst 1]
    set sfctr [lindex $ctrlst 2]
    set sfctrerr [lindex $ctrlst 3]
    set nsfctrb [lindex $ctrlstb 0]
    set nsfctrberr [lindex $ctrlstb 1]
    set sfctrb [lindex $ctrlstb 2]
    set sfctrberr [lindex $ctrlstb 3]
    if {$sfctrb > $sfctr} {
        lappend frwarnings "ERROR: background spin-flip countrate is higher than at peak"
        return 0
    }
    if {$nsfctrb > $nsfctr} {
        lappend frwarnings "ERROR: background non-spin-flip countrate is higher than at peak"
        return 0
    }
    set dn [expr $nsfctr - $nsfctrb]
    set ds [expr $sfctr - $sfctrb]
    set frc [expr $dn/$ds]
    set nsferrsq [expr $nsfctrerr*$nsfctrerr]
    set nsfberrsq [expr $nsfctrberr*$nsfctrberr]
    set sferrsq [expr $sfctrerr*$sfctrerr]
    set sfberrsq [expr $sfctrberr*$sfctrberr]
    set frerr [expr ($nsferrsq + $nsfberrsq)/($dn*$dn) + ($sferrsq + $sfberrsq)/($ds*$ds)]
    set frcerr [expr $fr*$frerr]
    return 1
}

proc linefit {xyelsts ci refit newci prevoutliers} {
	# fit line to xye lists but exclude any prevoutliers
	# record outliers that excede k=ci confidence interval
	# refit if requested
	# return slope slopeErr y0 y0Err (fitted value at first pt)
	# also return chi and the list of relative deviations from the fit
	set n [llength $xyelsts]
	if {$n < 2} {return {}}
	set previ {}
	foreach prev $prevoutliers {
		set i [lindex $prev 0]
		lappend previ $i
	}
	# get indices in xyelsts that we will fit
	set fi {}
	for {set i 0} {$i < $n} {incr i} {
		if {[lsearch $previ $i] >= 0} {continue}
		lappend fi $i
	}
	set nprevi [llength $previ]
	set nf [llength $fi]
	if {$nf < 2} {return {}}
	set p0 [lindex $xyelsts [lindex $fi 0]]
	set x0 [lindex $p0 0]
	set y0 [lindex $p0 1]
	set e0 [lindex $p0 2]
	if {$nf == 2} {
		set p1 [lindex $xyelsts [lindex $fi 1]]
		set x1 [lindex $p1 0]
		set y1 [lindex $p1 1]
		set e1 [lindex $p1 2]
		set dx [expr $x1 - $x0]
		if {[expr abs(dx)] <= 0.} {return {}}
		set dy [expr $y1 - $y0]
		set slope [expr $dy/$dx]
		return [list [list $slope 0] [list $y0 0] 0 {} $prevoutliers]
	}
	set sum 0
	set sumxsq 0
	set sumy 0
	set sumx 0
	set sumxy 0
	set pend [lindex $xyelsts [lindex $fi end]]
	set xend [lindex $pend 0]
	set xrange [expr 1.*($xend - $x0)]
	foreach pi $fi {
		set p [lindex $xyelsts $pi]
		# in case x is large use x0 as origin
		# this also makes intercept the y value at x0
		set x [expr [lindex $p 0] - $x0]
		set y [lindex $p 1]
		set e [lindex $p 2]
		set esq [expr $e*$e]
		if {$esq <= 0} {continue}
		set sum [expr $sum + 1./$esq]
		set sumxsq [expr $sumxsq + ($x*$x)/$esq]
		set sumy [expr $sumy + $y/$esq]
		set sumx [expr $sumx + $x/$esq]
		set sumxy [expr $sumxy + ($x*$y)/$esq]
	}
	set delt [expr $sum*$sumxsq - $sumx*$sumx]
	if {[expr abs($delt)] <= 0.} {return {}}
	set yint [expr ($sumxsq*$sumy - $sumx*$sumxy)/$delt]
	set slope [expr ($sum*$sumxy - $sumx*$sumy)/$delt]
	set sigyint [expr sqrt($sumxsq/$delt)]
	set sigslope [expr sqrt($sum/$delt)]
	set chisq 0
	set outliers {}
	set newxyelsts {}
	foreach pi $fi {
		set p [lindex $xyelsts $pi]
		# in case x is large use x0 as origin
		# this also makes intercept the y value at x0
		set x [lindex $p 0]
		set dx [expr $x - $x0]
		set xf [expr $dx/$xrange]
		set y [lindex $p 1]
		set e [lindex $p 2]
		set esq [expr $e*$e]
		if {$esq <= 0} {continue}
		set dy [expr $y - $yint - $slope*$dx]
		set re [expr $dy/$e]
		if {[expr abs($re)] > $ci} {
			lappend outliers [list $pi $re $xf $x $y $e]
		} else {
			lappend newxyelsts $p
		}
		set chisq [expr $chisq + $re*$re]
	}
	if {[llength $outliers] > 0 && $refit == 1} {
		return [linefit $xyelsts $newci 0 0 $outliers]
	}
	set chired [expr sqrt($chisq/($nf))]
	return [list [list $slope $sigslope] [list $yint $sigyint] $chired $outliers $prevoutliers]
}

proc expfit {xyelsts ci refit newci prevoutliers} {
	# xyelsts of list x y e must be in increasing x
	# ci is the outlier reporting confidence interval
	# if refit the outliers are removed and the data refit
	# both outlier list get returned
	# fit y(x) = y0 exp(-(x)/T)
	# ln(y(x)/y0) = (x)/T
	# ln(y(x)) = ln(y0) - x/T
	# sigslny = (1/y)sigy
	#puts "expfit starting prevoutliers= $prevoutliers"
	set n [llength $xyelsts]
	if {$n < 2} {return {}}
	set previ {}
	foreach prev $prevoutliers {
		set i [lindex $prev 0]
		lappend previ $i
	}
	#puts "expfit starting previndices= $previ"
	# get indices in xyelsts that we will fit
	set fi {}
	for {set i 0} {$i < $n} {incr i} {
		if {[lsearch $previ $i] >= 0} {continue}
		lappend fi $i
	}
	#puts "expfit fi= $fi"
	set nprevi [llength $previ]
	set nf [llength $fi]
	if {$nf < 2} {return {}}

	set p0 [lindex $xyelsts [lindex $fi 0]]
	set x0 [lindex $p0 0]
	set y0 [lindex $p0 1]
	set e0 [lindex $p0 2]
	set pend [lindex $xyelsts [lindex $fi end]]
	set xend [lindex $pend 0]
	set xrange [expr 1.0*($xend - $x0)]
	# here we can use a truncated list for the loglinefit since we calc outliers after the fit
	set xlnyelsts {}
	foreach pi $fi {
		set p [lindex $xyelsts $pi]
		set x [expr [lindex $p 0] - $x0]
		set y [lindex $p 1]
		set e [expr [lindex $p 2]/$y]
		set y [expr log($y)]
		lappend xlnyelsts [list $x $y $e]
	}
	set lnrs [linefit $xlnyelsts $ci 0 0 {}]
	# yint = ln(y0)
	# slope = -1/T
	set slopee [lindex $lnrs 0]
	set slope [lindex $slopee 0]
	set slopeErr [lindex $slopee 1]
	set yinte [lindex $lnrs 1]
	set yint [lindex $yinte 0]
	set yintErr [lindex $yinte 1]
	set chi [lindex $lnrs 2]
	# set outliers [lindex $lnrs 3]
	set T [expr -1./$slope]
	set Terr [expr $T*($slopeErr/$slope)]
	# M is y0
	set M [expr exp($yint)]
	#set rT [expr $Terr/$T]
	set Merr [expr $M*$yintErr]
	set chisq 0
	set eoutliers {}
	set newxyelsts {}
	foreach pi $fi {
		set p [lindex $xyelsts $pi]
		set x [lindex $p 0]
		set dx [expr $x - $x0]
		set xf [expr $dx/$xrange]
		set y [lindex $p 1]
		set e [lindex $p 2]
		if {$e <= 0} {continue}
		set dy [expr $y - $M*exp(-1.*$dx/$T)]
		set re [expr $dy/$e]
		if {[expr abs($re)] > $ci} {
			lappend eoutliers [list $pi $re $xf $x $y $e]
		} else {
			lappend newxyelsts $p
		}
		set chisq [expr $chisq + $re*$re]
	}
	#puts "expfit eoutliers= $eoutliers"
	if {$refit == 1} {
		set neweres [expfit $xyelsts $newci 0 0 $eoutliers]
		#puts "expfit refit result= $neweres"
		return $neweres
	}
	set chired [expr sqrt($chisq/($nf))]
	return [list [list $T $Terr] [list $M $Merr] $chired $eoutliers $prevoutliers]
}



proc processFRlst {cellpi frlst fblst sb vq} {
    global maxfrwait frwarnings
    global frc frcerr
	#puts "processFRlst start frlst= $frlst  fblst= $fblst"
    set frwarnings {}
    pipeWrite "p cgs $sb"
    pipeWrite "p cgv $vq"
    set frcts [lindex $frlst 1]
    set nsft [lindex $frcts 0]
    set sft [lindex $frcts 1]
    set utc [lindex $frlst 2]
    set frp [lindex $frlst 3]
    set frperr [lindex $frlst 4]
    set ctrlst [lindex $frlst 5]
    pipeWrite "p cgfc 0 $nsft"
    pipeWrite "p cgfc 1 $sft"
    # i forgot that we need the raw countRates to make the background correction
    # so add the ctrates to frlst
    # strictly speaking the frb is supposed to measured with same flipper states
    # as peak fr. Just return a warning if they are not.
    # This is irrelevant for he3 filters where flipEff = 1
    set frc 0
    set frcerr 0
    set frcor 0
    if {[llength $fblst] > 0} {
        set frbcts [lindex $fblst 1]
        set nsftb [lindex $frbcts 0]
        set sftb [lindex $frbcts 1]
        if {$nsft != $nsftb || $sft != $sftb} {
            lappend frwarnings "WARNING: peak and background flip-ratios measured with different flip-states"
        }
        set utcb [lindex $fblst 2]
        if {[expr abs($utc - $utcb) > $maxfrwait]} {
            lappend frwarnings "WARNING: peak and background flip-ratios measured more than $maxfrwait secs apart"
        }
        set ctrlstb [lindex $fblst 5]
        if {![frbgCor $ctrlst $ctrlstb]} {return 0}
        set frcor 1
    }
    #puts "processFRlst about to pipeWrite p cr $frcor $cellpi $utc $frp $frperr $frc $frcerr"
    pipeWrite "p cr $frcor $cellpi $utc $frp $frperr $frc $frcerr"
    # this logs the fr data and updates efficiency or polarization
    return 1
}

proc combineFRdata {a b} {
    if {[llength $a] < 1} {
        set ab $b
    } elseif {[llength $b] < 1} {
        set ab $a
    } else {
        set ra [lindex $a 3]
        set raerr [lindex $a 4]
        set rb [lindex $b 3]
        set rberr [lindex $b 4]
        set r [expr 0.5*($ra + $rb)]
        set rerr [expr 0.5*sqrt($raerr*$raerr + $rberr*$rberr)]
        set a [lreplace $a 3 3 $r]
        set a [lreplace $a 4 4 $rerr]
        set ab $a
    }
    return $ab
}

proc combineFR {fa fb} {
    # fb should have the later datestr utc
    set fret $fb
    # combine the V0 Q0 V1 Q1 results
    set v0a [lindex $fa 6]
    set v0b [lindex $fb 6]
    set v0 [combineFRdata $v0a $v0b]
    set q0a [lindex $fa 8]
    set q0b [lindex $fb 8]
    set q0 [combineFRdata $q0a $q0b]

    set v1a [lindex $fa 10]
    set v1b [lindex $fb 10]
    set v1 [combineFRdata $v1a $v1b]
    set q1a [lindex $fa 12]
    set q1b [lindex $fb 12]
    set q1 [combineFRdata $q1a $q1b]

    # NB for he3 flipping with spin-filters the flip-efficiency should be 1
    # so which flipstates are used to calculate the flip-ratio dont matter
    set fret [lreplace $fret 6 6 $v0]
    set fret [lreplace $fret 8 8 $q0]
    set fret [lreplace $fret 10 10 $v1]
    set fret [lreplace $fret 12 12 $q1]
    return $fret
}


proc setCellParams {cel ityp} {
    # set cell params for current cell
	global cells
	global dbCols pbCols

    global flipEff flipEffErr
    global rfFlipLoss
	
	if {! [info exists cells($cel)]} {
		Dialog_Warn "$cel not in dicitionary"
		return
	}
	set it [expr abs($ityp) - 1]
	set cp [string index "PA" $it]
	# declare the celldata array global to access it
	set ca $cells($cel)
	global $ca
	#set ca $cells($cel)
	#puts "cells($cel)= [array get $ca]"
	array set loca [array get $ca]
	#puts "loca= [array get loca]"
	#puts "setCellParams $cel  before pipeWrite p cd -99 name $cel  with ca= $cells($cel)"
	#return
	pipeWrite "p cd -99 name $cel"
	#puts "setCellParams cd -99 name $cel OK returning"
	#return
	# on bt7 must also set the type for 3-axis
	pipeWrite "p cd -99 type 3"
	pipeWrite "p cd -99 PorA $cp"
	#puts "setCellParams returning before data loop"
	#return
	# set this cell efficiency stuff from current global values

	set feff $flipEff($it)
	set fefferr $flipEffErr($it)
	set rfloss $rfFlipLoss($it)
	pipeWrite "p cd -99 rfLossPerFlip $rfloss"
	pipeWrite "p cd -99 feff $feff"
	pipeWrite "p cd -99 ferr $fefferr"
	# now add essential params from cell database
	foreach d $dbCols p $pbCols {
	    #puts "about to pipeWrite p cd -99 $p $ca\($d)"
	    set value $loca($d)
		pipeWrite "p cd -99 $p $value"
	}
}


proc logEfficiency {c lst args} {
	# for single cell 
    global efficiencyLog
    global readlines errlines warnlines
    global nread nerr nwarn
    global txt
    global transblock
    global beamsize
    global Ei
    
    puts "logEfficiency starting lst= $lst llength= [llength $lst] args= $args llengthargs= [llength $args]"
    set utc [lindex $lst 1]
    set datestr [lindex $lst 0]
    #set datestr [clock format $utc -format {%m/%d/%y %H:%M}]
    set typ [lindex $lst 2]
    # which cell is in beam and do we have a forced install
    set ityp 1
    if [string match *a* $typ] {set ityp 2}
    if [string match *i* $typ] {set ityp [expr -1*$ityp]}
    set opacity 0
    if [string match *o* $typ] {set opacity 1}
    set ctl [lindex $lst 3]
    set cel [lindex $lst 4]
    # if cell= nocell then ctl will be empty
    set ct 0
    set sc 1
    if {[llength $ctl] > 2} {
    	set ct [lindex $ctl 1]
    	set sc [lindex $ctl 2]
    }

    
    if {$c == "ct"} {
        set event [list $datestr $utc ct $typ $cel $ct $sc]
        set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-10g %-6g" $datestr $utc ct $typ $cel $ct $sc]
        # delay txt insert until all trans blocks and fr blocks are collected
        lappend transblock $estr\n
        #$txt(2) insert end $estr\n
        lappend efficiencyLog $event
        return $event
    }
    if {$c == "tr"} {
    	# handle nocell. Never should be doing efficiency calc with noCell since FR undefined
    	# so we shouldnt have to put the nocell into tclpbs for calculation
    	if {$cel == "nocell"} {
            set ei $Ei
            set tr 1.
            set trerr 0.
            set pol 1.
            set polerr 0.
            set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-6.4f %-6.4f  %-6.4f %-6.4f  %-6g" $datestr $utc tp $typ $cel $tr $trerr $pol $polerr $ei]
            set event [concat $datestr $utc $cel $typ $ei $tr $trerr $pol $polerr]
            lappend transblock $estr\n
            #$txt(2) insert end $estr\n
            lappend efficiencyLog $event
    	
            #set rl [lindex $readlines 1]
            set pi 1.
            set pie 0.
            set pf 1.
            set pfe 0.
            set T 10000.
            set Te 0.
            set dT 24.
            set event [concat $datestr $utc $typ decay $pi $pie $pf $pfe $T $Te $dT]
            set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-6.4f %-6.4f  %-6.4f %-6.4f  %-6g %-6g  %-6g" $datestr $utc pd $typ $cel $pi $pie $pf $pfe $T $Te $dT]
            lappend transblock $estr\n
            #$txt(2) insert end $estr\n
            lappend efficiencyLog $event
            return $event    	
    	}
    	# args always puts passed stuff into list
        set mtlst [lindex $args 0]
        set mtctl [lindex $mtlst 3]
        set mtct [lindex $mtctl 1]
        set mtsc [lindex $mtctl 2]
        #puts "mtct= $mtct  mtsc= $mtsc  ct= $ct  sc= $sc"
        # tell tclpbs what celltype and cellname is in the beam
        # in tclpbs this gets a new expcell instance
        pipeWrite "p cgb $ityp $cel"
        # need to define parameters for the cell in the beam
        #puts "logEfficiency tr returning after p cgb $ityp $cel"
        #return
        set itypa [expr abs($ityp)]
        set ipa [expr $itypa - 1]
        # get back the cellindex so we set its beam geometry
        pipeWrite "p cgbn $ipa"
        set rl [lindex $readlines 0]
        set celli [lindex $rl 3]
        beamAreas
        setBeamGeo $celli $ipa $beamsize
        setCellParams $cel $itypa
        # compute the transmission and its sig err  where sigCOUNTS = sqrt(COUNTS)
        set tran [expr ($ct/$sc)/($mtct/$mtsc)]
        set resqct [expr 1./$ct]
        set resqmtct [expr 1./$mtct]
        set transig [expr $tran*sqrt($resqct + $resqmtct)]
        #puts "logEfficiency tr returning after setCellParams $cel"
        #return
        # now add this transmission data
        if {$opacity} {
            pipeWrite "p cto $celli $tran $transig"
        } else {
            # p cta will print all current transmission data
            #puts "logEfficiency tr return before to  p cta $utc $tran $transig"
            #return
            # do a sanity check by calling transUnpol for celli at say pol=0.7 lambda=2.4
            puts "check transUnpol before cta"
            pipeWrite "p cti $celli 0.7 2.4"
            puts [lindex $readlines 0]
            puts [lindex $readlines 1]
            puts "about to p cta $celli $utc $tran $transig"
            pipeWrite "p cta $celli $utc $tran $transig"
            puts "returned from p cta $celli with nread= $nread"
        }
       

        # get back the he3pol and decay for this last trans
        if {$nerr > 0 || $nwarn > 0 || $nread < 1} {
        	puts "errors or warnings from p cta cell=$celli"
            if {$nerr > 0} {
                Dialog_Warn [join $errlines "\n"]
            } elseif {$nwarn > 0} {
                Dialog_Warn [join $warnlines "\n"]
            }
        	puts "printing failed cellindex=$celli"
        	pipeWrite "p hi $celli"
        	foreach ln $readlines {puts $ln}
            return {}
        }
        puts "logEfficiency tr added data via p cta cell=$celli OK nread= $nread"
        if {$opacity} {
            # prints opacity opacity1err cellname position
            set rl [lindex $readlines 0]
            set op [lindex $rl 3]
            set operr [lindex $rl 4]
            set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-6g %-6g" $datestr $utc op $typ $cel $op $operr]
            set event [concat $datestr $utc $rl]
            lappend efficiencyLog $event
            lappend transblock $estr\n
            #$txt(2) insert end $estr\n
            return $event
        } else {
            # there are two header lines followed by the data
            # secs ei trans transerr he3pol he3polErr
            if {$nread < 3} {
                Dialog_Warn "transmission has NO data"
                return {}
            }
            puts "logEfficiency about to do tp"
            set nd [expr $nread - 2]
            set il [expr $nread - 1]
            set rl [lindex $readlines $il]
            set ei [lindex $rl 1]
            set tr [lindex $rl 2]
            set trerr [lindex $rl 3]
            if {[expr abs($trerr)] > 1.} {set trerr 1.}
            set pol [lindex $rl 4]
            set polerr [lindex $rl 5]
            if {[expr abs($polerr)] > 1.} {set polerr 1.}
            set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-6.4f %-6.4f  %-6.4f %-6.4f  %-6g" $datestr $utc tp $typ $cel $tr $trerr $pol $polerr $ei]
            set event [concat $datestr $utc $cel $typ [lrange $rl 1 5]]
            lappend transblock $estr\n
            #$txt(2) insert end $estr\n
            lappend efficiencyLog $event
            # check for decay if we get 2 data points
            puts "printing cellindex=$celli"
        	pipeWrite "p hi $celli"
        	foreach ln $readlines {puts $ln}
            if {$nd < 2} {return $event}
            pipeWrite "p ch $celli"
            # return header line and data line:
            # isfit Pi Pierr Pf Pferr T Terr deltaHrs cellname
            if {$nread < 2} {return {}}
        	if {$nerr > 0 || $nwarn > 0} {
            	if {$nerr > 0} {
            	    Dialog_Warn [join "ERROR: getting pol decay from trans" "\n" $errlines "\n"]
            	} elseif {$nwarn > 0} {
                	Dialog_Warn [join $warnlines "\n"]
            	}
            	return {}
        	}
            set rl [lindex $readlines 1]
            set pi [lindex $rl 1]
            set pie [lindex $rl 2]
            set pf [lindex $rl 3]
            set pfe [lindex $rl 4]
            set T [lindex $rl 5]
            set Te [lindex $rl 6]
            set dT [lindex $rl 7]
            set event [concat $datestr $utc $typ decay $rl]
            set estr [format "%-s %-s %-4.4s %-3.3s %-14.14s  %-6.4f %-6.4f  %-6.4f %-6.4f  %-6g %-6g  %-6g" $datestr $utc pd $typ $cel $pi $pie $pf $pfe $T $Te $dT]
            lappend transblock $estr\n
            #$txt(2) insert end $estr\n
            lappend efficiencyLog $event
            return $event
        }
    }
}


proc combineCounts {translist fr chk} {
	global maxtranswait maxfrwait
	#puts "combineCounts starting translist= $translist"
	# if chk flag is set just return errors
	set nt [llength $translist]
	if {$nt < 1} {
	    if {$chk} {return "EMPTY transmission list"}
	    return {}
	}
	if {$nt < 2} {
		if {$chk} {
			return ""
		} else {
			return [lindex $translist 0]
		}
	}
	set utc 0
	set cts 0
	set sec 0
	# should check that ct (p a mt o) for this translist all have same value
	# We obviate this check by never combining a cell measured with differrent p a o
	# NB it would still be possible to move a cells postion between p a with two different
	# transmissions
	# should check that utc are not too far apart for combining, for mt doesnt matter
	set utcmin 9999999999
	set utcmax 0
	for {set i 0} {$i < $nt} {incr i} {
		set ti [lindex $translist $i]
		# all the ct will be same typ
		set ct [lindex $ti 2]
		set ctl [lindex $ti 3]
		set utc [lindex $ti 1]
		if {$utc < $utcmin} {set utcmin $utc}
		if {$utc > $utcmax} {set utcmax $utc}
		set cts [expr $cts + [lindex $ctl 1]]
		set sec [expr $sec + [lindex $ctl 2]]
	}
	if {$chk} {
	    set maxwait $maxtranswait
	    set maxs transmission
	    if {$fr} {
	        set maxwait $maxfrwait
	        set maxs flip-ratio
	    }
		if {($utcmax - $utcmin) > $maxwait} {
			return "WARNING: combining $maxs measurements at times > $maxwait secs"
		} else {
			return ""
		}
	
	}
	# fix up the first trans element as the combined one and return that
	set ti [lindex $translist 0]
	set ctl [list $utc $cts $sec]
	set ti [lreplace $ti 1 1 $utc]
	set ti [lreplace $ti 3 3 $ctl]
	return $ti
}


proc findPD {tb typ nam utci} {
    # try to find the pd entry of typ (ap) for cellname=name in the transblock tb.
    # Also record the corresponding install start utc=utci
    # return the pd line(it is a list) if found 
    # decided to return just the relevant cellfile data as array keyed list
    # see findMPD for monitor decay
    set nl [llength $tb]
    set dlst {}
    if {$nl < 1} {return $dlst}
    for {set kk 0} {$kk < $nl} {incr kk} {
        set ln [lindex $tb $kk]
        set llpd [llength $ln]
        if {$llpd < 13} {continue}
        set cmd [lindex $ln 3]
        if {$cmd != "pd"} {continue}
        set typk [string index [lindex $ln 4] 0]
        if {$typk != $typ} {continue}
        set namk [lindex $ln 5]
        if {$namk != $nam} {continue}
        lappend dlst utci $utci
        lappend dlst utcf [lindex $ln 2]
        lappend dlst iPol [lindex $ln 6]
        lappend dlst iPolErr [lindex $ln 7]
        lappend dlst fPol [lindex $ln [expr $llpd - 5]]
        lappend dlst fPolErr [lindex $ln [expr $llpd - 4]]
        lappend dlst Thr [lindex $ln [expr $llpd - 3]]
        lappend dlst ThrErr [lindex $ln [expr $llpd - 2]]
        lappend dlst hrs [lindex $ln [expr $llpd - 1]]
        return $dlst
    }
    return $dlst
}
proc findMPD {tb nam utci} {
    # try to find the mpd entry for p(polarizer) for cellname=nam in the transblock tb.
    # Also record the corresponding install start utc=utci
    # return the pd line(it is a list) if found 
    # decided to return just the relevant cellfile data as array keyed list
    set nl [llength $tb]
    set dlst {}
    if {$nl < 1} {return $dlst}
    set nl1 [expr $nl - 1]
    # for mpd start search from end of block
    for {set kk $nl1} {$kk >= 0} {incr kk -1} {
        set ln [lindex $tb $kk]
        set llpd [llength $ln]
        if {$llpd < 13} {continue}
        set cmd [lindex $ln 3]
        if {$cmd != "mpd"} {continue}
        set typk [string index [lindex $ln 4] 0]
        if {$typk != "p"} {continue}
        set namk [lindex $ln 5]
        if {$namk != $nam} {continue}
        lappend dlst utci $utci
        lappend dlst utcf [lindex $ln 2]
        lappend dlst iPol [lindex $ln 6]
        lappend dlst iPolErr [lindex $ln 7]
        lappend dlst fPol [lindex $ln [expr $llpd - 5]]
        lappend dlst fPolErr [lindex $ln [expr $llpd - 4]]
        lappend dlst Thr [lindex $ln [expr $llpd - 3]]
        lappend dlst ThrErr [lindex $ln [expr $llpd - 2]]
        lappend dlst hrs [lindex $ln [expr $llpd - 1]]
        return $dlst
    }
    return $dlst
}

proc pdBreak {pdlst utc1 utc2} {
	# if we have to breakup a data block due to geometry change
	# break the pd into matching blocks
	# Here return the block for utc range utc1 utc2
    # return new pdlst derived from pdlst utci
    # but setting parameters equivalent for the interval utc1 utc2
    # let (t-ti)/(tf-ti) = x
    # P(t) = Pi(Pf/Pi)^x
    # dP/dPi = (P/Pi)(1 - x)
    # dP/dPf = (P/Pf)x
    # sigsqP = ((P/Pi)(1-x))^2 sigsqPi + ((P/Pf)x)^2 sigsqPf or
    # sigsqP/P^2 = (1-x)^2 sigsqPi/Pi^2 + x^2 sigsqPf/Pf^2
    #puts "called pdBreak pdlst= $pdlst"
    array set lpd $pdlst
    set Pi $lpd(iPol)
    set Pf $lpd(fPol)
    set Pie 0
    if {$Pi > 0} {set Pie [expr $lpd(iPolErr)/$Pi]}
    set Pfe 0
    if {$Pf > 0} {set Pfe [expr $lpd(fPolErr)/$Pf]}
    set Piesq [expr $Pie*$Pie]
    set Pfesq [expr $Pfe*$Pfe]
    set Pr 0
    if {$Pi > 0} {set Pr [expr $Pf/$Pi]}
    set tf $lpd(utcf)
    set ti $lpd(utci)
    puts "pdBreak tf= $tf ti= $ti"
    set dt [expr $tf - $ti]
    set x1 [expr ($utc1 - $ti)/$dt]
    set x2 [expr ($utc2 - $ti)/$dt]
    set P1 [expr $Pi*pow($Pr,$x1)]
    set P2 [expr $Pi*pow($Pr,$x2)]
    set P1err [expr (1-$x1)*(1-$x1)*$Piesq + $x1*$x1*$Pfesq]
    set P1err [expr $P1*sqrt($P1err)]
    set P2err [expr (1-$x2)*(1-$x2)*$Piesq + $x2*$x2*$Pfesq]
    set P2err [expr $P2*sqrt($P2err)]
    set lpd(iPol) $P1
    set lpd(fPol) $P2
    set lpd(iPolErr) $P1err
    set lpd(fPolErr) $P2err
    set lpd(utci) $utc1
    set lpd(utcf) $utc2
    set hrs [expr ($utc2 - $utc1)/3600.]
    set lpd(hrs) $hrs
    return [array get lpd]    
}

proc equalGeomLists {g1 g2 fac} {
    # g1 g2 are two keyed geom lists
    # convert then to arrays and compare each array element with same key
    # to accuracy fac
    array set a1 $g1
    array set a2 $g2
    set keys [array names a1]
    foreach key $keys {
        set x1 $a1($key)
        set x2 $a2($key)
        set xave [expr ($x1 + $x2)/2.]
        set xdif [expr abs($x1 - $x2)]
        if {[expr abs($xave)] > 0} {
            if {[expr $xdif/$xave] > $fac} {return 0}
        } else {
            if {$xdif > $fac} {return 0}
        }
    }
    return 1
}


proc parseTOcellfiles {} {
    global transblocks newfrblocks
    global installs ninstalls
    global npairs pairsinstalls pairsfiles pairsflipratios pairsGeom
    global effs effsList bflds temps
    global distinctEnv distinctEff
    global transportEffCI VQmodes
    global mratDecayCI
    global finstalls installgeoms installdata installfiles finstallfiles installeffs pairsForIns
    global transSeg
    global geomAcc
    global txt nb effB
	global cellFileDataFiles
    # start by weeding out installs that have no datafiles
    # each install should be en entry in a cellfile except if there are no data files
    # install= datestr utc typ cellname
    set warnings {}
    
    # go through transblocks and locate the installs by blockindex
    set ntrnsblks [llength $transblocks]
    set nt1 [expr $ntrnsblks - 1]
    array set instblk {}
    for {set j 0} {$j < $ntrnsblks} {incr j} {
        set tb [lindex $transblocks $j]
        set nl [llength $tb]
        if {$nl < 2} {continue}
        set istar [expr $nl - 2]
        for {set i $istar} {$i >= 0} {incr i -1} {
            set ln [lindex $tb $i]
            set inss [lindex $ln 2]
            if {[scan $inss install=%d insnum] < 1} {break}
            set instblk($insnum) $j
        }
    }

    set n1 [expr $ninstalls - 1]
    set finstalls {}
    set installfiles {}
    set installdata {}
    set installgeoms {}
    set lastTblk 0
    for {set i 0} {$i < $ninstalls} {incr i} {
        set insnum [expr $i + 1]
        if {![info exists instblk($insnum)]} {
            Dialog_Warn "ERROR: transblock for install=$insnum doesnt exist"
            return
        }
        set lastTblk $instblk($insnum)
        set ins [lindex $installs $i]
        set utci [lindex $ins 1]
        set typ [string index [lindex $ins 2] 0]
        set typi 0
        if {$typ == "a"} {set typi 1}
        set nam [lindex $ins 3]
        set utcf 9999999999
        set uninsnum 0
        # we also look for the cells polarization and decay data
        set pdok 0
        set mpdok 0
        for {set j [expr $i + 1]} {$j < $ninstalls} {incr j} {
            set insf [lindex $installs $j]
            set utcf [lindex $insf 1]
            set typf [string index [lindex $insf 2] 0]
            if {$typ == $typf} {
                set uninsnum [expr $j + 1]
                if {![info exists instblk($uninsnum)]} {
                	puts "missing transblock ins=$i"
                    Dialog_Warn "ERROR: transblock for uninstall=$uninsnum doesnt exist"
                    return
                }
                set tb [lindex $transblocks $instblk($uninsnum)]
                set pd [findPD $tb $typ $nam $utci]
                set mpd [findMPD $tb $nam $utci]
                if {[llength $pd] > 0} {set pdok 1}
                if {[llength $mpd] > 0} {set mpdok 1}
                break
            }
        }
        # if cant find the uninstall for typ nam cell from the next instance of cell typ
        # search starting from last transblock
        # looking for pd typ cellname
        # NB this is the case for last install of type
        # cant find typ in the install list maybe it is in last transblock?
        if {$uninsnum == 0} {
            for {set k $nt1} {$k > $lastTblk} {incr k -1} {
                set tb [lindex $transblocks $k]
                set mpd [findMPD $tb $nam $utci]
                if {[llength $mpd] > 0} {
                    set mpdok 1
                    set utcf [lindex $mpd 3]
                }
                set pd [findPD $tb $typ $nam $utci]
                # pd comes back as array list starting as utci valueutci utcf valueutcf
                if {[llength $pd] > 0} {
                    set pdok 1
                    set utcf [lindex $pd 3]
                }
                if {$pdok == 1 && $mpdok == 1} {break}
            }
        }
        if {$pdok == 1 && $mpdok == 1} {
        	# compare the decays
        	array set pda $pd
        	array set mpda $mpd
        	set dec $pda(Thr)
        	set decErr $pda(ThrErr)
        	set mdec $mpda(Thr)
        	set mdecErr $mpda(ThrErr)
        	set ddec [expr abs($dec - $mdec)]
        	# use the larger error to get the CI for this diff
        	set err $decErr
        	if {$mdecErr > $err} {set err $mdecErr}
        	set cidec [expr $ddec/$err]
        	if {$cidec > $mratDecayCI} {
        		lappend warnings "WARNING: mrat-decay and detector-transmission decay differ by ci= $cidec for install=$insnum typ=$typ cell=$nam"
        	}
        }
        if {$pdok == 0} {
         	if {$typ == "a" || ($typ == "p" && $mpdok == 0)} {
            	lappend warnings "ERROR: missing polarization and decay data for install=$insnum typ=$typ cell=$nam"
            	#puts "missing pd for install= $i"
            	continue
            } elseif {$typ == "p"} {
            	set pd $mpd
            	set pdok 1
            	lappend warnings "WARNING: missing detector-transmission polarization and decay data replaced by mrat data for install=$insnum typ=$typ cell=$nam"
            }
        }
		puts "install=$insnum cell=$nam typ=$typ utci=$utci utcf=$utcf looking for files"
        set iflst [filesINutcRange $utci $utcf]
        set nfrr [frINutcRange $utci $utcf]
        # if there are no files or flipratios for this install skip
        if {[llength $iflst] < 1 && $nfrr < 1} {
        	lappend warnings "WARNING: install=$insnum typ=$typ cell=$nam has no files or flipping-ratios. Skipping in cellfile"
        	#puts "writeCellFile skipping ins=$i nofile nfr=$nfrr"
        	continue
        }

        # find pairs for this install index
        set pairsForIns {}
        for {set k 0} {$k < [llength $pairsinstalls]} {incr k} {
            set nfp [lindex $pairsfiles $k]
            set nrp [lindex $pairsflipratios $k]
            if {$nfp < 1 && $nrp < 1} {continue}
            set inspair [lindex $pairsinstalls $k]
            set pins [lindex $inspair 0]
            set ains [lindex $inspair 1]
            if {$typ == "a" && $ains == $i} {
                lappend pairsForIns $k
            } elseif {$typ == "p" && $pins == $i} {
                lappend pairsForIns $k
            }
        }
        if {[llength $pairsForIns] < 1} {
            lappend warnings "WARNING: no pairs with datafiles or flip-ratios for install=$insnum typ=$typ cell=$nam. Skipping in cellfile"
            continue
        }
        #puts "writeCellFile ins=$i pairsForIns= $pairsForIns"
        # so now pairsForIns is list of valid pair indices (must be consecutive) that use the current install i which has type typ pORa typeindex=typi
        # best practice only change beam geom at cell changes where transmissions will get msrd
        # but try to handle the case that geom changes in middle of a polarized-beam data block between init and fin trans msrs
        # if there are multiple beam geometries for this install  it will get broken into the geom pieces assuming the geoms differ
        # by more than geomAcc
        # this install covers utci utcf
        set geomsForInstall {}
        set subpds {}
        set subfiles {}
        set subgeoms {}
        set subdateu {}
        set nsub 0
        foreach pii $pairsForIns {
            set piig [lindex $pairsGeom $pii]
            # get the first geom
            set piik [lindex $piig 0]
            # the first pair geom utc is for the pair start which is usually a little after the install transmission
            # so use the install utci
            # if we have to divide up this install the pair utcs should serve fine as intermediate times
            set utc0 $utci
            set pag [lindex $piik 2]
            set geo0 [lindex $pag $typi]
            for {set k 1} {$k < [llength $piig]} {incr k} {
                set piik [lindex $piig $k]
                set utck [lindex $piik 1]
                set filsk [filesINutcRange $utc0 $utck]
                if {[llength $filsk] < 1} {continue}
                set pag [lindex $piik 2]
                set geok [lindex $pag $typi]
                if [equalGeomLists $geo0 $geok $geomAcc] {
                    set geo0 $geok
                    continue
                }
                # pii is a pair index
                # append an install derived from current but for utc interval utc0 utck
                puts "call pdBreak pd= $pd"
                set pdk [pdBreak $pd $utc0 $utck]
                lappend subpds $pdk
                lappend subfiles $filsk
                lappend subgeoms $geo0
                # get the cellfile formatted datestr
                set date0 [utcTOdatestr $utc0 0]
                lappend subdateu [list $date0 $utc0 $utck]
                set utc0 $utck
                set geo0 $geok
                incr nsub
            }
            # append the last interval utck utcf
            set filsk [filesINutcRange $utc0 $utcf]
            puts "filesINutcRange $utc0 $utcf = $filsk"
            if {[llength $filsk] > 0} {
                set pdk [pdBreak $pd $utc0 $utcf]
                lappend subpds $pdk
                lappend subfiles $filsk
                lappend subgeoms $geo0
                set date0 [utcTOdatestr $utc0 0]
                lappend subdateu [list $date0 $utc0 $utcf]
                incr nsub
            }
		}
		if {$nsub > 1} {
			lappend warnings "WARNING: divided by beam-geom changes install= $ins cell=$nam\nbeam changes= $subgeoms"
		}
        # now append an install for each sub
		for {set k 0} {$k < $nsub} {incr k} {
		    set finstall [list $nam $typ [lindex $subdateu $k]]
		    set finsdata [lindex $subpds $k]
		    set finsfils [lindex $subfiles $k]
		    set finsgeom [lindex $subgeoms $k]
		    lappend finstalls $finstall
		    lappend installdata $finsdata
		    lappend installfiles $finsfils
		    lappend installgeoms $finsgeom
		}
		puts "writeCellFile about to continue install loop ins=$i ninstalls=$ninstalls"
    }
    set nfinstalls [llength $finstalls]
    if {$nfinstalls < 2} {
        $txt(3) insert end "ERROR: no cellpairs with data."
        #nb select 3
        selectTab 3
        butdisable
        #$effB configure -bg grey -state disabled
        return
    }

    # Now get the chronological list of transport from the FRdata
    # separated into V1 and Q1 lists
    # start with all msrd FR even for blocks with no data files
    # check for env dependence
    # and if env dependence check for consistency across blocks
    # 0 is for Vmode  1 is for Qmode
    
    array set effs {0 {} 1 {}}
    array set effsList {0 {} 1 {}}
    array set bflds {0 {} 1 {}}
    array set temps {0 {} 1 {}}
    


    set nfrblks [llength $newfrblocks]
    set ntrnsblks [llength $transblocks]
    for {set i 0} {$i < $nfrblks} {incr i} {
        set frb [lindex $newfrblocks $i]
        set nfrbl [llength $frb]
        # sanity checking for valid fr and trans blocks. supposed to have start files stop lines
        # then FRdata blocks start with FRdata then header then datalines
        if {$nfrbl < 6} {continue}
        # skip the last two lines which have files and stop
        set nfrbl [expr $nfrbl - 2]
        set j 1
        while {$j < $nfrbl} {
            set frl [lindex $frb $j]
            #puts "frblock $i line $j = $frl"
            # we only want FR for experiment condition
            if {[lindex $frl 0] == "FRdata" && [lindex $frl 2] == 1} {
                set m [lindex $frl 4]
                #puts "  processing line $j mode= $m"
                incr j 2
                while {$j < $nfrbl} {
                    set frj [lindex $frb $j]
                    #puts "  processing line $j = $frj"
                    if {[lsearch $frj "FRdata"] == 0} {
                    	# found end of FRdata lines. So back up j
                        incr j -1
                        break
                    }
                    #puts "  getting data from line $j"
                    set utc [lindex $frj 0]
                    set FR [lindex $frj 1]
                    set FRerr [lindex $frj 2]
                    set teff [lindex $frj 4]
                    set tefferr [lindex $frj 5]
                    set b [lindex $frj 10]
                    set t [lindex $frj 11]
                    puts "  data b= $b  t= $t"
                    set frjlst [list $i $utc $teff $tefferr $FR $FRerr $b $t]
                    lappend effsList($m) $frjlst
                    lappend effs($m) [list $i $utc $teff $tefferr]
                    lappend bflds($m) $b
                    lappend temps($m) $t
                    incr j
                }
            }
            incr j
        }
    }
    # find the distinct envs using envTempTol envBfldTol envTempZeroTol  for both VQ modes
    array set distinctEnv {0 {} 1 {}}
    foreach mode {0 1} {
        set nenv [llength $temps($mode)]
        if {$nenv < 1} {continue}
        set frenv [list [lindex $bflds($mode) 0] [lindex $temps($mode) 0] [list [lindex $effs($mode) 0]]]
        puts "init frenv mode=$mode = $frenv"
        lappend distinctEnv($mode) $frenv
        for {set k 1} {$k < $nenv} {incr k} {
            set frenv [list [lindex $bflds($mode) $k] [lindex $temps($mode) $k] [list [lindex $effs($mode) $k]]]
            puts "kth frenv mode=$mode = $frenv"
            if [newDistinctEnv $mode $frenv] {lappend distinctEnv($mode) $frenv}         
        }
    }
    # if env makes differnce in transport efficiency, make
    # cellfiles for each env and VQ mode (assuming they also differ)
    # first check that each mode env is consistent by computing weighted average eff and error
    # and checking for outliers based on this
    # warn on outliers and append ave err to each unique env
    foreach mode {0 1} {
        for {set i 0} {$i < [llength $distinctEnv($mode)]} {incr i} {
            set uenv [lindex $distinctEnv($mode) $i]
            set b [lindex $uenv 0]
            set t [lindex $uenv 1]
            set efflst [lindex $uenv 2]
            set neff [llength $efflst]
            if {$neff < 2} {
                set effk [lindex $efflst 0]
                set ave [lindex $effk 2]
                set err [lindex $effk 3]
                lappend uenv [format %.4f $ave] [format %.4f $err]
                set distinctEnv($mode) [lreplace $distinctEnv($mode) $i $i $uenv]
                continue
            }
            set efferrs {}
            foreach effk $efflst {
                lappend efferrs [list [lindex $effk 2] [lindex $effk 3]]
            }
            set aveerr [averageErr $efferrs]
            set ave [lindex $aveerr 0]
            set err [lindex $aveerr 1]
            # get the list of normdev
            set normdevs [normDevs $ave $efferrs]
            set outliers {}
            for {set k 0} {$k < $neff} {incr k} {
                if {[lindex $normdevs $k] > $transportEffCI} {lappend outliers $k}
            }
            if {[llength $outliers] > 0} {
                set errmsg "WARNING: tranportEff for $VQmodes($mode) mode b=$b t=$t has the following outliers with CI=$transportEffCI from aveTransport=$ave :"
                foreach kout $outliers {
                    set effk [lindex $efflst $kout]
                    set effutc [lindex $effk 1]
                    set effval [lindex $effk 2]
                    set efferr [lindex $effk 3]
                    set ds [utcTOdatestr $effutc 1]
                    set errmsg "$errmsg\neff=$effval err=$efferr measured at $ds $effutc"
                }
                set errmsg "$errmsg\nIf you can determine if any of these are bad measurements, delete them from the console Log.\nFor now average will be used."
                lappend warnings $errmsg
            }
            lappend uenv [format %.4f $ave] [format %.4f $err]
            set distinctEnv($mode) [lreplace $distinctEnv($mode) $i $i $uenv]
        }
    }
    # have reported outliers and added ave err to transport lists.
    # next check if unique env have different transport by comparing averages
    # distinctEff for each mode is a list of lists+Avesum+Errsum+ave+err
    # the lists part of lists+Avesum+.. are b t eff err  and the Avesum.. part
    # has the current Sum eff/err^2  Sum 1/err^2 ave err so we can update ave when new ave err are included
    array set distinctEff {0 {} 1 {}}
	puts "distinctEnv= [array get distinctEnv]"
    foreach mode {0 1} {
        set neff [llength $distinctEnv($mode)]
        if {$neff < 1} {continue}
        set uenv [lindex $distinctEnv($mode) 0]
        # get b t eff err
        set eff [lindex $uenv 3]
        set efferr [lindex $uenv 4]
        set sumerr [expr 1./($efferr*$efferr)]
        set sumeff [expr $eff*$sumerr]
        set efflstl [list [list [list [lindex $uenv 0] [lindex $uenv 1] [lindex $uenv 3] [lindex $uenv 4]]] $sumeff $sumerr $eff $efferr]
        lappend distinctEff($mode) $efflstl
        for {set k 1} {$k < $neff} {incr k} {
            set uenv [lindex $distinctEnv($mode) $k]
            set eff [lindex $uenv 3]
            set efferr [lindex $uenv 4]
            set sumerr [expr 1./($efferr*$efferr)]
            set sumeff [expr $eff*$sumerr]
            set efflstl [list [list [list [lindex $uenv 0] [lindex $uenv 1] [lindex $uenv 3] [lindex $uenv 4]]] $sumeff $sumerr $eff $efferr]
            if [newDistinctEff $mode $efflstl] {lappend distinctEff($mode) $efflstl}         
        }
    }
	# if there is no env dependence to transport, then look up transport efficiency for each finstall
	# in the effs array for VQ 01 modes by utc
	# save the global trans ave err
	puts "distinctEff= [array get distinctEff]"
	set nVeff [llength $distinctEff(0)]
	set nQeff [llength $distinctEff(1)]
	array set VQeff {0 {} 1 {}}
	array set finstallfiles {}
	if {$nVeff < 2 && $nQeff < 2} {
		# no env dep. Get global transport
		set Vok 0
		set Qok 0
		if {$nVeff > 0} {
			set distincteff [lindex $distinctEff(0) 0]
			set VQeff(0) [list [lindex $distincteff 3] [lindex $distincteff 4]]
			set Vok 1
		}
		if {$nQeff > 0} {
			set distincteff [lindex $distinctEff(1) 0]
			set VQeff(1) [list [lindex $distincteff 3] [lindex $distincteff 4]]
			set Qok 1
		}
		if {$Vok == 0 && $Qok == 0} {
			lappend warnings "ERROR: No transport efficienies found."
			return
		}
		if {$Vok == 0} {
			set VQeff(0) $VQeff(1)
			set Vok 1
		}
		if {$Qok == 0} {
			set VQeff(1) $VQeff(0)
			set Qok 1
		}
		puts "VQeff= [array get VQeff]"
		array set installeffs {0 {} 1 {}}
		foreach fins $finstalls {
			set dat [lindex $fins 2]
			set uti [lindex $dat 1]
			set utf [lindex $dat 2]
			array set efferra {0 {} 1 {}}
			foreach m {0 1} {
				set elst [effsINutcRange $m $uti $utf]
				if {[llength $elst] > 0} {
					set efferra($m) [averageErr $elst]
				}
			}
			if {[llength $efferra(0)] == 0} {
				if {[llength $efferra(1)] > 0} {
					set efferra(0) $efferra(1)
				} else {
					set efferra(0) $VQeff(0)
				}
			}
			if {[llength $efferra(1)] == 0} {
				if {[llength $efferra(0)] > 0} {
					set efferra(1) $efferra(0)
				} else {
					set efferra(1) $VQeff(1)
				}
			}
			puts "for dat=$dat install efferra= [array get efferra]"
			foreach m {0 1} {
				lappend installeffs($m) $efferra($m)
			}
		}
		foreach m {0 1} {
			set finstallfiles($VQmodes($m)) [filesVQ $VQmodes($m)]	
			set fnam [writeCellFile $VQmodes($m) $finstalls $installdata $installgeoms $installeffs($m)]
			$txt(7) insert end "datafiles for cellfile $fnam\n"
			set cellFileDataFiles($fnam) $finstallfiles($VQmodes($m))
			foreach fil $finstallfiles($VQmodes($m)) {
				$txt(7) insert end "$fil\n"
			}
			$txt(7) insert end "\n"
		}
	} else {
		# write cellfile foreach distinctEff. Check it B doesnt vary or T doesnt vary
		# to help with file names
		array set bs {0 {} 1 {}}
		array set ts {0 {} 1 {}}
		array set bz {0 0 1 0}
		array set tz {0 0 1 0}
	
		foreach m {0 1} {
			foreach e $distinctEnv($m) {
				lappend bs($m) [lindex $e 0]
				lappend ts($m) [lindex $e 1]			
			}
		}
		foreach m {0 1} {
			set dvs [distinctValues bs($m) 1]
			if {[llength $dvs] == 1 && [lindex $dvs 0] == 0} {set bz($m) 1}
			set dvs [distinctValues ts($m) 1]
			if {[llength $dvs] == 1 && [lindex $dvs 0] == 0} {set tz($m) 1}
		}
		
		#array set enveffs {0 {} 1 {}}
		foreach m {0 1} {
			foreach de $distinctEff($m) {
				set envlst [lindex $de 0]
				set efer [list [lindex $de 3] [lindex $de 4]]
				set enveffs {}
				for {set k 0} {$k < $nfinstalls} {incr k} {
					lappend enveffs $efer
				}
				set blst {}
				set tlst {}
				foreach e $envlst {
					lappend blst [truncEnv [lindex $e 0]]
					lappend tlst [truncEnv [lindex $e 1]]
				}

				if {$bz($m) == 1} {
					set envName $VQmodes($m)_T=[dashList $tlst]
				} elseif {$tz($m) == 1} {
					set envName $VQmodes($m)_B=[dashList $blst]
				} else {
					set envName $VQmodes($m)_B=[dashList $blst]T=[dashList $tlst]
				}
				foreach e $envlst {
					set b [truncEnv [lindex $e 0]]
					set t [truncEnv [lindex $e 1]]
					set envFiles [filesINenvRange $b 0 $t 0]
					#puts "for env=$e files= $envFiles"
					lappend finstallfiles($envName) $envFiles
				}
				# NB all insalls use the env efficiency with env in the envName used for cellfile name
				puts "about to writeCellFile for m=$m env=$de envName=$envName finstalls=$finstalls"
				set fnam [writeCellFile $envName $finstalls $installdata $installgeoms $enveffs]
				$txt(7) insert end "datafiles for cellfile $fnam\n"
				set cellFileDataFiles($fnam) $finstallfiles($envName)
				set fillst $finstallfiles($envName)
				foreach fil [lindex $fillst 0] {
					# turns out finstallfiles(env) gives a list of a list of the files
					$txt(7) insert end "$fil\n"
				}
				$txt(7) insert end "\n"
			}
		}
	}
    if {[llength $warnings] > 0} {
    	$txt(4) insert end "\n\nCreating CellFile ERRORS and WARNINGS\n"
        foreach w $warnings {
            $txt(4) insert end $w\n\n
        }
        #Dialog_Warn [join $warnings "\n"]
    }
    butenable 4
    #$effB configure -bg grey
    #nb select 3
    selectTab 3
}

proc writeCellFiles {} {
	global cellFileNames
	global cellFileData
	global cellFileDataFiles
	foreach fil $cellFileNames {
		set fp [open $fil w]
		foreach ln $cellFileData($fil) {
			puts $fp $ln
		}
		close $fp
		set fp [open $fil-files w]
		foreach ln $cellFileDataFiles($fil) {
			puts $fp $ln
		}
		close $fp
	}
	# also write the filelists for each cellfile as $fil
}

set cellFileNames {}
array set cellFileData {}
array set cellFileDataFiles {}

proc writeCellFile {envname ins data geoms efferrs} {
    global exptid startDate
    global cells
    global sampleMosMin
    global txt
    global transSeg
    global cellfileLabels
    global cfspc
    global cellFileNames
    global cellFileData
    
    set tx $txt(3)
	set fnam $exptid\_$startDate\_$envname
	lappend cellFileNames $fnam
	#set cellFileData($fnam) {}
	# save all the lines and then format into neat columns including label
	set flines {}
	lappend flines $cellfileLabels
	$tx insert end "cellfile $fnam\n"
	puts "writeCellFile env=$envname ins= $ins efferrs=$efferrs"
	foreach i $ins d $data g $geoms e $efferrs {
		set id [lindex $i 2]
		set dt [lindex $id 0]
		set utci [lindex $id 1]
		set name [lindex $i 0]
		puts "install name= $name for date=$dt"
		set line "$name [string toupper [lindex $i 1]] [lindex $dt 0] [lindex $dt 1] $utci"
		array set da $d
		set terr $da(ThrErr)
		if {$transSeg == 1} {set terr [expr -1.*$da(fPolErr)]}
		set line "$line [format %.4f $da(iPol)] [format %.4f $da(iPolErr)] [format %.4f $da(Thr)] [format %.4f $terr]"
		# the full transport efficiency is to be divided between polarizer and analyzer so take sqrt
		set teffsq [lindex $e 0]
		set teffsqErr [lindex $e 1]
		set teff [expr sqrt($teffsq)]
		set terr [expr 0.5*$teffsqErr/$teff]
		set teff [format %.4f $teff]
		set terr [format %.4f $terr]
		array set ga $g
		if {![info exists cells($name)]} {
			Dialog_Warn "cell $name is not in the database.\nWill use fake data. PLEASE FIX THIS"
			array set lca {glassTrans1 0.86 Lcm 10 Hcm 10 rCURVcm 25 opacity1 1 opacity1SD 0}
		} else {
			#puts "cellname= $name"
			set ca $cells($name)
			global $ca
			#puts "celldb arraylist= [array get $ca]"
			array set lca [array get $ca]
		}
		set line "$line [format %.2f $ga(beamArea)] $teff $terr [format %.4f $ga(feff)] [format %.4f $ga(ferr)]"
		set line "$line [format %.4f $lca(glassTrans1)] 0 [format %.2f $lca(Lcm)] [format %.2f $lca(Hcm)] [format %.2f $lca(rCURVcm)]"
		set line "$line [format %.4f $lca(opacity1)] [format %.4f $lca(opacity1SD)]"
		set line "$line 60 3.35416 [format %.0f $ga(Hcols)] 9999 [format %.3f $ga(Vcols)] 9999"
		set line "$line [format %.3f $da(hrs)]"
		set line "$line [format %.1f $ga(S3)] [format %.1f $ga(DD)] [format %.1f $ga(SH)] 0 0 0 [format %.6f $ga(rfLossPerFlip)] [format %.0f $ga(SampleMos)]"
		lappend flines $line
	}
	#puts "flines= $flines"
	set newflines [formatlines $flines $cfspc]
	#puts "formattedlines= $newflines"
	foreach newl $newflines {
		$tx insert end "$newl\n"
	}
	set cellFileData($fnam) $newflines
	$tx insert end "\n"
	return $fnam
}

proc formatlines {lines csp} {
	set nl [llength $lines]
	if {$nl < 1} {return {}}
	set sp [format %-$csp.$csp\s " "]
	set nc [llength [lindex $lines 0]]
	array set cw {}
	for {set i 0} {$i < $nc} {incr i} {set cw($i) 0}
	foreach line $lines {
		set lnc [llength $line]
		if {$lnc != $nc} {
			puts "line doesnt have $nc columns = $line"
			return {}
		}
		for {set i 0} {$i < $nc} {incr i} {
			set wdl [string length [lindex $line $i]]
			if {$wdl > $cw($i)} {set cw($i) $wdl}
		}
	}
	set newlines {}
	set nc1 [expr $nc - 1]
	foreach line $lines {
		set newline ""
		for {set i 0} {$i < $nc} {incr i} {
			set wd [lindex $line $i]			
			set wdf [format %-$cw($i).$cw($i)\s $wd]
			if {$i < $nc1} {
				set newline "$newline$wdf$sp"
			} else {
				set newline "$newline$wdf"
			}
		}
		lappend newlines $newline		
	}
	return $newlines
}

proc filesINutcRange {utci utcf} {
    global pbfiles
    set rfiles {}
    set seq 0
    if {$utcf <= $utci} {return $rfiles}
    foreach f $pbfiles {
        set futc [lindex $f 1]
        if [valueInMinMax $futc $utci $utcf] {
            set seq 1
            lappend rfiles $f
        } else {
            if {$seq == 1} {return $rfiles}
        }
    }
    return $rfiles
}
proc nfilesINutcRange {utci utcf} {
    return [llength [filesINutcRange $utci $utcf]]
}

proc frINutcRange {utci utcf} {
    global flipratios
    set nin 0
    foreach utc $flipratios {
        if {$utc >= $utci && $utc <= $utcf} {incr nin}
    }
    return $nin
}

proc effsINutcRange {m utci utcf} {
	global effs
	set effsR {}
	foreach e $effs($m) {
		set ut [lindex $e 1]
		if {$ut >= $utci && $ut <= $utcf} {
			lappend effsR [list [lindex $e 2] [lindex $e 3]]
		}
	}
	return $effsR	
}

proc filesINenvRange {bfld brng temp trng} {
	global pbfiles
    set db [expr $brng/2.]
    set dt [expr $trng/2.]
    set bmin [expr $bfld - $db]
    set bmax [expr $bfld + $db]
    set tmin [expr $temp + $dt]
    set tmax [expr $temp - $dt]
    set rfiles {}
    foreach f $pbfiles {
        set bf [lindex $f 8]
        set br [expr [lindex $f 9]/2.]
        set bfmn [expr $bf - $br]
        set bfmx [expr $bf + $br]
        set tf [lindex $f 10]
        set tr [expr [lindex $f 11]/2.]
        set tfmn [expr $tf - $tr]
        set tfmx [expr $tf + $tr]
        if {[rangesOverlap  $bfmn $bfmx $bmin $bmax] && [rangesOverlap $tfmn $tfmx $tmin $tmax]} {
            lappend rfiles $f
        }
    }
    return $rfiles
}

proc filesVQ {vq} {
	global pbfiles
	set rfiles {}
	foreach f $pbfiles {
		set fvq [lindex $f 2]
		if {$fvq == $vq} {
			lappend rfiles [lindex $f 7]
		}
	}
	return $rfiles
}


proc truncEnv {x} {
	set x [string trimright $x 0]
	set x [string trimright $x .]
	if {[string length $x] < 1} {return 0}
	return $x
}

proc dashList {xlst} {
	set nx [llength $xlst]
	if {$nx < 1} {return 0}
	set ds [lindex $xlst 0]
	for {set i 1} {$i < $nx} {incr i} {
		set ds "$ds-[lindex $xlst $i]"
	}
	return $ds
}

proc sameEnv {envk envl} {
    global envTempTol envTempZeroTol envBfldTol
    set bdif [expr abs([lindex $envk 0] - [lindex $envl 0])]
    set bsame 0
    if {$bdif <= $envBfldTol} {set bsame 1}
    set tk [lindex $envk 1]
    set tl [lindex $envl 1]
    set tol $envTempTol
    if {$tk <= 0 || $tl <= 0} {set tol $envTempZeroTol}
    set tdif [expr abs($tk - $tl)]
    set tsame 0
    if {$tdif <= $tol} {set tsame 1}
    if {$tsame == 1 && $bsame == 1} {return 1}
    return 0   
}
proc newDistinctEnv {mode envl} {
    global distinctEnv
    set nu [llength $distinctEnv($mode)]
    for {set k 0} {$k < $nu} {incr k} {
        set envk [lindex $distinctEnv($mode) $k]
        if {[sameEnv $envk $envl]} {
            set efflstk [lindex $envk 2]
            set efflstl [lindex $envl 2]
            # list from envl has only one member. Add that member to the current unique envk efflstk
            lappend efflstk [lindex $efflstl 0]
            set envk [lreplace $envk 2 2 $efflstk]
            set distinctEnv($mode) [lreplace $distinctEnv($mode) $k $k $envk]
            return 0
        }
    }
    return 1
}

proc sameEffvalues {effk effl} {
	# here effk and effl are lists of eff efferr
	global transportEffCI
	set avek [lindex $effk 0]
	set errk [lindex $effk 1]
	set avel [lindex $effl 0]
	set errl [lindex $effl 1]
	set dave [expr abs($avek - $avel)]
	# use the average err
    set errave [expr 0.5*($errk + $errl)]
    if {[expr $dave/$errave] <= $transportEffCI} {return 1}
    return 0
}
proc sameEff {effk effl} {
    # effk and effl are lists with valueslist effsum errsum ave err
    global transportEffCI
    set avek [lindex $effk 3]
    set avel [lindex $effl 3]
    set errk [lindex $effk 4]
    set errl [lindex $effl 4]
    set cmp [sameEffvalues [list $avek $errk] [list $avel $errl]]
    puts "sameEff  sameEffvalues=$cmp  for 1= $avek $errk  2= $avel $errl"
    return $cmp
}
proc newDistinctEff {mode effll} {
    # effll has list list b t effave efferr followed by effsum errsum ave err
    # where ave err are new values
    # return 1=newDistinct if effl doesnt match any existing elements of distinctEff(mode)
    global distinctEff
    set nu [llength $distinctEff($mode)]
    for {set k 0} {$k < $nu} {incr k} {
        set effk [lindex $distinctEff($mode) $k]
        if {[sameEff $effk $effll]} {
        	# add effl to effk
        	set effllst [lindex $effll 0]
        	set ave [lindex $effll 3]
        	set err [lindex $effll 4]
        	set errsq [expr 1./($err*$err)]
        	set effsum [lindex $effk 1]
        	set errsum [lindex $effk 2]
        	set effsum [expr $effsum + $ave*$errsq]
        	set errsum [expr $errsum + $errsq]
        	set ave [expr $effsum/$errsum]
        	set err [expr 1./sqrt($errsum)]
        	set effklst [lindex $effk 0]
            lappend effklst [lindex $effllst 0]
            set effk [list $effklst $effsum $errsum $ave $err]
            set distinctEff($mode) [lreplace $distinctEff($mode) $k $k $effk]
            return 0
        }
    }
    return 1
}

proc distinctValues {xlst trunc} {
	set dvs {}
	set nx [llength $xlst]
	if {$nx < 1} return $dvs
	set x [lindex $xlst 0]
	if {$trunc} {set x [truncEnv $x]}
	lappend dvs $x
	for {set i 1} {$i < $nx} {incr i} {
		set x [lindex $xlst $i]
		if {$trunc} {set x [truncEnv $x]}
		if {[lsearch -exact -real $dvs $x] >= 0} {continue}
		lappend dvs $x
	}
	return $dvs
}

proc normDevs {ave xelst} {
    set nds {}
    foreach xe $xelst {
        set x [lindex $xe 0]
        set e [lindex $xe 1]
        lappend nds [expr abs($x - $ave)/$e]
    }
    return $nds
}

proc averageErr {xelst} {
    # xelst is list of lists= value error
    set xsum 0
    set esum 0
    foreach xe $xelst {
        set x [lindex $xe 0]
        set e [lindex $xe 1]
        set esq [expr $e*$e]
        if {$esq < 0} {set esq 1.}
        set esq [expr 1./$esq]
        set esum [expr $esum + $esq]
        set xsum [expr $xsum + $x*$esq]
    }
    set ave [expr $xsum/$esum]
    set err [expr sqrt(1./$esum)]
    return [list $ave $err]
}

#---------------------------------------------------------------------------------------
# communication with Tclpbs package, including debug stuff
set fptstack {}
set curFPT ""
set readfinished 0
set nread 0
set nerr 0
set nwarn 0
set readlines {}
set errlines {} 
set warnlines {}
set readline ""

proc newfilestack {filename} {
    global fptstack curFPT readline
    if [catch "open $filename r" fileId] {
		set readline "ERROR: failed to open $filename"
		pipeRead
		return 0
    } else {
		set curFPT $fileId
		set fptstack [linsert $fptstack 0 $fileId]
    }
    return 1
}

proc filestack {filename} {
    global readline curFPT fptstack
    # called from pipeWrite in library mode to implement cmndfile reading
    # if open filename succeeds increment the filestack
    # and start reading lines from this file
    # and passing them to pipeWrite
    # when EOF is reached close the file and remove it from the stack
    # if an @file is found in the current file it will be trapped
    # by pipeWrite to recall this filestack proc
    # and increment the stack
    # the stack is just implemented as a list
    # with new fileptrs inserted at the beginning of the list
    # so that when the file is closed just remove the list first element
    # and proceed with the new first element if it exists

    # newfilestack will try to open the file and set curFPT
    # if the fileopen fails newfilestack returns 0

    if { ! [newfilestack $filename] } { return }

    # as long as there are lines from curFPT
    # or there are more files on the stack the following loop will run

    while { 1 } {
	set nb [gets $curFPT cmd]
	if { $nb < 0 } {
	    # eof  close curFPT and remove it from the stack
	    close $curFPT
	    set fptstack [lreplace $fptstack 0 0]
	    # if stack is empty we are done with cmnd files
	    if { [llength $fptstack] < 1 } { return }
	    set curFPT [lindex $fptstack 0]
	} elseif { $nb > 0 } {
	    if { [string match @* $cmd] } {
		set cmd [string trimleft $cmd @]
		newfilestack $cmd
		# if newfilestack succeeds curFPT will update
		# if it failes an error message get printed and no update
	    } else {
		pipeWrite $cmd
	    }
	}
	# this will skip blank lines in cmnd files
    }
}

# pbscript pipe write
# this must now also handle Tclpbs pbsinit and pbs
# This always uses Tclpbs package, not pipe

# tried using open the commandline program on pipe, but could NOT get this to work. Hangs up
# maybe have to configure the pipe nonblocking or maybe have to flush stdout
proc openio {cmd} {
	global fptCL programCL
	if {[string length $fptCL] < 1} {
		if [catch {set fptCL [open |$programCL r+]}] {
			Dialog_Warn "ERROR: Failed pipe to $programCL"
			exit
		}
	}
	puts $fptCL $cmd
	flush $fptCL
	set lines [read $fptCL]
	#puts "CL read returned lines= $lines"
    foreach readline [split $lines \n] {
	   	pipeRead
    }
    return
}

proc pipeWrite {cmd} {
    global readfinished
    global nread nerr nwarn
    global readlines errlines warnlines
    global DEBUG debugFPT
    global useTclpbs
    global readline
    global outtxt debugFPT
    global CL fptCL fptCLout programCL fileCL fileCLout execrunning sleepms

    # clear the readlines list and the readfinished flag
    set readlines {}
    set errlines ""
    set warnlines ""
    set nread 0
    set nerr 0
    set nwarn 0
    set readfinished 0
    
    #puts "starting pipeWrite"
    #flush stdout

    if { $DEBUG && [info exists debugFPT] } {
	    $outtxt insert end "$cmd\n"
	    puts $debugFPT $cmd
	    flush $debugFPT
	    # since debug can run into crashes we save the file by closing it after every write
	    close $debugFPT
	    # then reopen
	    set debugFPT [open debugCMDS a+]
    }


    # handle Tclpbs scripting extension library
    # NB cmndfile reading via @filename needs to be handled here
    # as libpbscript is using a dyn string for the result
    if { [string match @* $cmd] } {
	    set cmd [string trimleft $cmd @]
	    filestack $cmd
	    return
    }
	# if CL is 1 use the cmndline program on a pipe
	# This is not working
	# Try exec programCL -fp in background
	if {$CL == 1} {
		#openio $cmd
		puts "pipeWrite starting CL mode"
		if {$execrunning == 0} {
			# the -fp flag will define the io files and set the filepoll flag
			if [catch {exec $programCL -fp $fileCL &}] {
				puts "exec $programCL failed"
				exit
			}
			set execrunning 1
		}
		# fileCL is relative filename wrt to current working directory
		if [catch {set fptCL [open $fileCL w]}] {
			puts "open $fileCL w  failed"
			exit
		}
		puts $fptCL $cmd
		close $fptCL
		# we must not open the fileCLout until programCL has written
		# as soon as programCL poll reads fptCL it removes any last fileCLout
		# so now poll for the fileCLout
		# programCL is supposed to have same working directory so we should still be able
		# to use relative io file names as we do in the exec
		set secs 0
		set secscheck 10
		set dsecs [expr $sleepms/1000.]
		while {$secs < 1000} {
			if [catch {set fptCLout [open $fileCLout r]}] {
				after $sleepms
				set secs [expr $secs + $dsecs]
				if {$secs > $secscheck} {
					puts "open $fileCLout waited $secs seconds"
					set secscheck [expr $secscheck + 10]
				}
			}
			
		}
		set lines [read $fptCLout]
		close $fptCLout
	} else {

    	# if not @cmndfile execute cmd and get result in lines
    	set lines [pbs $cmd]
    }
    # separate the lines into global readline for each one
    # and call pipeRead each line
    # this generates the readlines list and any errlines warnlines
    foreach readline [split $lines \n] {
	    pipeRead
    }
    # either remove the vwait readfinished or just set it in pipeRead
    # NB there is no fileevent loop polling for Tclpbs
    # so vwait will not work
}


proc waitRead {maxms} {
    # could replace the vwait after pipeWrite with this proc; waitRead maxms
    # if problems with pipe death due to timing.
    global readfinished
    if {$maxms > 0} {
	after $maxms set readfinished -1
    }
    vwait readfinished
    if { $readfinished < 0 } {
	Dialog_Warn "read time limit exceded. pbscript pipe may have died"
    } elseif {$maxms > 0} {
	after cancel set readfinished -1
    }
}

# pbscript callback for reading
# normally skip all blank lines in output to parse correctly
# if pipeRead is used with formatted output just turn off the skipblanklines
set skipblanklines 1
proc pipeRead {} {
    # called via fileevent when pipe ready for reading a line
    # or called directly when Tclpbs ext is used
    global pipe
    global readfinished
    global readlines errlines warnlines
    global nread nerr nwarn
    global DEBUG sessionrecover saverecoverErr
    global skipblanklines
    global readline


	# Tclpbs
	# the split cmnd has discarded the nl
	set line $readline
	set nb [string length $line]

	if { $DEBUG } {
	    global outtxt
	    $outtxt insert end "$line\n"
	}
	if { $skipblanklines } {
	    set line [string trim $line]
	    if { [string length $line] < 1 } { return }
	}
	# end of output=END is specific to pbscript
	# collect readlines until END
	if [string match $line "END"] {
	    set readfinished 1
	} else {
	    if { [string match ERROR* $line] } {
		    append errlines "$line\n"
		    incr nerr
	    } elseif { [string match WARN* $line] } {
		    append warnlines "$line\n"
		    incr nwarn
	    } else {
		    lappend readlines $line
		    incr nread
	    }
	}
}

proc quit {} {
	global CL fptCL
	if {$CL == 1} {
		close $fptCL
	}
	exit
}
